module config

    use ISO_FORTRAN_ENV
    implicit none

    integer, parameter :: NCONFIG = 512
    integer, parameter :: LENVALUES = 2048
    integer, parameter :: LENCONFIG = 128
    integer, parameter, private :: dp = REAL64

    interface stringToScalar
        module procedure stringToLogicalScalar
        module procedure stringToIntegerScalar
        module procedure stringToRealScalar
    end interface stringToScalar

    interface stringToArray
        module procedure stringToIntegerArray
        module procedure stringToRealArray
    end interface stringToArray

contains

    subroutine readLine(funit, status, line)
        implicit none
        integer, intent(in) :: funit
        integer, intent(out) :: status
        character(len = :), allocatable, intent(out) :: line
        integer :: size
        logical :: init
        character(len = 256) :: linefixed
        init = .true.
        do
            read(funit, "(a)", advance = 'NO', iostat = status, size = size) linefixed
            if ( IS_IOSTAT_END(status) ) exit
            if ( init ) then
                line = linefixed(1:size)
                init = .false.
            else
                line = line // linefixed(1:size)
            end if
            if ( IS_IOSTAT_EOR(status) ) then
                status = 0
                exit
            end if
        end do
    end subroutine readLine

    subroutine readconfig(file_name, var_name, var_value, var_group, var_id)

        implicit none

        character(len = *),   intent(in)  :: file_name
        character(len = LENCONFIG), intent(out) :: var_name(NCONFIG), var_group(NCONFIG), var_id(NCONFIG)
        character(len = LENVALUES), intent(out) :: var_value(NCONFIG)

        ! define variable
        integer :: statIn, unit, iX, jX, nX, ixDot
        character(len = :), allocatable :: trimmed_line, line
        character(len = LENCONFIG)  :: group, id

        group = 'default'

        open(newunit = unit, file = file_name, status = "old")

        nX = 0

        do

            call readLine(unit, statIn, line)

            ! reached file end
            if (statIn /=0 ) exit

            ! get rid of spaces
            trimmed_line = trim(adjustl(line))
            call compact(trimmed_line)

            ! skip empty lines
            if (trimmed_line == "") cycle

            ! skip comments
            if (trimmed_line(1:1) == "#") cycle

            if (trimmed_line(1:1) == "[") then
                ixDot = scan(trimmed_line, ".")
                if (ixDot>0) then
                    iX = scan(trimmed_line, "]")
                    id = trimmed_line(ixDot + 1 : iX - 1)
                    group = trimmed_line(2 : ixDot - 1)
                else
                    iX = scan(trimmed_line, "]")
                    id = ""
                    group = trimmed_line(2 : iX - 1)
                end if
                cycle
            end if

            ! find the variable name (moving forward)
            jX = len_trim(trimmed_line)
            iX = scan(trimmed_line, " ")
            nX = nX + 1

            if (nX > NCONFIG) then
                write(*, '(A)') 'WARNING: no storage space for record; increase NCONFIG'
                exit
            end if

            var_name(nX) = trimmed_line(1 : iX - 1)
            var_value(nX) = trimmed_line(iX + 1 : jX)
            var_group(nX) = group
            var_id(nX) = id

        end do

        close(unit)

        if (nX < NCONFIG) then
            var_id(nX + 1) = ''
            var_name(nX + 1) = ''
            var_value(nX + 1) = ''
            var_group(nX + 1) = ''
        end if

    end subroutine readconfig

    subroutine compact(str)

        implicit none

        character(len = *), intent(inout) :: str
        character(len = 1) :: ch
        character(len = len_trim(str)) :: outstr

        integer :: iX, jX
        integer :: ich, isp
        integer :: lenstr

        str = adjustl(str)
        lenstr = len_trim(str)
        outstr = ' '
        isp = 0
        jX = 0

        do iX = 1, lenstr
            ch = str(iX:iX)
            ich = iachar(ch)

            select case(ich)
            case(9, 32)     ! space or tab character
                if(isp == 0) then
                    jX = jX + 1
                    outstr(jX : jX) = ' '
                end if
                isp = 1
            case(33:)      ! not a space, quote, or control character
                jX = jX + 1
                outstr(jX : jX) = ch
                isp = 0
            end select

        end do

        str = adjustl(outstr)

    end subroutine compact

    subroutine stringToRealArray(str, arr, narr)
        use, intrinsic :: ieee_arithmetic
        implicit none
        character(len=*), intent(in) :: str
        real(dp), intent(out) :: arr(:)
        integer, intent(out) :: narr
        integer :: lenstr, lenarr, ipos
        integer :: iX, kX, istat
        real(dp) :: errcode

        errcode = IEEE_VALUE(0.0_dp, IEEE_QUIET_NAN)
        lenstr = len(str)
        lenarr = size(arr)

        if (lenstr == 0) return

        kX = 1
        iX = 1

        do

            if (iX > lenarr) then
                write(*, '(A)') 'WARNING: not enough storage to copy parsed string to array.'
                iX = lenarr
                exit
            end if

            if (kX > lenstr) exit

            ! absolute position of next separator
            ipos = index(str(kX:lenstr), ',')

            if (ipos == 0) then
                ! no more separators, read to end and exit
                read (str(kX : lenstr), *, iostat = istat) arr(iX)
                if (istat > 0) arr(iX) = errcode
                exit
            elseif (ipos == 1) then
                ! if the first character is a separator, read as zero
                arr(iX) = 0.0_dp
                kX = kX + 1
            else
                ! read from current position to separator
                read (str(kX:kX + ipos - 2), *, iostat = istat) arr(iX)
                if (istat > 0) arr(iX) = errcode
                kX = kX + ipos
                if (kX > lenstr) then
                    iX = iX + 1
                    if (iX > lenarr) then
                        write(*, '(A)') 'WARNING: not enough storage to copy parsed string to array.'
                        iX = lenarr
                        exit
                    else
                        arr(iX) = 0.0_dp
                        exit
                    end if
                end if
            end if

            iX = iX + 1

        end do

        narr = iX

    end subroutine stringToRealArray

    subroutine stringToIntegerArray(str, arr, narr)
        use, intrinsic :: ieee_arithmetic
        implicit none
        character(len = *), intent(in) :: str
        integer, intent(out) :: arr(:)
        integer, intent(out) :: narr
        integer :: lenstr, lenarr, ipos
        integer :: iX, kX, istat
        integer :: errcode

        errcode = -huge(0)
        lenstr = len(str)
        lenarr = size(arr)
        if (lenstr == 0) return
        kX = 1
        iX = 1

        do

            if (iX > lenarr) then
                write(*, '(A)') 'WARNING: not enough storage to copy parsed string to array.'
                iX = lenarr
                exit
            end if

            if (kX > lenstr) exit

            ! absolute position of next separator
            ipos = index(str(kX:lenstr), ',')

            if (ipos == 0) then
                ! no more separators, read to end and exit
                read (str(kX : lenstr), *, iostat = istat) arr(iX)
                if (istat > 0) arr(iX) = errcode
                exit
            elseif (ipos == 1) then
                ! if the first character is a separator, read as zero
                arr(iX) = 0
                kX = kX + 1
            else
                ! read from current position to separator
                read (str(kX : kX + ipos - 2), *, iostat = istat) arr(iX)
                if (istat > 0) arr(iX) = errcode
                kX = kX + ipos
                if (kX > lenstr) then
                    iX = iX + 1
                    if (iX > lenarr) then
                        write(*, '(A)') 'WARNING: not enough storage to copy parsed string to array.'
                        iX = lenarr
                        exit
                    else
                        arr(iX) = 0
                        exit
                    end if
                end if
            end if

            iX = iX + 1

        end do

        narr = iX

    end subroutine stringToIntegerArray


    subroutine stringToIntegerScalar(str, scalar)
        implicit none
        character(len = *), intent(in) :: str
        integer, intent(out) :: scalar
        integer :: istat
        read (str, *, iostat = istat) scalar
    end subroutine stringToIntegerScalar

    subroutine stringToRealScalar(str, scalar)
        implicit none
        character(len = *), intent(in) :: str
        real(dp), intent(out) :: scalar
        integer :: istat
        read (str, *, iostat = istat) scalar
    end subroutine stringToRealScalar

    subroutine stringToLogicalScalar(str, scalar)
        implicit none
        character(len = *), intent(in) :: str
        logical, intent(out) :: scalar
        integer :: istat
        read (str, *, iostat = istat) scalar
    end subroutine stringToLogicalScalar

    function getStringArrayLength(str) result(narr)
        implicit none
        character(len = *), intent(in) :: str
        integer :: narr
        integer :: lenstr
        integer :: ipos
        integer :: kX

        lenstr = len(str)

        if (lenstr == 0) then

            narr = 0
            return

        else

            narr = 1
            kX = 1
            do
                ipos = index(str(kX:), ',')
                if (ipos == 0) return
                narr = narr + 1
                kX = kX + ipos
            end do

        end if

    end function getStringArrayLength

    subroutine unknownVariableWarning(var_name, var_group)
        implicit none
        character(len = *), intent(in) :: var_name
        character(len = *), intent(in) :: var_group
        write(*,'(A)') 'WARNING: unrecognized variable name (' // trim(adjustl(var_name)) // &
            ') in group (' // trim(adjustl(var_group)) // ')'
    end subroutine unknownVariableWarning

    subroutine allEqual(lname, i1, i2, i3, i4, i5, i6, i7, i8, i9, i10)
        implicit none
        character(len=*), intent(in) :: lname
        integer, intent(in) :: i1, i2
        integer, intent(in), optional :: i3, i4, i5, i6, i7, i8, i9, i10
        logical :: err
        err = .false.

        if (i1 /= i2) err = .true.

        if (present(i3)) then
            if (i1 /= i3) err = .true.
        end if
        if (present(i4)) then
            if (i1 /= i4) err = .true.
        end if
        if (present(i5)) then
            if (i1 /= i5) err = .true.
        end if
        if (present(i6)) then
            if (i1 /= i6) err = .true.
        end if
        if (present(i7)) then
            if (i1 /= i7) err = .true.
        end if
        if (present(i8)) then
            if (i1 /= i8) err = .true.
        end if
        if (present(i9)) then
            if (i1 /= i9) err = .true.
        end if
        if (present(i10)) then
            if (i1 /= i10) err = .true.
        end if

        if (err) then
            write(*, '(A)') 'All values in ' // adjustl(trim(lname)) // ' are not equal.'
            stop
        end if

    end subroutine allEqual

end module config
