module environment
    use globaldef
    implicit none

    integer, parameter, private :: maxhhsize = 8

contains

    subroutine readFrac(fname, verbose, pM, pF, h)
        use config
        use mathfn
        implicit none
        character(len=*), intent(in) :: fname
        logical, intent(in) :: verbose

        ! individual type attibutes
        type(p_t), intent(inout) :: pM(NTYPE1)
        type(p_t), intent(inout) :: pF(NTYPE2)

        ! household type attibutes
        type(h_t) :: h(NTYPE1, NTYPE2)

        integer :: zX, iX
        integer :: tempInt, tempIntArray(2)

        character(len = LENCONFIG) :: var_name(NCONFIG)
        character(len = LENVALUES) :: var_value(NCONFIG)
        character(len = LENCONFIG) :: var_group(NCONFIG)
        character(len = LENCONFIG) :: var_id(NCONFIG)

        call readConfig(fname, var_name, var_value, var_group, var_id)

        do iX = 1, NCONFIG

            if (len_trim(var_id(iX)) > 0) then
                call stringToScalar(var_id(iX),zX)
            else
                zX = 1
            end if

            if ( var_name(iX) == '') exit

            if ( verbose ) then
                write(*, '(A)') '"' // trim(var_group(iX)) // '"' // ', "' // trim(var_name(iX)) // &
                                '"', ', "' // trim(var_value(iX)) // '"'
            end if

            select case(var_group(iX))
            case('NUMTYPES')
                select case(var_name(iX))
                case('NUM_MALE')
                    call stringToScalar(var_value(iX), tempInt)
                    if ( tempInt /= NTYPE1 ) then
                        write(*, '(A)') 'ERROR: input NUM_MALE (' // trim(adjustl(var_value(iX))) // &
                                        ') must equal NTYPE1 (' // intToStr(NTYPE1) // ')'
                        stop
                    end if
                case('NUM_FEMALE')
                    call stringToScalar(var_value(iX), tempInt)
                    if ( tempInt /= NTYPE2 ) then
                        write(*, '(A)') 'ERROR: input NUM_FEMALE (' // trim(adjustl(var_value(iX))) // &
                                        ') must equal NTYPE2 (' // intToStr(NTYPE2) // ')'
                        stop
                    end if
                case default
                    call unknownVariableWarning(var_name(iX), var_group(iX))
                end select
            case('MALE_TYPE')
                call stringToScalar(var_name(iX), tempInt)
                if (tempInt >= 1 .and. tempInt <= NTYPE1) then
                    call stringToScalar( var_value(iX), pM(tempInt)%frac )
                else
                    write(*, '(A)') 'ERROR: male type is out of range'
                    stop
                end if
            case('FEMALE_TYPE')
                call stringToScalar(var_name(iX), tempInt)
                if (tempInt >= 1 .and. tempInt <= NTYPE2) then
                    call stringToScalar( var_value(iX), pF(tempInt)%frac )
                else
                    write(*, '(A)') 'ERROR: female type is out of range'
                    stop
                end if
            case('MALE_KIDS')
                call stringToScalar(var_name(iX), tempInt)
                if (tempInt >= 1 .and. tempInt <= NTYPE1) then
                    call stringToScalar( var_value(iX), pM(tempInt)%probTypeK )
                else
                    write(*, '(A)') 'ERROR: male kids is out of range'
                    stop
                end if
            case('FEMALE_KIDS')
                call stringToScalar(var_name(iX), tempInt)
                if (tempInt >= 1 .and. tempInt <= NTYPE2) then
                    call stringToScalar( var_value(iX), pF(tempInt)%probTypeK )
                else
                    write(*, '(A)') 'ERROR: female kids is out of range'
                    stop
                end if
            case('JOINT_KIDS')
                call stringToIntegerArray(var_name(iX), tempIntArray, tempInt)
                if (tempIntArray(1) >= 1 .and. tempIntArray(1) <= NTYPE1) then
                    if (tempIntArray(2) >= 1 .and. tempIntArray(1) <= NTYPE2) then
                        call stringToScalar( var_value(iX), h(tempIntArray(1), tempIntArray(2))%probTypeK )
                    else
                        write(*, '(A)') 'ERROR: joint kids (female index) is out of range'
                        stop
                    end if
                else
                    write(*, '(A)') 'ERROR: joint kids (male index) is out of range'
                    stop
                end if
            case('MALE_KIDS_TAX')
                call stringToScalar(var_name(iX), tempInt)
                if (tempInt >= 1 .and. tempInt <= NTYPE1) then
                    call stringToScalar( var_value(iX), pM(tempInt)%probKidTax )
                else
                    write(*, '(A)') 'ERROR: male kids tax is out of range'
                    stop
                end if
            case('FEMALE_KIDS_TAX')
                call stringToScalar(var_name(iX), tempInt)
                if (tempInt >= 1 .and. tempInt <= NTYPE2) then
                    call stringToScalar( var_value(iX), pF(tempInt)%probKidTax )
                else
                    write(*, '(A)') 'ERROR: female kids tax is out of range'
                    stop
                end if
            case default
                write(*, '(A)') 'WARNING: unrecognized group name (' // trim(adjustl(var_group(iX))) // ')'
            end select

        end do

    end subroutine readFrac

    subroutine readFracKids(fname, verbose, pM, pF, h, m, pMscale, pFscale)
        use config
        use mathfn
        implicit none
        character(len = *), intent(in) :: fname
        logical, intent(in) :: verbose

        ! individual type attibutes
        type(p_t), intent(inout) :: pM(NTYPE1, 0:1)
        type(p_t), intent(inout) :: pF(NTYPE2, 0:1)

        ! household type attibutes
        type(h_t), intent(inout) :: h(NTYPE1, NTYPE2, 0:1)

        ! market attibutes
        type(m_t), intent(inout) :: m

        ! scale population vectors
        real(dp), intent(in), optional :: pMscale(NTYPE1)
        real(dp), intent(in), optional :: pFscale(NTYPE2)

        integer :: zX, iX
        integer :: tempInt, tempIntArray(2)
        integer :: nCount

        integer :: pX1, pX2

        character(len = LENCONFIG) :: var_name(NCONFIG)
        character(len = LENVALUES) :: var_value(NCONFIG)
        character(len = LENCONFIG) :: var_group(NCONFIG)
        character(len = LENCONFIG) :: var_id(NCONFIG)

        real(dp) :: thisFrac

        call readConfig(fname, var_name, var_value, var_group, var_id)

        m%marketId = 0
        m%taxId = 0
        pM%probKidTax = 0.0_dp
        pF%probKidTax = 0.0_dp

        pF%nly = 0.0_dp
        pM%nly = 0.0_dp
        pF%wlf = 0.0_dp
        pM%wlf = 0.0_dp

        do iX = 1, NCONFIG

!             if (var_id(iX) /= '') then
            if (len_trim(var_id(iX)) > 0) then
                call stringToScalar(var_id(iX), zX)
            else
                zX = 1
            end if

            if ( var_name(iX) == '') exit

            if ( verbose ) then
                write(*, '(A)') '"' // trim(var_group(iX)) // '"' // ', "' // trim(var_name(iX)) // &
                                '"', ', "' // trim(var_value(iX)) // '"'
            end if

            select case(var_group(iX))
            case('MARKET')
                select case(var_name(iX))
                case('COUNT')
                    call stringToScalar( var_value(iX), nCount )
                case('SIZE')
                    call stringToScalar( var_value(iX), m%marketSize )
                case('ID')
                    call stringToScalar( var_value(iX), m%marketId )
                case('TAXID')
                    call stringToScalar( var_value(iX), m%taxId )
                case('NAME')
                    m%name = var_value(iX)
                case default
                    call unknownVariableWarning(var_name(iX), var_group(iX))
                end select
            case('NUMTYPES')
                select case(var_name(iX))
                case('NUM_MALE')
                    call stringToScalar(var_value(iX), tempInt)
                    if ( tempInt /= NTYPE1 ) then
                        write(*, '(A)') 'ERROR: input NUM_MALE (' // trim(adjustl(var_value(iX))) // &
                                        ') must equal NTYPE1 (' // intToStr(NTYPE1) // ')'
                        stop
                    end if
                case('NUM_FEMALE')
                    call stringToScalar(var_value(iX), tempInt)
                    if ( tempInt /= NTYPE2 ) then
                        write(*, '(A)') 'ERROR: input NUM_FEMALE (' // trim(adjustl(var_value(iX))) // &
                                        ') must equal NTYPE2 (' // intToStr(NTYPE2) // ')'
                        stop
                    end if
                case default
                    call unknownVariableWarning(var_name(iX), var_group(iX))
                end select
            case('MALE_TYPE')
                call stringToScalar(var_name(iX), tempInt)
                if (tempInt >= 1 .and. tempInt <= NTYPE1) then
                    call stringToScalar( var_value(iX), thisFrac )
                    pM(tempInt, 1)%frac = thisFrac
                else
                    write(*, '(A)') 'ERROR: male type is out of range'
                    stop
                end if
            case('FEMALE_TYPE')
                call stringToScalar(var_name(iX), tempInt)
                if (tempInt >= 1 .and. tempInt <= NTYPE2) then
                    call stringToScalar( var_value(iX), thisFrac )
                    pF(tempInt, 1)%frac = thisFrac
                else
                    write(*, '(A)') 'ERROR: female type is out of range'
                    stop
                end if
            case('MALE_NONLABOUR')
                call stringToScalar(var_name(iX), tempInt)
                if (tempInt >= 1 .and. tempInt <= NTYPE1) then
                    call stringToScalar( var_value(iX), thisFrac )
                    pM(tempInt, 1)%nly = thisFrac
                else
                    write(*, '(A)') 'ERROR: male non-labour is out of range'
                    stop
                end if
            case('FEMALE_NONLABOUR')
                call stringToScalar(var_name(iX), tempInt)
                if (tempInt >= 1 .and. tempInt <= NTYPE2) then
                    call stringToScalar( var_value(iX), thisFrac )
                    pF(tempInt, 1)%nly = thisFrac
                else
                    write(*, '(A)') 'ERROR: female non-labour is out of range'
                    stop
                end if
            case('WELFARE_NOKIDS')
                call stringToScalar(var_name(iX), tempInt)
                if (tempInt >= 1 .and. tempInt<=max(NTYPE1, NTYPE2)) then
                    call stringToScalar( var_value(iX), thisFrac )
                    if (tempInt <= NTYPE1) then
                        pM(tempInt, 0)%wlf = thisFrac
                    end if
                    if (tempInt <= NTYPE2) then
                        pF(tempInt, 0)%wlf = thisFrac
                    end if
                else
                    write(*, '(A)') 'ERROR: welfare no kids is out of range'
                    stop
                end if
            case('WELFARE_KIDS')
                call stringToScalar(var_name(iX), tempInt)
                if (tempInt >= 1 .and. tempInt <= max(NTYPE1, NTYPE2)) then
                    call stringToScalar( var_value(iX), thisFrac )
                    if (tempInt <= NTYPE1) then
                        pM(tempInt, 1)%wlf = thisFrac
                    end if
                    if (tempInt <= NTYPE2) then
                        pF(tempInt, 1)%wlf = thisFrac
                    end if
                else
                    write(*, '(A)') 'ERROR: welfare kids is out of range'
                    stop
                end if
            case('MALE_KIDS')
                call stringToScalar(var_name(iX), tempInt)
                if (tempInt >= 1 .and. tempInt <= NTYPE1) then
                    call stringToScalar( var_value(iX), thisFrac )
                    pM(tempInt, 1)%probTypeK = thisFrac
                else
                    write(*, '(A)') 'ERROR: male kids is out of range'
                    stop
                end if
            case('FEMALE_KIDS')
                call stringToScalar(var_name(iX), tempInt)
                if (tempInt >= 1 .and. tempInt <= NTYPE2) then
                    call stringToScalar( var_value(iX), thisFrac )
                    pF(tempInt, 1)%probTypeK = thisFrac
                else
                    write(*, '(A)') 'ERROR: female kids is out of range'
                    stop
                end if
            case('JOINT_KIDS')
                call stringToIntegerArray(var_name(iX), tempIntArray, tempInt)
                if (tempIntArray(1) >= 1 .and. tempIntArray(1) <= NTYPE1) then
                    if (tempIntArray(2) >= 1 .and. tempIntArray(2) <= NTYPE2) then
                        call stringToScalar( var_value(iX), thisFrac )
                        h(tempIntArray(1), tempIntArray(2), 1)%probTypeK = thisFrac
                    else
                        write(*, '(A)') 'ERROR: joint kids (female index) is out of range'
                        stop
                    end if
                else
                    write(*, '(A)') 'ERROR: joint kids (male index) is out of range'
                    stop
                end if
            case('MALE_KIDS_TAX')
                call stringToScalar(var_name(iX), tempInt)
                if (tempInt >= 1 .and. tempInt <= NTYPE1) then
                    call stringToScalar( var_value(iX), pM(tempInt, 1)%probKidTax )
                else
                    write(*, '(A)') 'ERROR: male kids tax is out of range'
                    stop
                end if
            case('FEMALE_KIDS_TAX')
                call stringToScalar(var_name(iX), tempInt)
                if (tempInt >= 1 .and. tempInt <= NTYPE2) then
                    call stringToScalar( var_value(iX), pF(tempInt, 1)%probKidTax )
                else
                    write(*, '(A)') 'ERROR: female kids tax is out of range'
                    stop
                end if

            case default
                write(*, '(A)') 'WARNING: unrecognized group name (' // trim(adjustl(var_group(iX))) // ')'
            end select

        end do

        ! -----------------------------------------

        do pX1 = 1, NTYPE1
            pM(pX1, 0)%frac = pM(pX1, 1)%frac
            pM(pX1, 0)%nly  = pM(pX1, 1)%nly
            pM(pX1, 0)%probTypeK = 1.0_dp - pM(pX1, 1)%probTypeK
        end do

        do pX2 = 1, NTYPE2
            pF(pX2, 0)%frac = pF(pX2, 1)%frac
            pF(pX2, 0)%nly  = pF(pX2, 1)%nly
            pF(pX2, 0)%probTypeK = 1.0_dp - pF(pX2, 1)%probTypeK
        end do
        do pX2 = 1, NTYPE2
            do pX1 = 1, NTYPE1
                h(pX1, pX2, 0)%probTypeK = 1.0_dp - h(pX1, pX2, 1)%probTypeK
            end do
        end do

        ! this perturbs the population vectors
        if (present(pMscale)) then
            pM(:, 0)%frac = pM(:, 0)%frac + pMscale
            pM(:, 1)%frac = pM(:, 1)%frac + pMscale
        end if

        if (present(pFscale)) then
            pF(:, 0)%frac = pF(:, 0)%frac + pFscale
            pF(:, 1)%frac = pF(:, 1)%frac + pFscale
        end if

        if (minval(pM(:, 0)%frac) < 0.0_dp) then
            stop "elements of pM%frac(:, 0) can't be negative"
        end if

        if (minval(pM(:, 1)%frac) < 0.0_dp) then
            stop "elements of pM%frac(:, 1) can't be negative"
        end if

        if (minval(pF(:, 0)%frac) < 0.0_dp) then
            stop "elements of pF%frac(:, 0) can't be negative"
        end if

        if (minval(pF(:, 1)%frac) < 0.0_dp) then
            stop "elements of pF%frac(:, 1) can't be negative"
        end if

        ! weekly terms

        pF%nly = pF%nly / 52.0_dp
        pM%nly = pM%nly / 52.0_dp
        pF%wlf = pF%wlf / 52.0_dp
        pM%wlf = pM%wlf / 52.0_dp

    end subroutine readFracKids

    subroutine readFoodStamps( fname, verbose, tax )

        implicit none

        character(len = *), intent(in) :: fname
        logical, intent(in) :: verbose
        type(tax_t), intent(inout) :: tax

        integer  :: numfpl_val, numfsp_ded, nummax_fspval
        integer  :: fpl_year, fpl_prices, fsp_year, fsp_prices
        real(dp) :: fpl_val(maxhhsize), fsp_ded(maxhhsize), fsp_dedadd, max_fspval(maxhhsize)
        real(dp) :: fpl_add, gross_scale, net_scale, max_fspadd, max_shelter

        call readEnvironmentFoodStamps( fname, verbose, fpl_year, fpl_prices, fpl_val, numfpl_val, fpl_add, fsp_year, fsp_prices, &
            gross_scale, net_scale, fsp_ded, numfsp_ded, fsp_dedadd, max_fspval, nummax_fspval, max_fspadd, max_shelter )

        ! weeklyize
        fpl_val = fpl_val / 52.0_dp
        fpl_add = fpl_add / 52.0_dp
        fsp_ded = fsp_ded / 4.33_dp
        fsp_dedadd = fsp_dedadd / 4.33_dp
        max_fspval = max_fspval / 4.33_dp
        max_fspadd = max_fspadd / 4.33_dp
        max_shelter = max_shelter / 4.33_dp

        ! single no kids
        tax%sinGrossElig = gross_scale * fpl_val(1)
        tax%sinNetElig = net_scale * fpl_val(1)
        tax%sinStdDed = fsp_ded(1)
        tax%sinMaxFsp = max_fspval(1)

        ! single kids
        tax%sinKidsGrossElig = gross_scale * fpl_val(3)
        tax%sinKidsNetElig = net_scale * fpl_val(3)
        tax%sinKidsStdDed = fsp_ded(3)
        tax%sinKidsMaxFsp = max_fspval(3)

        ! couples no kids
        tax%couGrossElig = gross_scale * fpl_val(2)
        tax%couNetElig = net_scale * fpl_val(2)
        tax%couStdDed = fsp_ded(2)
        tax%couMaxFsp = max_fspval(2)

        ! couples kids
        tax%couKidsGrossElig = gross_scale * fpl_val(4)
        tax%couKidsNetElig = net_scale * fpl_val(4)
        tax%couKidsStdDed = fsp_ded(4)
        tax%couKidsMaxFsp = max_fspval(4)

    end subroutine readFoodStamps

    subroutine readTaxes( fdir, mX, verbose, tax )
        use mathfn, only : intToStr
        implicit none
        character(len=*), intent(in) :: fdir
        integer, intent(in) :: mX
        logical, intent(in) :: verbose
        type(tax_t), intent(inout) :: tax

        character(len=16) :: tax_label
        integer :: tax_year, tax_prices, numtax_bands, numtax_rates, numtax_level, tax_state
        real(dp) :: tax_bands(MAXKINKS), tax_rates(MAXKINKS), tax_level(MAXKINKS)

        call readEnvironmentTaxes( fdir // '/taxw_cou_' // intToStr(mX) // '.dat', verbose, tax_label, tax_year, tax_prices, &
                                   tax_state, numtax_bands, numtax_rates, numtax_level, tax_bands, tax_rates, tax_level )

        call setEnvironmentTaxes( tax, tax_label, numtax_bands, tax_bands, tax_rates, tax_level, tax_state )

        call readEnvironmentTaxes( fdir // '/taxw_coukid_' // intToStr(mX) // '.dat', verbose, tax_label, tax_year, tax_prices, &
                                   tax_state, numtax_bands, numtax_rates, numtax_level, tax_bands, tax_rates, tax_level )

        call setEnvironmentTaxes( tax, tax_label, numtax_bands, tax_bands, tax_rates, tax_level, tax_state )

        call readEnvironmentTaxes( fdir // '/taxw_sin_' // intToStr(mX) // '.dat', verbose, tax_label, tax_year, tax_prices, &
                                   tax_state, numtax_bands, numtax_rates, numtax_level, tax_bands, tax_rates, tax_level )

        call setEnvironmentTaxes( tax, tax_label, numtax_bands, tax_bands, tax_rates, tax_level, tax_state )

        call readEnvironmentTaxes( fdir // '/taxw_sinkid_' // intToStr(mX) // '.dat', verbose, tax_label, tax_year, tax_prices, &
                                   tax_state, numtax_bands, numtax_rates, numtax_level, tax_bands, tax_rates, tax_level )

        call setEnvironmentTaxes( tax, tax_label, numtax_bands, tax_bands, tax_rates, tax_level, tax_state )

    end subroutine readTaxes

    subroutine setEnvironmentTaxes( tax, tax_label, numtax_bands, tax_bands, tax_rates, tax_level, tax_state )
        implicit none
        type(tax_t), intent(inout) :: tax
        character(len=16), intent(in) :: tax_label

        integer,  intent(in) :: numtax_bands, tax_state
        real(dp), intent(in) :: tax_bands(MAXKINKS), tax_rates(MAXKINKS), tax_level(MAXKINKS)
        real(dp) :: tax_bands_wk(MAXKINKS), tax_level_wk(MAXKINKS), tax_virtual_wk(MAXKINKS)
        integer :: tX

        ! weekly terms
        do tX = 1, numtax_bands
            tax_bands_wk(tX) = tax_bands(tX) / 52.0_dp
        end do

        ! reconstruct levels from rates
        tax_level_wk(1) = tax_level(1) / 52.0_dp
        do tX = 2, numtax_bands
            tax_level_wk(tX) = tax_level_wk(tX - 1) + ( tax_bands_wk(tX) - tax_bands_wk(tX - 1) ) * tax_rates(tX - 1)
        end do

        ! calculate virtual income
        tax_virtual_wk(1) = tax_level_wk(1)
        do tX = 2, numtax_bands
            tax_virtual_wk(tX) = tax_level_wk(tX) - tax_bands_wk(tX) * tax_rates(tX)
        end do

        select case (tax_label)
            case ('cou')
                tax%couBreaks(1:numtax_bands) = tax_bands_wk(1:numtax_bands)
                tax%couNet(1:numtax_bands) = tax_level_wk(1:numtax_bands)
                tax%couVirtual(1:numtax_bands) = tax_virtual_wk(1:numtax_bands)
                tax%couMtr(1:numtax_bands) = tax_rates(1:numtax_bands)
                tax%couNumKinks = numtax_bands
            case ('coukid')
                tax%couKidsBreaks(1:numtax_bands) = tax_bands_wk(1:numtax_bands)
                tax%couKidsNet(1:numtax_bands) = tax_level_wk(1:numtax_bands)
                tax%couKidsVirtual(1:numtax_bands) = tax_virtual_wk(1:numtax_bands)
                tax%couKidsMtr(1:numtax_bands) = tax_rates(1:numtax_bands)
                tax%couKidsNumKinks = numtax_bands
            case ('sin')
                tax%sinBreaks(1:numtax_bands) = tax_bands_wk(1:numtax_bands)
                tax%sinNet(1:numtax_bands) = tax_level_wk(1:numtax_bands)
                tax%sinVirtual(1:numtax_bands) = tax_virtual_wk(1:numtax_bands)
                tax%sinMtr(1:numtax_bands) = tax_rates(1:numtax_bands)
                tax%sinNumKinks = numtax_bands
            case ('sinkid')
                tax%sinKidsBreaks(1:numtax_bands) = tax_bands_wk(1:numtax_bands)
                tax%sinKidsNet(1:numtax_bands) = tax_level_wk(1:numtax_bands)
                tax%sinKidsVirtual(1:numtax_bands) = tax_virtual_wk(1:numtax_bands)
                tax%sinKidsMtr(1:numtax_bands) = tax_rates(1:numtax_bands)
                tax%sinKidsNumKinks = numtax_bands
            case default
                stop 'unrecognized tax label in setEnvironmentTaxes'
        end select

        tax%state = tax_state

    end subroutine setEnvironmentTaxes

    subroutine readEnvironmentTaxes( fname, verbose, tax_label, tax_year, tax_prices, tax_state, &
                                     numtax_bands, numtax_rates, numtax_level, tax_bands, tax_rates, tax_level )

        use config

        implicit none

        character(len = *), intent(in) :: fname
        logical, intent(in) :: verbose
        character(len = 16), intent(out) :: tax_label
        integer, intent(out) :: numtax_bands, numtax_rates, numtax_level
        integer, intent(out) :: tax_year, tax_prices, tax_state
        real(dp), intent(out) :: tax_bands(MAXKINKS), tax_rates(MAXKINKS), tax_level(MAXKINKS)

        integer :: iX, zX

        character(len = LENCONFIG) :: var_name(NCONFIG)
        character(len = LENVALUES) :: var_value(NCONFIG)
        character(len = LENCONFIG) :: var_group(NCONFIG)
        character(len = LENCONFIG) :: var_id(NCONFIG)


        call readConfig( fname, var_name, var_value, var_group, var_id )

        do iX = 1, NCONFIG

            if (var_id(iX) /= '') then
                call stringToScalar(var_id(iX), zX)
            else
                zX = 1
            end if

            if ( var_name(iX) == '') exit

            if ( verbose ) then
                write(*, '(A)') '"' // trim(var_group(iX)) // '"' // ', "' // trim(var_name(iX)) &
                    // '"', ', "' // trim(var_value(iX)) // '"'
            end if

            select case(var_group(iX))
            case('TAXES')
                select case(var_name(iX))
                case('TAX_BANDS')
                    call stringToArray(var_value(iX), tax_bands, numtax_bands)
                case('TAX_RATES')
                    call stringToArray(var_value(iX), tax_rates, numtax_rates)
                case('TAX_LEVEL')
                    call stringToArray(var_value(iX), tax_level, numtax_level)
                case('TAX_YEAR')
                    call stringToScalar(var_value(iX), tax_year)
                case('TAX_PRICES')
                    call stringToScalar(var_value(iX), tax_prices)
                case('TAX_STATE')
                    call stringToScalar(var_value(iX), tax_state)
                case('TAX_STATE_LABEL')
                    !state_label = var_value(iX)
                case('TAX_LABEL')
                    tax_label = var_value(iX)
                case default
                    call unknownVariableWarning(var_name(iX), var_group(iX))
                end select
            case default
                write(*, '(A)') 'WARNING: unrecognized group name (' // trim(adjustl(var_group(iX))) // ')'
            end select
        end do

        call allEqual( 'inctax',  numtax_bands, numtax_level, numtax_rates )

    end subroutine readEnvironmentTaxes

    subroutine readEnvironmentFoodStamps( fname, verbose, fpl_year, fpl_prices, fpl_val, numfpl_val, fpl_add, &
                                          fsp_year, fsp_prices, gross_scale, net_scale, fsp_ded, numfsp_ded, fsp_dedadd, &
                                          max_fspval, nummax_fspval, max_fspadd, max_shelter )

        use config

        implicit none
        character(len = *), intent(in) :: fname
        logical, intent(in) :: verbose

        integer, intent(out)  :: numfpl_val, numfsp_ded, nummax_fspval
        integer, intent(out) :: fpl_year, fpl_prices, fsp_year, fsp_prices
        real(dp), intent(out) :: fpl_val(maxhhsize), fsp_ded(maxhhsize), fsp_dedadd, max_fspval(maxhhsize)
        real(dp), intent(out) :: fpl_add, gross_scale, net_scale, max_fspadd, max_shelter

        integer :: iX, zX

        character(len = LENCONFIG) :: var_name(NCONFIG)
        character(len = LENVALUES) :: var_value(NCONFIG)
        character(len = LENCONFIG) :: var_group(NCONFIG)
        character(len = LENCONFIG) :: var_id(NCONFIG)

        call readConfig( fname, var_name, var_value, var_group, var_id )

        do iX = 1, NCONFIG

            if (var_id(iX) /= '') then
                call stringToScalar(var_id(iX), zX)
            else
                zX = 1
            end if

            if ( var_name(iX) == '') exit

            if ( verbose ) then
                write(*, '(A)') '"' // trim(var_group(iX)) // '"' // ', "' // trim(var_name(iX)) // &
                                '"', ', "' // trim(var_value(iX)) // '"'
            end if

            select case(var_group(iX))
            case('FPL')
                select case(var_name(iX))
                case('FPL_VAL')
                    call stringToArray(var_value(iX), fpl_val, numfpl_val)
                case('FPL_ADD')
                    call stringToScalar(var_value(iX), fpl_add)
                case('FPL_YEAR')
                    call stringToScalar(var_value(iX), fpl_year)
                case('FPL_PRICES')
                    call stringToScalar(var_value(iX), fpl_prices)
                case default
                    call unknownVariableWarning(var_name(iX), var_group(iX))
                end select
            case('FSP')
                select case(var_name(iX))
                case('GROSS_SCALE')
                    call stringToScalar(var_value(iX), gross_scale)
                case('NET_SCALE')
                    call stringToScalar(var_value(iX), net_scale)
                case('STD_DED')
                    call stringToArray(var_value(iX), fsp_ded, numfsp_ded)
                case('STD_ADD')
                    call stringToScalar(var_value(iX), fsp_dedadd)
                case('MAX_FSPVAL')
                    call stringToArray(var_value(iX), max_fspval, nummax_fspval)
                case('MAX_FSPADD')
                    call stringToScalar(var_value(iX), max_fspadd)
                case('MAX_SHELTER')
                    call stringToScalar(var_value(iX), max_shelter)
                case('FSP_YEAR')
                    call stringToScalar(var_value(iX), fsp_year)
                case('FSP_PRICES')
                    call stringToScalar(var_value(iX), fsp_prices)
                case default
                    call unknownVariableWarning(var_name(iX), var_group(iX))
                end select
            case default
                write(*, '(A)') 'WARNING: unrecognized group name (' // trim(adjustl(var_group(iX))) // ')'
            end select
        end do

    end subroutine readEnvironmentFoodStamps


    subroutine readWelfare( fdir, mX, verbose, tax )
        use mathfn, only : intToStr
        implicit none
        character(len = *), intent(in) :: fdir
        integer, intent(in) :: mX
        logical, intent(in) :: verbose
        type(tax_t), intent(inout) :: tax

        integer, parameter :: maxsize = 2

        integer :: gross_test, net_test
        integer :: ben_year, ben_prices
        real(dp) :: gross_needscale, net_needscale, elig_disregard_amt, elig_disregard_pct
        real(dp) :: need_standard(maxsize)
        integer :: numneed_standard
        integer :: max_ben
        real(dp) :: max_benval(maxsize)
        integer :: nummax_benval
        real(dp) :: payment_standard(maxsize)
        integer :: numpayment_standard
        real(dp) :: ben_disregard_amt, ben_disregard_pct, ben_ratio

        call readEnvironmentWelfare( fdir // '/welfare_' // intToStr(mX) // '.dat', verbose, ben_year, ben_prices,      &
                                     gross_test, net_test, gross_needscale, net_needscale, elig_disregard_amt, elig_disregard_pct, &
                                     need_standard, numneed_standard, max_ben, max_benval, nummax_benval, payment_standard, &
                                     numpayment_standard, ben_disregard_amt, ben_disregard_pct, ben_ratio )

        ! weeklyize
        need_standard = need_standard / 4.33_dp
        elig_disregard_amt = elig_disregard_amt / 4.33_dp
        max_benval = max_benval / 4.33_dp
        payment_standard = payment_standard / 4.33_dp
        ben_disregard_amt = ben_disregard_amt / 4.33_dp

        tax%sinKidsGrossTest = merge( .true., .false., gross_test > 0 )
        tax%sinKidsNetTest = merge( .true., .false., net_test > 0 )
        tax%sinKidsNeedStd = need_standard(1)
        tax%sinKidsGrossNeedScale = gross_needscale
        tax%sinKidsNetNeedScale = net_needscale
        tax%sinKidsEligDisAmt = elig_disregard_amt
        tax%sinKidsEligDisPct = elig_disregard_pct

        tax%sinKidsMaxBen = merge( .true., .false., max_ben > 0 )
        tax%sinKidsMaxBenVal = max_benval(1)
        tax%sinKidsPaymentStd = payment_standard(1)
        tax%sinKidsBenDisAmt = ben_disregard_amt
        tax%sinKidsBenDisPct = ben_disregard_pct
        tax%sinKidsBenRatio = ben_ratio

        tax%couKidsGrossTest = merge( .true., .false., gross_test > 0 )
        tax%couKidsNetTest = merge( .true., .false., net_test > 0 )
        tax%couKidsNeedStd= need_standard(2)
        tax%couKidsGrossNeedScale = gross_needscale
        tax%couKidsNetNeedScale = net_needscale
        tax%couKidsEligDisAmt = elig_disregard_amt
        tax%couKidsEligDisPct = elig_disregard_pct

        tax%couKidsMaxBen = merge( .true., .false., max_ben > 0 )
        tax%couKidsMaxBenVal = max_benval(2)
        tax%couKidsPaymentStd = payment_standard(2)
        tax%couKidsBenDisAmt = ben_disregard_amt
        tax%couKidsBenDisPct = ben_disregard_pct
        tax%couKidsBenRatio = ben_ratio

    end subroutine readWelfare

    subroutine readEnvironmentWelfare( fname, verbose, ben_year, ben_prices,                                   &
                                       gross_test, net_test,                                                   &
                                       gross_needscale, net_needscale, elig_disregard_amt, elig_disregard_pct, &
                                       need_standard, numneed_standard,                                        &
                                       max_ben, max_benval, nummax_benval,                                     &
                                       payment_standard, numpayment_standard,                                  &
                                       ben_disregard_amt, ben_disregard_pct, ben_ratio )

        use config

        implicit none
        integer, parameter :: maxsize = 2
        character(len = *), intent(in) :: fname
        logical, intent(in) :: verbose

        integer, intent(out) :: gross_test, net_test
        integer, intent(out) :: ben_year, ben_prices
        real(dp), intent(out) :: gross_needscale, net_needscale, elig_disregard_amt, elig_disregard_pct
        real(dp), intent(out) :: need_standard(maxsize)
        integer, intent(out) :: numneed_standard
        integer, intent(out) :: max_ben
        real(dp), intent(out) :: max_benval(maxsize)
        integer, intent(out) :: nummax_benval
        real(dp), intent(out) :: payment_standard(maxsize)
        integer, intent(out) :: numpayment_standard
        real(dp), intent(out) :: ben_disregard_amt, ben_disregard_pct, ben_ratio

        integer :: iX, zX

        character(len = LENCONFIG) :: var_name(NCONFIG)
        character(len = LENVALUES) :: var_value(NCONFIG)
        character(len = LENCONFIG) :: var_group(NCONFIG)
        character(len = LENCONFIG) :: var_id(NCONFIG)

        call readConfig( fname, var_name, var_value, var_group, var_id )

        do iX = 1, NCONFIG

            if (var_id(iX) /= '') then
                call stringToScalar(var_id(iX), zX)
            else
                zX = 1
            end if

            if ( var_name(iX) == '') exit

            if ( verbose ) then
                write(*, '(A)') '"' // trim(var_group(iX)) // '"' // ', "' // trim(var_name(iX)) // &
                                '"', ', "' // trim(var_value(iX)) // '"'
            end if

            select case(var_group(iX))
            case('STATE')
                ! state identifier, not used here
            case('ELIG_TEST')
                select case(var_name(iX))
                case('GROSS_TEST')
                    call stringToScalar(var_value(iX), gross_test)
                case('NET_TEST')
                    call stringToScalar(var_value(iX), net_test)
                case('NEED_STANDARD')
                    call stringToArray(var_value(iX), need_standard, numneed_standard)
                case('GROSS_NEEDSCALE')
                    call stringToScalar(var_value(iX), gross_needscale)
                case('NET_NEEDSCALE')
                    call stringToScalar(var_value(iX), net_needscale)
                case('DISREGARD_DOLLAR')
                    call stringToScalar(var_value(iX), elig_disregard_amt)
                case('DISREGARD_PERCENT')
                    call stringToScalar(var_value(iX), elig_disregard_pct)
                case default
                    call unknownVariableWarning(var_name(iX), var_group(iX))
                end select
            case('BENEFIT_AMOUNT')
                select case(var_name(iX))
                case('MAX_BEN')
                    call stringToScalar(var_value(iX), max_ben)
                case('MAX_BENVAL')
                    call stringToArray(var_value(iX), max_benval, nummax_benval)
                case('PAYMENT_STANDARD')
                    call stringToArray(var_value(iX), payment_standard, numpayment_standard)
                case('DISREGARD_DOLLAR')
                    call stringToScalar(var_value(iX), ben_disregard_amt)
                case('DISREGARD_PERCENT')
                    call stringToScalar(var_value(iX), ben_disregard_pct)
                case('BENEFIT_RATIO')
                    call stringToScalar(var_value(iX), ben_ratio)
                case('BENEFIT_YEAR')
                    call stringToScalar(var_value(iX), ben_year)
                case('BENEFIT_PRICES')
                    call stringToScalar(var_value(iX), ben_prices)
                case default
                    call unknownVariableWarning(var_name(iX), var_group(iX))
                end select
            case default
                write(*, '(A)') 'WARNING: unrecognized group name (' // trim(adjustl(var_group(iX))) // ')'
            end select
        end do

    end subroutine readEnvironmentWelfare


    subroutine readMedicaid( fdir, fnameval, mX, verbose, tax )
        use mathfn, only : intToStr
        implicit none
        character(len = *), intent(in) :: fdir
        character(len = *), intent(in) :: fnameval
        integer, intent(in) :: mX
        logical, intent(in) :: verbose
        type(tax_t), intent(inout) :: tax

        integer :: infant_elig, age1to5_elig, age6to19_elig
        integer :: parentWork_elig, parentNotWork_elig
        integer :: childlessWork_elig, childlessNotWork_elig
        real(dp) :: fpl_val(maxhhsize), fpl_add
        integer :: numfpl_val

        call readEnvironmentMedicaid( fdir // '/medicaid_' // intToStr(mX) // '.dat', verbose, &
                                      infant_elig, age1to5_elig, age6to19_elig, &
                                      parentWork_elig, parentNotWork_elig, &
                                      childlessWork_elig, childlessNotWork_elig, &
                                      fpl_val, numfpl_val, fpl_add )

        call readEnvironmentMedicaidValue( fnameval, verbose, tax%kidsMedicaidValue, &
                                           tax%adultMedicaidValue(0), &
                                           tax%adultMedicaidValue(SEX_MALE), &
                                           tax%adultMedicaidValue(SEX_FEMALE) )

        tax%singleMedicaidWork = fpl_val(1) * childlessWork_elig / 100.0_dp
        tax%singleMedicaidNotWork = fpl_val(1) * childlessNotWork_elig / 100.0_dp

        tax%singleKidsMedicaidChild = fpl_val(3) * age1to5_elig / 100.0_dp
        tax%singleKidsMedicaidParentWork = fpl_val(3) * parentWork_elig / 100.0_dp
        tax%singleKidsMedicaidParentNotWork = fpl_val(3) * parentNotWork_elig / 100.0_dp

        tax%coupleMedicaidWork = fpl_val(2) * childlessWork_elig / 100.0_dp
        tax%coupleMedicaidNotWork = fpl_val(2) * childlessNotWork_elig / 100.0_dp

        tax%coupleKidsMedicaidChild = fpl_val(4) * age1to5_elig / 100.0_dp
        tax%coupleKidsMedicaidParentWork = fpl_val(4) * parentWork_elig / 100.0_dp
        tax%coupleKidsMedicaidParentNotWork = fpl_val(4) * parentNotWork_elig / 100.0_dp



    end subroutine readMedicaid

    subroutine readEnvironmentMedicaidValue( fname, verbose, kids_value, adult_value, male_value, female_value )

        use config

        implicit none
        character(len = *), intent(in) :: fname
        logical, intent(in) :: verbose
        real(dp), intent(out) :: kids_value, adult_value, male_value, female_value

        character(len = LENCONFIG) :: var_name(NCONFIG)
        character(len = LENVALUES) :: var_value(NCONFIG)
        character(len = LENCONFIG) :: var_group(NCONFIG)
        character(len = LENCONFIG) :: var_id(NCONFIG)

        integer :: iX, zX

        call readConfig( fname, var_name, var_value, var_group, var_id )

        do iX = 1, NCONFIG

            if (var_id(iX) /= '') then
                call stringToScalar(var_id(iX), zX)
            else
                zX = 1
            end if

            if ( var_name(iX) == '') exit

            if ( verbose ) then
                write(*, '(A)') '"' // trim(var_group(iX)) // '"' // ', "' // trim(var_name(iX)) // &
                                '"', ', "' // trim(var_value(iX)) // '"'
            end if

            select case(var_group(iX))
            case('MEDICAID')
                select case(var_name(iX))
                case('CHILDREN')
                    call stringToScalar(var_value(iX), kids_value)
                case('ADULT')
                    call stringToScalar(var_value(iX), adult_value)
                case('ADULT_MALE')
                    call stringToScalar(var_value(iX), male_value)
                case('ADULT_FEMALE')
                    call stringToScalar(var_value(iX), female_value)
                case default
                    call unknownVariableWarning(var_name(iX), var_group(iX))
                end select
            case default
                write(*, '(A)') 'WARNING: unrecognized group name (' // trim(adjustl(var_group(iX))) // ')'
            end select
        end do

    end subroutine readEnvironmentMedicaidValue

    subroutine readEnvironmentMedicaid( fname, verbose, infant_elig, age1to5_elig, age6to19_elig, &
                                        parentWork_elig, parentNotWork_elig, childlessWork_elig, childlessNotWork_elig, &
                                        fpl_val, numfpl_val, fpl_add )

        use config

        implicit none
        character(len = *), intent(in) :: fname
        logical, intent(in) :: verbose

        integer, intent(out) :: infant_elig, age1to5_elig, age6to19_elig
        integer, intent(out) :: parentWork_elig, parentNotWork_elig
        integer, intent(out) :: childlessWork_elig, childlessNotWork_elig
        real(dp), intent(out) :: fpl_val(maxhhsize)
        integer, intent(out) :: numfpl_val
        real(dp), intent(out) :: fpl_add
        integer :: fpl_year, fpl_prices
        integer :: iX, zX

        character(len = LENCONFIG) :: var_name(NCONFIG)
        character(len = LENVALUES) :: var_value(NCONFIG)
        character(len = LENCONFIG) :: var_group(NCONFIG)
        character(len = LENCONFIG) :: var_id(NCONFIG)

        call readConfig( fname, var_name, var_value, var_group, var_id )

        do iX = 1, NCONFIG

            if (var_id(iX) /= '') then
                call stringToScalar(var_id(iX), zX)
            else
                zX = 1
            end if

            if ( var_name(iX) == '') exit

            if ( verbose ) then
                write(*, '(A)') '"' // trim(var_group(iX)) // '"' // ', "' // trim(var_name(iX)) // &
                                '"', ', "' // trim(var_value(iX)) // '"'
            end if

            select case(var_group(iX))
            case('STATE')
                ! state identifier, not used here
            case('FPL')
                select case(var_name(iX))
                case('FPL_VAL')
                    call stringToArray(var_value(iX), fpl_val, numfpl_val)
                case('FPL_ADD')
                    call stringToScalar(var_value(iX), fpl_add)
                case('FPL_YEAR')
                    call stringToScalar(var_value(iX), fpl_year)
                case('FPL_PRICES')
                    call stringToScalar(var_value(iX), fpl_prices)
                case default
                    call unknownVariableWarning(var_name(iX), var_group(iX))
                end select
            case('CHILDREN')
                select case(var_name(iX))
                case('INFANT')
                    call stringToScalar(var_value(iX), infant_elig)
                case('AGE1_5')
                    call stringToScalar(var_value(iX), age1to5_elig)
                case('AGE6_19')
                    call stringToScalar(var_value(iX), age6to19_elig)
                case default
                    call unknownVariableWarning(var_name(iX), var_group(iX))
                end select
            case('PARENT')
                select case(var_name(iX))
                case('NONWORKING')
                    call stringToScalar(var_value(iX), parentNotWork_elig)
                case('WORKING')
                    call stringToScalar(var_value(iX), parentWork_elig)
                case default
                    call unknownVariableWarning(var_name(iX), var_group(iX))
                end select
            case('CHILDLESS')
                select case(var_name(iX))
                case('NONWORKING')
                    call stringToScalar(var_value(iX), childlessNotWork_elig)
                case('WORKING')
                    call stringToScalar(var_value(iX), childlessWork_elig)
                case default
                    call unknownVariableWarning(var_name(iX), var_group(iX))
                end select
            case default
                write(*, '(A)') 'WARNING: unrecognized group name (' // trim(adjustl(var_group(iX))) // ')'
            end select
        end do

    end subroutine readEnvironmentMedicaid

end module environment
