module mathfn

    use ISO_FORTRAN_ENV
    use, intrinsic :: ieee_arithmetic

    integer, private, parameter :: sp = REAL32
    integer, private, parameter :: dp = REAL64

    ! constants
    real(dp), private, parameter :: root2_1 = 1.0_dp / sqrt(2.0_dp)
    real(dp), private, parameter :: root2pi_1 = 0.398942280401432677939946059934381868_dp

    interface realToStr
        module procedure realToStr_dp
        module procedure realToStr_sp
    end interface realToStr

    interface normcdf
        module procedure normcdfScalar
        module procedure normcdfStdScalar
        module procedure normcdfVector
        module procedure normcdfStdVector
    end interface normcdf

    interface normcdfInv
        module procedure normcdfInvScalar
        module procedure normcdfInvStdScalar
        module procedure normcdfInvVector
        module procedure normcdfInvStdVector
    end interface normcdfInv

    interface normpdf
        module procedure normpdfScalar
        module procedure normpdfStdScalar
        module procedure normpdfVector
        module procedure normpdfStdVector
    end interface normpdf

    interface lognpdf
        module procedure lognpdfScalar
        module procedure lognpdfStdScalar
        module procedure lognpdfVector
        module procedure lognpdfStdVector
    end interface lognpdf


    interface logisticcdf
        module procedure logisticcdfScalar
        module procedure logisticcdfStdScalar
        module procedure logisticcdfVector
        module procedure logisticcdfStdVector
    end interface logisticcdf

    interface logisticcdfInv
        module procedure logisticcdfInvScalar
        module procedure logisticcdfInvStdScalar
        module procedure logisticcdfInvVector
        module procedure logisticcdfInvStdVector
    end interface logisticcdfInv

    interface logisticpdf
        module procedure logisticpdfScalar
        module procedure logisticpdfStdScalar
        module procedure logisticpdfVector
        module procedure logisticpdfStdVector
    end interface logisticpdf

    interface vmean
        module procedure mean_1d
        module procedure meansel_1d
        module procedure meannull_1d
        module procedure meanselnull_1d
    end interface vmean

    interface vcorr
        module procedure corr_1d
        module procedure corrsel_1d
        module procedure corrnull_1d
        module procedure corrselnull_1d
    end interface vcorr

    interface vsdmean
        module procedure sdmean_1d
        module procedure sdmeansel_1d
        module procedure sdmeannull_1d
        module procedure sdmeanselnull_1d
    end interface vsdmean


contains

    elemental function rfp(x) result(y)
        implicit none
        real(dp), intent(in) :: x
        real(dp) :: y
        y = ceiling(x * 100.0_dp) / 100.0_dp
    end function rfp

    function vecToStr(x) result(str)
        implicit none
        real(dp), intent(in) :: x(:)
        character(len = :), allocatable :: str
        integer :: nx, ix
        character(len = 64) :: buf
        nx = size(x)

        if (nx == 0) then
            str = ''
            return
        end if

        write(buf, '(G25.16)') x(1)
        str = trim(adjustl(buf))
        if (nx>1) then
            do ix = 2, nx
                write(buf, *) x(ix)
                str = str // ', ' // trim(adjustl(buf))
            end do
        end if

    end function vecToStr

    ! cumsum
    ! ------------------------------------------------------
    ! this returns the cumumlative sum

    pure function cumsum(x, n) result(y)
        implicit none
        real(dp), intent(in) :: x(:)
        integer, intent(in) :: n
        real(dp) :: y(n)
        integer :: iX
        y(1) = x(1)
        do iX = 2, n
            y(iX) = y(iX - 1) + x(iX)
        end do
    end function cumsum


    ! realToStr_dp
    ! ------------------------------------------------------
    ! this returns a variable length string when converting
    ! by calling a intToStrLen when determing return length

    pure function realToStr_dp(x, rfmt)
        implicit none
        real(dp), intent(in) :: x
        character(len = *), intent(in) :: rfmt
        character(realToStrLen_dp(x, rfmt)) :: realToStr_dp
        character(64) :: tempStr64
        write (tempStr64, rfmt) x
        realToStr_dp = adjustl(tempStr64)
    end function realToStr_dp

    ! realToStr_sp
    ! ------------------------------------------------------
    ! this returns a variable length string when converting
    ! by calling a intToStrLen when determing return length

    pure function realToStr_sp(x, rfmt)
        implicit none
        real(sp), intent(in) :: x
        character(len = *), intent(in) :: rfmt
        character(realToStrLen_sp(x, rfmt)) :: realToStr_sp
        character(64) :: tempStr64
        write (tempStr64, rfmt) x
        realToStr_sp = adjustl(tempStr64)
    end function realToStr_sp

    ! realToStrLen_dp
    ! ------------------------------------------------------
    ! called by realToStr_dp to determine length

    pure function realToStrLen_dp(x, rfmt)
        implicit none
        real(dp), intent(in) :: x
        character(len = *), intent(in) :: rfmt
        integer :: realToStrLen_dp
        character(64) :: tempStr64
        write (tempStr64, rfmt) x
        realToStrLen_dp = len_trim(adjustl(tempStr64))
    end function realToStrLen_dp

    ! realToStrLen_sp
    ! ------------------------------------------------------
    ! called by realToStr_sp to determine length

    pure function realToStrLen_sp(x, rfmt)
        implicit none
        real(sp), intent(in) :: x
        character(len = *), intent(in) :: rfmt
        integer :: realToStrLen_sp
        character(64) :: tempStr64
        write (tempStr64, rfmt) x
        realToStrLen_sp = len_trim(adjustl(tempStr64))
    end function realToStrLen_sp

    pure subroutine matrixInverse(mat, matInv, n)

        implicit none

        integer, intent(in) :: n
        real(dp), intent(in) :: mat(n, n)
        real(dp), intent(out) :: matInv(n, n)

        real(dp) :: a(n, n), L(n, n), U(n, n), b(n), d(n), x(n)
        real(dp) :: coeff
        integer :: i, j, k

        a = mat

        ! step 0: initialization for matrices L and U and b
        L = 0.0_dp
        U = 0.0_dp
        b = 0.0_dp

        ! step 1: forward elimination
        do k = 1, n - 1
           do i = k + 1, n
                coeff = a(i, k) / a(k, k)
                L(i, k) = coeff
                do j = k + 1, n
                    a(i, j) = a(i, j) - coeff * a(k, j)
                end do
            end do
        end do

        ! Step 2: prepare L and U matrices
        ! L matrix is a matrix of the elimination coefficient
        ! + the diagonal elements are 1.0
        do i = 1, n
            L(i, i) = 1.0_dp
        end do

        ! U matrix is the upper triangular part of A
        do j = 1, n
            do i = 1, j
                U(i, j) = a(i, j)
            end do
        end do

        ! Step 3: compute columns of the inverse matrix C
        do k = 1, n

            b(k) = 1.0_dp
            d(1) = b(1)

            ! Step 3a: Solve Ld = b using the forward substitution
            do i = 2, n
                d(i) = b(i)
                do j = 1, i - 1
                    d(i) = d(i) - L(i, j) * d(j)
                end do
            end do

            ! Step 3b: Solve Ux=d using the back substitution
            x(n) = d(n) / U(n, n)
            do i = n - 1, 1, -1
                x(i) = d(i)
                do j = n, i + 1, -1
                    x(i) = x(i) - U(i, j) * x(j)
                end do
                x(i) = x(i) / u(i, i)
            end do

            ! Step 3c: fill the solutions x(n) into column k of C
            do i = 1, n
                matInv(i, k) = x(i)
            end do

            b(k) = 0.0_dp

        end do

    end subroutine matrixInverse

    ! calculates mean and standard deviation of x selected with sel. if not defined returns
    ! NAN
    ! -------------------------------------------------------------------------------------

    pure subroutine sdmeansel_1d(x, sel, sd, mean)
        implicit none
        real(dp), intent(in) :: x(:)
        logical, intent(in) :: sel(size(x))
        real(dp), intent(out) :: sd, mean
        real(dp) :: z(size(x))
        integer  :: ix, nsel
        real(dp) :: NAN

        nsel = 0
        do ix = 1, size(x)
          if (sel(ix)) then
            nsel = nsel + 1
            z(nsel) = x(ix)
          end if
        end do

        NAN = IEEE_VALUE(0.0_dp, IEEE_QUIET_NAN)
        call sdmeannull_1d(z(1:nsel), sd, mean, NAN, NAN)

    end subroutine sdmeansel_1d

    ! calculates mean and standard deviation of x selected with sel. if not defined returns
    ! respective nulldef
    ! -------------------------------------------------------------------------------------

    pure subroutine sdmeanselnull_1d(x, sel, sd, mean, nulldefsd, nulldefmean)
        implicit none
        real(dp), intent(in) :: x(:)
        logical, intent(in) :: sel(size(x))
        real(dp), intent(out) :: sd, mean
        real(dp), intent(in) :: nulldefsd, nulldefmean
        real(dp) :: z(size(x))
        integer :: ix, nsel

        nsel = 0
        do ix = 1, size(x)
            if (sel(ix)) then
                nsel = nsel + 1
                z(nsel) = x(ix)
            end if
        end do

        call sdmeannull_1d(z(1:nsel), sd, mean, nulldefsd, nulldefmean)

    end subroutine sdmeanselnull_1d

    ! calculates mean and standard deviation of x. if not defined returns respective NAN
    ! ------------------------------------------------------------------------------------

    pure subroutine sdmean_1d(x, sd, mean)
        implicit none
        real(dp), intent(in) :: x(:)
        real(dp), intent(out) :: sd, mean
        real(dp) :: NAN

        NAN = IEEE_VALUE(0.0_dp, IEEE_QUIET_NAN)
        call sdmeannull_1d(x, sd, mean, NAN, NAN)

    end subroutine sdmean_1d

    ! calculates mean and standard deviation of x. if not defined returns respective nulldef
    ! --------------------------------------------------------------------------------------

    pure subroutine sdmeannull_1d(x, sd, mean, nulldefsd, nulldefmean)
        implicit none
        real(dp), intent(in) :: x(:)
        real(dp), intent(out) :: sd, mean
        real(dp) :: ssqr
        integer :: nsel
        real(dp), intent(in) :: nulldefsd, nulldefmean

        nsel = size(x)

        select case(nsel)
        case(0)
            mean = nulldefmean
            sd = nulldefsd
        case(1)
            mean = sum(x)
            sd = 0.0_dp
        case default
            if (minval(x) == maxval(x)) then
                mean = x(1)
                sd = 0.0_dp
            else
                mean = sum(x) / real(nsel, dp)
                ssqr = sum(x * x)
                sd   = sqrt( (ssqr - real(nsel, dp) * mean * mean ) / real(nsel - 1, dp) )
            end if
        end select

    end subroutine sdmeannull_1d

    ! calculates correlation and covariance between x0 and x1. also returns standard deviations and
    ! means. if not defined returns NAN
    ! ---------------------------------------------------------------------------------------------

    pure subroutine corr_1d(x0, x1, sd0, mean0, sd1, mean1, corr, cov)
        implicit none
        real(dp), intent(in) :: x0(:)
        real(dp), intent(in) :: x1(size(x0))
        real(dp), intent(out) :: sd0, mean0
        real(dp), intent(out) :: sd1, mean1
        real(dp), intent(out) :: corr, cov
        real(dp) :: NAN

        NAN = IEEE_VALUE(0.0_dp, IEEE_QUIET_NAN)
        call corrnull_1d(x0, x1, sd0, mean0, sd1, mean1, corr, cov, NAN, NAN, NAN)

    end subroutine corr_1d


    ! calculates correlation and covariance between x0 and x1 selecting observations with sel.
    ! also returns standard deviations and means. if not defined returns NAN
    ! ----------------------------------------------------------------------------------------

    pure subroutine corrsel_1d(x0, x1, sel, sd0, mean0, sd1, mean1, corr, cov)
        implicit none
        real(dp), intent(in) :: x0(:)
        real(dp), intent(in) :: x1(size(x0))
        logical,  intent(in) :: sel(size(x0))
        real(dp), intent(out) :: sd0, mean0
        real(dp), intent(out) :: sd1, mean1
        real(dp), intent(out) :: corr, cov
        real(dp) :: NAN
        integer  :: ix, nsel
        real(dp) :: z0(size(x0)), z1(size(x0))

        nsel = 0
        do ix = 1, size(x0)
            if (sel(ix)) then
                nsel = nsel + 1
                z0(nsel) = x0(ix)
                z1(nsel) = x1(ix)
            end if
        end do

        NAN = IEEE_VALUE( 0.0_dp, IEEE_QUIET_NAN )

        call corrnull_1d( z0(1:nsel), z1(1:nsel), sd0, mean0, sd1, mean1, corr, cov, NAN, NAN, NAN )

    end subroutine corrsel_1d


    ! calculates correlation and covariance between x0 and x1. also returns standard deviations and
    ! means. if not defined returns respective nulldef
    ! ---------------------------------------------------------------------------------------------

    pure subroutine corrnull_1d(x0, x1, sd0, mean0, sd1, mean1, corr, cov, nulldefsd, nulldefmean, nulldefcov)
        implicit none
        real(dp), intent(in) :: x0(:)
        real(dp), intent(in) :: x1(size(x0))
        real(dp), intent(out) :: sd0, mean0
        real(dp), intent(out) :: sd1, mean1
        real(dp), intent(out) :: corr, cov
        real(dp), intent(in) :: nulldefsd, nulldefmean, nulldefcov
        integer  :: nsel

        nsel = size(x0)

        select case(nsel)
        case(0)
            sd0 = nulldefsd
            sd1 = nulldefsd
            mean0 = nulldefmean
            mean1 = nulldefmean
            cov = nulldefcov
            corr = nulldefcov
        case(1)
            sd0 = 0.0_dp
            sd1 = 0.0_dp
            mean0 = mean_1d(x0)
            mean1 = mean_1d(x1)
            cov = nulldefcov
            corr = nulldefcov
        case default
            call sdmeannull_1d(x0, sd0, mean0, nulldefsd, nulldefmean)
            call sdmeannull_1d(x1, sd1, mean1, nulldefsd, nulldefmean)
            if ( (sd0 > 0.0_dp) .and. (sd1 > 0.0_dp) ) then
                cov = ( sum(x0 * x1) - nsel * mean0 * mean1 ) / real(nsel - 1, dp)
                corr = cov / sd0 / sd1
            else
                cov = 0.0_dp
                corr = 0.0_dp
            end if
        end select

    end subroutine corrnull_1d


    ! calculates correlation and covariance between x0 and x1 selecting observations with sel.
    ! also returns standard deviations and means. if not defined returns respective nulldef
    ! ----------------------------------------------------------------------------------------

    pure subroutine corrselnull_1d(x0, x1, sel, sd0, mean0, sd1, mean1, corr, cov, nulldefsd, nulldefmean, nulldefcov)
        implicit none
        real(dp), intent(in) :: x0(:)
        real(dp), intent(in) :: x1(size(x0))
        logical,  intent(in) :: sel(size(x0))
        real(dp), intent(out) :: sd0, mean0
        real(dp), intent(out) :: sd1, mean1
        real(dp), intent(out) :: corr, cov
        real(dp), intent(in) :: nulldefsd, nulldefmean, nulldefcov
        integer :: ix, nsel
        real(dp) :: z0(size(x0)), z1(size(x0))

        nsel = 0
        do ix = 1, size(x0)
            if (sel(ix)) then
                nsel = nsel + 1
                z0(nsel) = x0(ix)
                z1(nsel) = x1(ix)
            end if
        end do

        call corrnull_1d( z0(1:nsel), z1(1:nsel), sd0, mean0, sd1, mean1, &
            corr, cov, nulldefsd, nulldefmean, nulldefcov )

    end subroutine corrselnull_1d

    ! calculates mean of x with selection vector sel. returns NAN if undefined
    ! ------------------------------------------------------------------------

    pure function meansel_1d(x,sel) result(y)
        !DEC$ ATTRIBUTES FORCEINLINE :: meansel_1d
        implicit none
        real(dp), intent(in) :: x(:)
        logical, intent(in) :: sel(size(x))
        real(dp) :: y
        real(dp) :: NAN

        NAN = IEEE_VALUE(0.0_dp, IEEE_QUIET_NAN)
        y = meanselnull_1d(x,sel,NAN)

    end function meansel_1d

    ! calculates mean of x with selection vector sel. returns nulldef if undefined
    ! ----------------------------------------------------------------------------

    pure function meanselnull_1d(x, sel, nulldef) result(y)
        !DEC$ ATTRIBUTES FORCEINLINE :: meanselnull_1d
        implicit none
        real(dp), intent(in) :: x(:)
        logical, intent(in) :: sel(size(x))
        real(dp), intent(in) :: nulldef
        real(dp) :: y
        real(dp) :: z(size(x))
        integer :: nsel, ix

        nsel = 0
        do ix = 1, size(x)
            if (sel(ix)) then
                nsel = nsel + 1
                z(nsel) = x(ix)
            end if
        end do

        select case(nsel)
        case(0)
            y = nulldef
        case default
            y = sum(z(1:nsel)) / nsel
        end select

    end function meanselnull_1d

    ! calculates mean of x. returns NAN if undefined
    ! ----------------------------------------------

    pure function mean_1d(x) result(y)
        !DEC$ ATTRIBUTES FORCEINLINE :: mean_1d
        implicit none
        real(dp), intent(in) :: x(:)
        real(dp) :: y
        integer :: nsel
        nsel = size(x)
        select case(nsel)
        case(0)
            y = IEEE_VALUE(0.0_dp, IEEE_QUIET_NAN)
        case default
            y = sum(x) / nsel
        end select
    end function mean_1d

    ! calculates mean of x. returns nulldef if undefined
    ! --------------------------------------------------

    pure function meannull_1d(x, nulldef) result(y)
        !DEC$ ATTRIBUTES FORCEINLINE :: meannull_1d
        implicit none
        real(dp), intent(in) :: x(:)
        real(dp), intent(in) :: nulldef
        real(dp) :: y
        integer :: nsel
        nsel = size(x)
        select case(nsel)
        case(0)
            y = nulldef
        case default
            y = sum(x) / real(nsel, dp)
        end select
    end function meannull_1d

    pure subroutine zeroinDouble(ax, bx, xZero, F, iuser, ruser, tol, ifail, f1, f2)

        implicit none

        real(dp), intent(in) :: ax, bx !interval
        real(dp), intent(in) :: tol
        integer,  intent(out) :: ifail
        integer,  intent(in) :: iuser(:)
        real(dp), intent(in) :: ruser(:)
        real(dp), intent(out) :: xZero

        real(dp), intent(in), optional :: f1, f2

        interface
            pure function F(x, iuser, ruser) result(y)
                use ISO_FORTRAN_ENV
                implicit none
                integer, parameter :: dp = REAL64
                real(dp), intent(in) :: x
                integer,  intent(in) :: iuser(:)
                real(dp), intent(in) :: ruser(:)
                real(dp) :: y
            end function F
        end interface

        integer,  parameter :: maxiter = 1000
        real(dp), parameter :: eps = epsilon(ax)
        real(dp), parameter :: big = huge(0.0_dp)

        real(dp) :: a, b, c, d, e
        real(dp) :: fa, fb, fc, tol1
        integer  :: iter, neval
        real(dp) :: xm, p, q, r, s, fZero

        tol1 = 1.0_dp + eps

        a = ax ! initialization
        b = bx

        if (present(f1)) then
            fa = f1
        else
            fa = F(a, iuser, ruser)
        end if

        if (present(f2)) then
            fb = f2
        else
            fb = F(b, iuser, ruser)
        end if

        neval=2
        if ( fa == 0.0_dp ) then
            xZero = a
            fZero = 0.0_dp
            ifail = 0
            return
        end if

        if ( fb == 0.0_dp ) then
            xZero = b
            fZero = 0.0_dp
            ifail = 0
            return
        end if

        if (fa * fb > 0.0_dp) then
            xZero = huge(a)
            fZero = huge(a)
            ifail = 2
            return
        end if

        c = a
        fc = fa
        d = b-a
        e = d

        do iter = 1, maxIter

            if ( (fb > 0.0_dp .and. fc > 0.0_dp) .or. (fb < 0.0_dp .and. fc < 0.0_dp) ) then
                c = a   ! we insist that b and c straddle the zero
                fc = fa
                d = b - a
                e = d
            end if

            if ( abs(fc) < abs(fb) ) then
                a = b  ! we insist that b be the better guess of b and c
                b = c
                c = a
                fa = fb
                fb = fc
                fc = fa
            end if

            tol1 = 2.0_dp * eps * abs(b) + 0.5_dp * tol ! convergence test
            xm = 0.5_dp * (c - b)

            if ( (abs(xm) <= tol1) .or. (fb == 0.0_dp) ) then
                xZero = b
                fZero = fb
                ifail = 0   ! SUCCESS! The proper way to leave
                return
            end if

            if ( (abs(e) < tol1) .or. (abs(fa) <= abs(fb)) ) then
                d = xm   ! bisection
                e = d
            else
                if (a == c) then
                    s = fb / fa   ! linear interpolation
                    p = 2.0_dp * xm * s
                    q = 1.0_dp - s
                else
                    q = fa / fc   ! inverse quadratic interpolation
                    r = fb / fc
                    s = fb / fa
                    p = s * (2.0_dp * xm * q * (q - r) - (b - a) * (r - 1.0_dp))
                    q = (q - 1.0_dp) * (r - 1.0_dp) * (s - 1.0_dp)
                end if

                if (p > 0.0_dp) then
                    q = -q   ! adjust signs
                end if

                p = abs(p)

                if ( (p + p >= (3.0_dp * xm * q - abs(tol1 * q))) .or. (p + p >= abs(e * q)) ) then
                    d = xm   ! don't interpolate. Use bisection
                    e = d
                else
                    e = d   ! OK, use interpolation
                    d = p / q
                end if
            end if

            a = b   ! complete step. a becomes the previous iteration
            fa = fb

            if ( abs(d) > tol1 ) then
                b = b + d
            else
                b = b + sign(tol1, xm)
            end if

            fb = F(b, iuser, ruser)   ! the newest and best value (we hope)
            neval = neval + 1         ! keep count of the function evaluations

        end do

        ! the loop should never terminate. If it does, return the last iteration and set ifail to 1
        xZero = b
        fZero = fb
        ifail = 1

        return

    end subroutine zeroinDouble


    ! calculates normal cdf (scalar input) with mean mu and standard deviation sigma
    ! ------------------------------------------------------------------------------

    pure function normcdfScalar(x, mu, sigma) result(y)
        !DEC$ ATTRIBUTES FORCEINLINE :: normcdfScalar
        implicit none
        real(dp), intent(in) :: x
        real(dp), intent(in) :: mu, sigma
        real(dp) :: y
        y = 0.5_dp * ( 1.0_dp + erf( root2_1 * (x - mu) / sigma ) )
    end function normcdfScalar

    ! calculates normal cdf at vector x of size n with mean mu and standard deviation sigma
    ! -------------------------------------------------------------------------------------

    pure function normcdfVector(x, n, mu, sigma) result(y)
        !DEC$ ATTRIBUTES FORCEINLINE :: normcdfVector
        implicit none
        integer, intent(in) :: n
        real(dp), intent(in) :: x(n)
        real(dp), intent(in) :: mu, sigma
        real(dp) :: y(n)
        y = 0.5_dp * ( 1.0_dp + erf( root2_1 * (x - mu) / sigma ) )
    end function normcdfVector

    ! calculates standard normal cdf with scalar input x
    ! --------------------------------------------------

    pure function normcdfStdScalar(x) result(y)
        !DEC$ ATTRIBUTES FORCEINLINE :: normcdfStdScalar
        implicit none
        real(dp), intent(in) :: x
        real(dp) :: y
        y = 0.5_dp * ( 1.0_dp + erf( root2_1 * x ) )
    end function normcdfStdScalar

    ! calculates standard normal cdf at vector x of size n
    ! ----------------------------------------------------

    pure function normcdfStdVector(x, n) result(y)
        !DEC$ ATTRIBUTES FORCEINLINE :: normcdfStdVector
        implicit none
        integer, intent(in) :: n
        real(dp), intent(in) :: x(n)
        real(dp) :: y(n)
        y = 0.5_dp * ( 1.0_dp + erf( root2_1*x ) )
    end function normcdfStdVector

    ! calculates standard normal pdf at vector x of size n
    ! ----------------------------------------------------

    pure function normpdfStdVector(x, n) result(y)
        !DEC$ ATTRIBUTES FORCEINLINE :: normpdfStdVector
        implicit none
        integer, intent(in)  :: n
        real(dp), intent(in) :: x(n)
        real(dp) :: y(n)
        y = root2pi_1 * exp(-0.5_dp * (x ** 2))
    end function normpdfStdVector

    ! calculates standard normal pdf with scalar input x
    ! --------------------------------------------------

    pure function normpdfStdScalar(x) result(y)
        !DEC$ ATTRIBUTES FORCEINLINE :: normpdfStdScalar
        implicit none
        real(dp), intent(in) :: x
        real(dp) :: y
        y = root2pi_1 * exp(-0.5_dp * (x ** 2))
    end function normpdfStdScalar

    ! calculates standard normal pdf at vector x of size n
    ! ----------------------------------------------------

    pure function normpdfVector(x, n, mu, sigma) result(y)
        !DEC$ ATTRIBUTES FORCEINLINE :: normpdfVector
        implicit none
        integer, intent(in) :: n
        real(dp), intent(in) :: x(n)
        real(dp), intent(in) :: mu, sigma
        real(dp) :: y(n)
        real(dp) :: z(n)
        z = (x - mu) / sigma
        y = root2pi_1 * exp(-0.5_dp * (z ** 2)) / sigma
    end function normpdfVector

    ! calculates standard normal pdf with scalar input x
    ! --------------------------------------------------

    pure function normpdfScalar(x, mu, sigma) result(y)
        !DEC$ ATTRIBUTES FORCEINLINE :: normpdfScalar
        implicit none
        real(dp), intent(in) :: x
        real(dp), intent(in) :: mu, sigma
        real(dp) :: y
        real(dp) :: z
        z = (x - mu) / sigma
        y = root2pi_1 * exp(-0.5_dp * (z ** 2)) / sigma
    end function normpdfScalar

    pure function lognpdfStdScalar(x) result(y)
        implicit none
        real(dp), intent(in) :: x
        real(dp) :: y
        y = root2pi_1 * exp(-0.5_dp * (log(x) ** 2))
    end function lognpdfStdScalar

    pure function lognpdfStdVector(x, n) result(y)
        implicit none
        integer, intent(in) :: n
        real(dp), intent(in) :: x(n)
        real(dp) :: y(n)
        y = root2pi_1 * exp(-0.5_dp * (log(x) ** 2))
    end function lognpdfStdVector

    pure function lognpdfVector(x, n, mu, sigma) result(y)
        implicit none
        integer, intent(in) :: n
        real(dp), intent(in) :: x(n)
        real(dp), intent(in) :: mu, sigma
        real(dp) :: y(n)
        real(dp) :: z(n)
        z = (log(x) - mu) / sigma
        y = root2pi_1 * exp(-0.5_dp * (z ** 2)) / sigma / x
    end function lognpdfVector

    pure function lognpdfScalar(x, mu, sigma) result(y)
        implicit none
        real(dp), intent(in) :: x
        real(dp), intent(in) :: mu, sigma
        real(dp) :: y
        real(dp) :: z
        z = (log(x) - mu) / sigma
        y = root2pi_1 * exp(-0.5_dp * (z ** 2)) / sigma / x
    end function lognpdfScalar


    ! returns size n array xlin with uniform spacing from xmin to xmax
    ! ----------------------------------------------------------------

    pure subroutine linSpace(xmin, xmax, n, xlin)
        implicit none
        real(dp), intent(in) :: xmin, xmax
        integer,  intent(in) :: n
        real(dp), intent(out) :: xlin(n)
        real(dp) :: xstep
        integer :: ix
        xstep = (xmax - xmin) / real(n - 1, dp)
        xlin(1) = xmin
        do ix = 2, n - 1
          xlin(ix) = xmin + xstep * (ix - 1)
        end do
        xlin(n) = xmax
    end subroutine linSpace

    ! approximation of the definite integral via the trapezoidal method
    ! -----------------------------------------------------------------

    pure function trapz(nodes, x, n) result(y)
        implicit none
        integer, intent(in) :: n
        real(dp), intent(in) :: nodes(n)
        real(dp), intent(in) :: x(n)
        real(dp) :: y

        y = 0.5_dp * sum( (nodes(2:n) - nodes(1:n - 1)) * ( x(2:n) + x(1:n - 1) ) )

    end function trapz

    pure recursive subroutine quickSort(x, n)
        implicit none
        integer, intent(in) :: n
        real(dp), dimension(n), intent(inout) :: x

        integer  :: left, right
        real(dp) :: pivot, temp
        integer  :: marker

        if (n > 1) then

            pivot = x(1)  ! simple 1st point pivot can cause very long execution times for some data sets (i.e. already sorted data)
            left = 0
            right = n + 1

            do while (left < right)
                right = right - 1
                do while (x(right) > pivot)
                    right = right - 1
                end do
                left = left + 1
                do while (x(left) < pivot)
                    left = left + 1
                end do
                if (left < right) then
                    temp = x(left)
                    x(left) = x(right)
                    x(right) = temp
                end if
            end do

            if (left == right) then
                marker = left + 1
            else
                marker = left
            end if

            call quickSort(x(:marker - 1), marker - 1)
            call quickSort(x(marker:), n - marker + 1)

        end if

    end subroutine quickSort

    pure function lininter(x0, y0, n, x) result(y)

        implicit none

        integer, intent(in) :: n
        real(dp), intent(in) :: x0(n)
        real(dp), intent(in) :: y0(n)
        real(dp), intent(in) :: x
        real(dp) :: y
        integer :: ix
        real(dp) :: a, b

        if ( x < x0(1) ) then !extrapolate left
            b = (y0(2) - y0(1)) / (x0(2) - x0(1))
            a = y0(1) - b * x0(1)
            y = a + b * x
        else if ( x >= x0(n) ) then !extrapolate right
            b = (y0(n) - y0(n - 1)) / (x0(n) - x0(n - 1))
            a = y0(n) - b * x0(n)
            y = a + b * x
        else !interpolate
            do ix = 2, n
                if ( (x >= x0(ix - 1)) .and. (x < x0(ix)) ) then
                    b = (y0(ix) - y0(ix - 1)) / (x0(ix) - x0(ix - 1))
                    a = y0(ix) - b * x0(ix)
                    y = a + b * x
                    exit
                end if
            end do
        end if

    end function lininter

    ! calculates logistic cdf (scalar input) with mean mu and scale parameter sigma
    ! ------------------------------------------------------------------------------

    pure function logisticcdfScalar(x, mu, sigma) result(y)
        !DEC$ ATTRIBUTES FORCEINLINE :: logisticcdfScalar
        implicit none
        real(dp), intent(in) :: x
        real(dp), intent(in) :: mu, sigma
        real(dp) :: y
        y = 1.0_dp / ( 1.0_dp + exp( -(x - mu) / sigma ) )
    end function logisticcdfScalar

    ! calculates logistic cdf at vector x of size n with mean mu and scale parameter sigma
    ! -------------------------------------------------------------------------------------

    pure function logisticcdfVector(x, n, mu, sigma) result(y)
        !DEC$ ATTRIBUTES FORCEINLINE :: logisticcdfVector
        implicit none
        integer, intent(in) :: n
        real(dp), intent(in) :: x(n)
        real(dp), intent(in) :: mu, sigma
        real(dp) :: y(n)
        y = 1.0_dp / ( 1.0_dp + exp( -(x - mu) / sigma ) )
    end function logisticcdfVector

    ! calculates standard logistic cdf with scalar input x
    ! --------------------------------------------------

    pure function logisticcdfStdScalar(x) result(y)
        !DEC$ ATTRIBUTES FORCEINLINE :: logisticcdfStdScalar
        implicit none
        real(dp), intent(in) :: x
        real(dp) :: y
        y = 1.0_dp / ( 1.0_dp + exp(-x) )
    end function logisticcdfStdScalar

    ! calculates standard logistic cdf at vector x of size n
    ! ----------------------------------------------------

    pure function logisticcdfStdVector(x, n) result(y)
        !DEC$ ATTRIBUTES FORCEINLINE :: logisticcdfStdVector
        implicit none
        integer, intent(in) :: n
        real(dp), intent(in) :: x(n)
        real(dp) :: y(n)
        y = 1.0_dp / ( 1.0_dp + exp(-x) )
    end function logisticcdfStdVector

    ! calculates standard logistic pdf at vector x of size n
    ! ------------------------------------------------------

    pure function logisticpdfStdVector(x, n) result(y)
        !DEC$ ATTRIBUTES FORCEINLINE :: logisticpdfStdVector
        implicit none
        integer, intent(in) :: n
        real(dp), intent(in) :: x(n)
        real(dp) :: z(n)
        real(dp) :: y(n)
        z = exp(-x)
        y = z / (1.0_dp + z) ** 2
    end function logisticpdfStdVector

    ! calculates standard logistic pdf with scalar input x
    ! ----------------------------------------------------

    pure function logisticpdfStdScalar(x) result(y)
        !DEC$ ATTRIBUTES FORCEINLINE :: logisticpdfStdScalar
        implicit none
        real(dp), intent(in) :: x
        real(dp) :: z
        real(dp) :: y
        z = exp(-x)
        y = z / (1.0_dp + z) ** 2
    end function logisticpdfStdScalar

    ! calculates logistic pdf with mean mu and scale sigma at vector x of size n
    ! --------------------------------------------------------------------------

    pure function logisticpdfVector(x, n, mu, sigma) result(y)
        !DEC$ ATTRIBUTES FORCEINLINE :: logisticpdfVector
        implicit none
        integer, intent(in) :: n
        real(dp), intent(in) :: x(n)
        real(dp), intent(in) :: mu, sigma
        real(dp) :: z(n)
        real(dp) :: y(n)
        z = exp(-(x - mu) / sigma)
        y = (z / sigma) / (1.0_dp + z) ** 2
    end function logisticpdfVector

    ! calculates logistic pdf with mean mu and scale sigma at scalar x
    ! ----------------------------------------------------------------

    pure function logisticpdfScalar(x, mu, sigma) result(y)
        !DEC$ ATTRIBUTES FORCEINLINE :: logisticpdfScalar
        implicit none
        real(dp), intent(in) :: x
        real(dp), intent(in) :: mu, sigma
        real(dp) :: y
        real(dp) :: z
        z = exp(-(x - mu) / sigma)
        y = (z / sigma) / (1.0_dp + z) ** 2
    end function logisticpdfScalar






    ! calculates logistic inverse cdf (scalar input) with mean mu and scale parameter sigma
    ! -------------------------------------------------------------------------------------

    pure function logisticcdfInvScalar(p, mu, sigma) result(y)
        !DEC$ ATTRIBUTES FORCEINLINE :: logisticcdfInvScalar
        implicit none
        real(dp), intent(in) :: p
        real(dp), intent(in) :: mu, sigma
        real(dp) :: y
        y = mu + sigma * log( p / (1.0_dp - p) )
    end function logisticcdfInvScalar

    ! calculates logistic inverse cdf at vector p of size n with mean mu and scale parameter sigma
    ! --------------------------------------------------------------------------------------------

    pure function logisticcdfInvVector(p, n, mu, sigma) result(y)
        !DEC$ ATTRIBUTES FORCEINLINE :: logisticcdfInvVector
        implicit none
        integer, intent(in) :: n
        real(dp), intent(in) :: p(n)
        real(dp), intent(in) :: mu, sigma
        real(dp) :: y(n)
        y = mu + sigma * log( p / (1.0_dp - p) )
    end function logisticcdfInvVector

    ! calculates standard logistic inverse cdf with scalar input p
    ! ------------------------------------------------------------

    pure function logisticcdfInvStdScalar(p) result(y)
        !DEC$ ATTRIBUTES FORCEINLINE :: logisticcdfInvStdScalar
        implicit none
        real(dp), intent(in) :: p
        real(dp) :: y
        y = log( p / (1.0_dp - p) )
    end function logisticcdfInvStdScalar

    ! calculates standard logistic inverse cdf at vector p of size n
    ! --------------------------------------------------------------

    pure function logisticcdfInvStdVector(p, n) result(y)
        !DEC$ ATTRIBUTES FORCEINLINE :: logisticcdfInvStdVector
        implicit none
        integer, intent(in) :: n
        real(dp), intent(in) :: p(n)
        real(dp) :: y(n)
        y = log( p / (1.0_dp - p) )
    end function logisticcdfInvStdVector



    ! calculates normal inverse cdf (scalar input) with mean mu and standard deviation sigma
    ! --------------------------------------------------------------------------------------

    pure function normcdfInvScalar(p, mu, sigma) result(y)
        !DEC$ ATTRIBUTES FORCEINLINE :: normcdfInvScalar
        implicit none
        real(dp), intent(in) :: p
        real(dp), intent(in) :: mu, sigma
        real(dp) :: y
        y = mu + sigma * normcdfInvElemental(p)
    end function normcdfInvScalar

    ! calculates normal inverse cdf at vector p of size n with mean mu and standard deviation sigma
    ! ---------------------------------------------------------------------------------------------

    pure function normcdfInvVector(p, n, mu, sigma) result(y)
        !DEC$ ATTRIBUTES FORCEINLINE :: normcdfInvVector
        implicit none
        integer, intent(in) :: n
        real(dp), intent(in) :: p(n)
        real(dp), intent(in) :: mu, sigma
        real(dp) :: y(n)
        integer :: ix
        do ix = 1, n
            y(ix) = mu + sigma * normcdfInvElemental(p(ix))
        end do
    end function normcdfInvVector

    ! calculates standard normal inverse cdf with scalar input p
    ! ----------------------------------------------------------

    pure function normcdfInvStdScalar(p) result(y)
        !DEC$ ATTRIBUTES FORCEINLINE :: normcdfInvStdScalar
        implicit none
        real(dp), intent(in) :: p
        real(dp) :: y
        y = normcdfInvElemental(p)
    end function normcdfInvStdScalar

    ! calculates standard normal inverse cdf at vector p of size n
    ! ------------------------------------------------------------

    pure function normcdfInvStdVector(p, n) result(y)
        !DEC$ ATTRIBUTES FORCEINLINE :: normcdfInvStdVector
        implicit none
        integer, intent(in) :: n
        real(dp), intent(in) :: p(n)
        real(dp) :: y(n)
        integer :: ix
        do ix = 1, n
            y(ix) = normcdfInvElemental(p(ix))
        end do
    end function normcdfInvStdVector

    pure function normcdfInvElemental(p) result(z)
        !DEC$ ATTRIBUTES FORCEINLINE :: normcdfInvElemental

        use, intrinsic :: IEEE_ARITHMETIC

        implicit none

        real(dp), intent(in) :: p
        real(dp) :: q, r, z
        real(dp) :: e, u

        real(dp), parameter :: a1 = -3.969683028665376e+01_dp, &
            a2 =  2.209460984245205e+02_dp, &
            a3 = -2.759285104469687e+02_dp, &
            a4 = 1.383577518672690e+02_dp,  &
            a5 = -3.066479806614716e+01_dp, &
            a6 = 2.506628277459239e+00_dp

        real(dp), parameter :: b1 = -5.447609879822406e+01_dp, &
            b2 = 1.615858368580409e+02_dp,  &
            b3 = -1.556989798598866e+02_dp, &
            b4 = 6.680131188771972e+01_dp,  &
            b5 = -1.328068155288572e+01_dp

        real(dp), parameter :: c1 = -7.784894002430293e-03_dp, &
            c2 = -3.223964580411365e-01_dp, &
            c3 = -2.400758277161838e+00_dp, &
            c4 = -2.549732539343734e+00_dp, &
            c5 = 4.374664141464968e+00_dp,  &
            c6 = 2.938163982698783e+00_dp

        real(dp), parameter :: d1 = 7.784695709041462e-03_dp, &
            d2 = 3.224671290700398e-01_dp, &
            d3 = 2.445134137142996e+00_dp, &
            d4 = 3.754408661907416e+00_dp

        ! sqrt(2.0_dp * pi)
        real(dp), parameter :: root2pi = 2.5066282746310005024_dp

        ! Define break-points.
        real(dp), parameter :: plow  = 0.02425_dp
        real(dp), parameter :: phigh = 1.0_dp - plow

        ! Rational approximation for central region
        if ( (p >= plow) .and. (p <= phigh) ) then
            q = p - 0.5_dp
            r = q ** 2
            z = (((((a1 * r + a2) * r + a3) * r + a4) * r + a5) * r + a6) * q &
                    / (((((b1 * r + b2) * r + b3) * r + b4) * r + b5) * r + 1.0_dp)
            return
        end if

        ! Rational approximation for lower region
        if ( (p > 0.0_dp) .and. (p < plow) ) then
            q = sqrt(-2.0_dp * log(p))
            z = (((((c1 * q + c2) * q + c3) * q + c4) * q + c5) * q + c6) &
                    / ((((d1 * q + d2) * q + d3) * q + d4) * q + 1.0_dp)
            return
        end if

        ! Rational approximation for upper region:
        if ( (p > phigh) .and. (p < 1.0_dp) ) then
            q = sqrt(-2.0_dp * log(1.0_dp - p))
            z = - (((((c1 * q + c2) * q + c3) * q + c4) * q + c5) * q + c6) &
                    / ((((d1 * q + d2) * q + d3) * q + d4) * q + 1.0_dp)
            return
        end if

        if (p == 0.0_dp) then
            z = IEEE_VALUE(z, IEEE_NEGATIVE_INF)
            return
        else if (p == 1.0_dp) then
            z = IEEE_VALUE(z, IEEE_POSITIVE_INF)
            return
        else
            z = IEEE_VALUE(z, IEEE_QUIET_NAN)
            return
        end if

        ! The relative error of the approximation has absolute value less than 1.15e-9
        ! One iteration of Halley's rational method (third order) gives full machine precision.

        if ( (p > 0.0_dp) .and. (p < 1.0_dp) ) then
            e = 0.5_dp * erfc(-z / sqrt(2.0_dp)) - p         ! error
            u = e * root2pi * exp(0.5_dp * z ** 2)           ! f(z) / df(z)
            !z = z - u;                                      ! Newton's method
            z = z - u / (1.0_dp + 0.5_dp * z * u)            ! Halley's method
        end if

    end function normcdfInvElemental



    ! intToStr
    ! ------------------------------------------------------
    ! this returns a variable length string when converting
    ! by calling a intToStrLen when determing return length

    pure function intToStr(N)
        implicit none
        integer, intent(in) :: N
        character(intToStrLen(N)) :: intToStr
        character(20) :: tempStr20
        write (tempStr20, '(I20)') N
        intToStr = adjustl(tempStr20)
    end function intToStr

    ! intToStrLen
    ! ------------------------------------------------------
    ! called by intToStr to determine length

    pure function intToStrLen(N)
        implicit none
        integer, intent(in) :: N
        integer :: intToStrLen
        character(20) :: tempStr20
        write (tempStr20, '(I20)') N
        intToStrLen = len_trim(adjustl(tempStr20))
    end function intToStrLen


    ! perform a random shuffle of array x of length n
    ! -----------------------------------------------

    subroutine shuffle(x, n)
        integer, intent(in) :: n
        integer, intent(inout) :: x(n)
        integer :: iX, jX, temp
        real :: unif
        do iX = n, 2, -1
            call random_number(unif)
            jX = int(unif * iX) + 1
            temp = x(jX)
            x(jX) = x(iX)
            x(iX) = temp
        end do
    end subroutine shuffle

    ! flip the order of array x of length n and save in xud
    ! -----------------------------------------------------

    pure subroutine flipud(x, xud, n)
        integer, intent(in) :: n
        real(dp), intent(in) :: x(n)
        real(dp), intent(out) :: xud(n)
        integer :: iX
        do iX = 1, n
            xud(iX) = x(n + 1 - iX)
        end do
    end subroutine flipud


    ! returns true if vectors x1 and x2 are the same
    ! ----------------------------------------------

    pure function vecEqual(x1, x2) result(y)
        implicit none
        real(dp), intent(in) :: x1(:)
        real(dp), intent(in) :: x2(:)
        logical :: y
        integer :: iX, n1, n2

        n1 = size(x1)
        n2 = size(x2)

        if (n1 /= n2) then
            y = .false.
        else
            y = .true.
            do iX = 1, n1
                if ( x1(iX) /= x2(iX) ) then
                    y = .false.
                    exit
                end if
            end do
        end if

    end function vecEqual

end module mathfn
