
set more off
set seed 47459100

cap log close mylogfile
log using "$log_files/ipums_ed$numed`'_age$minage$maxage$statetag.log", text replace name(mylogfile)

use "$ipums_data/acs2006.dta", clear
qui count

di as text "INITIAL SAMPLE SIZE `r(N)'"

tab year

di as text "SEL: selecting on year..."

keep if year >= $minyear_ipums & year <= $maxyear_ipums

/* *********************************************************************** */
/* *********************************************************************** */

gen byte agegrp = 0
replace  agegrp = 1 if ( age <  $minage | age > $maxage  )
replace  agegrp = 2 if ( age >= $minage & age <= $maxage )
label variable agegrp "Age group"
label define agegrp_lbl 0 "N/A" 1 "NOT in age group" 2 "in age group"
label values agegrp agegrp_lbl

/* *********************************************************************** */
/* *********************************************************************** */

if $numed == 3 {
    gen byte edulev = 1
    replace  edulev = 2 if (educd >= 65 & educd <= 81)
    replace  edulev = 3 if (educd >= 101 & educd <= 116)
    label variable edulev "Highest education level (new def)"
    label define edulev_lbl 1 "High school degree or below" 2 "Some college/associate's degree" 3 "Bachelor's degree and above"
    label values edulev edulev_lbl
}
else if $numed == 4 {
    gen byte edulev = 1
    replace  edulev = 2 if (educd == 62)
    replace  edulev = 3 if (educd >= 65 & educd <= 81)
    replace  edulev = 4 if (educd >= 101 & educd <= 116)
    label variable edulev "Highest education level (new def)"
    label define edulev_lbl 1 "Less than high school" 2 "High school degree" 3 "Some college/associate's degree" 4 "Bachelor's degree and above"
    label values edulev edulev_lbl
}


/* *********************************************************************** */
/* *********************************************************************** */

/* weeks worked last year is coded by intervals */
/* assign mid point */

gen double wkswork = 0
replace wkswork = 7 if wkswork2 == 1
replace wkswork = 20 if wkswork2 == 2
replace wkswork = 33 if wkswork2 == 3
replace wkswork = 43.5 if wkswork2 == 4
replace wkswork = 48.5 if wkswork2 == 5
replace wkswork = 51 if wkswork2 == 6
replace wkswork = wkswork1 if ( year >= 2001 & year <= 2007 )
label variable wkswork "Weeks worked last year, averaged or actual"
label define wkswork_lbl 0 "N/A"
label values wkswork wkswork_lbl

/* *********************************************************************** */
/* *********************************************************************** */

// annual hours worked
gen double ahrswork = wkswork * uhrswork
label variable ahrswork "Annual hours worked"
label define ahrswork_lbl 0 "N/A"
label values ahrswork ahrswork_lbl

/* *********************************************************************** */
/* *********************************************************************** */

// race
cap drop race
cap label drop race_lbl
gen byte race = 1
replace race = 2 if racesingd == 10
replace race = 3 if racesingd == 20
replace race = 4 if racesingd == 12
label variable race "Race group"
label define race_lbl 1 "Other or N/A" 2 "White" 3 "Black" 4 "Hispanic"
label values race race_lbl


/* *********************************************************************** */
/* *********************************************************************** */

// recode state
matrix define recodeFips ///
    = (1, 2, ., 3, 4, 5, ., 6, 7, 8, 9, 10, 11, ., 12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23,  ///
       24, 25, 26, 27, 28, 29, 30, 31, 32, 33, 34, 35, 36, 37, 38, 39, ., 40, 41, 42, 43, 44, 45, ///
       46, 47, ., 48, 49, 50, 51)
gen byte state = recodeFips[1, statefip]
run label_state51.do
label values state state_lbl

/* *********************************************************************** */
/* *********************************************************************** */

// keep head, spouse, unmarried partners, children

di as text "SEL: keep head, spouse, unmarried partners, children..."

keep if ( relate == 1 | relate == 2 | relate==3 | related == 1114 )
gen byte head = relate == 1

// child age (relate == 3 includes adopted and step children)
gen byte _kid05   = relate == 3 & age >= 0  & age <= 5
gen byte _kid611  = relate == 3 & age >= 6  & age <= 11
gen byte _kid1218 = relate == 3 & age >= 12 & age <  18

egen byte nkids05   = total(_kid05),   by(year serial)
egen byte nkids611  = total(_kid611),  by(year serial)
egen byte nkids1218 = total(_kid1218), by(year serial)

gen byte nkids = nkids05 + nkids611 + nkids1218
gen byte kids = nkids > 0
drop _kid05 _kid611 _kid1218

// drop child observations

di as text "SEL: dropping children..."
drop if relate == 3
egen byte numads = count(pernum !=0), by(year serial)
assert numads <= 2

sort year serial pernum

// keep head, spouse, and unmarried partners

gen byte married = ( marst == 1 | marst == 2 )
gen byte _cohabiting = (related == 1114)
egen byte cohabiting = max(_cohabiting), by(year serial)
drop _cohabiting

// drop if missing partner/spouse

di as text "SEL: dropping if missing spouse (married)..."

drop if married == 1    & numads == 1

di as text "SEL: dropping if missing spouse (cohabiting)..."

drop if cohabiting == 1 & numads == 1

gen byte single = married == 0 & cohabiting == 0 & numads == 1
gen byte couple = 1 - single

gen byte tot = married + cohabiting + single
assert tot == 1
drop tot

egen byte sex_sp = total(sex), by(year serial)
replace sex_sp = sex_sp - sex

// drop same-sex partnerships

di as text "SEL: dropping same sex partnerships..."

keep if (sex != sex_sp)

// age selection
egen long agegrp_sp = total(agegrp), by(year serial)
replace agegrp_sp = agegrp_sp - agegrp
qui compress agegrp_sp
assert agegrp_sp == 0 if single == 1

egen long age_sp = total(age), by(year serial)
replace age_sp = age_sp - age
compress age_sp

di as text "SEL: dropping outside age group (singles and head)..."

drop if agegrp == 1 & married == 0
drop if agegrp == 1 & married == 1 & head == 1
drop if agegrp_sp == 1 & married == 1 & head == 0

if $bothage == 1 {
    di as text "SEL: dropping outside age group (married spouse)..."
    drop if ( agegrp==1|agegrp_sp==1 ) & married == 1
    }

// always respect hard age limits (these are couples with sufficient age difference)
di as text "SEL: dropping couples outside hard age limits..."

drop if ( age < $minage_hard | age > $maxage_hard | age_sp < $minage_hard | age_sp > $maxage_hard ) & married == 1
drop if ( age < $minage_hard | age > $maxage_hard ) & married == 0

// work indicator
gen byte work = uhrswork > 0

// rename income variable
ren incbus00 incbus

replace incinvst = 0 if incinvst < 0
replace incother = 0 if incother < 0
replace incbus   = 0 if incbus < 0

// generate more spousal variables
//local type : type `var'
local spvar edulev kids uhrswork ahrswork work incwage incbus incinvst incother ///
    race disabwrk diffrem diffphys diffmob diffcare diffsens school

foreach var in `spvar' {
    egen double `var'_sp = total(`var'), by(year serial)
    replace `var'_sp = `var'_sp - `var'
    qui compress `var'_sp
    assert `var'_sp == 0 if single == 1
    }

assert kids == kids_sp if married == 1
drop kids_sp

recode incwage 9999999 = .
recode incwage_sp 9999999 = .

gen double incnly = incinvst + incother + incbus
gen double incnly_sp = incinvst_sp + incother_sp + incbus_sp
gen byte disab = diffrem == 2 | diffphys == 2 | diffmob == 2 | diffcare == 2 | diffsens == 2
gen byte disab_sp = diffrem_sp == 2 | diffphys_sp == 2 | diffmob_sp == 2 | diffcare_sp == 2 | diffsens_sp == 2

keep year serial state hhwt married cohabiting single kids edulev age uhrswork ahrswork incwage ///
    race sex edulev_sp age_sp uhrswork_sp ahrswork_sp ///
    incwage_sp race_sp sex_sp ///
    incnly incnly_sp ///
    disab disab_sp ///
    head nkids

if $whiteonly == 1 {
    di as text "SEL: dropping non-whites..."
    keep if ( married == 0 & race == 2 | (married == 1 & race == 2 & race_sp == 2) )
    }

/* *********************************************************************** */
/* *********************************************************************** */

sort year serial
merge m:1 year using "$price_data/prices_index.dta"
assert _merge !=1
keep if _merge == 3
drop _merge

local upratevar incnly incnly_sp incwage incwage_sp

foreach var in `upratevar' {
    replace `var' = `var' * priceind
    }

/* *********************************************************************** */
/* *********************************************************************** */

di as text "SEL: dropping disabled families..."

drop if ( married == 0 & disab == 1 )
drop if ( married == 1 & (disab == 1 | disab_sp == 1) )

// generate male and female types
forval e = 1 / $numed {
    qui gen byte type_M`e' = edulev == `e' if sex == 1
    }

forval e = 1 / $numed {
    qui gen byte type_F`e' = edulev == `e' if sex == 2
    }


/* *********************************************************************** */
/* *********************************************************************** */

run regions.do
qui su $market
if `r(max)' ~= $nummarket {
    di as error "nummarket ($nummarket) inconsistent with market variable count (`r(max)')"
    }

tab $market, gen(market)
qui compress market*
gen byte market0 = 1

egen double sum_hhwt = sum(hhwt), by(married sex $market)

egen double _pm = mean(sum_hhwt) if sex == 1, by(married $market)
egen double pm = mean(_pm), by(married $market)
egen double _pf = mean(sum_hhwt) if sex == 2, by(married $market)
egen double pf = mean(_pf), by(married $market)

gen double newwt = hhwt
replace newwt = hhwt * (pf + pm) / (2 * pf) * (pf / pm) if married == 0 & sex == 1
replace newwt = hhwt * (pf + pm) / (2 * pf) if married == 0 & sex == 2

egen double sum_newwt = sum(newwt), by(married sex $market)

if $nummarket == 4 {
    local taxid1 33
    local taxid2 14
    local taxid3 44
    local taxid4 5
    }

if $nummarket == 9 {
    local taxid1 22
    local taxid2 33
    local taxid3 14
    local taxid4 26
    local taxid5 10
    local taxid6 43
    local taxid7 44
    local taxid8 3
    local taxid9 5
    }

if $stateonly == 1 {
    forval m = 1 / $nummarket {
        drop if $market == `m' & state ~= `taxid`m''
    }
}

/* *********************************************************************** */
/* *********************************************************************** */

forval m = 0 / $nummarket {

    if `m' == 0 {
        local ifm "$market ~= ."
        local sfxm ""
        }
    else {
        local ifm "$market == `m'"
        local sfxm "_market`m'"
        }

    cap file close typefile
    qui file open typefile using "$input_file/type_ed$numed`'_age$minage$maxage$statetag`sfxm'.asc", text write replace

    if `m' > 0 {
        file write typefile "[MARKET]" _n _n
        local this_market_lbl : label $market`'_lbl `m'
        file write typefile "NAME `this_market_lbl'" _n
        file write typefile "ID `m'" _n
        file write typefile "TAXID `taxid`m''" _n
        su market`m' [aw = newwt], meanonly
        file write typefile "SIZE `r(mean)'" _n
        qui count if $market == `m' & (married == 0 | (married == 1 & sex == 1))
        file write typefile "COUNT `r(N)'" _n _n
        }
    else {
        file write typefile "[MARKET]" _n _n
        file write typefile "NAME All" _n
        file write typefile "ID 0" _n
        file write typefile "TAXID 0" _n
        file write typefile "SIZE 1" _n
        qui count if (married == 0 | (married == 1 & sex == 1))
        file write typefile "COUNT `r(N)'" _n _n
        }

    file write typefile "[NUMTYPES]" _n _n
    file write typefile "NUM_MALE $numed" _n
    file write typefile "NUM_FEMALE $numed" _n

    file write typefile _n
    file write typefile "[MALE_TYPE]" _n _n

    forval e = 1 / $numed {
        su type_M`e' if `ifm' [aw = newwt], meanonly
        file write typefile "`e' `r(mean)'" _n
        }

    file write typefile _n
    file write typefile "[FEMALE_TYPE]" _n _n

    forval e = 1 / $numed {
        su type_F`e' if `ifm' [aw = newwt], meanonly
        file write typefile "`e' `r(mean)'" _n
        }

    file write typefile _n
    file write typefile "[MALE_NONLABOUR]" _n _n

    forval e = 1 / $numed {
        _pctile incnly [aw = newwt] if sex == 1 & married == 0 & edulev == `e' & incnly >= 0, p(99)
        su incnly [aw = newwt] if `ifm' & sex == 1 & married == 0 & edulev == `e' & incnly >= 0 & incnly <= `r(r1)', meanonly
        file write typefile "`e' `r(mean)'" _n
        }

    file write typefile _n
    file write typefile "[FEMALE_NONLABOUR]" _n _n

    forval e = 1 / $numed {
        _pctile incnly [aw = newwt] if sex == 2 & married == 0 & edulev == `e' & incnly >= 0, p(99)
        su incnly [aw = newwt] if `ifm' & sex == 2 & married == 0 & edulev == `e' & incnly >= 0 & incnly <= `r(r1)', meanonly
        file write typefile "`e' `r(mean)'" _n
        }

    file write typefile _n
    file write typefile "[MALE_KIDS]" _n _n

    forval e = 1 / $numed {
        su kids [aw = newwt] if `ifm' & edulev == `e' & married == 0 & sex == 1, meanonly
        file write typefile "`e' `r(mean)'" _n
        }

    file write typefile _n
    file write typefile "[FEMALE_KIDS]" _n _n

    forval e = 1 / $numed {
        su kids [aw = newwt] if `ifm' & edulev == `e' & married == 0 & sex == 2, meanonly
        file write typefile "`e' `r(mean)'" _n
        }

    file write typefile _n
    file write typefile "[JOINT_KIDS]" _n _n
    file write typefile "# first index is for male, second index is for female" _n _n

    forval e = 1 / $numed {
        forval esp = 1 / $numed {
            su kids [aw = newwt] if `ifm' & edulev == `e' & edulev_sp == `esp' & married == 1 & sex == 1 & sex_sp == 2, meanonly
            file write typefile "`e',`esp' `r(mean)'" _n
            }
        }

    file write typefile _n
    file write typefile "[MALE_KIDS_TAX]" _n _n

    forval e = 1 / $numed {
        su cohabiting [aw = newwt] if `ifm' & edulev == `e' & married == 0 & sex == 1 & kids==1, meanonly
        file write typefile "`e' `= 1 - `r(mean)''" _n
        }

    file write typefile _n
    file write typefile "[FEMALE_KIDS_TAX]" _n _n

    forval e = 1 / $numed {
        file write typefile "`e' 1" _n
        }

    cap file close typefile

}

foreach sp in "" _sp {

    gen double hrdiscrete`sp' = .

    replace hrdiscrete`sp' = 0  if uhrswork`sp' <= 6
    replace hrdiscrete`sp' = 12 if uhrswork`sp' >  6  & uhrswork`sp' <= 18
    replace hrdiscrete`sp' = 24 if uhrswork`sp' >  18 & uhrswork`sp' <= 30
    replace hrdiscrete`sp' = 36 if uhrswork`sp' >  30 & uhrswork`sp' <= 42
    replace hrdiscrete`sp' = 48 if uhrswork`sp' >  42 & uhrswork`sp' <= 54
    replace hrdiscrete`sp' = 60 if uhrswork`sp' >  54 & uhrswork`sp' <.

    gen byte hours`sp' = .
    replace hours`sp' = 0 if hrdiscrete`sp' == 0
    replace hours`sp' = 1 if hrdiscrete`sp' >  0 & hrdiscrete`sp' <= 30
    replace hours`sp' = 2 if hrdiscrete`sp' > 30 & hrdiscrete`sp' <.


    }


/* *********************************************************************** */
/* *********************************************************************** */


// identify observations at household
gen byte hh = ( married == 0 | (married == 1 & sex == 1) )

save "$save_data/acs_intermediate_ed$numed`'_age$minage$maxage$statetag.dta", replace

gen byte type_sum = 0 if hh == 1

qui gen byte type_Mm_F0 = married == 0 & sex == 1 if hh == 1
qui gen byte type_M0_Ff = married == 0 & sex == 2 if hh == 1
qui gen byte type_Mm_Ff = married == 1

// joint marital outcomes
forval e = 1 / $numed {

    qui gen byte type_M`e'_F0 = edulev == `e' & married == 0 & sex == 1 if hh == 1
    qui gen byte type_M0_F`e' = edulev == `e' & married == 0 & sex == 2 if hh == 1
    qui replace type_sum = type_sum + type_M`e'_F0 + type_M0_F`e' if hh == 1

    forval esp = 1 / $numed {
        qui gen byte type_M`e'_F`esp' = edulev == `e' & edulev_sp == `esp' & married == 1 & sex == 1 & sex_sp == 2 if hh == 1
        replace type_sum = type_sum + type_M`e'_F`esp' if hh == 1
        }

    // married individual type integrating over spouse type
    qui gen byte type_M`e'_Ff = edulev == `e' & sex == 1 & sex_sp == 2 if married == 1
    qui gen byte type_Mm_F`e' = edulev == `e' & sex == 2 & sex_sp == 1 if married == 1

    forval k = 0 / 1 {
        forval h = 0 / 2 {
            qui gen byte hours`h'_k`k'_M`e'_F0 = hours == `h' if type_M`e'_F0 == 1 & kid == `k'
            qui gen byte hours`h'_k`k'_M0_F`e' = hours == `h' if type_M0_F`e' == 1 & kid == `k'
            }
        }
    forval k = 0 / 1 {
        forval h = 0 / 2 {
            qui gen byte hoursMh`h'_k`k'_M`e'_Ff = hours == `h' if type_M`e'_Ff == 1 & kid == `k'
            qui gen byte hoursFh`h'_k`k'_Mm_F`e' = hours == `h' if type_Mm_F`e' == 1 & kid == `k'
            }
        }

    forval h = 0 / 2 {
        qui gen byte hours`h'_M`e'_F0 = hours == `h' if type_M`e'_F0 == 1
        qui gen byte hours`h'_M0_F`e' = hours == `h' if type_M0_F`e' == 1
        }

    forval h = 0 / 2 {
        qui gen byte hoursMh`h'_M`e'_Ff = hours == `h' if type_M`e'_Ff == 1
        qui gen byte hoursFh`h'_Mm_F`e' = hours == `h' if type_Mm_F`e' == 1
        }

    forval esp = 1 / $numed {
        forval h = 0 / 2 {
            qui gen byte hoursMh`h'_M`e'_F`esp' = hours == `h' if edulev == `e' & edulev_sp == `esp' & married == 1 & sex == 1 & sex_sp == 2
            qui gen byte hoursFh`h'_M`e'_F`esp' = hours == `h' if edulev == `e' & edulev_sp == `esp' & married == 1 & sex == 2 & sex_sp == 1
            }
        }

    forval k = 0 / 1 {
        forval esp = 1 / $numed {
            forval h = 0 / 2 {
                forval hsp = 0 / 2 {
                    qui gen byte hoursMh`h'Fh`hsp'_k`k'_M`e'_F`esp' = hours == `h' & hours_sp == `hsp' if type_M`e'_F`esp' == 1 & kids == `k'
                    }
                }
            }
        }

    }

forval h = 0 / 2 {
    qui gen byte hours`h'_Mm_F0 = hours == `h' if type_Mm_F0 == 1
    qui gen byte hours`h'_M0_Ff = hours == `h' if type_M0_Ff == 1
    }

forval h = 0 / 2 {
    qui gen byte hoursMh`h'_Mm_Ff = hours == `h' if type_Mm_Ff == 1 & sex == 1
    qui gen byte hoursFh`h'_Mm_Ff = hours == `h' if type_Mm_Ff == 1 & sex == 2
    }

forval k = 0 / 1 {

    forval h = 0 / 2 {
        qui gen byte hours`h'_k`k'_Mm_F0 = hours == `h' if type_Mm_F0 == 1 & kid == `k' & sex == 1
        qui gen byte hours`h'_k`k'_M0_Ff = hours == `h' if type_M0_Ff == 1 & kid == `k' & sex == 2
        }

    forval h = 0 / 2 {
        qui gen byte hoursMh`h'_k`k'_Mm_Ff = hours == `h' if type_Mm_Ff == 1 & kid == `k' & sex == 1
        qui gen byte hoursFh`h'_k`k'_Mm_Ff = hours == `h' if type_Mm_Ff == 1 & kid == `k' & sex == 2
        }

    }

assert type_sum == 1 if hh == 1
drop type_sum

gen double wage = incwage / ahrswork
qui replace wage = . if uhrswork == 0
recode  wage (0=.)

forval sex = 1 / 2 {
    forval e = 1 / $numed {
        _pctile wage [aw = newwt] if sex == `sex' & edulev == `e', p(2 98)
        qui replace wage = . if sex == `sex' & edulev == `e' & wage < r(r1)
        qui replace wage = . if sex == `sex' & edulev == `e' & wage > r(r2)
        }
    }

egen double wage_sp = total(wage), by(year serial)
replace wage_sp = wage_sp - wage

su wage if sex == 1 [aw = newwt], de
su wage if sex == 2 [aw = newwt], de

gen double logwage = log(wage)
gen double logwage_sp = log(wage_sp)

gen double logwage_product = logwage*logwage_sp

foreach sp in "" _sp {
    gen double earndiscrete`sp' = 0
    replace earndiscrete`sp' = hrdiscrete`sp' * wage if  hrdiscrete`sp' > 0
    }

drop incwage incwage_sp
drop ahrswork ahrswork_sp incnly incnly_sp priceind wage wage_sp uhrswork_sp uhrswork hrdiscrete_sp

// for marriage market matching function
forval e = 1 / $numed {
    gen byte M`e'_F0 = edulev == `e' & married == 0 if sex == 1
    }

forval e = 1 / $numed {
    gen byte M0_F`e' = edulev == `e' & married == 0 if sex == 2
    }

forval esp = 1 / $numed {
    forval e = 1 / $numed {
        gen byte M`e'_F`esp' = edulev == `e' & edulev_sp == `esp' & married == 1 if sex == 1
        }
    }

local postlistf

// joint marital outcomes
forval e = 1 / $numed {
    local postlistf "`postlistf' jtype_M`e'_F0 jtype_M0_F`e'"
    forval esp = 1 / $numed {
        local postlistf "`postlistf' jtype_M`e'_F`esp'"
        }
    }

forval m = 1 / $nummarket {
    forval e = 1 / $numed {
        local postlistf "`postlistf' R`m'jtype_M`e'_F0 R`m'jtype_M0_F`e'"
        forval esp = 1 / $numed {
            local postlistf "`postlistf' R`m'jtype_M`e'_F`esp'"
            }
        }
    }

// labour supply by market (employment and conditional hours)
forval m = 1 / $nummarket {
    forval e = 1 / $numed {
        local postlistf "`postlistf' R`m'mhoursW_M`e'_F0 R`m'mhoursh_M`e'_F0"
        local postlistf "`postlistf' R`m'fhoursW_M0_F`e' R`m'fhoursh_M0_F`e'"
        local postlistf "`postlistf' R`m'mhoursW_M`e'_Ff R`m'mhoursh_M`e'_Ff"
        local postlistf "`postlistf' R`m'fhoursW_Mm_F`e' R`m'fhoursh_Mm_F`e'"
        }
    local postlistf "`postlistf' R`m'mhoursW_Mm_F0 R`m'mhoursh_Mm_F0"
    local postlistf "`postlistf' R`m'fhoursW_M0_Ff R`m'fhoursh_M0_Ff"
    local postlistf "`postlistf' R`m'mhoursW_Mm_Ff R`m'mhoursh_Mm_Ff"
    local postlistf "`postlistf' R`m'fhoursW_Mm_Ff R`m'fhoursh_Mm_Ff"
    }

// joint labour supply allocations
forval k = 0 / 1 {
    forval e = 1 / $numed {

        forval h = 0 / 2 {
            local postlistf "`postlistf' mhours`h'_k`k'_M`e'_F0 fhours`h'_k`k'_M0_F`e'"
            }

        forval h = 0 / 2 {
            local postlistf "`postlistf' mhours`h'_k`k'_M`e'_Ff fhours`h'_k`k'_Mm_F`e'"
            }

        forval esp = 1 / $numed {
            forval h = 0 / 2 {
                forval hsp = 0 / 2 {
                    local postlistf "`postlistf' jhoursMh`h'Fh`hsp'_k`k'_M`e'_F`esp'"
                    }
                }
            }

        local postlistf "`postlistf' mhours_k`k'_M`e'_F0__mean mhours_k`k'_M`e'_F0__sd"
        local postlistf "`postlistf' fhours_k`k'_M0_F`e'__mean fhours_k`k'_M0_F`e'__sd"
        local postlistf "`postlistf' mhours_k`k'_M`e'_Ff__mean mhours_k`k'_M`e'_Ff__sd"
        local postlistf "`postlistf' fhours_k`k'_Mm_F`e'__mean fhours_k`k'_Mm_F`e'__sd"

        }
    }

// male hours
forval k = 0 / 1 {
    forval esp = 1 / $numed {
        forval e = 1 / $numed {
            forval h = 0 / 2 {
                local postlistf "`postlistf' mhoursMh`h'_k`k'_M`e'_F`esp'"
                }
            }
        }
    }

// female hours
forval k = 0/1 {
    forval esp = 1 / $numed {
        forval e = 1 / $numed {
            forval hsp = 0 / 2 {
                local postlistf "`postlistf' fhoursFh`hsp'_k`k'_M`e'_F`esp'"
                }
            }
        }
    }

forval e = 1 / $numed {
    local postlistf "`postlistf' mhoursh_M`e'_F0__mean mhoursh_M`e'_F0__sd mwork_M`e'_F0 fhoursh_M0_F`e'__mean fhoursh_M0_F`e'__sd fwork_M0_F`e'"
    forval esp = 1 / $numed {
        local postlistf "`postlistf' mhoursh_M`e'_F`esp'__mean mhoursh_M`e'_F`esp'__sd mwork_M`e'_F`esp' fhoursh_M`e'_F`esp'__mean fhoursh_M`e'_F`esp'__sd fwork_M`e'_F`esp'"
        }
    }

// joint labour supply allocations (integrating own and spousal type)
forval k = 0 / 1 {
    forval h = 0 / 2 {
        local postlistf "`postlistf' mhours`h'_k`k'_sin fhours`h'_k`k'_sin"
        local postlistf "`postlistf' mhours`h'_k`k'_cou fhours`h'_k`k'_cou"
        }
    }

// male singles distribution of wages within hours cell and all workers
forval k = 0 / 1 {
    forval e = 1 / $numed {
        forval h = 1 / 2 {
            local postlistf "`postlistf' mlogwage_M`e'F0_k`k'_h`h'__mean mlogwage_M`e'F0_k`k'_h`h'__sd"
            }
        local postlistf "`postlistf' mlogwageMw_M`e'F0_k`k'__mean mlogwageMw_M`e'F0_k`k'__sd"
        }
    }

forval e = 1 / $numed {
    local postlistf "`postlistf' mlogwageMw_M`e'F0__mean mlogwageMw_M`e'F0__sd"
    }

// female singles distribution of wages within hours cell and all workers
forval k = 0 / 1 {
    forval e = 1 / $numed {
        forval h = 1 / 2 {
            local postlistf "`postlistf' flogwage_M0F`e'_k`k'_h`h'__mean flogwage_M0F`e'_k`k'_h`h'__sd"
            }
        local postlistf "`postlistf' flogwageFw_M0F`e'_k`k'__mean flogwageFw_M0F`e'_k`k'__sd"
        }
    }

forval e = 1 / $numed {
    local postlistf "`postlistf' flogwageFw_M0F`e'__mean flogwageFw_M0F`e'__sd"
    }

// married men distribution of wages within hours cell and all workers (this integrates over spousal state)
forval k = 0 / 1 {
    forval e = 1 / $numed {
        forval h = 1 / 2 {
            local postlistf "`postlistf' mlogwage_M`e'Ff_k`k'_h`h'__mean mlogwage_M`e'Ff_k`k'_h`h'__sd"
            }
        local postlistf "`postlistf' mlogwageMw_M`e'Ff_k`k'__mean mlogwageMw_M`e'Ff_k`k'__sd"
        }
    }

forval e = 1 / $numed {
    local postlistf "`postlistf' mlogwageMw_M`e'Ff__mean mlogwageMw_M`e'Ff__sd"
    }

// married women distribution of wages within hours cell (this integrates over spousal state)
forval k = 0 / 1 {
    forval e = 1 / $numed {
        forval h = 1 / 2 {
            local postlistf "`postlistf' flogwage_MmF`e'_k`k'_h`h'__mean flogwage_MmF`e'_k`k'_h`h'__sd"
            }
        local postlistf "`postlistf' flogwageFw_MmF`e'_k`k'__mean flogwageFw_MmF`e'_k`k'__sd"
        }
    }

forval e = 1 / $numed {
    local postlistf "`postlistf' flogwageFw_MmF`e'__mean flogwageFw_MmF`e'__sd"
    }

// married men distribution of wages by spousal employment state
forval k = 0 / 1 {
    forval e = 1 / $numed {
        forval w = 0 / 1 {
            local postlistf "`postlistf' mlogwageFw`w'_M`e'Ff_k`k'__mean  mlogwageFw`w'_M`e'Ff_k`k'__sd"
            }
        }
    }

forval e = 1 / $numed {
    forval w = 0 / 1 {
        local postlistf "`postlistf' mlogwageFw`w'_M`e'Ff__mean  mlogwageFw`w'_M`e'Ff__sd"
        }
    }


// married women distribution of wages by spousal employment state
forval k = 0 / 1 {
    forval e = 1 / $numed {
        forval w = 0 / 1 {
            local postlistf "`postlistf' flogwageMw`w'_MmF`e'_k`k'__mean flogwageMw`w'_MmF`e'_k`k'__sd"
            }
        }
    }

forval e = 1 / $numed {
    forval w = 0 / 1 {
        local postlistf "`postlistf' flogwageMw`w'_MmF`e'__mean flogwageMw`w'_MmF`e'__sd"
        }
    }

// product of wages within match
forval k = 0 / 1 {
    forval e = 1 / $numed {
        forval esp = 1 / $numed {
            local postlistf "`postlistf' jlogwage_M`e'F`esp'_k`k'__mean jlogwage_M`e'F`esp'_k`k'__sd jlogwage_M`e'F`esp'_k`k'__cov"
            }
        }
    }

forval e = 1 / $numed {
    forval esp = 1 / $numed {
        local postlistf "`postlistf' jlogwage_M`e'F`esp'__mean jlogwage_M`e'F`esp'__sd jlogwage_M`e'F`esp'__cov"
        }
    }


// earnings

forval e = 1 / $numed {
    local postlistf "`postlistf' mearnMw_M`e'F0__mean mearnMw_M`e'F0__sd"
    }

forval e = 1 / $numed {
    local postlistf "`postlistf' fearnFw_M0F`e'__mean fearnFw_M0F`e'__sd"
    }

forval e = 1 / $numed {
    local postlistf "`postlistf' mearnMw_M`e'Ff__mean mearnMw_M`e'Ff__sd"
    }

forval e = 1 / $numed {
    local postlistf "`postlistf' fearnFw_MmF`e'__mean fearnFw_MmF`e'__sd"
    }

// very aggregated moments for table

// 1. conditional earnings
local postlistf "`postlistf' mearnMw_MmF0__mean mearnMw_MmF0__sd"
local postlistf "`postlistf' fearnFw_M0Ff__mean fearnFw_M0Ff__sd"
local postlistf "`postlistf' mearnMw_MmFf__mean mearnMw_MmFf__sd"
local postlistf "`postlistf' fearnFw_MmFf__mean fearnFw_MmFf__sd"

// 2. unconditional earnings
local postlistf "`postlistf' mearn_MmF0__mean mearn_MmF0__sd"
local postlistf "`postlistf' fearn_M0Ff__mean fearn_M0Ff__sd"
local postlistf "`postlistf' mearn_MmFf__mean mearn_MmFf__sd"
local postlistf "`postlistf' fearn_MmFf__mean fearn_MmFf__sd"

// 3. employment
local postlistf "`postlistf' mwork_MmF0"
local postlistf "`postlistf' fwork_M0Ff"
local postlistf "`postlistf' mwork_MmFf"
local postlistf "`postlistf' fwork_MmFf"

// 4. conditional hours
local postlistf "`postlistf' mhourshMw_MmF0__mean mhourshMw_MmF0__sd"
local postlistf "`postlistf' fhourshFw_M0Ff__mean fhourshFw_M0Ff__sd"
local postlistf "`postlistf' mhourshMw_MmFf__mean mhourshMw_MmFf__sd"
local postlistf "`postlistf' fhourshFw_MmFf__mean fhourshFw_MmFf__sd"

// 5. unconditional hours
local postlistf "`postlistf' mhoursh_MmF0__mean mhoursh_MmF0__sd"
local postlistf "`postlistf' fhoursh_M0Ff__mean fhoursh_M0Ff__sd"
local postlistf "`postlistf' mhoursh_MmFf__mean mhoursh_MmFf__sd"
local postlistf "`postlistf' fhoursh_MmFf__mean fhoursh_MmFf__sd"


/* post file for bootstrap repetitions */

local postlist
foreach pval in `postlistf' {
    local postlist "`postlist' (`pval')"
    }

cap postclose momIPUMS

postfile momIPUMS int r double(`postlistf') using "$save_data/mom_ipums_ed$numed`'_age$minage$maxage$statetag.dta", replace

forval r = 0 / $bsrep {

    di "`r'"

    preserve

    if `r' > 0 {
        qui bsample
        }

    /* sample number */
    gen int r = `r'

    if `r' == 0 {
        di as text "calculating... type distribtion"
        }

    // joint marital outcomes
    forval e = 1 / $numed {
        su M`e'_F0 [aw = newwt], meanonly
        scalar jtype_M`e'_F0 = r(mean)
        su M0_F`e' [aw = newwt], meanonly
        scalar jtype_M0_F`e' = r(mean)
        forval esp = 1 / $numed {
            su M`e'_F`esp' [aw = newwt], meanonly
            scalar jtype_M`e'_F`esp' = r(mean)
            }
        }

    // joint marital outcomes by market
    forval m = 1 / $nummarket {
        forval e = 1 / $numed {
            su M`e'_F0 [aw = newwt] if $market == `m', meanonly
            scalar R`m'jtype_M`e'_F0 = r(mean)
            su M0_F`e' [aw = newwt] if $market == `m', meanonly
            scalar R`m'jtype_M0_F`e' = r(mean)
            forval esp = 1 / $numed {
                su M`e'_F`esp' [aw = newwt] if $market == `m', meanonly
                scalar R`m'jtype_M`e'_F`esp' = r(mean)
                }
            }
        }

    if `r' == 0 {
        di as text "calculating... joint labour supply allocations"
        }

    forval m = 1 / $nummarket {
        forval e = 1 / $numed {

            // single male
            su hours0_M`e'_F0 [aw = newwt] if $market == `m', meanonly
            scalar R`m'mhoursW_M`e'_F0 = 1 - `r(mean)'
            su hrdiscrete [aw = newwt] if hours > 0 & type_M`e'_F0 == 1 & $market == `m', meanonly
            scalar R`m'mhoursh_M`e'_F0 = `r(mean)'

            // single female
            su hours0_M0_F`e' [aw = newwt] if $market == `m', meanonly
            scalar R`m'fhoursW_M0_F`e' = 1 - `r(mean)'
            su hrdiscrete [aw = newwt] if hours > 0 & type_M0_F`e' == 1 & $market == `m', meanonly
            scalar R`m'fhoursh_M0_F`e' = `r(mean)'

            // married male
            su hoursMh0_M`e'_Ff [aw = newwt] if $market == `m', meanonly
            scalar R`m'mhoursW_M`e'_Ff = 1 - `r(mean)'
            su hrdiscrete [aw = newwt] if hours > 0 & type_M`e'_Ff == 1 & sex == 1 & $market == `m', meanonly
            scalar R`m'mhoursh_M`e'_Ff = `r(mean)'

            // married female
            su hoursFh0_Mm_F`e' [aw = newwt] if $market == `m', meanonly
            scalar R`m'fhoursW_Mm_F`e' = 1 - `r(mean)'
            su hrdiscrete [aw = newwt] if hours > 0 & type_Mm_F`e' == 1 & sex == 2 & $market == `m', meanonly
            scalar R`m'fhoursh_Mm_F`e' = `r(mean)'

            }

        // single male
        su hours0_Mm_F0 [aw = newwt] if $market == `m', meanonly
        scalar R`m'mhoursW_Mm_F0 = 1 - `r(mean)'
        su hrdiscrete [aw = newwt] if hours > 0 & type_Mm_F0 == 1 & $market == `m', meanonly
        scalar R`m'mhoursh_Mm_F0 = `r(mean)'

        // single female
        su hours0_M0_Ff [aw = newwt] if $market == `m', meanonly
        scalar R`m'fhoursW_M0_Ff = 1 - `r(mean)'
        su hrdiscrete [aw = newwt] if hours > 0 & type_M0_Ff == 1 & $market == `m', meanonly
        scalar R`m'fhoursh_M0_Ff = `r(mean)'

        // married male
        su hoursMh0_Mm_Ff [aw = newwt] if $market == `m', meanonly
        scalar R`m'mhoursW_Mm_Ff = 1 - `r(mean)'
        su hrdiscrete [aw = newwt] if hours > 0 & type_Mm_Ff == 1 & sex == 1 & $market == `m', meanonly
        scalar R`m'mhoursh_Mm_Ff = `r(mean)'

        // married female
        su hoursFh0_Mm_Ff [aw = newwt] if $market == `m', meanonly
        scalar R`m'fhoursW_Mm_Ff = 1 - `r(mean)'
        su hrdiscrete [aw = newwt] if hours > 0 & type_Mm_Ff == 1 & sex == 2 & $market == `m', meanonly
        scalar R`m'fhoursh_Mm_Ff = `r(mean)'

        }

    // joint labour supply allocations part-time / full-time
    forval k = 0 / 1 {
        forval e = 1 / $numed {
            forval h = 0 / 2 {

                su hours`h'_k`k'_M`e'_F0 [aw = newwt], meanonly
                scalar mhours`h'_k`k'_M`e'_F0 = `r(mean)'
                su hours`h'_k`k'_M0_F`e' [aw = newwt], meanonly
                scalar fhours`h'_k`k'_M0_F`e' = `r(mean)'

                su hoursMh`h'_k`k'_M`e'_Ff [aw = newwt], meanonly
                scalar mhours`h'_k`k'_M`e'_Ff = `r(mean)'
                su hoursFh`h'_k`k'_Mm_F`e' [aw = newwt], meanonly
                scalar fhours`h'_k`k'_Mm_F`e' = `r(mean)'

                }

            forval esp = 1 / $numed {
                forval h = 0 / 2 {
                    forval hsp = 0 / 2 {
                        su hoursMh`h'Fh`hsp'_k`k'_M`e'_F`esp' [aw = newwt], meanonly
                        scalar jhoursMh`h'Fh`hsp'_k`k'_M`e'_F`esp' = `r(mean)'
                        }
                    }
                }

            }
        }


    // male hours
    forval k = 0 / 1 {
        forval esp = 1 / $numed {
            forval e = 1 / $numed {
                forval h = 0 / 2 {
                    scalar mhoursMh`h'_k`k'_M`e'_F`esp' = jhoursMh`h'Fh0_k`k'_M`e'_F`esp'  + jhoursMh`h'Fh1_k`k'_M`e'_F`esp'  + jhoursMh`h'Fh2_k`k'_M`e'_F`esp'
                    }
                }
            }
        }

    // female hours
    forval k = 0/1 {
        forval esp = 1 / $numed {
            forval e = 1 / $numed {
                forval hsp = 0 / 2 {
                    scalar fhoursFh`hsp'_k`k'_M`e'_F`esp' = jhoursMh0Fh`hsp'_k`k'_M`e'_F`esp' + jhoursMh1Fh`hsp'_k`k'_M`e'_F`esp' + jhoursMh2Fh`hsp'_k`k'_M`e'_F`esp'
                    }
                }
            }
        }

    // joint labour supply allocations part-time / full-time (integrating own/spouse type)
    forval k = 0 / 1 {
        forval h = 0 / 2 {

            su hours`h'_k`k'_Mm_F0 [aw = newwt], meanonly
            scalar mhours`h'_k`k'_sin = `r(mean)'

            su hours`h'_k`k'_M0_Ff [aw = newwt], meanonly
            scalar fhours`h'_k`k'_sin = `r(mean)'

            su hoursMh`h'_k`k'_Mm_Ff [aw = newwt], meanonly
            scalar mhours`h'_k`k'_cou = `r(mean)'

            su hoursFh`h'_k`k'_Mm_Ff [aw = newwt], meanonly
            scalar fhours`h'_k`k'_cou = `r(mean)'

            }
        }


    // joint labour supply allocations
    forval k = 0 / 1 {
        forval e = 1 / $numed {

            qui su hrdiscrete [aw = newwt] if hours > 0 & type_M`e'_F0 == 1 & kid == `k'
            scalar mhours_k`k'_M`e'_F0__mean = `r(mean)'
            scalar mhours_k`k'_M`e'_F0__sd = `r(sd)'
            qui su hrdiscrete [aw = newwt] if hours > 0 & type_M0_F`e' == 1 & kid == `k'
            scalar fhours_k`k'_M0_F`e'__mean = `r(mean)'
            scalar fhours_k`k'_M0_F`e'__sd = `r(sd)'

            qui su hrdiscrete [aw = newwt] if hours > 0 & type_M`e'_Ff == 1 & kid == `k'
            scalar mhours_k`k'_M`e'_Ff__mean = `r(mean)'
            scalar mhours_k`k'_M`e'_Ff__sd = `r(sd)'
            qui su hrdiscrete [aw = newwt] if hours > 0 & type_Mm_F`e' == 1 & kid == `k'
            scalar fhours_k`k'_Mm_F`e'__mean = `r(mean)'
            scalar fhours_k`k'_Mm_F`e'__sd = `r(sd)'

            }
        }

    forval e = 1 / $numed {

        qui su hrdiscrete [aw = newwt] if hours > 0 & type_M`e'_F0 == 1
        scalar mhoursh_M`e'_F0__mean = `r(mean)'
        scalar mhoursh_M`e'_F0__sd = `r(sd)'

        qui su hours0_M`e'_F0 [aw = newwt]
        scalar mwork_M`e'_F0 = 1 - `r(mean)'

        qui su hrdiscrete [aw = newwt] if hours > 0 & type_M0_F`e' == 1
        scalar fhoursh_M0_F`e'__mean = `r(mean)'
        scalar fhoursh_M0_F`e'__sd = `r(sd)'

        qui su hours0_M0_F`e' [aw = newwt]
        scalar fwork_M0_F`e' = 1 - `r(mean)'

        forval esp = 1 / $numed {
            qui su hrdiscrete [aw = newwt] if hours > 0 & edulev == `e' & edulev_sp == `esp' & married == 1 & sex == 1 & sex_sp == 2
            scalar mhoursh_M`e'_F`esp'__mean = `r(mean)'
            scalar mhoursh_M`e'_F`esp'__sd = `r(sd)'

            qui su hoursMh0_M`e'_F`esp' [aw = newwt]
            scalar mwork_M`e'_F`esp' = 1 - `r(mean)'

            qui su hrdiscrete [aw = newwt] if hours > 0 & edulev == `e' & edulev_sp == `esp' & married == 1 & sex == 2 & sex_sp == 1
            scalar fhoursh_M`e'_F`esp'__mean = `r(mean)'
            scalar fhoursh_M`e'_F`esp'__sd = `r(sd)'

            qui su hoursFh0_M`e'_F`esp' [aw = newwt]
            scalar fwork_M`e'_F`esp' = 1 - `r(mean)'
            }

        }

    if `r' == 0 {
        di as text "calculating... wage distributions conditional on hours"
        }

    // male singles distribution of wages within hours cell
    forval k = 0 / 1 {
        forval e = 1 / $numed {
            forval h = 1 / 2 {
                qui su logwage [aw = newwt] if type_M`e'_F0 == 1 & kids == `k' & hours == `h'
                scalar mlogwage_M`e'F0_k`k'_h`h'__mean = r(mean)
                scalar mlogwage_M`e'F0_k`k'_h`h'__sd = r(sd)
                }
            }
        }

    // female singles distribution of wages within hours cell
    forval k = 0 / 1 {
        forval e = 1 / $numed {
            forval h = 1 / 2 {
                qui su logwage [aw = newwt] if type_M0_F`e' == 1 & kids == `k' & hours == `h'
                scalar flogwage_M0F`e'_k`k'_h`h'__mean = r(mean)
                scalar flogwage_M0F`e'_k`k'_h`h'__sd = r(sd)
                }
            }
        }


    // married men distribution of wages within hours cell (this integrates over spousal state)
    forval k = 0 / 1 {
        forval e = 1 / $numed {
            forval h = 1 / 2 {
                qui su logwage [aw = newwt] if type_M`e'_Ff == 1 & kids == `k' & hours == `h'
                scalar mlogwage_M`e'Ff_k`k'_h`h'__mean = r(mean)
                scalar mlogwage_M`e'Ff_k`k'_h`h'__sd = r(sd)
                }
            }
        }

    // married women distribution of wages within hours cell (this integrates over spousal state)
    forval k = 0 / 1 {
        forval e = 1 / $numed {
            forval h = 1 / 2 {
                qui su logwage [aw = newwt] if type_Mm_F`e' == 1 & kids == `k' & hours == `h'
                scalar flogwage_MmF`e'_k`k'_h`h'__mean = r(mean)
                scalar flogwage_MmF`e'_k`k'_h`h'__sd = r(sd)
                }
            }
        }

    if `r' == 0 {
        di as text "calculating... wage distributions all workers"
        }

    // male singles distribution of wages all workers
    forval k = 0 / 1 {
        forval e = 1 / $numed {
            qui su logwage [aw = newwt] if type_M`e'_F0 == 1 & kids == `k' & hours > 0
            scalar mlogwageMw_M`e'F0_k`k'__mean = r(mean)
            scalar mlogwageMw_M`e'F0_k`k'__sd = r(sd)
            }
        }

    forval e = 1 / $numed {
        qui su logwage [aw = newwt] if type_M`e'_F0 == 1 & hours > 0
        scalar mlogwageMw_M`e'F0__mean = r(mean)
        scalar mlogwageMw_M`e'F0__sd = r(sd)
        }

    // female singles distribution of wages all workers
    forval k = 0 / 1 {
        forval e = 1 / $numed {
            qui su logwage [aw = newwt] if type_M0_F`e' == 1 & kids == `k' & hours > 0
            scalar flogwageFw_M0F`e'_k`k'__mean = r(mean)
            scalar flogwageFw_M0F`e'_k`k'__sd = r(sd)
            }
        }

    forval e = 1 / $numed {
        qui su logwage [aw = newwt] if type_M0_F`e' == 1 & hours > 0
        scalar flogwageFw_M0F`e'__mean = r(mean)
        scalar flogwageFw_M0F`e'__sd = r(sd)
        }


    // married men distribution of wages all workers (this integrates over spousal state)
    forval k = 0 / 1 {
        forval e = 1 / $numed {
            qui su logwage [aw = newwt] if type_M`e'_Ff == 1 & kids == `k' & hours > 0
            scalar mlogwageMw_M`e'Ff_k`k'__mean = r(mean)
            scalar mlogwageMw_M`e'Ff_k`k'__sd = r(sd)
            }
        }

    forval e = 1 / $numed {
        qui su logwage [aw = newwt] if type_M`e'_Ff == 1 & hours > 0
        scalar mlogwageMw_M`e'Ff__mean = r(mean)
        scalar mlogwageMw_M`e'Ff__sd = r(sd)
        }


    // married women distribution of wages all workers (this integrates over spousal state)
    forval k = 0 / 1 {
        forval e = 1 / $numed {
            qui su logwage [aw = newwt] if type_Mm_F`e' == 1 & kids == `k' & hours > 0
            scalar flogwageFw_MmF`e'_k`k'__mean = r(mean)
            scalar flogwageFw_MmF`e'_k`k'__sd = r(sd)
            }
        }

    forval e = 1 / $numed {
        qui su logwage [aw = newwt] if type_Mm_F`e' == 1 & hours > 0
        scalar flogwageFw_MmF`e'__mean = r(mean)
        scalar flogwageFw_MmF`e'__sd = r(sd)
        }

    if `r' == 0 {
        di as text "calculating... wage distributions conditional on spousal employment state"
        }

    // married men distribution of wages by spousal employment state
    forval k = 0 / 1 {
        forval e = 1 / $numed {
            qui su logwage [aw = newwt] if type_M`e'_Ff == 1 & kids == `k' & hours > 0 & hours_sp == 0
            scalar mlogwageFw0_M`e'Ff_k`k'__mean = r(mean)
            scalar mlogwageFw0_M`e'Ff_k`k'__sd = r(sd)
            qui su logwage [aw = newwt] if type_M`e'_Ff == 1 & kids == `k' & hours > 0 & hours_sp > 0
            scalar mlogwageFw1_M`e'Ff_k`k'__mean = r(mean)
            scalar mlogwageFw1_M`e'Ff_k`k'__sd = r(sd)
            }
        }

    forval e = 1 / $numed {
        qui su logwage [aw = newwt] if type_M`e'_Ff == 1 & hours > 0 & hours_sp == 0
        scalar mlogwageFw0_M`e'Ff__mean = r(mean)
        scalar mlogwageFw0_M`e'Ff__sd = r(sd)
        qui su logwage [aw = newwt] if type_M`e'_Ff == 1 & hours > 0 & hours_sp > 0
        scalar mlogwageFw1_M`e'Ff__mean = r(mean)
        scalar mlogwageFw1_M`e'Ff__sd = r(sd)
        }

    // married women distribution of wages by spousal employment state
    forval k = 0 / 1 {
        forval e = 1 / $numed {
            qui su logwage [aw = newwt] if type_Mm_F`e' == 1 & kids == `k' & hours > 0 & hours_sp == 0
            scalar flogwageMw0_MmF`e'_k`k'__mean = r(mean)
            scalar flogwageMw0_MmF`e'_k`k'__sd = r(sd)
            qui su logwage [aw = newwt] if type_Mm_F`e' == 1 & kids == `k' & hours > 0 & hours_sp > 0
            scalar flogwageMw1_MmF`e'_k`k'__mean = r(mean)
            scalar flogwageMw1_MmF`e'_k`k'__sd = r(sd)
            }
        }

    forval e = 1 / $numed {
        qui su logwage [aw = newwt] if type_Mm_F`e' == 1 & hours > 0 & hours_sp == 0
        scalar flogwageMw0_MmF`e'__mean = r(mean)
        scalar flogwageMw0_MmF`e'__sd = r(sd)
        qui su logwage [aw = newwt] if type_Mm_F`e' == 1 & hours > 0 & hours_sp > 0
        scalar flogwageMw1_MmF`e'__mean = r(mean)
        scalar flogwageMw1_MmF`e'__sd = r(sd)
        }


    if `r' == 0 {
        di as text "calculating... joint wage distributions"
        }

    // product of wages within match
    forval k = 0 / 1 {
        forval e = 1 / $numed {
            forval esp = 1 / $numed {
                qui su logwage_product [aw = newwt] if type_M`e'_F`esp' == 1 & kids == `k' & hours > 0 & hours_sp > 0
                scalar jlogwage_M`e'F`esp'_k`k'__mean = r(mean)
                scalar jlogwage_M`e'F`esp'_k`k'__sd = r(sd)
                qui corr logwage logwage_sp [aw = newwt] if type_M`e'_F`esp' == 1 & kids == `k' & hours > 0 & hours_sp > 0, cov
                scalar jlogwage_M`e'F`esp'_k`k'__cov = r(cov_12)
                }
            }
        }

    forval e = 1 / $numed {
        forval esp = 1 / $numed {
            qui su logwage_product [aw = newwt] if type_M`e'_F`esp' == 1 & hours > 0 & hours_sp > 0
            scalar jlogwage_M`e'F`esp'__mean = r(mean)
            scalar jlogwage_M`e'F`esp'__sd = r(sd)
            qui corr logwage logwage_sp [aw = newwt] if type_M`e'_F`esp' == 1 & hours > 0 & hours_sp > 0, cov
            scalar jlogwage_M`e'F`esp'__cov = r(cov_12)
            }
        }

    if `r' == 0 {
        di as text "calculating... earnings distributions"
        }

    // earnings single men
    forval e = 1 / $numed {
        qui su earndiscrete [aw = newwt] if type_M`e'_F0 == 1 & hours > 0
        scalar mearnMw_M`e'F0__mean = r(mean)
        scalar mearnMw_M`e'F0__sd = r(sd)
        }

    // earnings married men
    forval e = 1 / $numed {
        qui su earndiscrete [aw = newwt] if type_M`e'_Ff == 1 & hours > 0
        scalar mearnMw_M`e'Ff__mean = r(mean)
        scalar mearnMw_M`e'Ff__sd = r(sd)
        }

    // earnings single women
    forval e = 1 / $numed {
        qui su earndiscrete [aw = newwt] if type_M0_F`e' == 1 & hours > 0
        scalar fearnFw_M0F`e'__mean = r(mean)
        scalar fearnFw_M0F`e'__sd = r(sd)
        }

    // earnings married women
    forval e = 1 / $numed {
        qui su earndiscrete [aw = newwt] if type_Mm_F`e' == 1 & hours > 0
        scalar fearnFw_MmF`e'__mean = r(mean)
        scalar fearnFw_MmF`e'__sd = r(sd)
        }

    if `r' == 0 {
        di as text "calculating... aggregated moments"
        }

    // work
    qui su hours0_Mm_F0 [aw = newwt], meanonly
    scalar mwork_MmF0 = 1 - `r(mean)'

    qui su hours0_M0_Ff [aw = newwt], meanonly
    scalar fwork_M0Ff = 1 - `r(mean)'

    qui su hoursMh0_Mm_Ff [aw = newwt], meanonly
    scalar mwork_MmFf = 1 - `r(mean)'

    qui su hoursFh0_Mm_Ff [aw = newwt], meanonly
    scalar fwork_MmFf = 1 - `r(mean)'

    qui su hours0_Mm_F0 [aw = newwt] if type_Mm_F0 == 1
    scalar mwork_MmF0 = 1 - `r(mean)'

    // conditional hours
    qui su hrdiscrete [aw = newwt] if type_Mm_F0 == 1 & hours > 0
    scalar mhourshMw_MmF0__mean = `r(mean)'
    scalar mhourshMw_MmF0__sd = `r(sd)'

    qui su hrdiscrete [aw = newwt] if type_M0_Ff == 1 & hours > 0
    scalar fhourshFw_M0Ff__mean = `r(mean)'
    scalar fhourshFw_M0Ff__sd = `r(sd)'

    qui su hrdiscrete [aw = newwt] if type_Mm_Ff == 1 & sex == 1 & hours > 0
    scalar mhourshMw_MmFf__mean = `r(mean)'
    scalar mhourshMw_MmFf__sd = `r(sd)'

    qui su hrdiscrete [aw = newwt] if type_Mm_Ff == 1 & sex == 2 & hours > 0
    scalar fhourshFw_MmFf__mean = `r(mean)'
    scalar fhourshFw_MmFf__sd = `r(sd)'

    // unconditional hours
    qui su hrdiscrete [aw = newwt] if type_Mm_F0 == 1
    scalar mhoursh_MmF0__mean = `r(mean)'
    scalar mhoursh_MmF0__sd = `r(sd)'

    qui su hrdiscrete [aw = newwt] if type_M0_Ff == 1
    scalar fhoursh_M0Ff__mean = `r(mean)'
    scalar fhoursh_M0Ff__sd = `r(sd)'

    qui su hrdiscrete [aw = newwt] if type_Mm_Ff == 1 & sex == 1
    scalar mhoursh_MmFf__mean = `r(mean)'
    scalar mhoursh_MmFf__sd = `r(sd)'

    qui su hrdiscrete [aw = newwt] if type_Mm_Ff == 1 & sex == 2
    scalar fhoursh_MmFf__mean = `r(mean)'
    scalar fhoursh_MmFf__sd = `r(sd)'

    // conditional earnings
    qui su earndiscrete [aw = newwt] if type_Mm_F0 == 1 & hours > 0
    scalar mearnMw_MmF0__mean = r(mean)
    scalar mearnMw_MmF0__sd = r(sd)

    qui su earndiscrete [aw = newwt] if type_M0_Ff == 1 & hours > 0
    scalar fearnFw_M0Ff__mean = r(mean)
    scalar fearnFw_M0Ff__sd = r(sd)

    qui su earndiscrete [aw = newwt] if type_Mm_Ff == 1 & sex == 1 & hours > 0
    scalar mearnMw_MmFf__mean = r(mean)
    scalar mearnMw_MmFf__sd = r(sd)

    qui su earndiscrete [aw = newwt] if type_Mm_Ff == 1 & sex == 2 & hours > 0
    scalar fearnFw_MmFf__mean = r(mean)
    scalar fearnFw_MmFf__sd = r(sd)

    // unconditional earnings
    qui su earndiscrete [aw = newwt] if type_Mm_F0 == 1
    scalar mearn_MmF0__mean = r(mean)
    scalar mearn_MmF0__sd = r(sd)

    qui su earndiscrete [aw = newwt] if type_M0_Ff == 1
    scalar fearn_M0Ff__mean = r(mean)
    scalar fearn_M0Ff__sd = r(sd)

    qui su earndiscrete [aw = newwt] if type_Mm_Ff == 1 & sex == 1
    scalar mearn_MmFf__mean = r(mean)
    scalar mearn_MmFf__sd = r(sd)

    qui su earndiscrete [aw = newwt] if type_Mm_Ff == 1 & sex == 2
    scalar fearn_MmFf__mean = r(mean)
    scalar fearn_MmFf__sd = r(sd)

    post momIPUMS (`r') `postlist'

    restore

    }

postclose momIPUMS

local labh0 "UN"
local labh1 "PT"
local labh2 "FT"

use "$save_data/mom_ipums_ed$numed`'_age$minage$maxage$statetag.dta", clear

// type distribution
forval e = 1 / $numed {
    cap label var jtype_M`e'_F0 "type distribution: frac (m=`e',f=0)"
    cap label var jtype_M0_F`e' "type distribution: frac (m=0,f=`e')"
    forval esp = 1 / $numed {
        cap label var jtype_M`e'_F`esp' "type distribution: frac (m=`e',f=`esp')"
        }
    }

forval m = 1 / $nummarket {
    forval e = 1 / $numed {
        cap label var R`m'jtype_M`e'_F0 "Market `m' type distribution: frac (m=`e',f=0)"
        cap label var R`m'jtype_M0_F`e' "Market `m' type distribution: frac (m=0,f=`e')"
        forval esp = 1 / $numed {
            cap label var R`m'jtype_M`e'_F`esp' "Market `m' type distribution: frac (m=`e',f=`esp')"
            }
        }
    }

// joint labour supply allocations
forval k = 0 / 1 {
    forval e = 1 / $numed {
        forval h = 0 / 2 {
            cap label var mhours`h'_k`k'_M`e'_F0 "male work hours: frac Mh=`labh`h'' Fh=.  [type m=`e',f=0,k=`k']"
            cap label var fhours`h'_k`k'_M0_F`e' "fem. work hours: frac Mh=.  Fh=`labh`h'' [type m=0,f=`e',k=`k']"
            }

        forval h = 0 / 2 {
            cap label var mhours`h'_k`k'_M`e'_Ff "male work hours: frac Mh=`labh`h'' Fh=+  [type m=`e',f=+,k=`k']"
            cap label var fhours`h'_k`k'_Mm_F`e' "fem. work hours: frac Mh=+  Fh=`labh`h'' [type m=+,f=`e',k=`k']"
            }

        forval esp = 1 / $numed {
            forval h = 0 / 2 {
                forval hsp = 0 / 2 {
                    cap label var jhoursMh`h'Fh`hsp'_k`k'_M`e'_F`esp' "jnt. work hours: frac Mh=`labh`h'' Fh=`labh`hsp'' [type m=`e',f=`esp',k=`k']"
                    }
                }
            }
        }
    }

// joint labour supply allocations (integrating over own and spousal type)
forval k = 0 / 1 {
    forval h = 0 / 2 {
        cap label var mhours`h'_k`k'_sin "male work hours: frac Mh=`labh`h'' Fh=.  [type m=+,f=0,k=`k']"
        cap label var fhours`h'_k`k'_sin "fem. work hours: frac Mh=.  Fh=`labh`h'' [type m=0,f=+,k=`k']"
        }

    forval h = 0 / 2 {
        cap label var mhours`h'_k`k'_cou "male work hours: frac Mh=`labh`h'' Fh=+  [type m=+,f=+,k=`k']"
        cap label var fhours`h'_k`k'_cou "fem. work hours: frac Mh=+  Fh=`labh`h'' [type m=+,f=+,k=`k']"
        }
    }


// male singles distribution of wages within hours cell
forval k = 0 / 1 {
    forval e = 1 / $numed {
        forval h = 1 / 2 {
            cap label var mlogwage_M`e'F0_k`k'_h`h'__mean "male log wages (mean) [type m=`e',f=0,k=`k' Mh=`labh`h'',Fh=. ]"
            cap label var mlogwage_M`e'F0_k`k'_h`h'__sd   "male log wages (s.d.) [type m=`e',f=0,k=`k' Mh=`labh`h'',Fh=. ]"
            }
        }
    }

// male singles distribution of wages all workers
forval k = 0 / 1 {
    forval e = 1 / $numed {
        cap label var mlogwageMw_M`e'F0_k`k'__mean "male log wages (mean) [type m=`e',f=0,k=`k' Mw=1,Fw=. ]"
        cap label var mlogwageMw_M`e'F0_k`k'__sd   "male log wages (s.d.) [type m=`e',f=0,k=`k' Mw=1,Fw=. ]"
        }
    }

forval e = 1 / $numed {
    cap label var mlogwageMw_M`e'F0__mean "male log wages (mean) [type m=`e',f=0, Mw=1,Fw=. ]"
    cap label var mlogwageMw_M`e'F0__sd   "male log wages (s.d.) [type m=`e',f=0, Mw=1,Fw=. ]"
    }

// female singles distribution of wages within hours cell
forval k = 0 / 1 {
    forval e = 1 / $numed {
        forval h = 1 / 2 {
            cap label var flogwage_M0F`e'_k`k'_h`h'__mean "fem. log wages (mean) [type m=0,f=`e',k=`k' Mh=. ,Fh=`labh`h'']"
            cap label var flogwage_M0F`e'_k`k'_h`h'__sd   "fem. log wages (s.d.) [type m=0,f=`e',k=`k' Mh=. ,Fh=`labh`h'']"
            }
        }
    }


// female singles distribution of wages all workers
forval k = 0 / 1 {
    forval e = 1 / $numed {
        cap label var flogwageFw_M0F`e'_k`k'__mean "fem. log wages (mean) [type m=0,f=`e',k=`k' Mw=. ,Fw=1]"
        cap label var flogwageFw_M0F`e'_k`k'__sd   "fem. log wages (s.d.) [type m=0,f=`e',k=`k' Mw=. ,Fw=1]"
        }
    }

forval e = 1 / $numed {
    cap label var flogwageFw_M0F`e'__mean "fem. log wages (mean) [type m=0,f=`e', Mw=. ,Fw=1]"
    cap label var flogwageFw_M0F`e'__sd   "fem. log wages (s.d.) [type m=0,f=`e', Mw=. ,Fw=1]"
    }


// married men distribution of wages within hours cell (this integrates over spousal state)
forval k = 0 / 1 {
    forval e = 1 / $numed {
        forval h = 1 / 2 {
            cap label var mlogwage_M`e'Ff_k`k'_h`h'__mean "male log wages (mean) [type m=`e',f=+,k=`k' Mh=`labh`h'',Fh=+ ]"
            cap label var mlogwage_M`e'Ff_k`k'_h`h'__sd   "male log wages (s.d.) [type m=`e',f=+,k=`k' Mh=`labh`h'',Fh=+ ]"
            }
        }
    }

// married men distribution of wages all workers (this integrates over spousal state)
forval k = 0 / 1 {
    forval e = 1 / $numed {
        cap label var mlogwageMw_M`e'Ff_k`k'__mean "male log wages (mean) [type m=`e',f=+,k=`k' Mw=1,Fw=+ ]"
        cap label var mlogwageMw_M`e'Ff_k`k'__sd   "male log wages (s.d.) [type m=`e',f=+,k=`k' Mw=1,Fw=+ ]"
        }
    }

forval e = 1 / $numed {
    cap label var mlogwageMw_M`e'Ff__mean "male log wages (mean) [type m=`e',f=+, Mw=1,Fw=+ ]"
    cap label var mlogwageMw_M`e'Ff__sd   "male log wages (s.d.) [type m=`e',f=+, Mw=1,Fw=+ ]"
    }

// married women distribution of wages within hours cell (this integrates over spousal state)
forval k = 0 / 1 {
    forval e = 1 / $numed {
        forval h = 1 / 2 {
            cap label var flogwage_MmF`e'_k`k'_h`h'__mean "fem. log wages (mean) [type m=+,f=`e',k=`k' Mh=+ ,Fh=`labh`h'']"
            cap label var flogwage_MmF`e'_k`k'_h`h'__sd   "fem. log wages (s.d.) [type m=+,f=`e',k=`k' Mh=+ ,Fh=`labh`h'']"
            }
        }
    }

// married women distribution of wages all workers (this integrates over spousal state)
forval k = 0 / 1 {
    forval e = 1 / $numed {
        cap label var flogwageFw_MmF`e'_k`k'__mean "fem. log wages (mean) [type m=+,f=`e',k=`k' Mw=+ ,Fw=1]"
        cap label var flogwageFw_MmF`e'_k`k'__sd   "fem. log wages (s.d.) [type m=+,f=`e',k=`k' Mw=+ ,Fw=1]"
        }
    }

forval e = 1 / $numed {
    cap label var flogwageFw_MmF`e'__mean "fem. log wages (mean) [type m=+,f=`e',Mw=+,Fw=1]"
    cap label var flogwageFw_MmF`e'__sd   "fem. log wages (s.d.) [type m=+,f=`e',Mw=+,Fw=1]"
    }

// married men distribution of wages by spousal employment state
forval k = 0 / 1 {
    forval e = 1 / $numed {
        forval w = 0 / 1 {
            cap label var mlogwageFw`w'_M`e'Ff_k`k'__mean "male log wages (mean) [type m=`e',f=+,k=`k' Mw=1,Fw=`w']"
            cap label var mlogwageFw`w'_M`e'Ff_k`k'__sd   "male log wages (s.d.) [type m=`e',f=+,k=`k' Mw=1,Fw=`w']"
            }
        }
    }

forval e = 1 / $numed {
    forval w = 0 / 1 {
        cap label var mlogwageFw`w'_M`e'Ff__mean "male log wages (mean) [type m=`e',f=+ Mw=1,Fw=`w']"
        cap label var mlogwageFw`w'_M`e'Ff__sd   "male log wages (s.d.) [type m=`e',f=+ Mw=1,Fw=`w']"
        }
    }

// married women distribution of wages by spousal employment state
forval k = 0 / 1 {
    forval e = 1 / $numed {
        forval w = 0 / 1 {
            cap label var flogwageMw`w'_MmF`e'_k`k'__mean "male log wages (mean) [type m=+,f=`e',k=`k' Mw=`w',Fw=1]"
            cap label var flogwageMw`w'_MmF`e'_k`k'__sd   "male log wages (s.d.) [type m=+,f=`e',k=`k' Mw=`w',Fw=1]"
            }
        }
    }

forval e = 1 / $numed {
    forval w = 0 / 1 {
        cap label var flogwageMw`w'_MmF`e'__mean "male log wages (mean) [type m=+,f=`e' Mw=`w',Fw=1]"
        cap label var flogwageMw`w'_MmF`e'__sd   "male log wages (s.d.) [type m=+,f=`e' Mw=`w',Fw=1]"
        }
    }

// product of wages within match
forval k = 0 / 1 {
    forval e = 1 / $numed {
        forval esp = 1 / $numed {
            cap label var jlogwage_M`e'F`esp'_k`k'__mean "product log wages (mean) [type m=`e',f=`esp',k=`k' Mw=1,Fw=1]"
            cap label var jlogwage_M`e'F`esp'_k`k'__sd   "product log wages (s.d.) [type m=`e',f=`esp',k=`k' Mw=1,Fw=1]"
            cap label var jlogwage_M`e'F`esp'_k`k'__cov  "male/fm log wages (cov.) [type m=`e',f=`esp',k=`k' Mw=1,Fw=1]"
            }
        }
    }

forval e = 1 / $numed {
    forval esp = 1 / $numed {
        cap label var jlogwage_M`e'F`esp'__mean "product log wages (mean) [type m=`e',f=`esp' Mw=1,Fw=1]"
        cap label var jlogwage_M`e'F`esp'__sd   "product log wages (s.d.) [type m=`e',f=`esp' Mw=1,Fw=1]"
        cap label var jlogwage_M`e'F`esp'__cov  "male/fm log wages (cov.) [type m=`e',f=`esp' Mw=1,Fw=1]"
        }
    }

// joint labour supply allocations
forval k = 0 / 1 {
    forval e = 1 / $numed {

        cap label var mhours_k`k'_M`e'_F0__mean "male work hours (mean) [type m=`e',f=0,k=`k', Mw=1,Fw=.]"
        cap label var mhours_k`k'_M`e'_F0__sd   "male work hours (s.d.) [type m=`e',f=0,k=`k', Mw=1,Fw=.]"
        cap label var fhours_k`k'_M0_F`e'__mean "fem. work hours (mean) [type m=0,f=`e',k=`k', Mw=1,Fw=.]"
        cap label var fhours_k`k'_M0_F`e'__sd   "fem. work hours (s.d.) [type m=0,f=`e',k=`k', Mw=1,Fw=.]"
        cap label var mhours_k`k'_M`e'_Ff__mean "male work hours (mean) [type m=`e',f=+,k=`k', Mw=1,Fw=+]"
        cap label var mhours_k`k'_M`e'_Ff__sd   "male work hours (s.d.) [type m=`e',f=+,k=`k', Mw=1,Fw=+]"
        cap label var fhours_k`k'_Mm_F`e'__mean "fem. work hours (mean) [type m=+,f=`e',k=`k', Mw=+,Fw=1]"
        cap label var fhours_k`k'_Mm_F`e'__sd   "fem. work hours (s.d.) [type m=+,f=`e',k=`k', Mw=+,Fw=1]"

        }
    }

// earnings
forval e = 1 / $numed {
    cap label var mearnMw_M`e'F0__mean "male earnings (mean) [type m=`e',f=0, Mw=1,Fw=. ]"
    cap label var mearnMw_M`e'F0__sd   "male earnings (s.d.) [type m=`e',f=0, Mw=1,Fw=. ]"
    cap label var mearnMw_M`e'Ff__mean "male earnings (mean) [type m=`e',f=+, Mw=1,Fw=+ ]"
    cap label var mearnMw_M`e'Ff__sd   "male earnings (s.d.) [type m=`e',f=+, Mw=1,Fw=+ ]"
    }

forval e = 1 / $numed {
    cap label var fearnFw_M0F`e'__mean "fem. earnings (mean) [type m=0,f=`e', Mw=.,Fw=1 ]"
    cap label var fearnFw_M0F`e'__sd   "fem. earnings (s.d.) [type m=0,f=`e', Mw=.,Fw=1 ]"
    cap label var fearnFw_MmF`e'__mean "fem. earnings (mean) [type m=+,f=`e', Mw=+,Fw=1 ]"
    cap label var fearnFw_MmF`e'__sd   "fem. earnings (s.d.) [type m=+,f=`e', Mw=+,Fw=1 ]"
    }

// aggregated

cap label var mwork_MmF0 "male work [type m=+,f=0 ]"
cap label var mwork_MmFf "male work [type m=+,f=+ ]"
cap label var fwork_M0Ff "female work [type m=0,f=0 ]"
cap label var fwork_MmFf "female work [type m=+,f=+ ]"

cap label var mhourshMw_MmF0__mean "male work hours (mean) [type m=+,f=0, Mw=1,Fw=.]"
cap label var mhourshMw_MmF0__sd   "male work hours (s.d.) [type m=+,f=0, Mw=1,Fw=.]"
cap label var fhourshFw_M0Ff__mean "fem. work hours (mean) [type m=0,f=+, Mw=1,Fw=.]"
cap label var fhourshFw_M0Ff__sd   "fem. work hours (s.d.) [type m=0,f=+, Mw=1,Fw=.]"

cap label var mhourshMw_MmFf__mean "male work hours (mean) [type m=+,f=+, Mw=1,Fw=+]"
cap label var mhourshMw_MmFf__sd   "male work hours (s.d.) [type m=+,f=+, Mw=1,Fw=+]"
cap label var fhourshFw_MmFf__mean "fem. work hours (mean) [type m=+,f=+, Mw=+,Fw=1]"
cap label var fhourshFw_MmFf__sd   "fem. work hours (s.d.) [type m=+,f=+, Mw=+,Fw=1]"

cap label var mhoursh_MmF0__mean "male unconditional work hours (mean) [type m=+,f=0, Mw=+,Fw=.]"
cap label var mhoursh_MmF0__sd   "male unconditional work hours (s.d.) [type m=+,f=0, Mw=+,Fw=.]"
cap label var fhoursh_M0Ff__mean "fem. unconditional work hours (mean) [type m=0,f=+, Mw=+,Fw=.]"
cap label var fhoursh_M0Ff__sd   "fem. unconditional work hours (s.d.) [type m=0,f=+, Mw=+,Fw=.]"

cap label var mhoursh_MmFf__mean "male unconditional work hours (mean) [type m=+,f=+, Mw=+,Fw=+]"
cap label var mhoursh_MmFf__sd   "male unconditional work hours (s.d.) [type m=+,f=+, Mw=+,Fw=+]"
cap label var fhoursh_MmFf__mean "fem. unconditional work hours (mean) [type m=+,f=+, Mw=+,Fw=+]"
cap label var fhoursh_MmFf__sd   "fem. unconditional work hours (s.d.) [type m=+,f=+, Mw=+,Fw=+]"

cap label var mearnMw_MmF0__mean "male earnings (mean) [type m=+,f=0, Mw=1,Fw=. ]"
cap label var mearnMw_MmF0__sd   "male earnings (s.d.) [type m=+,f=0, Mw=1,Fw=. ]"
cap label var mearnMw_MmFf__mean "male earnings (mean) [type m=+,f=+, Mw=1,Fw=+ ]"
cap label var mearnMw_MmFf__sd   "male earnings (s.d.) [type m=+,f=+, Mw=1,Fw=+ ]"

cap label var fearnFw_M0Ff__mean "fem. earnings (mean) [type m=0,f=+, Mw=.,Fw=1 ]"
cap label var fearnFw_M0Ff__sd   "fem. earnings (s.d.) [type m=0,f=+, Mw=.,Fw=1 ]"
cap label var fearnFw_MmFf__mean "fem. earnings (mean) [type m=+,f=+, Mw=+,Fw=1 ]"
cap label var fearnFw_MmFf__sd   "fem. earnings (s.d.) [type m=+,f=+, Mw=+,Fw=1 ]"

cap label var mearn_MmF0__mean "male unconditional earnings (mean) [type m=+,f=0, Mw=+,Fw=. ]"
cap label var mearn_MmF0__sd   "male unconditional earnings (s.d.) [type m=+,f=0, Mw=+,Fw=. ]"
cap label var mearn_MmFf__mean "male unconditional earnings (mean) [type m=+,f=+, Mw=+,Fw=+ ]"
cap label var mearn_MmFf__sd   "male unconditional earnings (s.d.) [type m=+,f=+, Mw=+,Fw=+ ]"

cap label var fearn_M0Ff__mean "fem. unconditional earnings (mean) [type m=0,f=+, Mw=.,Fw=+ ]"
cap label var fearn_M0Ff__sd   "fem. unconditional earnings (s.d.) [type m=0,f=+, Mw=.,Fw=+ ]"
cap label var fearn_MmFf__mean "fem. unconditional earnings (mean) [type m=+,f=+, Mw=+,Fw=+ ]"
cap label var fearn_MmFf__sd   "fem. unconditional earnings (s.d.) [type m=+,f=+, Mw=+,Fw=+ ]"

sort r
save, replace

log close mylogfile
