module moments

    use globaldef

    interface momentDistance
        module procedure momentDistanceWeight
        module procedure momentDistanceNoWeight
    end interface momentDistance

    interface checkStateIntegration
        module procedure checkStateIntegration1d
        module procedure checkStateIntegration2d
    end interface checkStateIntegration

contains

    subroutine labelMoments(label)
        use mathfn
        implicit none
        character(len = 128), intent(out) :: label(NMOM)
        integer :: iX1, iX2, iX3, iX4, iX5, iX6
        integer :: ixAll
        include 'momlab.f90'
    end subroutine labelMoments

    subroutine aggregateEallSingle(np, prob, eallVec, eall)
        implicit none
        integer, intent(in) :: np
        real(dp),     intent(in) :: prob(np)
        type(eall_t), intent(in) :: eallVec(np)
        type(eall_t), intent(out) :: eall

        integer  :: qX, hX1
        real(dp) :: group_mean1(np), group_sd1(np), group_prob1(np)

        eall%hours = 0.0_dp
        eall%work  = 0.0_dp
        eall%home  = 0.0_dp

        eall%Eu = 0.0_dp
        eall%Eswf = 0.0_dp
        eall%Etottax = 0.0_dp

        do qX = 1, np
            eall%hours = eall%hours + prob(qX) * eallVec(qX)%hours
            eall%work = eall%work  + prob(qX) * eallVec(qX)%work
            eall%home = eall%home  + prob(qX) * eallVec(qX)%home
            eall%Eu = eall%Eu  + prob(qX) * eallVec(qX)%Eu
            eall%Eswf = eall%Eswf  + prob(qX) * eallVec(qX)%Eswf
            eall%Etottax = eall%Etottax  + prob(qX) * eallVec(qX)%Etottax
        end do

        ! conditional work hours
        do qX = 1, np
            group_mean1(qX) = eallVec(qX)%hourshW
            group_sd1(qX) = eallVec(qX)%hourshWsd
            group_prob1(qX) = prob(qX) * eallVec(qX)%work(1)
        end do

        call momentIntegrateGroup( np, group_prob1, group_mean1, group_sd1, &
                                   eall%hourshW, eall%hourshWsd )

        ! unconditional work hours
        do qX = 1, np
            group_mean1(qX) = eallVec(qX)%hoursh
            group_sd1(qX) = eallVec(qX)%hourshsd
        end do

        call momentIntegrateGroup( np, prob, group_mean1, group_sd1, &
                                   eall%hoursh, eall%hourshsd )

        ! unconditional home hours
        do qX = 1, np
            group_mean1(qX) = eallVec(qX)%homeh
            group_sd1(qX) = eallVec(qX)%homehsd
        end do

        call momentIntegrateGroup( np, prob, group_mean1, group_sd1, &
                                   eall%homeh, eall%homehsd )

        ! accepted wages conditional on hours group
        do hX1 = 0, 2

            do qX = 1, np
                group_mean1(qX) = eallVec(qX)%wage(hX1)
                group_sd1(qX) = eallVec(qX)%wagesd(hX1)
                group_prob1(qX) = prob(qX) * eallVec(qX)%hours(hX1)
            end do

            call momentIntegrateGroup( np, group_prob1, group_mean1, group_sd1, &
                                       eall%wage(hX1), eall%wagesd(hX1) )

        end do

        ! accepted wages conditional on employment
        do qX = 1, np
            group_mean1(qX) = eallVec(qX)%wageWork
            group_sd1(qX) = eallVec(qX)%wagesdWork
            group_prob1(qX) = prob(qX) * eallVec(qX)%work(1)
        end do

        call momentIntegrateGroup( np, group_prob1, group_mean1, group_sd1, &
                                   eall%wageWork, eall%wagesdWork )

        ! home hours conditional on employment
        do hX1 = 0, 1

            do qX = 1, np
                group_mean1(qX) = eallVec(qX)%homehW(hX1)
                group_sd1(qX) = eallVec(qX)%homehWsd(hX1)
                group_prob1(qX) = prob(qX) * eallVec(qX)%work(hX1)
            end do

            call momentIntegrateGroup( np, group_prob1, group_mean1, group_sd1, &
                                       eall%homehW(hX1), eall%homehWsd(hX1) )

        end do

        ! home hours conditional on work hours
        do hX1 = 0, 2

            do qX = 1, np
                group_mean1(qX) = eallVec(qX)%homehH(hX1)
                group_sd1(qX) = eallVec(qX)%homehHsd(hX1)
                group_prob1(qX) = prob(qX) * eallVec(qX)%hours(hX1)
            end do

            call momentIntegrateGroup( np, group_prob1, group_mean1, group_sd1, &
                                       eall%homehH(hX1), eall%homehHsd(hX1) )

        end do

        ! unconditional earnings
        do qX = 1, np
            group_mean1(qX) = eallVec(qX)%earn
            group_sd1(qX) = eallVec(qX)%earnsd
        end do

        call momentIntegrateGroup( np, prob, group_mean1, group_sd1, &
                                   eall%earn, eall%earnsd )

        ! conditional earnings
        do qX = 1, np
            group_mean1(qX) = eallVec(qX)%earnW
            group_sd1(qX) = eallVec(qX)%earnWsd
            group_prob1(qX) = prob(qX) * eallVec(qX)%work(1)
        end do

        call momentIntegrateGroup( np, group_prob1, group_mean1, group_sd1, &
                                   eall%earnW, eall%earnWsd )

    end subroutine aggregateEallSingle


    pure function inverseMoments(momIn) result(momOut)
        implicit none
        type(mom_t), intent(in) :: momIn
        type(mom_t) :: momOut
        integer :: iX
        real(dp) :: momVec(NMOM)

        momVec = transfer( momIn, (/1.0_dp/) )

        do iX = 1, NMOM
            if ( momVec(iX) > 0.0_dp ) then
                momVec(iX) = 1.0_dp / momVec(iX) ** 2
            end if
        end do

        momOut = transfer( momVec, momIn )

    end function inverseMoments


    subroutine checkStateIntegration1d(x, lab)
        implicit none
        real(dp), intent(in) :: x(:)
        character(len = *), intent(in) :: lab
        integer :: n
        real(dp) :: y
        real(dp), parameter :: tol = 1.0e-8_dp
        n = size(x)
        y = sum(x)

        if ( (y > 1.0_dp + tol) .or. (y < 1.0_dp - tol) ) then
            write (*,*) ' state integration for ' // lab // ':'
            write (*,*) y
        end if

    end subroutine checkStateIntegration1d


    subroutine checkStateIntegration2d(x, lab)
        implicit none
        real(dp), intent(in) :: x(:, :)
        character(len = *), intent(in) :: lab
        integer :: n1, n2
        real(dp) :: y
        real(dp), parameter :: tol = 1.0e-8_dp
        n1 = size(x, 1)
        n2 = size(x, 2)
        y = sum(x)

        if ( (y > 1.0_dp + tol) .or. (y < 1.0_dp - tol) ) then
            write (*,*) ' state integration for ' // lab // ':'
            write (*,*) y
        end if

    end subroutine checkStateIntegration2d



    subroutine writemom(funit, vmode, ind, mom1, mom2, momV, iX1, iX2, iX3, iX4, iX5)
        implicit none
        integer, intent(in) :: funit
        logical, intent(in) :: vmode
        logical, intent(in) :: ind
        real(dp), intent(in) :: mom1, mom2, momV
        integer, intent(in), optional :: iX1
        integer, intent(in), optional :: iX2
        integer, intent(in), optional :: iX3
        integer, intent(in), optional :: iX4
        integer, intent(in), optional :: iX5

        character(len = *), parameter :: fmt5 = '(5(I2,",",2X),F28.16,",",2X,F28.16)'
        character(len = *), parameter :: fmt4 = '(4(I2,",",2X),F28.16,",",2X,F28.16)'
        character(len = *), parameter :: fmt3 = '(3(I2,",",2X),F28.16,",",2X,F28.16)'
        character(len = *), parameter :: fmt2 = '(2(I2,",",2X),F28.16,",",2X,F28.16)'
        character(len = *), parameter :: fmt1 = '(1(I2,",",2X),F28.16,",",2X,F28.16)'
        character(len = *), parameter :: fmt0 = '(F28.16,",",2X,F28.16)'

        character(len = *), parameter :: vmt5 = '(5(I2,",",2X),F28.16,",",2X,F28.16,",",2X,F28.16)'
        character(len = *), parameter :: vmt4 = '(4(I2,",",2X),F28.16,",",2X,F28.16,",",2X,F28.16)'
        character(len = *), parameter :: vmt3 = '(3(I2,",",2X),F28.16,",",2X,F28.16,",",2X,F28.16)'
        character(len = *), parameter :: vmt2 = '(2(I2,",",2X),F28.16,",",2X,F28.16,",",2X,F28.16)'
        character(len = *), parameter :: vmt1 = '(1(I2,",",2X),F28.16,",",2X,F28.16,",",2X,F28.16)'
        character(len = *), parameter :: vmt0 = '(F28.16,",",2X,F28.16,",",2X,F28.16)'

        if ( ind ) then
            if ( present(iX2) .and. present(iX3) .and. present(iX4) .and. present(iX5) ) then
                if ( vmode ) then
                    write(funit,vmt5) iX1, iX2, iX3, iX4, iX5, mom1, mom2, momV
                else
                    write(funit,fmt5) iX1, iX2, iX3, iX4, iX5, mom1, mom2
                end if
            elseif ( present(iX2) .and. present(iX3) .and. present(iX4) ) then
                if ( vmode ) then
                    write(funit,vmt4) iX1, iX2, iX3, iX4, mom1, mom2, momV
                else
                    write(funit,fmt4) iX1, iX2, iX3, iX4, mom1, mom2
                end if
            elseif ( present(iX2) .and. present(iX3) ) then
                if ( vmode ) then
                    write(funit,vmt3) iX1, iX2, iX3, mom1, mom2, momV
                else
                    write(funit,fmt3) iX1, iX2, iX3, mom1, mom2
                end if
            elseif ( present(iX2) ) then
                if ( vmode ) then
                    write(funit,vmt2) iX1, iX2, mom1, mom2, momV
                else
                    write(funit,fmt2) iX1, iX2, mom1, mom2
                end if
            elseif ( present(iX1) ) then
                if ( vmode ) then
                    write(funit,vmt1) iX1, mom1, mom2, momV
                else
                    write(funit,fmt1) iX1, mom1, mom2
                end if
            else
                if ( present(iX1) .or. present(iX2) .or. present(iX3) .or. present(iX4) .or. present(iX5) ) then
                    write(funit,*) 'writemom input error'
                else
                    if ( vmode ) then
                        write(funit,vmt0) mom1, mom2, momV
                    else
                        write(funit,fmt0) mom1, mom2
                    end if
                end if
            end if
        else
            if ( vmode ) then
                write(funit,vmt0) mom1, mom2, momV
            else
                write(funit,fmt0) mom1, mom2
            end if
        end if

    end subroutine writemom

    subroutine printMomentsCompareExtra(mom1, mom2, momV2, fdir, prefix)
        implicit none
        type(momExtra_t), intent(in) :: mom1, mom2
        type(momExtra_t), intent(in), optional :: momV2
        character(len = *), intent(in), optional :: fdir
        character(len = *), intent(in), optional :: prefix

        character(len = *), parameter :: fmt5 = '(5(I2,",",2X),F25.16,",",2X,F25.16)'
        character(len = *), parameter :: fmt4 = '(4(I2,",",2X),F25.16,",",2X,F25.16)'
        character(len = *), parameter :: fmt3 = '(3(I2,",",2X),F25.16,",",2X,F25.16)'
        character(len = *), parameter :: fmt2 = '(2(I2,",",2X),F25.16,",",2X,F25.16)'
        character(len = *), parameter :: fmt1 = '(1(I2,",",2X),F25.16,",",2X,F25.16)'

        type(momExtra_t) :: momV
        integer :: pX1, pX2, kX, hX1, hX2
        logical :: writefile
        integer :: funit
        logical :: vmode
        character(len = :), allocatable :: fprefix
        logical :: ind

        if ( present(fdir) ) then
            writefile = .true.
            ind = .true.
            if ( present(prefix) ) then
                fprefix = trim(adjustl(fdir)) // trim(adjustl(prefix)) // '/'
            else
                fprefix = trim(adjustl(fdir)) // '/'
            end if
        else
            writefile = .false.
            ind = .false.
            funit = IOUT
        end if

        if ( present( momV2) ) then
            vmode = .true.
            momV = momV2
        else
            vmode = .false.
        end if

        if ( .not. writefile ) then
            print *, '**** EXTRA IPUMS ****'
            print *
            print *, 'marginal labour supply (integrates over spousal state)'
            print *
            print *, 'mhours_Mf(0:2, NTYPE1, 0:1)'
        else
            open( newunit = funit, file = fprefix // 'mhours_Mf.csv', status = "replace" )
        end if

        do kX = 0, 1
            do pX1 = 1, NTYPE1
                do hX1 = 0, 2
                    call writemom( funit, vmode, ind, mom1%mhours_Mf(hX1, pX1, kX), mom2%mhours_Mf(hX1, pX1, kX), &
                                   momV%mhours_Mf(hX1, pX1, kX), hX1, pX1, kX )
                end do
            end do
        end do

        if (writefile) then
            close(funit)
        end if

        if ( .not. writefile ) then
            print *
            print *, 'fhours_mF(0:2, NTYPE2, 0:1)'
        else
            open( newunit = funit, file = fprefix // 'fhours_mF.csv', status = "replace" )
        end if

        do kX = 0, 1
            do pX2 = 1, NTYPE2
                do hX2 = 0, 2
                    call writemom( funit, vmode, ind, mom1%fhours_mF(hX2, pX2, kX), mom2%fhours_mF(hX2, pX2, kX), &
                                   momV%fhours_mF(hX2, pX2, kX), hX2, pX2, kX )
                end do
            end do
        end do

        if (writefile) then
            close(funit)
        end if

        if ( .not. writefile ) then
            print *
            print *, 'jhours_MF(0:2, 0:2, NTYPE1, NTYPE2, 0:1)'
        else
            open( newunit = funit, file = fprefix // 'jhours_MF.csv', status = "replace" )
        end if

        do kX = 0, 1
            do pX2 = 1, NTYPE2
                do pX1 = 1, NTYPE1
                    do hX2 = 0, 2
                        do hX1 = 0, 2
                            call writemom( funit, vmode, ind, mom1%jhours_MF(hX1, hX2, pX1, pX2, kX), mom2%jhours_MF(hX1, hX2, pX1, pX2, kX), &
                                           momV%jhours_MF(hX1, hX2, pX1, pX2, kX), hX1, hX2, pX1, pX2, kX )
                        end do
                    end do
                end do
            end do
        end do

        if (writefile) then
            close(funit)
        end if

        if ( .not. writefile ) then
            print *
            print *
            print *, 'marginal labour supply (integrates over own and spousal state)'
            print *
            print *, 'mhours_sin(0:2, 0:1)'
        else
            open( newunit = funit, file = fprefix // 'mhours_sin.csv', status = "replace" )
        end if

        do kX = 0, 1
            do hX1 = 0, 2
                call writemom( funit, vmode, ind, mom1%mhours_sin(hX1, kX), mom2%mhours_sin(hX1, kX), momV%mhours_sin(hX1, kX), hX1, kX )
            end do
        end do

        if (writefile) then
            close(funit)
        end if

        if ( .not. writefile ) then
            print *
            print *, 'fhours_sin(0:2, 0:1)'
        else
            open( newunit = funit, file = fprefix // 'fhours_sin.csv', status = "replace" )
        end if

        do kX = 0, 1
            do hX2 = 0, 2
                call writemom( funit, vmode, ind, mom1%fhours_sin(hX2, kX), mom2%fhours_sin(hX2, kX), momV%fhours_sin(hX2, kX), hX2, kX )
            end do
        end do

        if (writefile) then
            close(funit)
        end if

        if ( .not. writefile ) then
            print *
            print *, 'mhours_cou(0:2, 0:1)'
        else
            open( newunit = funit, file = fprefix // 'mhours_cou.csv', status = "replace" )
        end if

        do kX = 0, 1
            do hX1 = 0, 2
                call writemom( funit, vmode, ind, mom1%mhours_cou(hX1, kX), mom2%mhours_cou(hX1, kX), momV%mhours_cou(hX1, kX), hX1, kX )
            end do
        end do

        if (writefile) then
            close(funit)
        end if

        if ( .not. writefile ) then
            print *
            print *, 'fhours_cou(0:2, 0:1)'
        else
            open( newunit = funit, file = fprefix // 'fhours_cou.csv', status = "replace" )
        end if

        do kX = 0, 1
            do hX2 = 0, 2
                call writemom( funit, vmode, ind, mom1%fhours_cou(hX2, kX), mom2%fhours_cou(hX2, kX), momV%fhours_cou(hX2, kX), hX2, kX )
            end do
        end do

        if (writefile) then
            close(funit)
        end if

        if ( .not. writefile ) then
            print *
            print *
            print *, 'wage distribution (single men)'
            print *
            print *, 'mwage_M0(1:2, NTYPE1, 0:1)'
        else
            open( newunit = funit, file = fprefix // 'mwage_M0.csv', status = "replace" )
        end if

        do kX = 0, 1
            do pX1 = 1, NTYPE1
                do hX1 = 1, 2
                    call writemom( funit, vmode, ind, mom1%mwage_M0(hX1, pX1, kX), mom2%mwage_M0(hX1, pX1, kX), &
                                   momV%mwage_M0(hX1, pX1, kX), hX1, pX1, kX )
                end do
            end do
        end do

        if (writefile) then
            close(funit)
        end if

        if ( .not. writefile ) then
            print *
            print *, 'mwagesd_M0(1:2, NTYPE1, 0:1)'
        else
            open( newunit = funit, file = fprefix // 'mwagesd_M0.csv', status = "replace" )
        end if

        do kX = 0, 1
            do pX1 = 1, NTYPE1
                do hX1 = 1, 2
                    call writemom( funit, vmode, ind, mom1%mwagesd_M0(hX1, pX1, kX), mom2%mwagesd_M0(hX1, pX1, kX), &
                                   momV%mwagesd_M0(hX1, pX1, kX), hX1, pX1, kX )
                end do
            end do
        end do

        if (writefile) then
            close(funit)
        end if


        if ( .not. writefile ) then
            print *
            print *
            print *, 'wage distribution (single women)'
            print *
            print *, 'fwage_0F(1:2, NTYPE2, 0:1)'
        else
            open( newunit = funit, file = fprefix // 'fwage_0F.csv', status = "replace" )
        end if

        do kX = 0, 1
            do pX2 = 1, NTYPE2
                do hX2 = 1, 2
                    call writemom( funit, vmode, ind, mom1%fwage_0F(hX2, pX2, kX), mom2%fwage_0F(hX2, pX2, kX), &
                                   momV%fwage_0F(hX2, pX2, kX), hX2, pX2, kX )
                end do
            end do
        end do

        if (writefile) then
            close(funit)
        end if

        if ( .not. writefile ) then
            print *
            print *, 'fwagesd_0F(1:2,NTYPE2, 0:1)'
        else
            open( newunit = funit, file = fprefix // 'fwagesd_0F.csv', status = "replace" )
        end if

        do kX = 0, 1
            do pX2 = 1, NTYPE2
                do hX2 = 1, 2
                    call writemom( funit, vmode, ind, mom1%fwagesd_0F(hX2, pX2, kX), mom2%fwagesd_0F(hX2, pX2, kX), &
                                   momV%fwagesd_0F(hX2, pX2, kX), hX2, pX2, kX )
                end do
            end do
        end do

        if (writefile) then
            close(funit)
        end if

        if ( .not. writefile ) then
            print *
            print *
            print *, 'wage distribution (married men, integrates over spouse type/state)'
            print *
            print *, 'mwage_Mf(1:2, NTYPE1, 0:1)'
        else
            open( newunit = funit, file = fprefix // 'mwage_Mf.csv', status = "replace" )
        end if

        do kX = 0, 1
            do pX1 = 1, NTYPE1
                do hX1 = 1, 2
                    call writemom( funit, vmode, ind, mom1%mwage_Mf(hX1, pX1, kX), mom2%mwage_Mf(hX1, pX1, kX), &
                                   momV%mwage_Mf(hX1, pX1, kX), hX1, pX1, kX )
                end do
            end do
        end do

        if (writefile) then
            close(funit)
        end if

        if ( .not. writefile ) then
            print *
            print *, 'mwagesd_Mf(1:2, NTYPE1, 0:1)'
        else
            open( newunit = funit, file = fprefix // 'mwagesd_Mf.csv', status = "replace" )
        end if

        do kX = 0, 1
            do pX1 = 1, NTYPE1
                do hX1 = 1, 2
                    call writemom( funit, vmode, ind, mom1%mwagesd_Mf(hX1, pX1, kX), mom2%mwagesd_Mf(hX1, pX1, kX), &
                                   momV%mwagesd_Mf(hX1, pX1, kX), hX1, pX1, kX )
                end do
            end do
        end do

        if (writefile) then
            close(funit)
        end if

        if ( .not. writefile ) then
            print *
            print *
            print *, 'wage distribution (married women, integrates over spouse type/state)'
            print *
            print *, 'fwage_mF(1:2, NTYPE2, 0:1)'
        else
            open( newunit = funit, file = fprefix // 'fwage_mF.csv', status = "replace" )
        end if

        do kX = 0, 1
            do pX2 = 1, NTYPE2
                do hX2 = 1, 2
                    call writemom( funit, vmode, ind, mom1%fwage_mF(hX2, pX2, kX), mom2%fwage_mF(hX2, pX2, kX), &
                                   momV%fwage_mF(hX2, pX2, kX), hX2, pX2, kX )
                end do
            end do
        end do

        if (writefile) then
            close(funit)
        end if

        if ( .not. writefile ) then
            print *
            print *, 'fwagesd_mF(1:2,NTYPE2, 0:1)'
        else
            open( newunit = funit, file = fprefix // 'fwagesd_mF.csv', status = "replace" )
        end if

        do kX = 0, 1
            do pX2 = 1, NTYPE2
                do hX2 = 1, 2
                    call writemom( funit, vmode, ind, mom1%fwagesd_mF(hX2, pX2, kX), mom2%fwagesd_mF(hX2, pX2, kX), &
                                   momV%fwagesd_mF(hX2, pX2, kX), hX2, pX2, kX )
                end do
            end do
        end do

        if (writefile) then
            close(funit)
        end if

        if ( .not. writefile ) then
            print *
            print *
            print *, 'wage distribution (married men, by spousal employment state)'
            print *
            print *, 'mwageFw_Mf(0:1, NTYPE1, 0:1)'
        else
            open( newunit = funit, file = fprefix // 'mwageFw_Mf.csv', status = "replace" )
        end if

        do kX = 0, 1
            do pX1 = 1, NTYPE1
                do hX2 = 0, 1
                    call writemom( funit, vmode, ind, mom1%mwageFw_Mf(hX2, pX1, kX), mom2%mwageFw_Mf(hX2, pX1, kX), &
                                   momV%mwageFw_Mf(hX2, pX1, kX), hX2, pX1, kX )
                end do
            end do
        end do

        if (writefile) then
            close(funit)
        end if

        if ( .not. writefile ) then
            print *
            print *, 'mwageFwsd_Mf(0:1, NTYPE1, 0:1)'
        else
            open( newunit = funit, file = fprefix // 'mwageFwsd_Mf.csv', status = "replace" )
        end if

        do kX = 0, 1
            do pX1 = 1, NTYPE1
                do hX2 = 0, 1
                    call writemom( funit, vmode, ind, mom1%mwageFwsd_Mf(hX2, pX1, kX), mom2%mwageFwsd_Mf(hX2, pX1, kX), &
                                   momV%mwageFwsd_Mf(hX2, pX1, kX), hX2, pX1, kX )
                end do
            end do
        end do

        if (writefile) then
            close(funit)
        end if

        if ( .not. writefile ) then
            print *
            print *
            print *, 'wage distribution (married women, by spousal employment state)'
            print *
            print *, 'fwageMw_mF(0:1, NTYPE2, 0:1)'
        else
            open( newunit = funit, file = fprefix // 'fwageMw_mF.csv', status = "replace" )
        end if

        do kX = 0, 1
            do pX2 = 1, NTYPE2
                do hX1 = 0, 1
                    call writemom( funit, vmode, ind, mom1%fwageMw_mF(hX1, pX2, kX), mom2%fwageMw_mF(hX1, pX2, kX), &
                                   momV%fwageMw_mF(hX1, pX2, kX), hX1, pX2, kX )
                end do
            end do
        end do

        if (writefile) then
            close(funit)
        end if

        if ( .not. writefile ) then
            print *
            print *, 'fwageMwsd_mF(0:1,NTYPE2, 0:1)'
        else
            open( newunit = funit, file = fprefix // 'fwageMwsd_mF.csv', status = "replace" )
        end if

        do kX = 0, 1
            do pX2 = 1, NTYPE2
                do hX1 = 0, 1
                    call writemom( funit, vmode, ind, mom1%fwageMwsd_mF(hX1, pX2, kX), mom2%fwageMwsd_mF(hX1, pX2, kX), &
                                   momV%fwageMwsd_mF(hX1, pX2, kX), hX1, pX2, kX )
                end do
            end do
        end do

        if (writefile) then
            close(funit)
        end if

        if ( .not. writefile ) then
            print *
            print *
            print *, 'wage distribution all workers (single men)'
            print *
            print *, 'mwageMw_M0(NTYPE1, 0:1)'
        else
            open( newunit = funit, file = fprefix // 'mwageMw_M0.csv', status = "replace" )
        end if

        do kX = 0, 1
            do pX1 = 1, NTYPE1
                call writemom( funit, vmode, ind, mom1%mwageMw_M0(pX1, kX), mom2%mwageMw_M0(pX1, kX), momV%mwageMw_M0(pX1, kX), pX1, kX )
            end do
        end do

        if (writefile) then
            close(funit)
        end if

        if ( .not. writefile ) then
            print *
            print *, 'mwagesdMw_M0(NTYPE1, 0:1)'
        else
            open( newunit = funit, file = fprefix // 'mwagesdMw_M0.csv', status = "replace" )
        end if

        do kX = 0, 1
            do pX1 = 1, NTYPE1
                call writemom( funit, vmode, ind, mom1%mwagesdMw_M0(pX1, kX), mom2%mwagesdMw_M0(pX1, kX), &
                               momV%mwagesdMw_M0(pX1, kX), pX1, kX )
            end do
        end do

        if (writefile) then
            close(funit)
        end if

        if ( .not. writefile ) then
            print *
            print *
            print *, 'wage distribution all workers (single women)'
            print *
            print *, 'fwageFw_0F(NTYPE2, 0:1)'
        else
            open( newunit = funit, file = fprefix // 'fwageFw_0F.csv', status = "replace" )
        end if

        do kX = 0, 1
            do pX2 = 1, NTYPE2
                call writemom( funit, vmode, ind, mom1%fwageFw_0F(pX2, kX), mom2%fwageFw_0F(pX2, kX), momV%fwageFw_0F(pX2, kX), pX2, kX )
            end do
        end do

        if (writefile) then
            close(funit)
        end if

        if ( .not. writefile ) then
            print *
            print *, 'fwagesdFw_0F(NTYPE2, 0:1)'
        else
            open( newunit = funit, file = fprefix // 'fwagesdFw_0F.csv', status = "replace" )
        end if

        do kX = 0, 1
            do pX2 = 1, NTYPE2
                call writemom( funit, vmode, ind, mom1%fwagesdFw_0F(pX2, kX), mom2%fwagesdFw_0F(pX2, kX), &
                               momV%fwagesdFw_0F(pX2, kX), pX2, kX )
            end do
        end do

        if (writefile) then
            close(funit)
        end if

        if ( .not. writefile ) then
            print *
            print *
            print *, 'wage distribution all workers (married men, integrates over spouse type/state)'
            print *
            print *, 'mwageMw_Mf(NTYPE1, 0:1)'
        else
            open( newunit = funit, file = fprefix // 'mwageMw_Mf.csv', status = "replace" )
        end if

        do kX = 0, 1
            do pX1 = 1, NTYPE1
                call writemom( funit, vmode, ind, mom1%mwageMw_Mf(pX1, kX), mom2%mwageMw_Mf(pX1, kX), momV%mwageMw_Mf(pX1, kX), pX1, kX )
            end do
        end do

        if (writefile) then
            close(funit)
        end if

        if ( .not. writefile ) then
            print *
            print *, 'mwagesdMw_Mf(NTYPE1, 0:1)'
        else
            open( newunit = funit, file = fprefix // 'mwagesdMw_Mf.csv', status = "replace" )
        end if

        do kX = 0, 1
            do pX1 = 1, NTYPE1
                call writemom( funit, vmode, ind, mom1%mwagesdMw_Mf(pX1, kX), mom2%mwagesdMw_Mf(pX1, kX), &
                               momV%mwagesdMw_Mf(pX1, kX), pX1, kX )
            end do
        end do

        if (writefile) then
            close(funit)
        end if

        if ( .not. writefile ) then
            print *
            print *
            print *, 'wage distribution all workers (married women, integrates over spouse type/state)'
            print *
            print *, 'fwageFw_mF(NTYPE2, 0:1)'
        else
            open( newunit = funit, file = fprefix // 'fwageFw_mF.csv', status = "replace" )
        end if

        do kX = 0, 1
            do pX2 = 1, NTYPE2
                call writemom( funit, vmode, ind, mom1%fwageFw_mF(pX2, kX), mom2%fwageFw_mF(pX2, kX), momV%fwageFw_mF(pX2, kX), pX2, kX )
            end do
        end do

        if (writefile) then
            close(funit)
        end if

        if ( .not. writefile ) then
            print *
            print *, 'fwagesdFw_mF(NTYPE2, 0:1)'
        else
            open( newunit = funit, file = fprefix // 'fwagesdFw_mF.csv', status = "replace" )
        end if

        do kX = 0, 1
            do pX2 = 1, NTYPE2
                call writemom( funit, vmode, ind, mom1%fwagesdFw_mF(pX2, kX), mom2%fwagesdFw_mF(pX2, kX), &
                               momV%fwagesdFw_mF(pX2, kX), pX2, kX )
            end do
        end do

        if (writefile) then
            close(funit)
        end if

        if ( .not. writefile ) then
            print *
            print *
            print *, 'product of wages within a match'
            print *
            print *, 'jwage_MF(NTYPE1, NTYPE2, 0:1)'
        else
            open( newunit = funit, file = fprefix // 'jwage_MF.csv', status = "replace" )
        end if

        do kX = 0, 1
            do pX2 = 1, NTYPE2
                do pX1 = 1, NTYPE1
                    call writemom( funit, vmode, ind, mom1%jwage_MF(pX1, pX2, kX), mom2%jwage_MF(pX1, pX2, kX), &
                                   momV%jwage_MF(pX1, pX2, kX), pX1, pX2, kX )
                end do
            end do
        end do

        if (writefile) then
            close(funit)
        end if

        if ( .not. writefile ) then
            print *
            print *, 'jwagesd_MF(NTYPE1, NTYPE2, 0:1)'
        else
            open( newunit = funit, file = fprefix // 'jwagesd_MF.csv', status = "replace" )
        end if

        do kX = 0, 1
            do pX2 = 1, NTYPE2
                do pX1 = 1, NTYPE1
                    call writemom( funit, vmode, ind, mom1%jwagesd_MF(pX1, pX2, kX), mom2%jwagesd_MF(pX1, pX2, kX), &
                                   momV%jwagesd_MF(pX1, pX2, kX), pX1, pX2, kX )
                end do
            end do
        end do

        if (writefile) then
            close(funit)
        end if

        if ( .not. writefile ) then
            print *
            print *, 'jwagecov_MF(NTYPE1, NTYPE2, 0:1)'
        else
            open( newunit = funit, file = fprefix // 'jwagecov_MF.csv', status = "replace" )
        end if

        do kX = 0, 1
            do pX2 = 1, NTYPE2
                do pX1 = 1, NTYPE1
                    call writemom( funit, vmode, ind, mom1%jwagecov_MF(pX1, pX2, kX), mom2%jwagecov_MF(pX1, pX2, kX), &
                                   momV%jwagecov_MF(pX1, pX2, kX), pX1, pX2, kX )
                end do
            end do
        end do

        if (writefile) then
            close(funit)
        end if

        if ( .not. writefile ) then
            print *
            print *

            print *, 'home production hours by own employment (single men)'
            print *
            print *, 'mhomehMw_m0(0:1, 0:1)'
        else
            open( newunit = funit, file = fprefix // 'mhomehMw_m0.csv', status = "replace" )
        end if

        do kX = 0, 1
            do hX1 = 0, 1
                call writemom( funit, vmode, ind, mom1%mhomehMw_m0(hX1, kX), mom2%mhomehMw_m0(hX1, kX), momV%mhomehMw_m0(hX1, kX), hX1, kX )
            end do
        end do

        if (writefile) then
            close(funit)
        end if

        if ( .not. writefile ) then
            print *
            print *, 'mhomehMwsd_m0(0:1, 0:1)'
        else
            open( newunit = funit, file = fprefix // 'mhomehMwsd_m0.csv', status = "replace" )
        end if

        do kX = 0, 1
            do hX1 = 0, 1
                call writemom( funit, vmode, ind, mom1%mhomehMwsd_m0(hX1, kX), mom2%mhomehMwsd_m0(hX1, kX), &
                               momV%mhomehMwsd_m0(hX1, kX), hX1, kX )
            end do
        end do

        if (writefile) then
            close(funit)
        end if

        if ( .not. writefile ) then
            print *
            print *

            print *, 'home production hours by own hours (single women)'
            print *
            print *, 'fhomehFh_0f(0:2, 0:1)'
        else
            open( newunit = funit, file = fprefix // 'fhomehFh_0f.csv', status = "replace" )
        end if

        do kX = 0, 1
            do hX2 = 0, 2
                call writemom( funit, vmode, ind, mom1%fhomehFh_0f(hX2, kX), mom2%fhomehFh_0f(hX2, kX), momV%fhomehFh_0f(hX2, kX), hX2, kX )
            end do
        end do

        if (writefile) then
            close(funit)
        end if

        if ( .not. writefile ) then
            print *
            print *, 'fhomehFhsd_0f(0:2, 0:1)'
        else
            open( newunit = funit, file = fprefix // 'fhomehFhsd_0f.csv', status = "replace" )
        end if

        do kX = 0, 1
            do hX2 = 0, 2
                call writemom( funit, vmode, ind, mom1%fhomehFhsd_0f(hX2, kX), mom2%fhomehFhsd_0f(hX2, kX), &
                               momV%fhomehFhsd_0f(hX2, kX), hX2, kX )
            end do
        end do

        if (writefile) then
            close(funit)
        end if

        if ( .not. writefile ) then
            print *
            print *
            print *, 'home production hours by own employment (married men)'
            print *
            print *, 'mhomehMw_mf(0:1, 0:1)'
        else
            open( newunit = funit, file = fprefix // 'mhomehMw_mf.csv', status = "replace" )
        end if

        do kX = 0, 1
            do hX1 = 0, 1
                call writemom( funit, vmode, ind, mom1%mhomehMw_mf(hX1, kX), mom2%mhomehMw_mf(hX1, kX), &
                               momV%mhomehMw_mf(hX1, kX), hX1, kX )
            end do
        end do

        if (writefile) then
            close(funit)
        end if

        if ( .not. writefile ) then
            print *
            print *, 'mhomehMwsd_mf(0:1,0:1)'
        else
            open( newunit = funit, file = fprefix // 'mhomehMwsd_mf.csv', status = "replace" )
        end if

        do kX = 0, 1
            do hX1 = 0, 1
                call writemom( funit, vmode, ind, mom1%mhomehMwsd_mf(hX1, kX), mom2%mhomehMwsd_mf(hX1, kX), &
                               momV%mhomehMwsd_mf(hX1, kX), hX1, kX )
            end do
        end do

        if (writefile) then
            close(funit)
        end if

        if ( .not. writefile ) then

            print *
            print *

            print *, 'home production hours by own hours (married women)'
            print *
            print *, 'fhomehFh_mf(0:2, 0:1)'
        else
            open( newunit = funit, file = fprefix // 'fhomehFh_mf.csv', status = "replace" )
        end if

        do kX = 0, 1
            do hX2 = 0, 2
                call writemom( funit, vmode, ind, mom1%fhomehFh_mf(hX2, kX), mom2%fhomehFh_mf(hX2, kX), &
                               momV%fhomehFh_mf(hX2, kX), hX2, kX )
            end do
        end do

        if (writefile) then
            close(funit)
        end if

        if ( .not. writefile ) then
            print *
            print *, 'fhomehFhsd_mf(0:2, 0:1)'
        else
            open( newunit = funit, file = fprefix // 'fhomehFhsd_mf.csv', status = "replace" )
        end if

        do kX = 0, 1
            do hX2 = 0, 2
                call writemom( funit, vmode, ind, mom1%fhomehFhsd_mf(hX2, kX), mom2%fhomehFhsd_mf(hX2, kX), &
                               momV%fhomehFhsd_mf(hX2, kX), hX2, kX )
            end do
        end do

        if (writefile) then
            close(funit)
        end if

        if ( .not. writefile ) then
            print *
            print *

            print *, 'home production hours by spouse employment (married men)'
            print *
            print *, 'mhomehFw_mf(0:1, 0:1)'
        else
            open( newunit = funit, file = fprefix // 'mhomehFw_mf.csv', status = "replace" )
        end if

        do kX = 0, 1
            do hX2 = 0, 1
                call writemom( funit, vmode, ind, mom1%mhomehFw_mf(hX2, kX), mom2%mhomehFw_mf(hX2, kX), &
                               momV%mhomehFw_mf(hX2, kX), hX2, kX )
            end do
        end do

        if (writefile) then
            close(funit)
        end if

        if ( .not. writefile ) then
            print *
            print *, 'mhomehFhsd_mf(0:1, 0:1)'
        else
            open( newunit = funit, file = fprefix // 'mhomehFwsd_mf.csv', status = "replace" )
        end if

        do kX = 0, 1
            do hX2 = 0, 1
                call writemom( funit, vmode, ind, mom1%mhomehFwsd_mf(hX2, kX), mom2%mhomehFwsd_mf(hX2, kX), &
                               momV%mhomehFwsd_mf(hX2, kX), hX2, kX )
            end do
        end do

        if (writefile) then
            close(funit)
        end if

        if ( .not. writefile ) then
            print *
            print *

            print *, 'home production hours by spouse employment (married women)'
            print *
            print *, 'fhomehMw_mf(0:1, 0:1)'
        else
            open( newunit = funit, file = fprefix // 'fhomehMw_mf.csv', status = "replace" )
        end if

        do kX = 0, 1
            do hX1 = 0, 1
                call writemom( funit, vmode, ind, mom1%fhomehMw_mf(hX1, kX), mom2%fhomehMw_mf(hX1, kX), &
                               momV%fhomehMw_mf(hX1, kX), hX1, kX )
            end do
        end do

        if (writefile) then
            close(funit)
        end if

        if ( .not. writefile ) then
            print *
            print *, 'fhomehMwsd_mf(0:1, 0:1)'
        else
            open( newunit = funit, file = fprefix // 'fhomehMwsd_mf.csv', status = "replace" )
        end if

        do kX = 0, 1
            do hX1 = 0, 1
                call writemom( funit, vmode, ind, mom1%fhomehMwsd_mf(hX1, kX), mom2%fhomehMwsd_mf(hX1, kX), &
                               momV%fhomehMwsd_mf(hX1, kX), hX1, kX )
            end do
        end do

        if (writefile) then
            close(funit)
        end if

        if ( .not. writefile ) then
            print *
            print *, 'marginal home time (integrates over own and spousal state)'
            print *
            print *, 'mhome_sin(0:2, 0:1)'
        else
            open( newunit = funit, file = fprefix // 'mhome_sin.csv', status = "replace" )
        end if

        do kX = 0, 1
            do hX1 = 1, NHH
                call writemom( funit, vmode, ind, mom1%mhome_sin(hX1, kX), mom2%mhome_sin(hX1, kX), momV%mhome_sin(hX1, kX), hX1, kX )
            end do
        end do

        if (writefile) then
            close(funit)
        end if

        if ( .not. writefile ) then
            print *
            print *, 'fhome_sin(0:2, 0:1)'
        else
            open( newunit = funit, file = fprefix // 'fhome_sin.csv', status = "replace" )
        end if

        do kX = 0, 1
            do hX2 = 1, NHH
                call writemom( funit, vmode, ind, mom1%fhome_sin(hX2, kX), mom2%fhome_sin(hX2, kX), momV%fhome_sin(hX2, kX), hX2, kX )
            end do
        end do

        if (writefile) then
            close(funit)
        end if

        if ( .not. writefile ) then
            print *
            print *, 'mhome_cou(0:2, 0:1)'
        else
            open( newunit = funit, file = fprefix // 'mhome_cou.csv', status = "replace" )
        end if

        do kX = 0, 1
            do hX1 = 1, NHH
                call writemom( funit, vmode, ind, mom1%mhome_cou(hX1, kX), mom2%mhome_cou(hX1, kX), momV%mhome_cou(hX1, kX), hX1, kX )
            end do
        end do

        if (writefile) then
            close(funit)
        end if

        if ( .not. writefile ) then
            print *
            print *, 'fhome_cou(0:2, 0:1)'
        else
            open( newunit = funit, file = fprefix // 'fhome_cou.csv', status = "replace" )
        end if

        do kX = 0, 1
            do hX2 = 1, NHH
                call writemom( funit, vmode, ind, mom1%fhome_cou(hX2, kX), mom2%fhome_cou(hX2, kX), momV%fhome_cou(hX2, kX), hX2, kX )
            end do
        end do

        if (writefile) then
            close(funit)
        end if

        if ( .not. writefile ) then
            print *
            print *
            print *, 'type distribution'
            print *
            print *, 'jtype_M0(NTYPE1)'
        else
            open( newunit = funit, file = fprefix // 'jtype_M0.csv', status = "replace" )
        end if

        do pX1 = 1, NTYPE1
            call writemom( funit, vmode, ind, mom1%jtype_M0(pX1), mom2%jtype_M0(pX1), momV%jtype_M0(pX1), pX1 )
        end do

        if (writefile) then
            close(funit)
        end if

        if ( .not. writefile ) then
            print *
            print *, 'jtype_0F(NTYPE2)'
        else
            open( newunit = funit, file = fprefix // 'jtype_0F.csv', status = "replace" )
        end if

        do pX2 = 1, NTYPE2
             call writemom( funit, vmode, ind, mom1%jtype_0F(pX2), mom2%jtype_0F(pX2), momV%jtype_0F(pX2), pX2 )
        end do

        if (writefile) then
            close(funit)
        end if

        if ( .not. writefile ) then
            print *
            print *, 'jtype_MF(NTYPE1, NTYPE2)'
        else
            open( newunit = funit, file = fprefix // 'jtype_MF.csv', status = "replace" )
        end if

        do pX2 = 1, NTYPE2
            do pX1 = 1, NTYPE1
                 call writemom( funit, vmode, ind, mom1%jtype_MF(pX1, pX2), mom2%jtype_MF(pX1, pX2), momV%jtype_MF(pX1, pX2), pX1, pX2 )
            end do
        end do

        if (writefile) then
            close(funit)
        end if


        if ( .not. writefile ) then
            print *
            print *, 'mwork_cou_knk'
        else
            open( newunit = funit, file = fprefix // 'mwork_cou_knk.csv', status = "replace" )
        end if
        call writemom( funit, vmode, ind, mom1%mwork_cou_knk, mom2%mwork_cou_knk, momV%mwork_cou_knk )

        if (writefile) then
            close(funit)
        end if

        if ( .not. writefile ) then
            print *
            print *, 'mhoursh_cou_knk'
        else
            open( newunit = funit, file = fprefix // 'mhoursh_cou_knk.csv', status = "replace" )
        end if
        call writemom( funit, vmode, ind, mom1%mhoursh_cou_knk, mom2%mhoursh_cou_knk, momV%mhoursh_cou_knk )

        if (writefile) then
            close(funit)
        end if

        if ( .not. writefile ) then
            print *
            print *, 'mhourshMw_cou_knk'
        else
            open( newunit = funit, file = fprefix // 'mhourshMw_cou_knk.csv', status = "replace" )
        end if
        call writemom( funit, vmode, ind, mom1%mhourshMw_cou_knk, mom2%mhourshMw_cou_knk, momV%mhourshMw_cou_knk )

        if (writefile) then
            close(funit)
        end if

        if ( .not. writefile ) then
            print *
            print *, 'mhomeh_cou_knk'
        else
            open( newunit = funit, file = fprefix // 'mhomeh_cou_knk.csv', status = "replace" )
        end if
        call writemom( funit, vmode, ind, mom1%mhomeh_cou_knk, mom2%mhomeh_cou_knk, momV%mhomeh_cou_knk )

        if (writefile) then
            close(funit)
        end if

        if ( .not. writefile ) then
            print *
            print *, 'mearnMw_cou_knk'
        else
            open( newunit = funit, file = fprefix // 'mearnMw_cou_knk.csv', status = "replace" )
        end if
        call writemom( funit, vmode, ind, mom1%mearnMw_cou_knk, mom2%mearnMw_cou_knk, momV%mearnMw_cou_knk )

        if (writefile) then
            close(funit)
        end if



        if ( .not. writefile ) then
            print *
            print *, 'fwork_cou_knk'
        else
            open( newunit = funit, file = fprefix // 'fwork_cou_knk.csv', status = "replace" )
        end if
        call writemom( funit, vmode, ind, mom1%fwork_cou_knk, mom2%fwork_cou_knk, momV%fwork_cou_knk )

        if (writefile) then
            close(funit)
        end if

        if ( .not. writefile ) then
            print *
            print *, 'fhoursh_cou_knk'
        else
            open( newunit = funit, file = fprefix // 'fhoursh_cou_knk.csv', status = "replace" )
        end if
        call writemom( funit, vmode, ind, mom1%fhoursh_cou_knk, mom2%fhoursh_cou_knk, momV%fhoursh_cou_knk )

        if (writefile) then
            close(funit)
        end if

        if ( .not. writefile ) then
            print *
            print *, 'fhourshFw_cou_knk'
        else
            open( newunit = funit, file = fprefix // 'fhourshFw_cou_knk.csv', status = "replace" )
        end if
        call writemom( funit, vmode, ind, mom1%fhourshFw_cou_knk, mom2%fhourshFw_cou_knk, momV%fhourshFw_cou_knk )

        if (writefile) then
            close(funit)
        end if

        if ( .not. writefile ) then
            print *
            print *, 'fhomeh_cou_knk'
        else
            open( newunit = funit, file = fprefix // 'fhomeh_cou_knk.csv', status = "replace" )
        end if
        call writemom( funit, vmode, ind, mom1%fhomeh_cou_knk, mom2%fhomeh_cou_knk, momV%fhomeh_cou_knk )

        if (writefile) then
            close(funit)
        end if

        if ( .not. writefile ) then
            print *
            print *, 'fearnFw_cou_knk'
        else
            open( newunit = funit, file = fprefix // 'fearnFw_cou_knk.csv', status = "replace" )
        end if
        call writemom( funit, vmode, ind, mom1%fearnFw_cou_knk, mom2%fearnFw_cou_knk, momV%fearnFw_cou_knk )

        if (writefile) then
            close(funit)
        end if


        if ( .not. writefile ) then
            print *
            print *, 'mwork_sin_knk'
        else
            open( newunit = funit, file = fprefix // 'mwork_sin_knk.csv', status = "replace" )
        end if
        call writemom( funit, vmode, ind, mom1%mwork_sin_knk, mom2%mwork_sin_knk, momV%mwork_sin_knk )

        if (writefile) then
            close(funit)
        end if

        if ( .not. writefile ) then
            print *
            print *, 'mhoursh_sin_knk'
        else
            open( newunit = funit, file = fprefix // 'mhoursh_sin_knk.csv', status = "replace" )
        end if
        call writemom( funit, vmode, ind, mom1%mhoursh_sin_knk, mom2%mhoursh_sin_knk, momV%mhoursh_sin_knk )

        if (writefile) then
            close(funit)
        end if

        if ( .not. writefile ) then
            print *
            print *, 'mhourshMw_sin_knk'
        else
            open( newunit = funit, file = fprefix // 'mhourshMw_sin_knk.csv', status = "replace" )
        end if
        call writemom( funit, vmode, ind, mom1%mhourshMw_sin_knk, mom2%mhourshMw_sin_knk, momV%mhourshMw_sin_knk )

        if (writefile) then
            close(funit)
        end if

        if ( .not. writefile ) then
            print *
            print *, 'mhomeh_sin_knk'
        else
            open( newunit = funit, file = fprefix // 'mhomeh_sin_knk.csv', status = "replace" )
        end if
        call writemom( funit, vmode, ind, mom1%mhomeh_sin_knk, mom2%mhomeh_sin_knk, momV%mhomeh_sin_knk )

        if (writefile) then
            close(funit)
        end if

        if ( .not. writefile ) then
            print *
            print *, 'mearnMw_sin_knk'
        else
            open( newunit = funit, file = fprefix // 'mearnMw_sin_knk.csv', status = "replace" )
        end if
        call writemom( funit, vmode, ind, mom1%mearnMw_sin_knk, mom2%mearnMw_sin_knk, momV%mearnMw_sin_knk )

        if (writefile) then
            close(funit)
        end if

        if ( .not. writefile ) then
            print *
            print *, 'fwork_sin_knk'
        else
            open( newunit = funit, file = fprefix // 'fwork_sin_knk.csv', status = "replace" )
        end if
        call writemom( funit, vmode, ind, mom1%fwork_sin_knk, mom2%fwork_sin_knk, momV%fwork_sin_knk )

        if (writefile) then
            close(funit)
        end if

        if ( .not. writefile ) then
            print *
            print *, 'fhoursh_sin_knk'
        else
            open( newunit = funit, file = fprefix // 'fhoursh_sin_knk.csv', status = "replace" )
        end if
        call writemom( funit, vmode, ind, mom1%fhoursh_sin_knk, mom2%fhoursh_sin_knk, momV%fhoursh_sin_knk )

        if (writefile) then
            close(funit)
        end if

        if ( .not. writefile ) then
            print *
            print *, 'fhourshFw_sin_knk'
        else
            open( newunit = funit, file = fprefix // 'fhourshFw_sin_knk.csv', status = "replace" )
        end if
        call writemom( funit, vmode, ind, mom1%fhourshFw_sin_knk, mom2%fhourshFw_sin_knk, momV%fhourshFw_sin_knk )

        if (writefile) then
            close(funit)
        end if

        if ( .not. writefile ) then
            print *
            print *, 'fhomeh_sin_knk'
        else
            open( newunit = funit, file = fprefix // 'fhomeh_sin_knk.csv', status = "replace" )
        end if
        call writemom( funit, vmode, ind, mom1%fhomeh_sin_knk, mom2%fhomeh_sin_knk, momV%fhomeh_sin_knk )

        if (writefile) then
            close(funit)
        end if

        if ( .not. writefile ) then
            print *
            print *, 'fearnFw_sin_knk'
        else
            open( newunit = funit, file = fprefix // 'fearnFw_sin_knk.csv', status = "replace" )
        end if
        call writemom( funit, vmode, ind, mom1%fearnFw_sin_knk, mom2%fearnFw_sin_knk, momV%fearnFw_sin_knk )

        if (writefile) then
            close(funit)
        end if

        ! moved from mom_t
        if ( .not. writefile ) then
            print *
            print *, 'mhoursh_M_0(NTYPE1)'
        else
            open( newunit = funit, file = fprefix // '/mhoursh_M_0.csv', status = "replace" )
        end if

        do pX1 = 1, NTYPE1
            call writemom( funit, vmode, ind, mom1%mhoursh_M_0(pX1), mom2%mhoursh_M_0(pX1), momV%mhoursh_M_0(pX1), pX1 )
        end do

        if (writefile) then
            close(funit)
        end if

        if ( .not. writefile ) then
            print *
            print *, 'fhoursh_0_F(NTYPE2)'
        else
            open( newunit = funit, file = fprefix // '/fhoursh_0_F.csv', status = "replace" )
        end if

        do pX2 = 1, NTYPE2
            call writemom( funit, vmode, ind, mom1%fhoursh_0_F(pX2), mom2%fhoursh_0_F(pX2), momV%fhoursh_0_F(pX2), pX2 )
        end do

        if (writefile) then
            close(funit)
        end if

        if ( .not. writefile ) then
            print *
            print *, 'mwork_M_0(NTYPE1)'
        else
            open( newunit = funit, file = fprefix // '/mwork_M_0.csv', status = "replace" )
        end if

        do pX1 = 1, NTYPE1
            call writemom( funit, vmode, ind, mom1%mwork_M_0(pX1), mom2%mwork_M_0(pX1), momV%mwork_M_0(pX1), pX1 )
        end do

        if (writefile) then
            close(funit)
        end if

        if ( .not. writefile ) then
            print *
            print *, 'fwork_0_F(NTYPE2)'
        else
            open( newunit = funit, file = fprefix // '/fwork_0_F.csv', status = "replace" )
        end if

        do pX2 = 1, NTYPE2
            call writemom( funit, vmode, ind, mom1%fwork_0_F(pX2), mom2%fwork_0_F(pX2), momV%fwork_0_F(pX2), pX2 )
        end do

        if (writefile) then
            close(funit)
        end if

        if ( .not. writefile ) then
            print *
            print *, 'mhomeh_M_0(NTYPE1)'
        else
            open( newunit = funit, file = fprefix // '/mhomeh_M_0.csv', status = "replace" )
        end if

        do pX1 = 1, NTYPE1
            call writemom( funit, vmode, ind, mom1%mhomeh_M_0(pX1), mom2%mhomeh_M_0(pX1), momV%mhomeh_M_0(pX1), pX1 )
        end do

        if (writefile) then
            close(funit)
        end if

        if ( .not. writefile ) then
            print *
            print *, 'fhomeh_0_F(NTYPE2)'
        else
            open( newunit = funit, file = fprefix // '/fhomeh_0_F.csv', status = "replace" )
        end if

        do pX2 = 1, NTYPE2
            call writemom( funit, vmode, ind, mom1%fhomeh_0_F(pX2), mom2%fhomeh_0_F(pX2), momV%fhomeh_0_F(pX2), pX2 )
        end do

        if (writefile) then
            close(funit)
        end if

    end subroutine printMomentsCompareExtra

    subroutine printMomentsCompare(mom1, mom2, momV2, fdir)
        implicit none
        type(mom_t), intent(in) :: mom1, mom2
        type(mom_t), intent(in), optional :: momV2
        character(len = *), intent(in), optional :: fdir

        character(len = *), parameter :: fmt5 = '(5(I2,",",2X),F25.16,",",2X,F25.16)'
        character(len = *), parameter :: fmt4 = '(4(I2,",",2X),F25.16,",",2X,F25.16)'
        character(len = *), parameter :: fmt3 = '(3(I2,",",2X),F25.16,",",2X,F25.16)'
        character(len = *), parameter :: fmt2 = '(2(I2,",",2X),F25.16,",",2X,F25.16)'
        character(len = *), parameter :: fmt1 = '(1(I2,",",2X),F25.16,",",2X,F25.16)'

        type(mom_t) :: momV
        integer :: pX1, pX2, kX, hX1, hX2, mX
        logical :: writefile
        integer :: funit
        logical :: vmode
        logical :: ind

        if ( present(fdir) ) then
            writefile = .true.
            ind = .true.
        else
            writefile = .false.
            ind = .false.
            funit = IOUT
        end if

        if ( present( momV2) ) then
            vmode = .true.
            momV = momV2
        else
            vmode = .false.
        end if

        if ( .not. writefile ) then
            print *
            print *
            print *, 'Rjtype_M0(NTYPE1, NMARKET)'
        else
            open( newunit = funit, file = fdir // '/Rjtype_M0.csv', status = "replace" )
        end if

        do mX = 1, NMARKET
            do pX1 = 1, NTYPE1
                call writemom( funit, vmode, ind, mom1%Rjtype_M0(pX1, mX), mom2%Rjtype_M0(pX1, mX), momV%Rjtype_M0(pX1, mX), pX1, mX )
            end do
        end do

        if (writefile) then
            close(funit)
        end if

        if ( .not. writefile ) then
            print *
            print *, 'Rjtype_0F(NTYPE2, NMARKET)'
        else
            open( newunit = funit, file = fdir // '/Rjtype_0F.csv', status = "replace" )
        end if

        do mX = 1, NMARKET
            do pX2 = 1, NTYPE2
                 call writemom( funit, vmode, ind, mom1%Rjtype_0F(pX2, mX), mom2%Rjtype_0F(pX2, mX), momV%Rjtype_0F(pX2, mX), pX2, mX )
            end do
        end do

        if (writefile) then
            close(funit)
        end if

        if ( .not. writefile ) then
            print *
            print *, 'Rjtype_MF(NTYPE1, NTYPE2, NMARKET)'
        else
            open( newunit = funit, file = fdir // '/Rjtype_MF.csv', status = "replace" )
        end if

        do mX = 1, NMARKET
            do pX2 = 1, NTYPE2
                do pX1 = 1, NTYPE1
                     call writemom( funit, vmode, ind, mom1%Rjtype_MF(pX1, pX2, mX), mom2%Rjtype_MF(pX1, pX2, mX), &
                                    momV%Rjtype_MF(pX1, pX2, mX), pX1, pX2, mX )
                end do
            end do
        end do

        if (writefile) then
            close(funit)
        end if

        ! labour supply by market

        if ( .not. writefile ) then
            print *
            print *, 'Rmhours_M0(NTYPE1, NMARKET)'
        else
            open( newunit = funit, file = fdir // '/Rmhours_M0.csv', status = "replace" )
        end if

        do mX = 1, NMARKET
            do pX1 = 1, NTYPE1
                 call writemom( funit, vmode, ind, mom1%Rmhours_M0(pX1, mX), mom2%Rmhours_M0(pX1, mX), momV%Rmhours_M0(pX1, mX), pX1, mX )
            end do
        end do

        if (writefile) then
            close(funit)
        end if

        if ( .not. writefile ) then
            print *
            print *, 'Rfhours_0F(NTYPE2, NMARKET)'
        else
            open( newunit = funit, file = fdir // '/Rfhours_0F.csv', status = "replace" )
        end if

        do mX = 1, NMARKET
            do pX2 = 1, NTYPE2
                 call writemom( funit, vmode, ind, mom1%Rfhours_0F(pX2, mX), mom2%Rfhours_0F(pX2, mX), momV%Rfhours_0F(pX2, mX), pX2, mX )
            end do
        end do

        if (writefile) then
            close(funit)
        end if

        if ( .not. writefile ) then
            print *
            print *, 'Rmhours_Mf(NTYPE1, NMARKET)'
        else
            open( newunit = funit, file = fdir // '/Rmhours_Mf.csv', status = "replace" )
        end if

        do mX = 1, NMARKET
            do pX1 = 1, NTYPE1
                 call writemom( funit, vmode, ind, mom1%Rmhours_Mf(pX1, mX), mom2%Rmhours_Mf(pX1, mX), momV%Rmhours_Mf(pX1, mX), pX1, mX )
            end do
        end do

        if (writefile) then
            close(funit)
        end if

        if ( .not. writefile ) then
            print *
            print *, 'Rfhours_mF(NTYPE2, NMARKET)'
        else
            open( newunit = funit, file = fdir // '/Rfhours_mF.csv', status = "replace" )
        end if

        do mX = 1, NMARKET
            do pX2 = 1, NTYPE2
                 call writemom( funit, vmode, ind, mom1%Rfhours_mF(pX2, mX), mom2%Rfhours_mF(pX2, mX), momV%Rfhours_mF(pX2, mX), pX2, mX )
            end do
        end do

        if (writefile) then
            close(funit)
        end if

        ! employment status
        ! -----------------

        if ( .not. writefile ) then
            print *
            print *, 'Rmwork_M0(NTYPE1, NMARKET)'
        else
            open( newunit = funit, file = fdir // '/Rmwork_M0.csv', status = "replace" )
        end if

        do mX = 1, NMARKET
            do pX1 = 1, NTYPE1
                 call writemom( funit, vmode, ind, mom1%Rmwork_M0(pX1, mX), mom2%Rmwork_M0(pX1, mX), momV%Rmwork_M0(pX1, mX), pX1, mX )
            end do
        end do

        if (writefile) then
            close(funit)
        end if

        if ( .not. writefile ) then
            print *
            print *, 'Rfwork_0F(NTYPE2, NMARKET)'
        else
            open( newunit = funit, file = fdir // '/Rfwork_0F.csv', status = "replace" )
        end if

        do mX = 1, NMARKET
            do pX2 = 1, NTYPE2
                 call writemom( funit, vmode, ind, mom1%Rfwork_0F(pX2, mX), mom2%Rfwork_0F(pX2, mX), momV%Rfwork_0F(pX2, mX), pX2, mX )
            end do
        end do

        if (writefile) then
            close(funit)
        end if

        if ( .not. writefile ) then
            print *
            print *, 'Rmwork_Mf(NTYPE1, NMARKET)'
        else
            open( newunit = funit, file = fdir // '/Rmwork_Mf.csv', status = "replace" )
        end if

        do mX = 1, NMARKET
            do pX1 = 1, NTYPE1
                 call writemom( funit, vmode, ind, mom1%Rmwork_Mf(pX1, mX), mom2%Rmwork_Mf(pX1, mX), momV%Rmwork_Mf(pX1, mX), pX1, mX )
            end do
        end do

        if (writefile) then
            close(funit)
        end if

        if ( .not. writefile ) then
            print *
            print *, 'Rfwork_mF(NTYPE2, NMARKET)'
        else
            open( newunit = funit, file = fdir // '/Rfwork_mF.csv', status = "replace" )
        end if

        do mX = 1, NMARKET
            do pX2 = 1, NTYPE2
                 call writemom( funit, vmode, ind, mom1%Rfwork_mF(pX2, mX), mom2%Rfwork_mF(pX2, mX), momV%Rfwork_mF(pX2, mX), pX2, mX )
            end do
        end do

        if (writefile) then
            close(funit)
        end if

        if ( .not. writefile ) then
            print *
            print *
            print *, 'joint labour supply allocation'
            print *
            print *, 'mhours_M0(0:2, NTYPE1, 0:1)'
        else
            open( newunit = funit, file = fdir // '/mhours_M0.csv', status = "replace" )
        end if

        do kX = 0, 1
            do pX1 = 1, NTYPE1
                do hX1 = 0, 2
                    call writemom( funit, vmode, ind, mom1%mhours_M0(hX1, pX1, kX), mom2%mhours_M0(hX1, pX1, kX), &
                                   momV%mhours_M0(hX1, pX1, kX), hX1, pX1, kX )
                end do
            end do
        end do

        if (writefile) then
            close(funit)
        end if

        if ( .not. writefile ) then
            print *
            print *, 'fhours_0F(0:2,NTYPE2, 0:1)'
        else
            open( newunit = funit, file = fdir // '/fhours_0F.csv', status = "replace" )
        end if

        do kX = 0, 1
            do pX2 = 1, NTYPE2
                do hX2 = 0, 2
                    call writemom( funit, vmode, ind, mom1%fhours_0F(hX2, pX2, kX), mom2%fhours_0F(hX2, pX2, kX), &
                                   momV%fhours_0F(hX2, pX2, kX), hX2, pX2, kX )
                end do
            end do
        end do

        if (writefile) then
            close(funit)
        end if

        if ( .not. writefile ) then
            print *
            print *
            print *, 'marginal labour supply allocation'
            print *
            print *, 'mhours_MF(0:2, NTYPE1, NTYPE2, 0:1)'
        else
            open( newunit = funit, file = fdir // '/mhours_MF.csv', status = "replace" )
        end if

        do kX = 0, 1
            do pX2 = 1, NTYPE2
                do pX1 = 1, NTYPE1
                    do hX1 = 0, 2
                        call writemom( funit, vmode, ind, mom1%mhours_MF(hX1, pX1, pX2, kX), mom2%mhours_MF(hX1, pX1, pX2, kX), &
                                       momV%mhours_MF(hX1, pX1, pX2, kX), hX1, pX1, pX2, kX )
                    end do
                end do
            end do
        end do

        if (writefile) then
            close(funit)
        end if

        if ( .not. writefile ) then
            print *
            print *, 'fhours_MF(0:2, NTYPE1, NTYPE2, 0:1)'
        else
            open( newunit = funit, file = fdir // '/fhours_MF.csv', status = "replace" )
        end if

        do kX = 0, 1
            do pX2 = 1, NTYPE2
                do pX1 = 1, NTYPE1
                    do hX2 = 0, 2
                        call writemom( funit, vmode, ind, mom1%fhours_MF(hX2, pX1, pX2, kX), mom2%fhours_MF(hX2, pX1, pX2, kX), &
                                       momV%fhours_MF(hX2, pX1, pX2, kX), hX2, pX1, pX2, kX )
                    end do
                end do
            end do
        end do

        if (writefile) then
            close(funit)
        end if

        if ( .not. writefile ) then
            print *
            print *
            print *, 'labour supply distribution'
            print *
            print *, 'mhoursh_M0(NTYPE1, 0:1)'
        else
            open( newunit = funit, file = fdir // '/mhoursh_M0.csv', status = "replace" )
        end if

        do kX = 0, 1
            do pX1 = 1, NTYPE1
                call writemom( funit, vmode, ind, mom1%mhoursh_M0(pX1, kX), mom2%mhoursh_M0(pX1, kX), momV%mhoursh_M0(pX1, kX), pX1, kX )
            end do
        end do

        if (writefile) then
            close(funit)
        end if

        if ( .not. writefile ) then
            print *
            print *, 'mhourshsd_M0(NTYPE1, 0:1)'
        else
            open( newunit = funit, file = fdir // '/mhourshsd_M0.csv', status = "replace" )
        end if

        do kX = 0, 1
            do pX1 = 1, NTYPE1
                call writemom( funit, vmode, ind, mom1%mhourshsd_M0(pX1, kX), mom2%mhourshsd_M0(pX1, kX), &
                               momV%mhourshsd_M0(pX1, kX), pX1, kX )
            end do
        end do

        if (writefile) then
            close(funit)
        end if

        if ( .not. writefile ) then
            print *
            print *, 'fhoursh_0F(NTYPE2, 0:1)'
        else
            open( newunit = funit, file = fdir // '/fhoursh_0F.csv', status = "replace" )
        end if

        do kX = 0, 1
            do pX2 = 1, NTYPE2
                call writemom( funit, vmode, ind, mom1%fhoursh_0F(pX2, kX), mom2%fhoursh_0F(pX2, kX), momV%fhoursh_0F(pX2, kX), pX2, kX )
            end do
        end do

        if (writefile) then
            close(funit)
        end if

        if ( .not. writefile ) then
            print *
            print *, 'fhourshsd_0F(NTYPE2, 0:1)'
        else
            open( newunit = funit, file = fdir // '/fhourshsd_0F.csv', status = "replace" )
        end if

        do kX = 0, 1
            do pX2 = 1, NTYPE2
                call writemom( funit, vmode, ind, mom1%fhourshsd_0F(pX2, kX), mom2%fhourshsd_0F(pX2, kX), &
                               momV%fhourshsd_0F(pX2, kX), pX2, kX )
            end do
        end do

        if (writefile) then
            close(funit)
        end if

        if ( .not. writefile ) then
            print *
            print *, 'mhoursh_Mf(NTYPE1, 0:1)'
        else
            open( newunit = funit, file = fdir // '/mhoursh_Mf.csv', status = "replace" )
        end if

        do kX = 0, 1
            do pX1 = 1, NTYPE1
                call writemom( funit, vmode, ind, mom1%mhoursh_Mf(pX1, kX), mom2%mhoursh_Mf(pX1, kX), momV%mhoursh_Mf(pX1, kX), pX1, kX )
            end do
        end do

        if (writefile) then
            close(funit)
        end if

        if ( .not. writefile ) then
            print *
            print *, 'mhourshsd_Mf(NTYPE1, 0:1)'
        else
            open( newunit = funit, file = fdir // '/mhourshsd_Mf.csv', status = "replace" )
        end if

        do kX = 0, 1
            do pX1 = 1, NTYPE1
                call writemom( funit, vmode, ind, mom1%mhourshsd_Mf(pX1, kX), mom2%mhourshsd_Mf(pX1, kX), &
                               momV%mhourshsd_Mf(pX1, kX), pX1, kX )
            end do
        end do

        if (writefile) then
            close(funit)
        end if

        if ( .not. writefile ) then
            print *
            print *, 'fhoursh_mF(NTYPE2, 0:1)'
        else
            open( newunit = funit, file = fdir // '/fhoursh_mF.csv', status = "replace" )
        end if

        do kX = 0, 1
            do pX2 = 1, NTYPE2
                call writemom( funit, vmode, ind, mom1%fhoursh_mF(pX2, kX), mom2%fhoursh_mF(pX2, kX), momV%fhoursh_mF(pX2, kX), pX2, kX )
            end do
        end do

        if (writefile) then
            close(funit)
        end if

        if ( .not. writefile ) then
            print *
            print *, 'fhourshsd_mF(NTYPE2, 0:1)'
        else
            open( newunit = funit, file = fdir // '/fhourshsd_mF.csv', status = "replace" )
        end if

        do kX = 0, 1
            do pX2 = 1, NTYPE2
                call writemom( funit, vmode, ind, mom1%fhourshsd_mF(pX2, kX), mom2%fhourshsd_mF(pX2, kX), &
                               momV%fhourshsd_mF(pX2, kX), pX2, kX )
            end do
        end do

        if (writefile) then
            close(funit)
        end if

        if ( .not. writefile ) then
            print *
            print *, 'mhoursh_M_F(NTYPE1, NTYPE2)'
        else
            open( newunit = funit, file = fdir // '/mhoursh_M_F.csv', status = "replace" )
        end if

        do pX2 = 1, NTYPE2
            do pX1 = 1, NTYPE1
                call writemom( funit, vmode, ind, mom1%mhoursh_M_F(pX1, pX2), mom2%mhoursh_M_F(pX1, pX2), &
                               momV%mhoursh_M_F(pX1, pX2), pX1, pX2 )
            end do
        end do

        if (writefile) then
            close(funit)
        end if

        if ( .not. writefile ) then
            print *
            print *, 'fhoursh_M_F(NTYPE1, NTYPE2)'
        else
            open( newunit = funit, file = fdir // '/fhoursh_M_F.csv', status = "replace" )
        end if

        do pX2 = 1, NTYPE2
            do pX1 = 1, NTYPE1
                call writemom( funit, vmode, ind, mom1%fhoursh_M_F(pX1, pX2), mom2%fhoursh_M_F(pX1, pX2), &
                               momV%fhoursh_M_F(pX1, pX2), pX1, pX2 )
            end do
        end do

        if (writefile) then
            close(funit)
        end if

        if ( .not. writefile ) then
            print *
            print *, 'mwork_M_F(NTYPE1, NTYPE2)'
        else
            open( newunit = funit, file = fdir // '/mwork_M_F.csv', status = "replace" )
        end if

        do pX2 = 1, NTYPE2
            do pX1 = 1, NTYPE1
                call writemom( funit, vmode, ind, mom1%mwork_M_F(pX1, pX2), mom2%mwork_M_F(pX1, pX2), momV%mwork_M_F(pX1, pX2), pX1, pX2 )
            end do
        end do

        if (writefile) then
            close(funit)
        end if

        if ( .not. writefile ) then
            print *
            print *, 'fwork_M_F(NTYPE1, NTYPE2)'
        else
            open( newunit = funit, file = fdir // '/fwork_M_F.csv', status = "replace" )
        end if

        do pX2 = 1, NTYPE2
            do pX1 = 1, NTYPE1
                call writemom( funit, vmode, ind, mom1%fwork_M_F(pX1, pX2), mom2%fwork_M_F(pX1, pX2), momV%fwork_M_F(pX1, pX2), pX1, pX2 )
            end do
        end do

        if (writefile) then
            close(funit)
        end if

        if ( .not. writefile ) then
            print *
            print *
            print *, 'wage distribution all workers (single men, integrates over kids)'
            print *
            print *, 'mwageMwAll_M0(NTYPE1)'
        else
            open( newunit = funit, file = fdir // '/mwageMwAll_M0.csv', status = "replace" )
        end if

        do pX1 = 1, NTYPE1
            call writemom( funit, vmode, ind, mom1%mwageMwAll_M0(pX1), mom2%mwageMwAll_M0(pX1), momV%mwageMwAll_M0(pX1), pX1 )
        end do

        if (writefile) then
            close(funit)
        end if

        if ( .not. writefile ) then
            print *
            print *, 'mwagesdMwAll_M0(NTYPE1)'
        else
            open( newunit = funit, file = fdir // '/mwagesdMwAll_M0.csv', status = "replace" )
        end if

        do pX1 = 1, NTYPE1
            call writemom( funit, vmode, ind, mom1%mwagesdMwAll_M0(pX1), mom2%mwagesdMwAll_M0(pX1), momV%mwagesdMwAll_M0(pX1), pX1 )
        end do

        if (writefile) then
            close(funit)
        end if

        if ( .not. writefile ) then
            print *
            print *
            print *, 'wage distribution all workers (single women, integrates over kids)'
            print *
            print *, 'fwageFwAll_0F(NTYPE2)'
        else
            open( newunit = funit, file = fdir // '/fwageFwAll_0F.csv', status = "replace" )
        end if

        do pX2 = 1, NTYPE2
            call writemom( funit, vmode, ind, mom1%fwageFwAll_0F(pX2), mom2%fwageFwAll_0F(pX2), momV%fwageFwAll_0F(pX2), pX2 )
        end do

        if (writefile) then
            close(funit)
        end if

        if ( .not. writefile ) then
            print *
            print *, 'fwagesdFwAll_0F(NTYPE2)'
        else
            open( newunit = funit, file = fdir // '/fwagesdFwAll_0F.csv', status = "replace" )
        end if

        do pX2 = 1, NTYPE2
            call writemom( funit, vmode, ind, mom1%fwagesdFwAll_0F(pX2), mom2%fwagesdFwAll_0F(pX2), momV%fwagesdFwAll_0F(pX2), pX2 )
        end do

        if (writefile) then
            close(funit)
        end if

        if ( .not. writefile ) then
            print *
            print *
            print *, 'wage distribution all workers (married men, integrates over spouse type/state and kids)'
            print *
            print *, 'mwageMwAll_Mf(NTYPE1)'
        else
            open( newunit = funit, file = fdir // '/mwageMwAll_Mf.csv', status = "replace" )
        end if

        do pX1 = 1, NTYPE1
            call writemom( funit, vmode, ind, mom1%mwageMwAll_Mf(pX1), mom2%mwageMwAll_Mf(pX1), momV%mwageMwAll_Mf(pX1), pX1 )
        end do

        if (writefile) then
            close(funit)
        end if

        if ( .not. writefile ) then
            print *
            print *, 'mwagesdMwAll_Mf(NTYPE1)'
        else
            open( newunit = funit, file = fdir // '/mwagesdMwAll_Mf.csv', status = "replace" )
        end if

        do pX1 = 1, NTYPE1
            call writemom( funit, vmode, ind, mom1%mwagesdMwAll_Mf(pX1), mom2%mwagesdMwAll_Mf(pX1), momV%mwagesdMwAll_Mf(pX1), pX1 )
        end do

        if (writefile) then
            close(funit)
        end if

        if ( .not. writefile ) then
            print *
            print *
            print *, 'wage distribution all workers (married women, integrates over spouse type/state and kids)'
            print *
            print *, 'fwageFwAll_mF(NTYPE2)'
        else
            open( newunit = funit, file = fdir // '/fwageFwAll_mF.csv', status = "replace" )
        end if

        do pX2 = 1, NTYPE2
            call writemom( funit, vmode, ind, mom1%fwageFwAll_mF(pX2), mom2%fwageFwAll_mF(pX2), momV%fwageFwAll_mF(pX2), pX2 )
        end do

        if (writefile) then
            close(funit)
        end if

        if ( .not. writefile ) then
            print *
            print *, 'fwagesdFwAll_mF(NTYPE2)'
        else
            open( newunit = funit, file = fdir // '/fwagesdFwAll_mF.csv', status = "replace" )
        end if

        do pX2 = 1, NTYPE2
            call writemom( funit, vmode, ind, mom1%fwagesdFwAll_mF(pX2), mom2%fwagesdFwAll_mF(pX2), momV%fwagesdFwAll_mF(pX2), pX2 )
        end do

        if (writefile) then
            close(funit)
        end if













        if ( .not. writefile ) then
            print *
            print *
            print *, 'earnings distribution all workers (single men, integrates over kids)'
            print *
            print *, 'mearnMwAll_M0(NTYPE1)'
        else
            open( newunit = funit, file = fdir // '/mearnMwAll_M0.csv', status = "replace" )
        end if

        do pX1 = 1, NTYPE1
            call writemom( funit, vmode, ind, mom1%mearnMwAll_M0(pX1), mom2%mearnMwAll_M0(pX1), momV%mearnMwAll_M0(pX1), pX1 )
        end do

        if (writefile) then
            close(funit)
        end if

        if ( .not. writefile ) then
            print *
            print *, 'mearnsdMwAll_M0(NTYPE1)'
        else
            open( newunit = funit, file = fdir // '/mearnsdMwAll_M0.csv', status = "replace" )
        end if

        do pX1 = 1, NTYPE1
            call writemom( funit, vmode, ind, mom1%mearnsdMwAll_M0(pX1), mom2%mearnsdMwAll_M0(pX1), momV%mearnsdMwAll_M0(pX1), pX1 )
        end do

        if (writefile) then
            close(funit)
        end if



        if ( .not. writefile ) then
            print *
            print *
            print *, 'earnings distribution all workers (single women, integrates over kids)'
            print *
            print *, 'fearnFwAll_0F(NTYPE2)'
        else
            open( newunit = funit, file = fdir // '/fearnFwAll_0F.csv', status = "replace" )
        end if

        do pX2 = 1, NTYPE2
            call writemom( funit, vmode, ind, mom1%fearnFwAll_0F(pX2), mom2%fearnFwAll_0F(pX2), momV%fearnFwAll_0F(pX2), pX2 )
        end do

        if (writefile) then
            close(funit)
        end if

        if ( .not. writefile ) then
            print *
            print *, 'fearnsdFwAll_0F(NTYPE2)'
        else
            open( newunit = funit, file = fdir // '/fearnsdFwAll_0F.csv', status = "replace" )
        end if

        do pX2 = 1, NTYPE2
            call writemom( funit, vmode, ind, mom1%fearnsdFwAll_0F(pX2), mom2%fearnsdFwAll_0F(pX2), momV%fearnsdFwAll_0F(pX2), pX2 )
        end do

        if (writefile) then
            close(funit)
        end if



        if ( .not. writefile ) then
            print *
            print *
            print *, 'earnings distribution all workers (married men, integrates over spouse type/state and kids)'
            print *
            print *, 'mearnMwAll_Mf(NTYPE1)'
        else
            open( newunit = funit, file = fdir // '/mearnMwAll_Mf.csv', status = "replace" )
        end if

        do pX1 = 1, NTYPE1
            call writemom( funit, vmode, ind, mom1%mearnMwAll_Mf(pX1), mom2%mearnMwAll_Mf(pX1), momV%mearnMwAll_Mf(pX1), pX1 )
        end do

        if (writefile) then
            close(funit)
        end if

        if ( .not. writefile ) then
            print *
            print *, 'mearnsdMwAll_Mf(NTYPE1)'
        else
            open( newunit = funit, file = fdir // '/mearnsdMwAll_Mf.csv', status = "replace" )
        end if

        do pX1 = 1, NTYPE1
            call writemom( funit, vmode, ind, mom1%mearnsdMwAll_Mf(pX1), mom2%mearnsdMwAll_Mf(pX1), momV%mearnsdMwAll_Mf(pX1), pX1 )
        end do

        if (writefile) then
            close(funit)
        end if


        if ( .not. writefile ) then
            print *
            print *
            print *, 'earnings distribution all workers (married women, integrates over spouse type/state and kids)'
            print *
            print *, 'fearnFwAll_mF(NTYPE2)'
        else
            open( newunit = funit, file = fdir // '/fearnFwAll_mF.csv', status = "replace" )
        end if

        do pX2 = 1, NTYPE2
            call writemom( funit, vmode, ind, mom1%fearnFwAll_mF(pX2), mom2%fearnFwAll_mF(pX2), momV%fearnFwAll_mF(pX2), pX2 )
        end do

        if (writefile) then
            close(funit)
        end if

        if ( .not. writefile ) then
            print *
            print *, 'fearnsdFwAll_mF(NTYPE2)'
        else
            open( newunit = funit, file = fdir // '/fearnsdFwAll_mF.csv', status = "replace" )
        end if

        do pX2 = 1, NTYPE2
            call writemom( funit, vmode, ind, mom1%fearnsdFwAll_mF(pX2), mom2%fearnsdFwAll_mF(pX2), momV%fearnsdFwAll_mF(pX2), pX2 )
        end do

        if (writefile) then
            close(funit)
        end if

        ! **** ATUS ****

        if ( .not. writefile ) then
            print *
            print *
            print *, 'marginal distribution of home production time (single men/women)'
            print *
            print *, 'mhome_M0(NHH, NTYPE1, 0:1)'
        else
            open( newunit = funit, file = fdir // '/mhome_M0.csv', status = "replace" )
        end if

        do kX = 0, 1
            do pX1 = 1, NTYPE1
                do hX1 = 1, NHH
                    call writemom( funit, vmode, ind, mom1%mhome_M0(hX1, pX1, kX), mom2%mhome_M0(hX1, pX1, kX), &
                                   momV%mhome_M0(hX1, pX1, kX), hX1, pX1, kX )
                end do
            end do
        end do

        if (writefile) then
            close(funit)
        end if

        if ( .not. writefile ) then
            print *
            print *, 'fhome_0F(NHH, NTYPE2, 0:1)'
        else
            open( newunit = funit, file = fdir // '/fhome_0F.csv', status = "replace" )
        end if

        do kX = 0, 1
            do pX2 = 1, NTYPE2
                do hX2 = 1, NHH
                    call writemom( funit, vmode, ind, mom1%fhome_0F(hX2, pX2, kX), mom2%fhome_0F(hX2, pX2, kX), &
                                   momV%fhome_0F(hX2, pX2, kX), hX2, pX2, kX )
                end do
            end do
        end do

        if (writefile) then
            close(funit)
        end if

        if ( .not. writefile ) then
            print *
            print *
            print *, 'marginal distribution of home production time (married men/women)'
            print *
            print *, 'mhome_MF(NHH, NTYPE1, 0:1)'
        else
            open( newunit = funit, file = fdir // '/mhome_MF.csv', status = "replace" )
        end if

        do kX = 0, 1
            do pX1 = 1, NTYPE1
                do hX1 = 1, NHH
                    call writemom( funit, vmode, ind, mom1%mhome_MF(hX1, pX1, kX), mom2%mhome_MF(hX1, pX1, kX), &
                                   momV%mhome_MF(hX1, pX1, kX), hX1, pX1, kX )
                end do
            end do
        end do

        if (writefile) then
            close(funit)
        end if

        if ( .not. writefile ) then
            print *
            print *, 'fhome_MF(NHH, NTYPE2, 0:1)'
        else
            open( newunit = funit, file = fdir // '/fhome_MF.csv', status = "replace" )
        end if

        do kX = 0, 1
            do pX2 = 1, NTYPE2
                do hX2 = 1, NHH
                    call writemom( funit, vmode, ind, mom1%fhome_MF(hX2, pX2, kX), mom2%fhome_MF(hX2, pX2, kX), &
                                   momV%fhome_MF(hX2, pX2, kX), hX2, pX2, kX )
                end do
            end do
        end do

        if (writefile) then
            close(funit)
        end if

        if ( .not. writefile ) then
            print *
            print *
            print *, 'home production hours (single men)'
            print *
            print *, 'mhomeh_M0(NTYPE1, 0:1)'
        else
            open( newunit = funit, file = fdir // '/mhomeh_M0.csv', status = "replace" )
        end if

        do kX = 0, 1
            do pX1 = 1, NTYPE1
                call writemom( funit, vmode, ind, mom1%mhomeh_M0(pX1, kX), mom2%mhomeh_M0(pX1, kX), momV%mhomeh_M0(pX1, kX), pX1, kX )
            end do
        end do

        if (writefile) then
            close(funit)
        end if

        if ( .not. writefile ) then
            print *
            print *, 'mhomehsd_M0(NTYPE1, 0:1)'
        else
            open( newunit = funit, file = fdir // '/mhomehsd_M0.csv', status = "replace" )
        end if

        do kX = 0, 1
            do pX1 = 1, NTYPE1
                call writemom( funit, vmode, ind, mom1%mhomehsd_M0(pX1, kX), mom2%mhomehsd_M0(pX1, kX), &
                               momV%mhomehsd_M0(pX1, kX), pX1, kX )
            end do
        end do

        if (writefile) then
            close(funit)
        end if

        if ( .not. writefile ) then
            print *
            print *
            print *, 'home production hours (single women)'
            print *
            print *, 'fhomeh_0F(NTYPE2, 0:1)'
        else
            open( newunit = funit, file = fdir // '/fhomeh_0F.csv', status = "replace" )
        end if

        do kX = 0, 1
            do pX2 = 1, NTYPE2
                call writemom( funit, vmode, ind, mom1%fhomeh_0F(pX2, kX), mom2%fhomeh_0F(pX2, kX), momV%fhomeh_0F(pX2, kX), pX2, kX )
            end do
        end do

        if (writefile) then
            close(funit)
        end if

        if ( .not. writefile ) then
            print *
            print *, 'fhomehsd_0F(NTYPE2, 0:1)'
        else
            open( newunit = funit, file = fdir // '/fhomehsd_0F.csv', status = "replace" )
        end if

        do kX = 0, 1
            do pX2 = 1, NTYPE2
                call writemom( funit, vmode, ind, mom1%fhomehsd_0F(pX2, kX), mom2%fhomehsd_0F(pX2, kX), &
                               momV%fhomehsd_0F(pX2, kX), pX2, kX )
            end do
        end do

        if (writefile) then
            close(funit)
        end if

        if ( .not. writefile ) then
            print *
            print *
            print *, 'home production hours (married men)'
            print *
            print *, 'mhomeh_Mf(NTYPE1, 0:1)'
        else
            open( newunit = funit, file = fdir // '/mhomeh_Mf.csv', status = "replace" )
        end if

        do kX = 0, 1
            do pX1 = 1, NTYPE1
                call writemom( funit, vmode, ind, mom1%mhomeh_Mf(pX1, kX), mom2%mhomeh_Mf(pX1, kX), momV%mhomeh_Mf(pX1, kX), pX1, kX )
            end do
        end do

        if (writefile) then
            close(funit)
        end if

        if ( .not. writefile ) then
            print *
            print *, 'mhomehsd_Mf(NTYPE1, 0:1)'
        else
            open( newunit = funit, file = fdir // '/mhomehsd_Mf.csv', status = "replace" )
        end if

        do kX = 0, 1
            do pX1 = 1, NTYPE1
                call writemom( funit, vmode, ind, mom1%mhomehsd_Mf(pX1, kX), mom2%mhomehsd_Mf(pX1, kX), &
                               momV%mhomehsd_Mf(pX1, kX), pX1, kX )
            end do
        end do

        if (writefile) then
            close(funit)
        end if

        if ( .not. writefile ) then
            print *
            print *
            print *, 'home production hours (married women)'
            print *
            print *, 'fhomeh_mF(NTYPE2, 0:1)'
        else
            open( newunit = funit, file = fdir // '/fhomeh_mF.csv', status = "replace" )
        end if

        do kX = 0, 1
            do pX2 = 1, NTYPE2
                call writemom( funit, vmode, ind, mom1%fhomeh_mF(pX2, kX), mom2%fhomeh_mF(pX2, kX), momV%fhomeh_mF(pX2, kX), pX2, kX )
            end do
        end do

        if (writefile) then
            close(funit)
        end if

        if ( .not. writefile ) then
            print *
            print *, 'fhomehsd_mF(NTYPE2, 0:1)'
        else
            open( newunit = funit, file = fdir // '/fhomehsd_mF.csv', status = "replace" )
        end if

        do kX = 0, 1
            do pX2 = 1, NTYPE2
                call writemom( funit, vmode, ind, mom1%fhomehsd_mF(pX2, kX), mom2%fhomehsd_mF(pX2, kX), &
                               momV%fhomehsd_mF(pX2, kX), pX2, kX )
            end do
        end do

        if (writefile) then
            close(funit)
        end if

        ! home hours by match
        if ( .not. writefile ) then
            print *
            print *, 'mhomeh_M_F(NTYPE1, NTYPE2)'
        else
            open( newunit = funit, file = fdir // '/mhomeh_M_F.csv', status = "replace" )
        end if

        do pX2 = 1, NTYPE2
            do pX1 = 1, NTYPE1
                call writemom( funit, vmode, ind, mom1%mhomeh_M_F(pX1, pX2), mom2%mhomeh_M_F(pX1, pX2), &
                               momV%mhomeh_M_F(pX1, pX2), pX1, pX2 )
            end do
        end do

        if (writefile) then
            close(funit)
        end if

        if ( .not. writefile ) then
            print *
            print *, 'fhomeh_M_F(NTYPE1_NTYPE2)'
        else
            open( newunit = funit, file = fdir // '/fhomeh_M_F.csv', status = "replace" )
        end if

        do pX2 = 1, NTYPE2
            do pX1 = 1, NTYPE1
                call writemom( funit, vmode, ind, mom1%fhomeh_M_F(pX1, pX2), mom2%fhomeh_M_F(pX1, pX2), &
                               momV%fhomeh_M_F(pX1, pX2), pX1, pX2 )
            end do
        end do

        if (writefile) then
            close(funit)
        end if

        ! home hours by kids, integrating spouse

        if ( .not. writefile ) then
            print *
            print *
        end if

    end subroutine printMomentsCompare



    subroutine momentIntegrateGroup( n, frac, group_mean, group_sd, total_mean, total_sd )
        implicit none
        integer,  intent(in)  :: n
        real(dp), intent(in)  :: frac(n)
        real(dp), intent(in)  :: group_mean(n)
        real(dp), intent(in)  :: group_sd(n)
        real(dp), intent(out) :: total_mean
        real(dp), intent(out) :: total_sd

        real(dp) :: group_ESq(n), group_var(n), group_meanSq(n)
        real(dp) :: total_ESq, total_var
        real(dp) :: sumfrac

        sumfrac = sum(frac)

        if (sumfrac>0.0_dp) then

            ! first, recover Ex^2 for each group
            group_var = group_sd ** 2
            group_meanSq = group_mean ** 2
            group_ESq = group_var + group_meanSq
            total_mean = sum( group_mean*frac ) / sumfrac
            total_ESq = sum( group_ESq*frac ) / sumfrac
            total_var = max( total_ESq - total_mean ** 2, 0.0_dp )
            total_sd = sqrt(total_var)

        else

            total_mean = 0.0_dp
            total_sd = 0.0_dp

        end if

    end subroutine momentIntegrateGroup


    subroutine aggregateMomentsPoints(m, qM, qF, eallMQ, eallFQ, eallQ, mom, momExtra)
        implicit none
        type(m_t),     intent(in)  :: m
        real(dp),      intent(in)  :: qM(NPOINT1)
        real(dp),      intent(in)  :: qF(NPOINT2)
        type(eall_t),  intent(in)  :: eallMQ(NTYPE1, NPOINT1, 0:1)
        type(eall_t),  intent(in)  :: eallFQ(NTYPE2, NPOINT2, 0:1)
        type(eall2_t), intent(in)  :: eallQ(NTYPE1, NTYPE2, NPOINT1, NPOINT2, 0:1)
        type(mom_t),   intent(out) :: mom
        type(momExtra_t), intent(out), optional :: momExtra

        integer :: kX
        type(eall_t)  :: eallM(NTYPE1, 0:1)
        type(eall_t)  :: eallF(NTYPE2, 0:1)
        type(eall2_t) :: eall(NTYPE1, NTYPE2, 0:1)

        if (NPOINT1 > 1 .or. NPOINT2 > 1) then
            call aggregateMomentsHeterogeneitySingle(qM, qF, eallMQ, eallFQ, eallF, eallM)
            call aggregateMomentsHeterogeneity(qM, qF, eallQ, eall)
        else
            do kX = 0, 1
                eallM(:, kX) = eallMQ(:, 1, kX)
                eallF(:, kX) = eallFQ(:, 1, kX)
                eall(:, :, kX) = eallQ(:, :, 1, 1, kX)
            end do
        end if

        if (present(momExtra)) then
            call aggregateMoments2(m, eallM, eallF, eall, mom, momExtra)
        else
            call aggregateMoments2(m, eallM, eallF, eall, mom)
        end if

    end subroutine aggregateMomentsPoints


    subroutine aggregateMomentsSinglesPoints(m, qM, qF, eallMQ, eallFQ, mom)
        implicit none
        type(m_t),     intent(in)  :: m
        real(dp),      intent(in)  :: qM(NPOINT1)
        real(dp),      intent(in)  :: qF(NPOINT2)
        type(eall_t),  intent(in)  :: eallMQ(NTYPE1, NPOINT1, 0:1)
        type(eall_t),  intent(in)  :: eallFQ(NTYPE2, NPOINT2, 0:1)
        type(mom_t),   intent(out) :: mom

        type(eall_t)  :: eallM(NTYPE1, 0:1)
        type(eall_t)  :: eallF(NTYPE2, 0:1)

        call aggregateMomentsHeterogeneitySingle(qM, qF, eallMQ, eallFQ, eallF, eallM)
        call aggregateMomentsSingles(m, eallM, eallF, mom)

    end subroutine aggregateMomentsSinglesPoints

    subroutine aggregateMomentsHeterogeneitySingle(qM, qF, eallMQ, eallFQ, eallF, eallM)
        implicit none
        real(dp),     intent(in) :: qM(NPOINT1)
        real(dp),     intent(in) :: qF(NPOINT2)
        type(eall_t), intent(in) :: eallMQ(NTYPE1, NPOINT1, 0:1)
        type(eall_t), intent(in) :: eallFQ(NTYPE2, NPOINT2, 0:1)
        type(eall_t), intent(out) :: eallM(NTYPE1, 0:1)
        type(eall_t), intent(out) :: eallF(NTYPE2, 0:1)

        integer  :: kX, pX1, pX2, qX1, qX2, hX1
        real(dp) :: group_mean1(NPOINT1), group_sd1(NPOINT1)
        real(dp) :: group_mean2(NPOINT2), group_sd2(NPOINT2)


        do kX = 0, 1
            do pX1 = 1, NTYPE1
                eallM(pX1, kX)%hours = 0.0_dp
                eallM(pX1, kX)%work  = 0.0_dp
                eallM(pX1, kX)%home  = 0.0_dp
                do qX1 = 1, NPOINT1
                    eallM(pX1, kX)%hours = eallM(pX1, kX)%hours + qM(qX1) * eallMQ(pX1, qX1, kX)%hours
                    eallM(pX1, kX)%work  = eallM(pX1, kX)%work  + qM(qX1) * eallMQ(pX1, qX1, kX)%work
                    eallM(pX1, kX)%home  = eallM(pX1, kX)%home  + qM(qX1) * eallMQ(pX1, qX1, kX)%home
                end do
            end do
        end do

        do kX = 0, 1
            do pX1 = 1, NTYPE1

                do qX1 = 1, NPOINT1
                    group_mean1(qX1) = eallMQ(pX1, qX1, kX)%hourshW
                    group_sd1(qX1) = eallMQ(pX1, qX1, kX)%hourshWsd
                end do

                call momentIntegrateGroup( NPOINT1, qM, group_mean1, group_sd1, &
                                           eallM(pX1, kX)%hourshW, eallM(pX1, kX)%hourshWsd )

                do qX1 = 1, NPOINT1
                    group_mean1(qX1) = eallMQ(pX1, qX1, kX)%hoursh
                    group_sd1(qX1) = eallMQ(pX1, qX1, kX)%hourshsd
                end do

                call momentIntegrateGroup( NPOINT1, qM, group_mean1, group_sd1, &
                                           eallM(pX1, kX)%hoursh, eallM(pX1, kX)%hourshsd )

                do qX1 = 1, NPOINT1
                    group_mean1(qX1) = eallMQ(pX1, qX1, kX)%homeh
                    group_sd1(qX1) = eallMQ(pX1, qX1, kX)%homehsd
                end do

                call momentIntegrateGroup( NPOINT1, qM, group_mean1, group_sd1, &
                                           eallM(pX1, kX)%homeh, eallM(pX1, kX)%homehsd )

                do hX1 = 0, 2

                    do qX1 = 1, NPOINT1
                        group_mean1(qX1) = eallMQ(pX1, qX1, kX)%wage(hX1)
                        group_sd1(qX1) = eallMQ(pX1, qX1, kX)%wagesd(hX1)
                    end do

                    call momentIntegrateGroup( NPOINT1, qM, group_mean1, group_sd1, &
                                               eallM(pX1, kX)%wage(hX1), eallM(pX1, kX)%wagesd(hX1) )

                end do

                do qX1 = 1, NPOINT1
                    group_mean1(qX1) = eallMQ(pX1, qX1, kX)%wageWork
                    group_sd1(qX1) = eallMQ(pX1, qX1, kX)%wagesdWork
                end do

                call momentIntegrateGroup( NPOINT1, qM, group_mean1, group_sd1, &
                                           eallM(pX1, kX)%wageWork, eallM(pX1, kX)%wagesdWork )

                do hX1 = 0, 1

                    do qX1 = 1, NPOINT1
                        group_mean1(qX1) = eallMQ(pX1, qX1, kX)%homehW(hX1)
                        group_sd1(qX1) = eallMQ(pX1, qX1, kX)%homehWsd(hX1)
                    end do

                    call momentIntegrateGroup( NPOINT1, qM, group_mean1, group_sd1, &
                                               eallM(pX1, kX)%homehW(hX1), eallM(pX1, kX)%homehWsd(hX1) )

                end do

                do hX1 = 0, 2

                    do qX1 = 1, NPOINT1
                        group_mean1(qX1) = eallMQ(pX1, qX1, kX)%homehH(hX1)
                        group_sd1(qX1) = eallMQ(pX1, qX1, kX)%homehHsd(hX1)
                    end do

                    call momentIntegrateGroup( NPOINT1, qM, group_mean1, group_sd1, &
                                               eallM(pX1, kX)%homehH(hX1), eallM(pX1, kX)%homehHsd(hX1) )

                end do

            end do
        end do



        do kX = 0, 1
            do pX2 = 1, NTYPE2

                eallF(pX2, kX)%hours = 0.0_dp
                eallF(pX2, kX)%work  = 0.0_dp
                eallF(pX2, kX)%home  = 0.0_dp

                do qX2 = 1, NPOINT2
                    eallF(pX2, kX)%hours = eallF(pX2, kX)%hours + qF(qX2) * eallFQ(pX2, qX2, kX)%hours
                    eallF(pX2, kX)%work  = eallF(pX2, kX)%work  + qF(qX2) * eallFQ(pX2, qX2, kX)%work
                    eallF(pX2, kX)%home  = eallF(pX2, kX)%home  + qF(qX2) * eallFQ(pX2, qX2, kX)%home
                end do
            end do
        end do

        do kX = 0, 1
            do pX2 = 1, NTYPE2

                do qX2 = 1, NPOINT2
                    group_mean2(qX2) = eallFQ(pX2, qX2, kX)%hourshW
                    group_sd2(qX2) = eallFQ(pX2, qX2, kX)%hourshWsd
                end do

                call momentIntegrateGroup( NPOINT2, qF, group_mean2, group_sd2, &
                                           eallF(pX2, kX)%hourshW, eallF(pX2, kX)%hourshWsd )

                do qX2 = 1, NPOINT2
                    group_mean2(qX2) = eallFQ(pX2, qX2, kX)%hoursh
                    group_sd2(qX2) = eallFQ(pX2, qX2, kX)%hourshsd
                end do

                call momentIntegrateGroup( NPOINT2, qF, group_mean2, group_sd2, &
                                           eallF(pX2, kX)%hoursh, eallF(pX2, kX)%hourshsd )

                do qX2 = 1, NPOINT2
                    group_mean2(qX2) = eallFQ(pX2, qX2, kX)%homeh
                    group_sd2(qX2) = eallFQ(pX2, qX2, kX)%homehsd
                end do

                call momentIntegrateGroup( NPOINT2, qF, group_mean2, group_sd2, &
                                           eallF(pX2, kX)%homeh, eallF(pX2, kX)%homehsd )

                do hX1 = 0, 2

                    do qX2 = 1, NPOINT2
                        group_mean2(qX2) = eallFQ(pX2, qX2, kX)%wage(hX1)
                        group_sd2(qX2) = eallFQ(pX2, qX2, kX)%wagesd(hX1)
                    end do

                    call momentIntegrateGroup( NPOINT2, qF, group_mean2, group_sd2, &
                                               eallF(pX2, kX)%wage(hX1), eallF(pX2, kX)%wagesd(hX1) )

                end do

                do qX2 = 1, NPOINT2
                    group_mean2(qX2) = eallFQ(pX2, qX2, kX)%wageWork
                    group_sd2(qX2) = eallFQ(pX2, qX2, kX)%wagesdWork
                end do

                call momentIntegrateGroup( NPOINT2, qF, group_mean2, group_sd2, &
                                           eallF(pX2, kX)%wageWork, eallF(pX2, kX)%wagesdWork )

                do hX1 = 0, 1

                    do qX2 = 1, NPOINT2
                        group_mean2(qX2) = eallFQ(pX2, qX2, kX)%homehW(hX1)
                        group_sd2(qX2) = eallFQ(pX2, qX2, kX)%homehWsd(hX1)
                    end do

                    call momentIntegrateGroup( NPOINT2, qF, group_mean2, group_sd2, &
                                               eallF(pX2, kX)%homehW(hX1), eallF(pX2, kX)%homehWsd(hX1) )

                end do

                do hX1 = 0, 2

                    do qX2 = 1, NPOINT2
                        group_mean2(qX2) = eallFQ(pX2, qX2, kX)%homehH(hX1)
                        group_sd2(qX2) = eallFQ(pX2, qX2, kX)%homehHsd(hX1)
                    end do

                    call momentIntegrateGroup( NPOINT2, qF, group_mean2, group_sd2, &
                                               eallF(pX2, kX)%homehH(hX1), eallF(pX2, kX)%homehHsd(hX1) )

                end do

            end do
        end do

    end subroutine aggregateMomentsHeterogeneitySingle

    subroutine aggregateMomentsHeterogeneity(qM, qF, eallQ, eall)
        implicit none
        real(dp),      intent(in)  :: qM(NPOINT1)
        real(dp),      intent(in)  :: qF(NPOINT2)
        type(eall2_t), intent(in)  :: eallQ(NTYPE1, NTYPE2, NPOINT1, NPOINT2, 0:1)
        type(eall2_t), intent(out) :: eall(NTYPE1, NTYPE2, 0:1)

        integer :: pX1, pX2, qX1, qX2, hX1, hX2, kX, qX
        real(dp) :: q(NPOINT1, NPOINT2), qVec(NPOINTALL)
        real(dp) :: group_mean(NPOINTALL), group_sd(NPOINTALL)
        real(dp) :: group_mean1(NPOINTALL)
        real(dp) :: group_mean2(NPOINTALL)

        do qX2 = 1, NPOINT2
            do qX1 = 1, NPOINT1
                qX = qX1 + (qX2 - 1) * NPOINT1
                qVec(qX) = qM(qX1) * qF(qX2)
                q(qX1, qX2) = qVec(qX)
            end do
        end do

        do kX = 0, 1
            do pX2 = 1, NTYPE2
                do pX1 = 1, NTYPE1

                    eall(pX1, pX2, kX)%jhours = 0.0_dp
                    eall(pX1, pX2, kX)%mhours = 0.0_dp
                    eall(pX1, pX2, kX)%fhours = 0.0_dp
                    eall(pX1, pX2, kX)%mwork  = 0.0_dp
                    eall(pX1, pX2, kX)%fwork  = 0.0_dp
                    eall(pX1, pX2, kX)%mhome  = 0.0_dp
                    eall(pX1, pX2, kX)%fhome  = 0.0_dp

                    do qX2 = 1, NPOINT2
                        do qX1 = 1, NPOINT1
                            eall(pX1, pX2, kX)%jhours = eall(pX1, pX2, kX)%jhours + q(qX1, qX2) * eallQ(pX1, pX2, qX1, qX2, kX)%jhours
                            eall(pX1, pX2, kX)%mhours = eall(pX1, pX2, kX)%mhours + q(qX1, qX2) * eallQ(pX1, pX2, qX1, qX2, kX)%mhours
                            eall(pX1, pX2, kX)%fhours = eall(pX1, pX2, kX)%fhours + q(qX1, qX2) * eallQ(pX1, pX2, qX1, qX2, kX)%fhours
                            eall(pX1, pX2, kX)%mwork  = eall(pX1, pX2, kX)%mwork  + q(qX1, qX2) * eallQ(pX1, pX2, qX1, qX2, kX)%mwork
                            eall(pX1, pX2, kX)%fwork  = eall(pX1, pX2, kX)%fwork  + q(qX1, qX2) * eallQ(pX1, pX2, qX1, qX2, kX)%fwork
                            eall(pX1, pX2, kX)%mhome  = eall(pX1, pX2, kX)%mhome  + q(qX1, qX2) * eallQ(pX1, pX2, qX1, qX2, kX)%mhome
                            eall(pX1, pX2, kX)%fhome  = eall(pX1, pX2, kX)%fhome  + q(qX1, qX2) * eallQ(pX1, pX2, qX1, qX2, kX)%fhome
                        end do
                    end do
                end do
            end do
        end do

        do kX = 0, 1
            do pX2 = 1, NTYPE2
                do pX1 = 1, NTYPE1

                    ! conditional hours distribution
                    do qX2 = 1, NPOINT2
                        do qX1 = 1, NPOINT1
                            qX = qX1 + (qX2 - 1) * NPOINT1
                            group_mean(qX) = eallQ(pX1, pX2, qX1, qX2, kX)%mhourshMw
                            group_sd(qX) = eallQ(pX1, pX2, qX1, qX2, kX)%mhourshMwsd
                        end do
                    end do

                    call momentIntegrateGroup( NPOINTALL, qVec, group_mean, group_sd, &
                                               eall(pX1, pX2, kX)%mhourshMw, eall(pX1, pX2, kX)%mhourshMwsd )

                    do qX2 = 1, NPOINT2
                        do qX1 = 1, NPOINT1
                            qX = qX1 + (qX2 - 1) * NPOINT1
                            group_mean(qX) = eallQ(pX1, pX2, qX1, qX2, kX)%fhourshFw
                            group_sd(qX) = eallQ(pX1, pX2, qX1, qX2, kX)%fhourshFwsd
                        end do
                    end do

                    call momentIntegrateGroup( NPOINTALL, qVec, group_mean, group_sd, &
                                               eall(pX1, pX2, kX)%fhourshFw, eall(pX1, pX2, kX)%fhourshFwsd )

                    ! unconditional hours distribution
                    do qX2 = 1, NPOINT2
                        do qX1 = 1, NPOINT1
                            qX = qX1 + (qX2 - 1) * NPOINT1
                            group_mean(qX) = eallQ(pX1, pX2, qX1, qX2, kX)%mhoursh
                            group_sd(qX) = eallQ(pX1, pX2, qX1, qX2, kX)%mhourshsd
                        end do
                    end do

                    call momentIntegrateGroup( NPOINTALL, qVec, group_mean, group_sd, &
                                               eall(pX1, pX2, kX)%mhoursh, eall(pX1, pX2, kX)%mhourshsd )

                    do qX2 = 1, NPOINT2
                        do qX1 = 1, NPOINT1
                            qX = qX1 + (qX2 - 1) * NPOINT1
                            group_mean(qX) = eallQ(pX1, pX2, qX1, qX2, kX)%fhoursh
                            group_sd(qX) = eallQ(pX1, pX2, qX1, qX2, kX)%fhourshsd
                        end do
                    end do

                    call momentIntegrateGroup( NPOINTALL, qVec, group_mean, group_sd, &
                                               eall(pX1, pX2, kX)%fhoursh, eall(pX1, pX2, kX)%fhourshsd )

                    do hX1 = 0, 2

                        do qX2 = 1, NPOINT2
                            do qX1 = 1, NPOINT1
                                qX = qX1 + (qX2 - 1) * NPOINT1
                                group_mean(qX) = eallQ(pX1, pX2, qX1, qX2, kX)%mwage(hX1)
                                group_sd(qX) = eallQ(pX1, pX2, qX1, qX2, kX)%mwagesd(hX1)
                            end do
                        end do

                        call momentIntegrateGroup( NPOINTALL, qVec, group_mean, group_sd, &
                                                   eall(pX1, pX2, kX)%mwage(hX1), eall(pX1, pX2, kX)%mwagesd(hX1) )

                    end do

                    do qX2 = 1, NPOINT2
                        do qX1 = 1, NPOINT1
                            qX = qX1 + (qX2 - 1) * NPOINT1
                            group_mean(qX) = eallQ(pX1, pX2, qX1, qX2, kX)%mwageMw
                            group_sd(qX) = eallQ(pX1, pX2, qX1, qX2, kX)%mwagesdMw
                        end do
                    end do

                    call momentIntegrateGroup( NPOINTALL, qVec, group_mean, group_sd, &
                                               eall(pX1, pX2, kX)%mwageMw, eall(pX1, pX2, kX)%mwagesdMw )

                    do hX2 = 0, 2

                        do qX2 = 1, NPOINT2
                            do qX1 = 1, NPOINT1
                                qX = qX1 + (qX2 - 1) * NPOINT1
                                group_mean(qX) = eallQ(pX1, pX2, qX1, qX2, kX)%fwage(hX2)
                                group_sd(qX) = eallQ(pX1, pX2, qX1, qX2, kX)%fwagesd(hX2)
                            end do
                        end do

                        call momentIntegrateGroup( NPOINTALL, qVec, group_mean, group_sd, &
                                                   eall(pX1, pX2, kX)%fwage(hX2), eall(pX1, pX2, kX)%fwagesd(hX2) )

                    end do

                    do qX2 = 1, NPOINT2
                        do qX1 = 1, NPOINT1
                            qX = qX1 + (qX2 - 1) * NPOINT1
                            group_mean(qX) = eallQ(pX1, pX2, qX1, qX2, kX)%fwageFw
                            group_sd(qX) = eallQ(pX1, pX2, qX1, qX2, kX)%fwagesdFw
                        end do
                    end do

                    call momentIntegrateGroup( NPOINTALL, qVec, group_mean, group_sd, &
                                               eall(pX1, pX2, kX)%fwageFw, eall(pX1, pX2, kX)%fwagesdFw )

                    do hX1 = 0, 1

                        do qX2 = 1, NPOINT2
                            do qX1 = 1, NPOINT1
                                qX = qX1 + (qX2 - 1) * NPOINT1
                                group_mean(qX) = eallQ(pX1, pX2, qX1, qX2, kX)%mwageFw(hX1)
                                group_sd(qX) = eallQ(pX1, pX2, qX1, qX2, kX)%mwageFwsd(hX1)
                            end do
                        end do

                        call momentIntegrateGroup( NPOINTALL, qVec, group_mean, group_sd, &
                                                   eall(pX1, pX2, kX)%mwageFw(hX1), eall(pX1, pX2, kX)%mwageFwsd(hX1) )

                    end do

                    do hX2 = 0, 1

                        do qX2 = 1, NPOINT2
                            do qX1 = 1, NPOINT1
                                qX = qX1 + (qX2 - 1) * NPOINT1
                                group_mean(qX) = eallQ(pX1, pX2, qX1, qX2, kX)%fwageMw(hX2)
                                group_sd(qX) = eallQ(pX1, pX2, qX1, qX2, kX)%fwageMwsd(hX2)
                            end do
                        end do

                        call momentIntegrateGroup( NPOINTALL, qVec, group_mean, group_sd, &
                                                   eall(pX1, pX2, kX)%fwageMw(hX2), eall(pX1, pX2, kX)%fwageMwsd(hX2) )

                    end do

                    do qX2 = 1, NPOINT2
                        do qX1 = 1, NPOINT1
                            qX = qX1 + (qX2 - 1) * NPOINT1
                            group_mean(qX) = eallQ(pX1, pX2, qX1, qX2, kX)%jwage
                            group_sd(qX) = eallQ(pX1, pX2, qX1, qX2, kX)%jwagesd
                        end do
                    end do

                    call momentIntegrateGroup( NPOINTALL, qVec, group_mean, group_sd, &
                                               eall(pX1, pX2, kX)%jwage, eall(pX1, pX2, kX)%jwagesd )

                    do qX2 = 1, NPOINT2
                        do qX1 = 1, NPOINT1
                            qX = qX1 + (qX2 - 1) * NPOINT1
                            group_mean(qX) = eallQ(pX1, pX2, qX1, qX2, kX)%mhomeh
                            group_sd(qX) = eallQ(pX1, pX2, qX1, qX2, kX)%mhomehsd
                        end do
                    end do

                    call momentIntegrateGroup( NPOINTALL, qVec, group_mean, group_sd, &
                                               eall(pX1, pX2, kX)%mhomeh, eall(pX1, pX2, kX)%mhomehsd )

                    do qX2 = 1, NPOINT2
                        do qX1 = 1, NPOINT1
                            qX = qX1 + (qX2 - 1) * NPOINT1
                            group_mean(qX) = eallQ(pX1, pX2, qX1, qX2, kX)%fhomeh
                            group_sd(qX) = eallQ(pX1, pX2, qX1, qX2, kX)%fhomehsd
                        end do
                    end do

                    call momentIntegrateGroup( NPOINTALL, qVec, group_mean, group_sd, &
                                               eall(pX1, pX2, kX)%fhomeh, eall(pX1, pX2, kX)%fhomehsd )

                    do hX1 = 0, 1

                        do qX2 = 1, NPOINT2
                            do qX1 = 1, NPOINT1
                                qX = qX1 + (qX2 - 1) * NPOINT1
                                group_mean(qX) = eallQ(pX1, pX2, qX1, qX2, kX)%mhomehMw(hX1)
                                group_sd(qX) = eallQ(pX1, pX2, qX1, qX2, kX)%mhomehMwsd(hX1)
                            end do
                        end do

                        call momentIntegrateGroup( NPOINTALL, qVec, group_mean, group_sd, &
                                                   eall(pX1, pX2, kX)%mhomehMw(hX1), eall(pX1, pX2, kX)%mhomehMwsd(hX1) )

                    end do

                    do hX1 = 0, 2

                        do qX2 = 1, NPOINT2
                            do qX1 = 1, NPOINT1
                                qX = qX1 + (qX2 - 1) * NPOINT1
                                group_mean(qX) = eallQ(pX1, pX2, qX1, qX2, kX)%mhomehMh(hX1)
                                group_sd(qX) = eallQ(pX1, pX2, qX1, qX2, kX)%mhomehMhsd(hX1)
                            end do
                        end do

                        call momentIntegrateGroup( NPOINTALL, qVec, group_mean, group_sd, &
                                                   eall(pX1, pX2, kX)%mhomehMh(hX1), eall(pX1, pX2, kX)%mhomehMhsd(hX1) )

                    end do

                    do hX2 = 0, 1

                        do qX2 = 1, NPOINT2
                            do qX1 = 1, NPOINT1
                                qX = qX1 + (qX2 - 1) * NPOINT1
                                group_mean(qX) = eallQ(pX1, pX2, qX1, qX2, kX)%mhomehFw(hX2)
                                group_sd(qX) = eallQ(pX1, pX2, qX1, qX2, kX)%mhomehFwsd(hX2)
                            end do
                        end do

                        call momentIntegrateGroup( NPOINTALL, qVec, group_mean, group_sd, &
                                                   eall(pX1, pX2, kX)%mhomehFw(hX2), eall(pX1, pX2, kX)%mhomehFwsd(hX2) )

                    end do

                    do hX2 = 0, 2

                        do qX2 = 1, NPOINT2
                            do qX1 = 1, NPOINT1
                                qX = qX1 + (qX2 - 1) * NPOINT1
                                group_mean(qX) = eallQ(pX1, pX2, qX1, qX2, kX)%mhomehFh(hX2)
                                group_sd(qX) = eallQ(pX1, pX2, qX1, qX2, kX)%mhomehFhsd(hX2)
                            end do
                        end do

                        call momentIntegrateGroup( NPOINTALL, qVec, group_mean, group_sd, &
                                                   eall(pX1, pX2, kX)%mhomehFh(hX2), eall(pX1, pX2, kX)%mhomehFhsd(hX2) )

                    end do


                    do hX2 = 0, 1

                        do qX2 = 1, NPOINT2
                            do qX1 = 1, NPOINT1
                                qX = qX1 + (qX2 - 1) * NPOINT1
                                group_mean(qX) = eallQ(pX1, pX2, qX1, qX2, kX)%fhomehFw(hX2)
                                group_sd(qX) = eallQ(pX1, pX2, qX1, qX2, kX)%fhomehFwsd(hX2)
                            end do
                        end do

                        call momentIntegrateGroup( NPOINTALL, qVec, group_mean, group_sd, &
                                                   eall(pX1, pX2, kX)%fhomehFw(hX2), eall(pX1, pX2, kX)%fhomehFwsd(hX2) )

                    end do

                    do hX2 = 0, 2

                        do qX2 = 1, NPOINT2
                            do qX1 = 1, NPOINT1
                                qX = qX1 + (qX2 - 1) * NPOINT1
                                group_mean(qX) = eallQ(pX1, pX2, qX1, qX2, kX)%fhomehFh(hX2)
                                group_sd(qX) = eallQ(pX1, pX2, qX1, qX2, kX)%fhomehFhsd(hX2)
                            end do
                        end do

                        call momentIntegrateGroup( NPOINTALL, qVec, group_mean, group_sd, &
                                                   eall(pX1, pX2, kX)%fhomehFh(hX2), eall(pX1, pX2, kX)%fhomehFhsd(hX2) )

                    end do

                    do hX1 = 0, 1

                        do qX2 = 1, NPOINT2
                            do qX1 = 1, NPOINT1
                                qX = qX1 + (qX2 - 1) * NPOINT1
                                group_mean(qX) = eallQ(pX1, pX2, qX1, qX2, kX)%fhomehMw(hX1)
                                group_sd(qX) = eallQ(pX1, pX2, qX1, qX2, kX)%fhomehMwsd(hX1)
                            end do
                        end do

                        call momentIntegrateGroup( NPOINTALL, qVec, group_mean, group_sd, &
                                                   eall(pX1, pX2, kX)%fhomehMw(hX1), eall(pX1, pX2, kX)%fhomehMwsd(hX1) )

                    end do

                    do hX1 = 0, 2

                        do qX2 = 1, NPOINT2
                            do qX1 = 1, NPOINT1
                                qX = qX1 + (qX2 - 1) * NPOINT1
                                group_mean(qX) = eallQ(pX1, pX2, qX1, qX2, kX)%fhomehMh(hX1)
                                group_sd(qX) = eallQ(pX1, pX2, qX1, qX2, kX)%fhomehMhsd(hX1)
                            end do
                        end do

                        call momentIntegrateGroup( NPOINTALL, qVec, group_mean, group_sd, &
                                                   eall(pX1, pX2, kX)%fhomehMh(hX1), eall(pX1, pX2, kX)%fhomehMhsd(hX1) )

                    end do

                    ! log-wage covariance
                    do qX2 = 1, NPOINT2
                        do qX1 = 1, NPOINT1
                            qX = qX1 + (qX2 - 1) * NPOINT1
                            group_mean1(qX) = eallQ(pX1, pX2, qX1, qX2, kX)%mwageFw(1)
                            group_mean2(qX) = eallQ(pX1, pX2, qX1, qX2, kX)%fwageMw(1)
                            group_mean(qX) = eallQ(pX1, pX2, qX1, qX2, kX)%jwage
                        end do
                    end do

                    eall(pX1, pX2, kX)%jwagecov = sum( qVec * group_mean ) - sum(qVec * group_mean1) * sum(qVec * group_mean2)

                end do ! pX1
            end do ! pX2
        end do ! kX

    end subroutine aggregateMomentsHeterogeneity

    subroutine aggregateMomentsSingles(m,eallM,eallF,mom,momExtra)

        implicit none

        type(m_t),    intent(in) :: m
        type(eall_t), intent(in) :: eallM(NTYPE1, 0:1)
        type(eall_t), intent(in) :: eallF(NTYPE2, 0:1)
        type(mom_t), intent(out) :: mom
        type(momExtra_t), intent(out), optional :: momExtra

        integer :: pX1, pX2, hX1, hX2, kX
        real(dp) :: group_mean_condM(NTYPE1), group_sd_condM(NTYPE1)
        real(dp) :: group_mean_condF(NTYPE2), group_sd_condF(NTYPE2)
        real(dp) :: frac_M0(NTYPE1), frac_0F(NTYPE2)
        real(dp) :: thisFrac1(NTYPE1), thisFrac2(NTYPE2)
        real(dp) :: group_mean_kids(0:1), group_sd_kids(0:1), group_frac_kids(0:1)

        call momentsInitialize( mom, 0.0_dp )

        frac_M0 = m%numSingle1 / sum(m%numSingle1)
        frac_0F = m%numSingle2 / sum(m%numSingle2)

        ! joint labour supply allocation
        do kX = 0, 1
            do pX1 = 1, NTYPE1
                mom%mhours_M0(:, pX1, kX) = eallM(pX1, kX)%hours
            end do
        end do

        do kX = 0, 1
            do pX2 = 1, NTYPE2
                mom%fhours_0F(:, pX2, kX) = eallF(pX2, kX)%hours
            end do
        end do

        ! labour supply
        do kX = 0, 1
            do pX1 = 1, NTYPE1
                mom%mhoursh_M0(pX1, kX) = eallM(pX1, kX)%hourshW
                mom%mhourshsd_M0(pX1, kX) = eallM(pX1, kX)%hourshWsd
            end do
        end do

        do kX = 0, 1
            do pX2 = 1, NTYPE2
                mom%fhoursh_0F(pX2, kX) = eallF(pX2, kX)%hourshW
                mom%fhourshsd_0F(pX2, kX) = eallF(pX2, kX)%hourshWsd
            end do
        end do


        ! wage distribution all workers (single men)
        do pX1 = 1, NTYPE1
            do kX = 0, 1
                group_mean_kids(kX) = eallM(pX1, kX)%wageWork
                group_sd_kids(kX) = eallM(pX1, kX)%wagesdWork
                group_frac_kids(kX) = m%numSingleKid1(pX1, kX)
            end do

            call momentIntegrateGroup( 2, group_frac_kids, group_mean_kids, group_sd_kids, &
                                       mom%mwageMwAll_M0(pX1), mom%mwagesdMwAll_M0(pX1) )

        end do



        ! wage distribution all workers (single women)
        do pX2 = 1, NTYPE2
            do kX = 0, 1
                group_mean_kids(kX) = eallF(pX2, kX)%wageWork
                group_sd_kids(kX) = eallF(pX2, kX)%wagesdWork
                group_frac_kids(kX) = m%numSingleKid2(pX2, kX)
            end do

            call momentIntegrateGroup( 2, group_frac_kids, group_mean_kids, group_sd_kids, &
                                       mom%fwageFwAll_0F(pX2), mom%fwagesdFwAll_0F(pX2) )

        end do


        ! **** ATUS ****


        ! marginal distribution of home production time (single men)

        do kX = 0, 1
            do pX1 = 1, NTYPE1
                mom%mhome_M0(:, pX1, kX) = eallM(pX1, kX)%home
            end do
        end do

        ! marginal distribution of home production time (single women)

        do kX = 0, 1
            do pX2 = 1, NTYPE2
                mom%fhome_0F(:, pX2, kX) = eallF(pX2, kX)%home
            end do
        end do

        ! home production hours (single men)
        do kX = 0, 1
            do pX1 = 1, NTYPE1
                mom%mhomeh_M0(pX1, kX) = eallM(pX1, kX)%homeh
                mom%mhomehsd_M0(pX1, kX) = eallM(pX1, kX)%homehsd
            end do
        end do

        ! home production hours (single women)
        do kX = 0, 1
            do pX2 = 1, NTYPE2
                mom%fhomeh_0F(pX2, kX) = eallF(pX2, kX)%homeh
                mom%fhomehsd_0F(pX2, kX) = eallF(pX2, kX)%homehsd
            end do
        end do


        if ( present(momExtra) ) then

            ! unconditional hours distribution (single men)
            do kX = 0, 1
                do pX1 = 1, NTYPE1
                    momExtra%mhoursuh_M0(pX1, kX) = eallM(pX1, kX)%hoursh
                    momExtra%mhoursuhsd_M0(pX1, kX) = eallM(pX1, kX)%hourshsd
                end do
            end do

            ! unconditional hours distribution (single women)
            do kX = 0, 1
                do pX2 = 1, NTYPE2
                    momExtra%fhoursuh_0F(pX2, kX) = eallF(pX2, kX)%hoursh
                    momExtra%fhoursuhsd_0F(pX2, kX) = eallF(pX2, kX)%hourshsd
                end do
            end do

            ! wage distribution (single men)
            do kX = 0, 1
                do pX1 = 1, NTYPE1
                    do hX1 = 1, 2
                        momExtra%mwage_M0(hX1, pX1, kX) = eallM(pX1, kX)%wage(hX1)
                    end do
                end do
            end do

            do kX = 0, 1
                do pX1 = 1, NTYPE1
                    do hX1 = 1, 2
                        momExtra%mwagesd_M0(hX1, pX1, kX) = eallM(pX1, kX)%wagesd(hX1)
                    end do
                end do
            end do

            ! wage distribution (single women)
            do kX = 0, 1
                do pX2 = 1, NTYPE2
                    do hX2 = 1, 2
                        momExtra%fwage_0F(hX2, pX2, kX) = eallF(pX2, kX)%wage(hX2)
                    end do
                end do
            end do

            do kX = 0, 1
                do pX2 = 1, NTYPE2
                    do hX2 = 1, 2
                        momExtra%fwagesd_0F(hX2, pX2, kX) = eallF(pX2, kX)%wagesd(hX2)
                    end do
                end do
            end do

            ! wage distribution all workers (single men)
            do kX = 0, 1
                do pX1 = 1, NTYPE1
                    momExtra%mwageMw_M0(pX1, kX) = eallM(pX1, kX)%wageWork
                end do
            end do

            do kX = 0, 1
                do pX1 = 1, NTYPE1
                    momExtra%mwagesdMw_M0(pX1, kX) = eallM(pX1, kX)%wagesdWork
                end do
            end do


            ! wage distribution all workers (single women)
            do kX = 0, 1
                do pX2 = 1, NTYPE2
                    momExtra%fwageFw_0F(pX2, kX) = eallF(pX2, kX)%wageWork
                end do
            end do

            do kX = 0, 1
                do pX2 = 1, NTYPE2
                    momExtra%fwagesdFw_0F(pX2, kX) = eallF(pX2, kX)%wagesdWork
                end do
            end do

            ! home production hours by own employment (single men)
            do kX = 0, 1
                do hX1 = 0, 1

                    do pX1 = 1, NTYPE1
                        group_mean_condM(pX1) = eallM(pX1, kX)%homehW(hX1)
                        group_sd_condM(pX1) = eallM(pX1, kX)%homehWsd(hX1)
                        thisFrac1(pX1) = frac_M0(pX1)*eallM(pX1, kX)%work(hX1)
                    end do

                    call momentIntegrateGroup( NTYPE1, thisFrac1, group_mean_condM, group_sd_condM, &
                                               momExtra%mhomehMw_m0(hX1, kX), momExtra%mhomehMwsd_m0(hX1, kX) )

                end do
            end do

            ! home production hours by own hours (single women)
            do kX = 0, 1
                do hX2 = 0, 2

                    do pX2 = 1, NTYPE2
                        group_mean_condF(pX2) = eallF(pX2, kX)%homehH(hX2)
                        group_sd_condF(pX2) = eallF(pX2, kX)%homehHsd(hX2)
                        thisFrac2(pX2) = frac_0F(pX2) * eallF(pX2, kX)%hours(hX2)
                    end do

                    call momentIntegrateGroup( NTYPE2, thisFrac2, group_mean_condF, group_sd_condF, &
                                               momExtra%fhomehFh_0f(hX2, kX), momExtra%fhomehFhsd_0f(hX2, kX) )

                end do
            end do

        end if

    end subroutine aggregateMomentsSingles



    ! calculate distance metric

    pure function momentDistanceWeight(momData, momModel, momWeight) result(y)
        implicit none
        type(mom_t), intent(in) :: momData, momModel, momWeight
        real(dp) :: y
        real(dp) :: momDataVec(NMOM), momModelVec(NMOM), momWeightVec(NMOM), momDistVec(NMOM)

        momDataVec = transfer( momData, (/1.0_dp/) )
        momModelVec = transfer( momModel, (/1.0_dp/) )
        momWeightVec = transfer( momWeight, (/1.0_dp/) )

        momDistVec = momDataVec - momModelVec
        y = sum(momWeightVec * momDistVec ** 2)

    end function momentDistanceWeight

    subroutine weightingMatrix(vmom, modeMom, weightMom, rescale)
        implicit none
        type(mom_t), intent(in) :: vmom, modeMom
        type(mom_t), intent(out) :: weightMom
        real(dp) :: vecVmom(NMOM), vecModeMom(NMOM), vecWeightMom(NMOM)
        real(dp), intent(in), optional :: rescale
        integer :: iX

        vecVmom = transfer( vmom, (/1.0_dp/) )
        vecModeMom = transfer( modeMom, (/1.0_dp/) )

        do iX = 1, NMOM
            if ( vecVmom(iX) > 0.0_dp ) then
                vecWeightMom(iX) = 1.0_dp / ( vecVmom(iX) ** 2 )
            else
                vecWeightMom(iX) = 0.0_dp
            end if
        end do

        if (present(rescale)) then
            do iX = 1, NMOM
                if ( approxEqual( vecModeMom(iX), 2.0_dp, 1.0e-12_dp ) ) then
                    vecWeightMom(iX) = vecWeightMom(iX) * rescale
                end if
            end do
        end if

        weightMom = transfer( vecWeightMom, vmom )

    end subroutine weightingMatrix

    subroutine momentsInitialize(mom, val)
        implicit none
        type(mom_t), intent(out) :: mom
        real(dp),    intent(in)  :: val

        real(dp) :: momentsVec(NMOM)
        momentsVec = val
        mom = transfer( momentsVec, mom )

    end subroutine momentsInitialize

    pure function approxEqual(x1, x2, tol) result(y)
        implicit none
        real(dp), intent(in) :: x1, x2, tol
        logical :: y
        if ( abs(x1 - x2) < tol ) then
            y = .true.
        else
            y = .false.
        end if
    end function approxEqual

    pure function momentDistanceNoWeight(momData, momModel) result(y)
        implicit none
        type(mom_t), intent(in) :: momData, momModel
        real(dp) :: y
        real(dp) :: momDataVec(NMOM), momModelVec(NMOM), momDistVec(NMOM)

        momDataVec = transfer( momData, (/1.0_dp/) )
        momModelVec = transfer( momModel, (/1.0_dp/) )

        momDistVec = momDataVec - momModelVec
        y = sum(momDistVec ** 2)

    end function momentDistanceNoWeight


    subroutine readMomentFiles(fdir, opt, mom, vmom, modemom)

        implicit none

        character(len = *), intent(in) :: fdir
        type(opt_t), intent(in) :: opt
        type(mom_t), intent(out) :: mom
        type(mom_t), intent(out) :: vmom
        type(mom_t), intent(out) :: modemom
        integer :: funit, istat

        integer :: pX1, pX2
        integer :: hX1, hX2
        integer :: kX, mX

        character(len = :), allocatable :: subdir

        subdir = trim(adjustl(opt%locname)) // '/'

        ! **** IPUMS ****


        ! type distribution

        open(newunit = funit, file = fdir // '/moments/' // subdir // 'Rjtype_M0.asc', status = "old")
        do mX = 1, NMARKET
            do pX1 = 1, NTYPE1
                read(funit, *, iostat = istat) mom%Rjtype_M0(pX1, mX), vmom%Rjtype_M0(pX1, mX), modemom%Rjtype_M0(pX1, mX)
            end do
        end do
        close(funit)

        open(newunit = funit, file = fdir // '/moments/' // subdir // 'Rjtype_0F.asc', status = "old")
        do mX = 1, NMARKET
            do pX2 = 1, NTYPE2
                read(funit, *, iostat = istat) mom%Rjtype_0F(pX2, mX), vmom%Rjtype_0F(pX2, mX), modemom%Rjtype_0F(pX2, mX)
            end do
        end do
        close(funit)

        open(newunit = funit, file = fdir // '/moments/' // subdir // 'Rjtype_MF.asc', status = "old")
        do mX = 1, NMARKET
            do pX2 = 1, NTYPE2
                do pX1 = 1, NTYPE1
                    read(funit, *, iostat = istat) mom%Rjtype_MF(pX1, pX2, mX), vmom%Rjtype_MF(pX1, pX2, mX), &
                                                   modemom%Rjtype_MF(pX1, pX2, mX)
                end do
            end do
        end do
        close(funit)

        ! joint labour supply allocation

        open(newunit = funit, file = fdir // '/moments/' // subdir // 'Rmhoursmean_M0.asc', status = "old")
        do mX = 1, NMARKET
            do pX1 = 1, NTYPE1
                read(funit, *, iostat = istat) mom%Rmhours_M0(pX1, mX), vmom%Rmhours_M0(pX1, mX), modemom%Rmhours_M0(pX1, mX)
            end do
        end do
        close(funit)

        open(newunit = funit, file = fdir // '/moments/' // subdir // 'Rfhoursmean_0F.asc', status = "old")
        do mX = 1, NMARKET
            do pX2 = 1, NTYPE2
                read(funit, *, iostat = istat) mom%Rfhours_0F(pX2, mX), vmom%Rfhours_0F(pX2, mX), modemom%Rfhours_0F(pX2, mX)
            end do
        end do
        close(funit)

        open(newunit = funit, file = fdir // '/moments/' // subdir // 'Rmhoursmean_Mf.asc', status = "old")
        do mX = 1, NMARKET
            do pX1 = 1, NTYPE1
                read(funit, *, iostat = istat) mom%Rmhours_Mf(pX1, mX), vmom%Rmhours_Mf(pX1, mX), modemom%Rmhours_Mf(pX1, mX)
            end do
        end do
        close(funit)

        open(newunit = funit, file = fdir // '/moments/' // subdir // 'Rfhoursmean_mF.asc', status = "old")
        do mX = 1, NMARKET
            do pX2 = 1, NTYPE2
                read(funit, *, iostat = istat) mom%Rfhours_mF(pX2, mX), vmom%Rfhours_mF(pX2, mX), modemom%Rfhours_mF(pX2, mX)
            end do
        end do
        close(funit)

        ! employment status
        ! -----------------

        open(newunit = funit, file = fdir // '/moments/' // subdir // 'Rmwork_M0.asc', status = "old")
        do mX = 1, NMARKET
            do pX1 = 1, NTYPE1
                read(funit, *, iostat = istat) mom%Rmwork_M0(pX1, mX), vmom%Rmwork_M0(pX1, mX), modemom%Rmwork_M0(pX1, mX)
            end do
        end do
        close(funit)

        open(newunit = funit, file = fdir // '/moments/' // subdir // 'Rfwork_0F.asc', status = "old")
        do mX = 1, NMARKET
            do pX2 = 1, NTYPE2
                read(funit, *, iostat = istat) mom%Rfwork_0F(pX2, mX), vmom%Rfwork_0F(pX2, mX), modemom%Rfwork_0F(pX2, mX)
            end do
        end do
        close(funit)

        open(newunit = funit, file = fdir // '/moments/' // subdir // 'Rmwork_Mf.asc', status = "old")
        do mX = 1, NMARKET
            do pX1 = 1, NTYPE1
                read(funit, *, iostat = istat) mom%Rmwork_Mf(pX1, mX), vmom%Rmwork_Mf(pX1, mX), modemom%Rmwork_Mf(pX1, mX)
            end do
        end do
        close(funit)

        open(newunit = funit, file = fdir // '/moments/' // subdir // 'Rfwork_mF.asc', status = "old")
        do mX = 1, NMARKET
            do pX2 = 1, NTYPE2
                read(funit, *, iostat = istat) mom%Rfwork_mF(pX2, mX), vmom%Rfwork_mF(pX2, mX), modemom%Rfwork_mF(pX2, mX)
            end do
        end do
        close(funit)

        open(newunit = funit, file = fdir // '/moments/' // subdir // 'mhours_M0.asc', status = "old")
        do kX = 0, 1
            do pX1 = 1, NTYPE1
                do hX1 = 0, 2
                    read(funit, *, iostat = istat) mom%mhours_M0(hX1, pX1, kX), vmom%mhours_M0(hX1, pX1, kX), &
                                                   modemom%mhours_M0(hX1, pX1, kX)
                end do
                call checkStateIntegration( mom%mhours_M0(:, pX1, kX), 'mhours_M0' )
            end do
        end do
        close(funit)


        open(newunit = funit, file = fdir // '/moments/' // subdir // 'fhours_0F.asc', status = "old")
        do kX = 0, 1
            do pX2 = 1, NTYPE2
                do hX2 = 0, 2
                    read(funit, *, iostat = istat) mom%fhours_0F(hX2, pX2, kX), vmom%fhours_0F(hX2, pX2, kX), &
                                                   modemom%fhours_0F(hX2, pX2, kX)
                end do
                call checkStateIntegration( mom%fhours_0F(:, pX2, kX), 'fhours_0F' )
            end do
        end do
        close(funit)


        open(newunit = funit, file = fdir // '/moments/' // subdir // 'mhours_MF.asc', status = "old")
        do kX = 0, 1
            do pX2 = 1, NTYPE2
                do pX1 = 1, NTYPE1
                    do hX1 = 0, 2
                        read(funit, *, iostat = istat) mom%mhours_MF(hX1, pX1, pX2, kX), vmom%mhours_MF(hX1, pX1, pX2, kX), &
                                                       modemom%mhours_MF(hX1, pX1, pX2, kX)
                    end do
                    call checkStateIntegration( mom%mhours_MF(:, pX1, pX2, kX), 'mhours_MF' )
                end do
            end do
        end do
        close(funit)

        open(newunit = funit, file = fdir // '/moments/' // subdir // 'fhours_MF.asc', status = "old")
        do kX = 0, 1
            do pX2 = 1, NTYPE2
                do pX1 = 1, NTYPE1
                    do hX2 = 0, 2
                        read(funit, *, iostat = istat) mom%fhours_MF(hX2, pX1, pX2, kX), vmom%fhours_MF(hX2, pX1, pX2, kX), &
                                                       modemom%fhours_MF(hX2, pX1, pX2, kX)
                    end do
                    call checkStateIntegration( mom%mhours_MF(:, pX1, pX2, kX), 'mhours_MF' )
                end do
            end do
        end do
        close(funit)



        open(newunit = funit, file = fdir // '/moments/' // subdir // 'mhoursmean_M0.asc', status = "old")
        do kX = 0, 1
            do pX1 = 1, NTYPE1
                read(funit, *, iostat = istat) mom%mhoursh_M0(pX1, kX), vmom%mhoursh_M0(pX1, kX), modemom%mhoursh_M0(pX1, kX)
            end do
        end do
        close(funit)

        open(newunit = funit, file = fdir // '/moments/' // subdir // 'mhourssd_M0.asc', status = "old")
        do kX = 0, 1
            do pX1 = 1, NTYPE1
                read(funit, *, iostat = istat) mom%mhourshsd_M0(pX1, kX), vmom%mhourshsd_M0(pX1, kX), modemom%mhourshsd_M0(pX1, kX)
            end do
        end do
        close(funit)

        open(newunit = funit, file = fdir // '/moments/' // subdir // 'fhoursmean_0F.asc', status = "old")
        do kX = 0, 1
            do pX2 = 1, NTYPE2
                read(funit, *, iostat = istat) mom%fhoursh_0F(pX2, kX), vmom%fhoursh_0F(pX2, kX), modemom%fhoursh_0F(pX2, kX)
            end do
        end do
        close(funit)

        open(newunit = funit, file = fdir // '/moments/' // subdir // 'fhourssd_0F.asc', status = "old")
        do kX = 0, 1
            do pX2 = 1, NTYPE2
                read(funit, *, iostat = istat) mom%fhourshsd_0F(pX2, kX), vmom%fhourshsd_0F(pX2, kX), modemom%fhourshsd_0F(pX2, kX)
            end do
        end do
        close(funit)

        open(newunit = funit, file = fdir // '/moments/' // subdir // 'mhoursmean_Mf.asc', status = "old")
        do kX = 0, 1
            do pX1 = 1, NTYPE1
                read(funit, *, iostat = istat) mom%mhoursh_Mf(pX1, kX), vmom%mhoursh_Mf(pX1, kX), modemom%mhoursh_Mf(pX1, kX)
            end do
        end do
        close(funit)

        open(newunit = funit, file = fdir // '/moments/' // subdir // 'mhourssd_Mf.asc', status = "old")
        do kX = 0, 1
            do pX1 = 1, NTYPE1
                read(funit, *, iostat = istat) mom%mhourshsd_Mf(pX1, kX), vmom%mhourshsd_Mf(pX1, kX), modemom%mhourshsd_Mf(pX1, kX)
            end do
        end do
        close(funit)

        open(newunit = funit, file = fdir // '/moments/' // subdir // 'fhoursmean_mF.asc', status = "old")
        do kX = 0, 1
            do pX2 = 1, NTYPE2
                read(funit, *, iostat = istat) mom%fhoursh_mF(pX2, kX), vmom%fhoursh_mF(pX2, kX), modemom%fhoursh_mF(pX2, kX)
            end do
        end do
        close(funit)

        open(newunit = funit, file = fdir // '/moments/' // subdir // 'fhourssd_mF.asc', status = "old")
        do kX = 0, 1
            do pX2 = 1, NTYPE2
                read(funit, *, iostat = istat) mom%fhourshsd_mF(pX2, kX), vmom%fhourshsd_mF(pX2, kX), modemom%fhourshsd_mF(pX2, kX)
            end do
        end do
        close(funit)


        open(newunit = funit, file = fdir // '/moments/' // subdir // 'mhoursh_M_F.asc', status = "old")
        do pX2 = 1, NTYPE2
            do pX1 = 1, NTYPE1
                read(funit, *, iostat = istat) mom%mhoursh_M_F(pX1, pX2), vmom%mhoursh_M_F(pX1, pX2), modemom%mhoursh_M_F(pX1, pX2)
            end do
        end do
        close(funit)

        open(newunit = funit, file = fdir // '/moments/' // subdir // 'fhoursh_M_F.asc', status = "old")
        do pX2 = 1, NTYPE2
            do pX1 = 1, NTYPE1
                read(funit, *, iostat = istat) mom%fhoursh_M_F(pX1, pX2), vmom%fhoursh_M_F(pX1, pX2), modemom%fhoursh_M_F(pX1, pX2)
            end do
        end do
        close(funit)

        open(newunit = funit, file = fdir // '/moments/' // subdir // 'mwork_M_F.asc', status = "old")
        do pX2 = 1, NTYPE2
            do pX1 = 1, NTYPE1
                read(funit, *, iostat = istat) mom%mwork_M_F(pX1, pX2), vmom%mwork_M_F(pX1, pX2), modemom%mwork_M_F(pX1, pX2)
            end do
        end do
        close(funit)

        open(newunit = funit, file = fdir // '/moments/' // subdir // 'fwork_M_F.asc', status = "old")
        do pX2 = 1, NTYPE2
            do pX1 = 1, NTYPE1
                read(funit, *, iostat = istat) mom%fwork_M_F(pX1, pX2), vmom%fwork_M_F(pX1, pX2), modemom%fwork_M_F(pX1, pX2)
            end do
        end do
        close(funit)


        ! wage distribution all workers (single men, integrates over kids)

        open(newunit = funit, file = fdir // '/moments/' // subdir // 'mwageMwAll_M0.asc', status = "old")
        do pX1 = 1, NTYPE1
            read(funit, *, iostat = istat) mom%mwageMwAll_M0(pX1), vmom%mwageMwAll_M0(pX1), modemom%mwageMwAll_M0(pX1)
        end do
        close(funit)

        open(newunit = funit, file = fdir // '/moments/' // subdir // 'mwagesdMwAll_M0.asc', status = "old")
        do pX1 = 1, NTYPE1
            read(funit, *, iostat = istat) mom%mwagesdMwAll_M0(pX1), vmom%mwagesdMwAll_M0(pX1), modemom%mwagesdMwAll_M0(pX1)
        end do
        close(funit)


        ! wage distribution all workers (single women, integrates over kids)

        open(newunit = funit, file = fdir // '/moments/' // subdir // 'fwageFwAll_0F.asc', status = "old")
        do pX1 = 1, NTYPE1
            read(funit, *, iostat = istat) mom%fwageFwAll_0F(pX1), vmom%fwageFwAll_0F(pX1), modemom%fwageFwAll_0F(pX1)
        end do
        close(funit)

        open(newunit = funit, file = fdir // '/moments/' // subdir // 'fwagesdFwAll_0F.asc', status = "old")
        do pX2 = 1, NTYPE2
            read(funit, *, iostat = istat) mom%fwagesdFwAll_0F(pX2), vmom%fwagesdFwAll_0F(pX2), modemom%fwagesdFwAll_0F(pX2)
        end do
        close(funit)



        ! wage distribution all workers (married men, integrates over spouse type/state and kids)

        open(newunit = funit, file = fdir // '/moments/' // subdir // 'mwageMwAll_Mf.asc', status = "old")
        do pX1 = 1, NTYPE1
            read(funit, *, iostat = istat) mom%mwageMwAll_Mf(pX1), vmom%mwageMwAll_Mf(pX1), modemom%mwageMwAll_Mf(pX1)
        end do
        close(funit)

        open(newunit = funit, file = fdir // '/moments/' // subdir // 'mwagesdMwAll_Mf.asc', status = "old")
        do pX1 = 1, NTYPE1
            read(funit, *, iostat = istat) mom%mwagesdMwAll_Mf(pX1), vmom%mwagesdMwAll_Mf(pX1), modemom%mwagesdMwAll_Mf(pX1)
        end do
        close(funit)


        ! wage distribution all workers (married women, integrates over spouse type/state and kids)

        open(newunit = funit, file = fdir // '/moments/' // subdir // 'fwageFwAll_mF.asc', status = "old")
        do pX2 = 1, NTYPE2
            read(funit, *, iostat = istat) mom%fwageFwAll_mF(pX2), vmom%fwageFwAll_mF(pX2), modemom%fwageFwAll_mF(pX2)
        end do
        close(funit)

        open(newunit = funit, file = fdir // '/moments/' // subdir // 'fwagesdFwAll_mF.asc', status = "old")
        do pX2 = 1, NTYPE2
            read(funit, *, iostat = istat) mom%fwagesdFwAll_mF(pX2), vmom%fwagesdFwAll_mF(pX2), modemom%fwagesdFwAll_mF(pX2)
        end do
        close(funit)



        ! earnings distribution all workers (single men, integrates over kids)

        open(newunit = funit, file = fdir // '/moments/' // subdir // 'mearnMwAll_M0.asc', status = "old")
        do pX1 = 1, NTYPE1
            read(funit, *, iostat = istat) mom%mearnMwAll_M0(pX1), vmom%mearnMwAll_M0(pX1), modemom%mearnMwAll_M0(pX1)
        end do
        close(funit)

        open(newunit = funit, file = fdir // '/moments/' // subdir // 'mearnsdMwAll_M0.asc', status = "old")
        do pX1 = 1, NTYPE1
            read(funit, *, iostat = istat) mom%mearnsdMwAll_M0(pX1), vmom%mearnsdMwAll_M0(pX1), modemom%mearnsdMwAll_M0(pX1)
        end do
        close(funit)


        ! earnings distribution all workers (single women, integrates over kids)

        open(newunit = funit, file = fdir // '/moments/' // subdir // 'fearnFwAll_0F.asc', status = "old")
        do pX1 = 1, NTYPE1
            read(funit, *, iostat = istat) mom%fearnFwAll_0F(pX1), vmom%fearnFwAll_0F(pX1), modemom%fearnFwAll_0F(pX1)
        end do
        close(funit)

        open(newunit = funit, file = fdir // '/moments/' // subdir // 'fearnsdFwAll_0F.asc', status = "old")
        do pX2 = 1, NTYPE2
            read(funit, *, iostat = istat) mom%fearnsdFwAll_0F(pX2), vmom%fearnsdFwAll_0F(pX2), modemom%fearnsdFwAll_0F(pX2)
        end do
        close(funit)



        ! earnings distribution all workers (married men, integrates over spouse type/state and kids)

        open(newunit = funit, file = fdir // '/moments/' // subdir // 'mearnMwAll_Mf.asc', status = "old")
        do pX1 = 1, NTYPE1
            read(funit, *, iostat = istat) mom%mearnMwAll_Mf(pX1), vmom%mearnMwAll_Mf(pX1), modemom%mearnMwAll_Mf(pX1)
        end do
        close(funit)

        open(newunit = funit, file = fdir // '/moments/' // subdir // 'mearnsdMwAll_Mf.asc', status = "old")
        do pX1 = 1, NTYPE1
            read(funit, *, iostat = istat) mom%mearnsdMwAll_Mf(pX1), vmom%mearnsdMwAll_Mf(pX1), modemom%mearnsdMwAll_Mf(pX1)
        end do
        close(funit)


        ! earnings distribution all workers (married women, integrates over spouse type/state and kids)

        open(newunit = funit, file = fdir // '/moments/' // subdir // 'fearnFwAll_mF.asc', status = "old")
        do pX2 = 1, NTYPE2
            read(funit, *, iostat = istat) mom%fearnFwAll_mF(pX2), vmom%fearnFwAll_mF(pX2), modemom%fearnFwAll_mF(pX2)
        end do
        close(funit)

        open(newunit = funit, file = fdir // '/moments/' // subdir // 'fearnsdFwAll_mF.asc', status = "old")
        do pX2 = 1, NTYPE2
            read(funit, *, iostat = istat) mom%fearnsdFwAll_mF(pX2), vmom%fearnsdFwAll_mF(pX2), modemom%fearnsdFwAll_mF(pX2)
        end do
        close(funit)



        ! **** ATUS ****



        ! marginal distribution of home production time (single men/women)

        open(newunit = funit, file = fdir // '/moments/' // subdir // 'mhome_M0.asc', status = "old")
        do kX = 0, 1
            do pX1 = 1, NTYPE1
                do hX1 = 1, NHH
                    read(funit, *, iostat = istat) mom%mhome_M0(hX1, pX1, kX), vmom%mhome_M0(hX1, pX1, kX), modemom%mhome_M0(hX1, pX1, kX)
                end do
                call checkStateIntegration( mom%mhome_M0(:, pX1, kX), 'mhome_M0' )
            end do
        end do
        close(funit)

        open(newunit = funit, file = fdir // '/moments/' // subdir // 'fhome_0F.asc', status = "old")
        do kX = 0, 1
            do pX2 = 1, NTYPE2
                do hX2 = 1, NHH
                    read(funit, *, iostat = istat) mom%fhome_0F(hX2, pX2, kX), vmom%fhome_0F(hX2, pX2, kX), modemom%fhome_0F(hX2, pX2, kX)
                end do
                call checkStateIntegration( mom%fhome_0F(:, pX2, kX), 'fhome_0F' )
            end do
        end do
        close(funit)



        ! marginal distribution of home production time (married men/women)

        open(newunit = funit, file = fdir // '/moments/' // subdir // 'mhome_Mf.asc', status = "old")
        do kX = 0, 1
            do pX1 = 1, NTYPE1
                do hX1 = 1, NHH
                    read(funit, *, iostat = istat) mom%mhome_Mf(hX1, pX1, kX), vmom%mhome_Mf(hX1, pX1, kX), modemom%mhome_Mf(hX1, pX1, kX)
                end do
                call checkStateIntegration( mom%mhome_Mf(:, pX1, kX), 'mhome_Mf' )
            end do
        end do
        close(funit)

        open(newunit = funit, file = fdir // '/moments/' // subdir // 'fhome_mF.asc', status = "old")
        do kX = 0, 1
            do pX2 = 1, NTYPE2
                do hX2 = 1, NHH
                    read(funit, *, iostat = istat) mom%fhome_mF(hX2, pX2, kX), vmom%fhome_mF(hX2, pX2, kX), modemom%fhome_mF(hX2, pX2, kX)
                end do
                call checkStateIntegration( mom%fhome_mF(:, pX2, kX), 'fhome_mF' )
            end do
        end do
        close(funit)



        ! home production hours (single men/women)

        open(newunit = funit, file = fdir // '/moments/' // subdir // 'mhomeh_M0.asc', status = "old")
        do kX = 0, 1
            do pX1 = 1, NTYPE1
                read(funit, *, iostat = istat) mom%mhomeh_M0(pX1, kX), vmom%mhomeh_M0(pX1, kX), modemom%mhomeh_M0(pX1, kX)
            end do
        end do
        close(funit)

        open(newunit = funit, file = fdir // '/moments/' // subdir // 'mhomehsd_M0.asc', status = "old")
        do kX = 0, 1
            do pX1 = 1, NTYPE1
                read(funit, *, iostat = istat) mom%mhomehsd_M0(pX1, kX), vmom%mhomehsd_M0(pX1, kX), modemom%mhomehsd_M0(pX1, kX)
            end do
        end do
        close(funit)

        open(newunit = funit, file = fdir // '/moments/' // subdir // 'fhomeh_0F.asc', status = "old")
        do kX = 0, 1
            do pX2 = 1, NTYPE2
                read(funit, *, iostat = istat) mom%fhomeh_0F(pX2, kX), vmom%fhomeh_0F(pX2, kX), modemom%fhomeh_0F(pX2, kX)
            end do
        end do
        close(funit)

        open(newunit = funit, file = fdir // '/moments/' // subdir // 'fhomehsd_0F.asc', status = "old")
        do kX = 0, 1
            do pX2 = 1, NTYPE2
                read(funit, *, iostat = istat) mom%fhomehsd_0F(pX2, kX), vmom%fhomehsd_0F(pX2, kX), modemom%fhomehsd_0F(pX2, kX)
            end do
        end do
        close(funit)


        ! home production hours (married men/women)

        open(newunit = funit, file = fdir // '/moments/' // subdir // 'mhomeh_Mf.asc', status = "old")
        do kX = 0, 1
            do pX1 = 1, NTYPE1
                read(funit, *, iostat = istat) mom%mhomeh_Mf(pX1, kX), vmom%mhomeh_Mf(pX1, kX), modemom%mhomeh_Mf(pX1, kX)
            end do
        end do
        close(funit)

        open(newunit = funit, file = fdir // '/moments/' // subdir // 'mhomehsd_Mf.asc', status = "old")
        do kX = 0, 1
            do pX1 = 1, NTYPE1
                read(funit, *, iostat = istat) mom%mhomehsd_Mf(pX1, kX), vmom%mhomehsd_Mf(pX1, kX), modemom%mhomehsd_Mf(pX1, kX)
            end do
        end do
        close(funit)

        open(newunit = funit, file = fdir // '/moments/' // subdir // 'fhomeh_mF.asc', status = "old")
        do kX = 0, 1
            do pX2 = 1, NTYPE2
                read(funit, *, iostat = istat) mom%fhomeh_mF(pX2, kX), vmom%fhomeh_mF(pX2, kX), modemom%fhomeh_mF(pX2, kX)
            end do
        end do
        close(funit)

        open(newunit = funit, file = fdir // '/moments/' // subdir // 'fhomehsd_mF.asc', status = "old")
        do kX = 0, 1
            do pX2 = 1, NTYPE2
                read(funit, *, iostat = istat) mom%fhomehsd_mF(pX2, kX), vmom%fhomehsd_mF(pX2, kX), modemom%fhomehsd_mF(pX2, kX)
            end do
        end do
        close(funit)

        ! home hours by match distribution
        open(newunit = funit, file = fdir // '/moments/' // subdir // 'mhomeh_M_F.asc', status = "old")
        do pX2 = 1, NTYPE2
            do pX1 = 1, NTYPE1
                read(funit, *, iostat = istat) mom%mhomeh_M_F(pX1, pX2), vmom%mhomeh_M_F(pX1, pX2), modemom%mhomeh_M_F(pX1, pX2)
            end do
        end do
        close(funit)

        open(newunit = funit, file = fdir // '/moments/' // subdir // 'fhomeh_M_F.asc', status = "old")
        do pX2 = 1, NTYPE2
            do pX1 = 1, NTYPE1
                read(funit, *, iostat = istat) mom%fhomeh_M_F(pX1, pX2), vmom%fhomeh_M_F(pX1, pX2), modemom%fhomeh_M_F(pX1, pX2)
            end do
        end do
        close(funit)

    end subroutine readMomentFiles


    subroutine readMomentFilesExtra2(fdir, ftag, mom)

        implicit none

        character(len = *), intent(in) :: fdir
        character(len = *), intent(in) :: ftag
        type(momExtra_t), intent(out) :: mom
        integer :: funit, istat

        integer :: pX1, pX2
        integer :: hX1, hX2
        integer :: kX

        character(len = :), allocatable :: fpath

        fpath = fdir // '/' // ftag // '/'

        ! **** IPUMS ****

        open(newunit = funit, file = fpath // 'jtype_M0.asc', status = "old")
        do pX1 = 1, NTYPE1
            read(funit, *, iostat = istat) mom%jtype_M0(pX1)
        end do
        close(funit)

        open(newunit = funit, file = fpath // 'jtype_0F.asc', status = "old")
        do pX2 = 1, NTYPE2
            read(funit, *, iostat = istat) mom%jtype_0F(pX2)
        end do
        close(funit)

        open(newunit = funit, file = fpath // 'jtype_MF.asc', status = "old")
        do pX2 = 1, NTYPE2
            do pX1 = 1, NTYPE1
                read(funit, *, iostat = istat) mom%jtype_MF(pX1, pX2)
            end do
        end do
        close(funit)

        ! joint labour supply allocation (integrates over spousal state)

        open(newunit = funit, file = fpath // 'mhours_Mf.asc', status = "old")
        do kX = 0, 1
            do pX1 = 1, NTYPE1
                do hX1 = 0, 2
                    read(funit, *, iostat = istat) mom%mhours_Mf(hX1, pX1, kX)
                end do
            end do
        end do
        close(funit)


        open(newunit = funit, file = fpath // 'fhours_mF.asc', status = "old")
        do kX = 0, 1
            do pX2 = 1, NTYPE2
                do hX2 = 0, 2
                    read(funit, *, iostat = istat) mom%fhours_mF(hX2, pX2, kX)
                end do
            end do
        end do
        close(funit)


        open(newunit = funit, file = fpath // 'jhours_MF.asc', status = "old")
        do kX = 0, 1
            do pX2 = 1, NTYPE2
                do pX1 = 1, NTYPE1
                    do hX2 = 0, 2
                        do hX1 = 0, 2
                            read(funit, *, iostat = istat) mom%jhours_MF(hX1, hX2, pX1, pX2, kX)
                        end do
                    end do
                end do
            end do
        end do
        close(funit)

        ! joint labour supply allocation (integrates over own and spousal state)

        open(newunit = funit, file = fpath // 'mhours_sin.asc', status = "old")
        do kX = 0, 1
            do hX1 = 0, 2
                read(funit, *, iostat = istat) mom%mhours_sin(hX1, kX)
            end do
        end do
        close(funit)

        open(newunit = funit, file = fpath // 'fhours_sin.asc', status = "old")
        do kX = 0, 1
            do hX2 = 0, 2
                read(funit, *, iostat = istat) mom%fhours_sin(hX2, kX)
            end do
        end do
        close(funit)

        open(newunit = funit, file = fpath // 'mhours_cou.asc', status = "old")
        do kX = 0, 1
            do hX1 = 0, 2
                read(funit, *, iostat = istat) mom%mhours_cou(hX1, kX)
            end do
        end do
        close(funit)

        open(newunit = funit, file = fpath // 'fhours_cou.asc', status = "old")
        do kX = 0, 1
            do hX2 = 0, 2
                read(funit, *, iostat = istat) mom%fhours_cou(hX2, kX)
            end do
        end do
        close(funit)

        ! wage distribution (single men)

        open(newunit = funit, file = fpath // 'mwage_M0.asc', status = "old")
        do kX = 0, 1
            do pX1 = 1, NTYPE1
                do hX1 = 1, 2
                    read(funit, *, iostat = istat) mom%mwage_M0(hX1, pX1, kX)
                end do
            end do
        end do
        close(funit)

        open(newunit = funit, file = fpath // 'mwagesd_M0.asc', status = "old")
        do kX = 0, 1
            do pX1 = 1, NTYPE1
                do hX1 = 1, 2
                    read(funit, *, iostat = istat) mom%mwagesd_M0(hX1, pX1, kX)
                end do
            end do
        end do
        close(funit)


        ! wage distribution (single women)

        open(newunit = funit, file = fpath // 'fwage_0F.asc', status = "old")
        do kX = 0, 1
            do pX1 = 1, NTYPE1
                do hX1 = 1, 2
                    read(funit, *, iostat = istat) mom%fwage_0F(hX1, pX1, kX)
                end do
            end do
        end do
        close(funit)

        open(newunit = funit, file = fpath // 'fwagesd_0F.asc', status = "old")
        do kX = 0, 1
            do pX2 = 1, NTYPE2
                do hX2 = 1, 2
                    read(funit, *, iostat = istat) mom%fwagesd_0F(hX2, pX2, kX)
                end do
            end do
        end do
        close(funit)


        ! wage distribution (married men, integrates over spouse type/state)

        open(newunit = funit, file = fpath // 'mwage_Mf.asc', status = "old")
        do kX = 0, 1
            do pX1 = 1, NTYPE1
                do hX1 = 1, 2
                    read(funit, *, iostat = istat) mom%mwage_Mf(hX1, pX1, kX)
                end do
            end do
        end do
        close(funit)

        open(newunit = funit, file = fpath // 'mwagesd_Mf.asc', status = "old")
        do kX = 0, 1
            do pX1 = 1, NTYPE1
                do hX1 = 1, 2
                    read(funit, *, iostat = istat) mom%mwagesd_Mf(hX1, pX1, kX)
                end do
            end do
        end do
        close(funit)


        ! wage distribution (married women, integrates over spouse type/state)

        open(newunit = funit, file = fpath // 'fwage_mF.asc', status = "old")
        do kX = 0, 1
            do pX2 = 1, NTYPE2
                do hX2 = 1, 2
                    read(funit, *, iostat = istat) mom%fwage_mF(hX2, pX2, kX)
                end do
            end do
        end do
        close(funit)

        open(newunit = funit, file = fpath // 'fwagesd_mF.asc', status = "old")
        do kX = 0, 1
            do pX2 = 1, NTYPE2
                do hX2 = 1, 2
                    read(funit, *, iostat = istat) mom%fwagesd_mF(hX2, pX2, kX)
                end do
            end do
        end do
        close(funit)


        ! wage distribution (married men, by spousal employment state)

        open(newunit = funit, file = fpath // 'mwageFw_Mf.asc', status = "old")
        do kX = 0, 1
            do pX1 = 1, NTYPE1
                do hX2 = 0, 1
                    read(funit, *, iostat = istat) mom%mwageFw_Mf(hX2, pX1, kX)
                end do
            end do
        end do
        close(funit)

        open(newunit = funit, file = fpath // 'mwageFwsd_Mf.asc', status = "old")
        do kX = 0, 1
            do pX1 = 1, NTYPE1
                do hX2 = 0, 1
                    read(funit, *, iostat = istat) mom%mwageFwsd_Mf(hX2, pX1, kX)
                end do
            end do
        end do
        close(funit)


        ! wage distribution (married women, by spousal employment state)

        open(newunit = funit, file = fpath // 'fwageMw_mF.asc', status = "old")
        do kX = 0, 1
            do pX2 = 1, NTYPE2
                do hX1 = 0, 1
                    read(funit, *, iostat = istat) mom%fwageMw_mF(hX1, pX2, kX)
                end do
            end do
        end do
        close(funit)

        open(newunit = funit, file = fpath // 'fwageMwsd_mF.asc', status = "old")
        do kX = 0, 1
            do pX2 = 1, NTYPE2
                do hX1 = 0, 1
                    read(funit, *, iostat = istat) mom%fwageMwsd_mF(hX1, pX2, kX)
                end do
            end do
        end do
        close(funit)


        ! product of wages within a match

        open(newunit = funit, file = fpath // 'jwagecov_MF.asc', status = "old")
        do kX = 0, 1
            do pX2 = 1, NTYPE2
                do pX1 = 1, NTYPE1
                    read(funit, *, iostat = istat) mom%jwagecov_MF(pX1, pX2, kX)
                end do
            end do
        end do
        close(funit)


        ! wage distribution all workers (single men)

        open(newunit = funit, file = fpath // 'mwageMw_M0.asc', status = "old")
        do kX = 0, 1
            do pX1 = 1, NTYPE1
                read(funit, *, iostat = istat) mom%mwageMw_M0(pX1, kX)
            end do
        end do
        close(funit)

        open(newunit = funit, file = fpath // 'mwagesdMw_M0.asc', status = "old")
        do kX = 0, 1
            do pX1 = 1, NTYPE1
                read(funit, *, iostat = istat) mom%mwagesdMw_M0(pX1, kX)
            end do
        end do
        close(funit)


        ! wage distribution all workers (single women)

        open(newunit = funit, file = fpath // 'fwageFw_0F.asc', status = "old")
        do kX = 0, 1
            do pX1 = 1, NTYPE1
                read(funit, *, iostat = istat) mom%fwageFw_0F(pX1, kX)
            end do
        end do
        close(funit)

        open(newunit = funit, file = fpath // 'fwagesdFw_0F.asc', status = "old")
        do kX = 0, 1
            do pX2 = 1, NTYPE2
                read(funit, *, iostat = istat) mom%fwagesdFw_0F(pX2, kX)
            end do
        end do
        close(funit)


        ! wage distribution all workers (married men, integrates over spouse type/state)

        open(newunit = funit, file = fpath // 'mwageMw_Mf.asc', status = "old")
        do kX = 0, 1
            do pX1 = 1, NTYPE1
                read(funit, *, iostat = istat) mom%mwageMw_Mf(pX1, kX)
            end do
        end do
        close(funit)

        open(newunit = funit, file = fpath // 'mwagesdMw_Mf.asc', status = "old")
        do kX = 0, 1
            do pX1 = 1, NTYPE1
                read(funit, *, iostat = istat) mom%mwagesdMw_Mf(pX1, kX)
            end do
        end do
        close(funit)


        ! wage distribution all workers (married women, integrates over spouse type/state)

        open(newunit = funit, file = fpath // 'fwageFw_mF.asc', status = "old")
        do kX = 0, 1
            do pX2 = 1, NTYPE2
                read(funit, *, iostat = istat) mom%fwageFw_mF(pX2, kX)
            end do
        end do
        close(funit)

        open(newunit = funit, file = fpath // 'fwagesdFw_mF.asc', status = "old")
        do kX = 0, 1
            do pX2 = 1, NTYPE2
                read(funit, *, iostat = istat) mom%fwagesdFw_mF(pX2, kX)
            end do
        end do
        close(funit)


        ! product of wages within a match

        open(newunit = funit, file = fpath // 'jwage_MF.asc', status = "old")
        do kX = 0, 1
            do pX2 = 1, NTYPE2
                do pX1 = 1, NTYPE1
                    read(funit, *, iostat = istat) mom%jwage_MF(pX1, pX2, kX)
                end do
            end do
        end do
        close(funit)

        open(newunit = funit, file = fpath // 'jwagesd_MF.asc', status = "old")
        do kX = 0, 1
            do pX2 = 1, NTYPE2
                do pX1 = 1, NTYPE1
                    read(funit, *, iostat = istat) mom%jwagesd_MF(pX1, pX2, kX)
                end do
            end do
        end do
        close(funit)



        ! **** ATUS ****

        ! home production allocation (integrates over own and spousal state)

        open(newunit = funit, file = fpath // 'mhome_sin.asc', status = "old")
        do kX = 0, 1
            do hX1 = 1, NHH
                read(funit, *, iostat = istat) mom%mhome_sin(hX1, kX)
            end do
        end do
        close(funit)

        open(newunit = funit, file = fpath // 'fhome_sin.asc', status = "old")
        do kX = 0, 1
            do hX2 = 1, NHH
                read(funit, *, iostat = istat) mom%fhome_sin(hX2, kX)
            end do
        end do
        close(funit)

        open(newunit = funit, file = fpath // 'mhome_cou.asc', status = "old")
        do kX = 0, 1
            do hX1 = 1, NHH
                read(funit, *, iostat = istat) mom%mhome_cou(hX1, kX)
            end do
        end do
        close(funit)

        open(newunit = funit, file = fpath // 'fhome_cou.asc', status = "old")
        do kX = 0, 1
            do hX2 = 1, NHH
                read(funit, *, iostat = istat) mom%fhome_cou(hX2, kX)
            end do
        end do
        close(funit)

        ! home production hours by own employment/hours (single men/women)

        open(newunit = funit, file = fpath // 'mhomehMw_m0.asc', status = "old")
        do kX = 0, 1
            do hX1 = 0, 1
                read(funit, *, iostat = istat) mom%mhomehMw_m0(hX1, kX)
            end do
        end do
        close(funit)

        open(newunit = funit, file = fpath // 'mhomehMwsd_m0.asc', status = "old")
        do kX = 0, 1
            do hX1 = 0, 1
                read(funit, *, iostat = istat) mom%mhomehMwsd_m0(hX1, kX)
            end do
        end do
        close(funit)


        open(newunit = funit, file = fpath // 'fhomehFh_0f.asc', status = "old")
        do kX = 0, 1
            do hX2 = 0, 2
                read(funit, *, iostat = istat) mom%fhomehFh_0f(hX2, kX)
            end do
        end do
        close(funit)

        open(newunit = funit, file = fpath // 'fhomehFhsd_0f.asc', status = "old")
        do kX = 0, 1
            do hX2 = 0, 2
                read(funit, *, iostat = istat) mom%fhomehFhsd_0f(hX2, kX)
            end do
        end do
        close(funit)


        ! home production hours by own employment/hours (married men/women)

        open(newunit = funit, file = fpath // 'mhomehMw_mf.asc', status = "old")
        do kX = 0, 1
            do hX1 = 0, 1
                read(funit, *, iostat = istat) mom%mhomehMw_mf(hX1, kX)
            end do
        end do
        close(funit)

        open(newunit = funit, file = fpath // 'mhomehMwsd_mf.asc', status = "old")
        do kX = 0, 1
            do hX1 = 0, 1
                read(funit, *, iostat = istat) mom%mhomehMwsd_mf(hX1, kX)
            end do
        end do
        close(funit)


        open(newunit = funit, file = fpath // 'fhomehFh_mf.asc', status = "old")
        do kX = 0, 1
            do hX2 = 0, 2
                read(funit, *, iostat = istat) mom%fhomehFh_mf(hX2, kX)
            end do
        end do
        close(funit)

        open(newunit = funit, file = fpath // 'fhomehFhsd_mf.asc', status = "old")
        do kX = 0, 1
            do hX2 = 0, 2
                read(funit, *, iostat = istat) mom%fhomehFhsd_mf(hX2, kX)
            end do
        end do
        close(funit)


        ! home production hours by spouse employment (married men/women)

        open(newunit = funit, file = fpath // 'mhomehFw_mf.asc', status = "old")
        do kX = 0, 1
            do hX2 = 0, 1
                read(funit, *, iostat = istat) mom%mhomehFw_mf(hX2, kX)
            end do
        end do
        close(funit)

        open(newunit = funit, file = fpath // 'mhomehFwsd_mf.asc', status = "old")
        do kX = 0, 1
            do hX2 = 0, 1
                read(funit, *, iostat = istat) mom%mhomehFwsd_mf(hX2, kX)
            end do
        end do
        close(funit)


        open(newunit = funit, file = fpath // 'fhomehMw_mf.asc', status = "old")
        do kX = 0, 1
            do hX1 = 0, 1
                read(funit, *, iostat = istat) mom%fhomehMw_mf(hX1, kX)
            end do
        end do
        close(funit)

        open(newunit = funit, file = fpath // 'fhomehMwsd_mf.asc', status = "old")
        do kX = 0, 1
            do hX1 = 0, 1
                read(funit, *, iostat = istat) mom%fhomehMwsd_mf(hX1, kX)
            end do
        end do
        close(funit)

        ! aggregated moments

        open(newunit = funit, file = fpath // 'mwork_m0.asc', status = "old")
        read(funit, *, iostat = istat) mom%mwork_sin_knk
        close(funit)
        open(newunit = funit, file = fpath // 'mwork_mf.asc', status = "old")
        read(funit, *, iostat = istat) mom%mwork_cou_knk
        close(funit)
        open(newunit = funit, file = fpath // 'fwork_0f.asc', status = "old")
        read(funit, *, iostat = istat) mom%fwork_sin_knk
        close(funit)
        open(newunit = funit, file = fpath // 'fwork_mf.asc', status = "old")
        read(funit, *, iostat = istat) mom%fwork_cou_knk
        close(funit)

        open(newunit = funit, file = fpath // 'mhourshMw_m0.asc', status = "old")
        read(funit, *, iostat = istat) mom%mhourshMw_sin_knk
        close(funit)
        open(newunit = funit, file = fpath // 'mhourshMw_mf.asc', status = "old")
        read(funit, *, iostat = istat) mom%mhourshMw_cou_knk
        close(funit)
        open(newunit = funit, file = fpath // 'fhourshFw_0f.asc', status = "old")
        read(funit, *, iostat = istat) mom%fhourshFw_sin_knk
        close(funit)
        open(newunit = funit, file = fpath // 'fhourshFw_mf.asc', status = "old")
        read(funit, *, iostat = istat) mom%fhourshFw_cou_knk
        close(funit)

        open(newunit = funit, file = fpath // 'mhoursh_m0.asc', status = "old")
        read(funit, *, iostat = istat) mom%mhoursh_sin_knk
        close(funit)
        open(newunit = funit, file = fpath // 'mhoursh_mf.asc', status = "old")
        read(funit, *, iostat = istat) mom%mhoursh_cou_knk
        close(funit)
        open(newunit = funit, file = fpath // 'fhoursh_0f.asc', status = "old")
        read(funit, *, iostat = istat) mom%fhoursh_sin_knk
        close(funit)
        open(newunit = funit, file = fpath // 'fhoursh_mf.asc', status = "old")
        read(funit, *, iostat = istat) mom%fhoursh_cou_knk
        close(funit)

        open(newunit = funit, file = fpath // 'mearnMw_m0.asc', status = "old")
        read(funit, *, iostat = istat) mom%mearnMw_sin_knk
        close(funit)
        open(newunit = funit, file = fpath // 'mearnMw_mf.asc', status = "old")
        read(funit, *, iostat = istat) mom%mearnMw_cou_knk
        close(funit)
        open(newunit = funit, file = fpath // 'fearnFw_0f.asc', status = "old")
        read(funit, *, iostat = istat) mom%fearnFw_sin_knk
        close(funit)
        open(newunit = funit, file = fpath // 'fearnFw_mf.asc', status = "old")
        read(funit, *, iostat = istat) mom%fearnFw_cou_knk
        close(funit)

        open(newunit = funit, file = fpath // 'mhomeh_m0.asc', status = "old")
        read(funit, *, iostat = istat) mom%mhomeh_sin_knk
        close(funit)
        open(newunit = funit, file = fpath // 'mhomeh_mf.asc', status = "old")
        read(funit, *, iostat = istat) mom%mhomeh_cou_knk
        close(funit)
        open(newunit = funit, file = fpath // 'fhomeh_0f.asc', status = "old")
        read(funit, *, iostat = istat) mom%fhomeh_sin_knk
        close(funit)
        open(newunit = funit, file = fpath // 'fhomeh_mf.asc', status = "old")
        read(funit, *, iostat = istat) mom%fhomeh_cou_knk
        close(funit)

        ! moved from mom
        open(newunit = funit, file = fpath // 'mhoursh_M_0.asc', status = "old")
        do pX1 = 1, NTYPE1
            read(funit, *, iostat = istat) mom%mhoursh_M_0(pX1)
        end do
        close(funit)

        open(newunit = funit, file = fpath // 'fhoursh_0_F.asc', status = "old")
        do pX2 = 1, NTYPE2
            read(funit, *, iostat = istat) mom%fhoursh_0_F(pX2)
        end do
        close(funit)

        open(newunit = funit, file = fpath // 'mwork_M_0.asc', status = "old")
        do pX1 = 1, NTYPE1
            read(funit, *, iostat = istat) mom%mwork_M_0(pX1)
        end do
        close(funit)

        open(newunit = funit, file = fpath // 'fwork_0_F.asc', status = "old")
        do pX2 = 1, NTYPE2
            read(funit, *, iostat = istat) mom%fwork_0_F(pX2)
        end do
        close(funit)

        open(newunit = funit, file = fpath // 'mhomeh_M_0.asc', status = "old")
        do pX1 = 1, NTYPE1
            read(funit, *, iostat = istat) mom%mhomeh_M_0(pX1)
        end do
        close(funit)

        open(newunit = funit, file = fpath // 'fhomeh_0_F.asc', status = "old")
        do pX2 = 1, NTYPE2
            read(funit, *, iostat = istat) mom%fhomeh_0_F(pX2)
        end do
        close(funit)

    end subroutine readMomentFilesExtra2


    subroutine readMomentFiles2(fdir,ftag,mom)

        implicit none

        character(len = *), intent(in) :: fdir
        character(len = *), intent(in) :: ftag
        type(mom_t), intent(out) :: mom
        integer :: funit, istat

        integer :: pX1, pX2
        integer :: hX1, hX2
        integer :: kX, mX

        character(len = :), allocatable :: fpath

        fpath = fdir // '/' // ftag // '/'

        ! **** IPUMS ****


        ! type distribution

        open(newunit = funit, file = fpath // 'Rjtype_M0.asc', status = "old")
        do mX = 1, NMARKET
            do pX1 = 1, NTYPE1
                read(funit, *, iostat = istat) mom%Rjtype_M0(pX1, mX)
            end do
        end do
        close(funit)

        open(newunit = funit, file = fpath // 'Rjtype_0F.asc', status = "old")
        do mX = 1, NMARKET
            do pX2 = 1, NTYPE2
                read(funit, *, iostat = istat) mom%Rjtype_0F(pX2, mX)
            end do
        end do
        close(funit)

        open(newunit = funit, file = fpath // 'Rjtype_MF.asc', status = "old")
        do mX = 1, NMARKET
            do pX2 = 1, NTYPE2
                do pX1 = 1, NTYPE1
                    read(funit, *, iostat = istat) mom%Rjtype_MF(pX1, pX2, mX)
                end do
            end do
        end do
        close(funit)

        open(newunit = funit, file = fpath // 'Rmhoursmean_M0.asc', status = "old")
        do mX = 1, NMARKET
            do pX1 = 1, NTYPE1
                read(funit, *, iostat = istat) mom%Rmhours_M0(pX1, mX)
            end do
        end do
        close(funit)

        open(newunit = funit, file = fpath // 'Rfhoursmean_0F.asc', status = "old")
        do mX = 1, NMARKET
            do pX2 = 1, NTYPE2
                read(funit, *, iostat = istat) mom%Rfhours_0F(pX2, mX)
            end do
        end do
        close(funit)

        open(newunit = funit, file = fpath // 'Rmhoursmean_Mf.asc', status = "old")
        do mX = 1, NMARKET
            do pX1 = 1, NTYPE1
                read(funit, *, iostat = istat) mom%Rmhours_Mf(pX1, mX)
            end do
        end do
        close(funit)

        open(newunit = funit, file = fpath // 'Rfhoursmean_mF.asc', status = "old")
        do mX = 1, NMARKET
            do pX2 = 1, NTYPE2
                read(funit, *, iostat = istat) mom%Rfhours_mF(pX2, mX)
            end do
        end do
        close(funit)

        open(newunit = funit, file = fpath // 'Rmwork_M0.asc', status = "old")
        do mX = 1, NMARKET
            do pX1 = 1, NTYPE1
                read(funit, *, iostat = istat) mom%Rmwork_M0(pX1, mX)
            end do
        end do
        close(funit)

        open(newunit = funit, file = fpath // 'Rfwork_0F.asc', status = "old")
        do mX = 1, NMARKET
            do pX2 = 1, NTYPE2
                read(funit, *, iostat = istat) mom%Rfwork_0F(pX2, mX)
            end do
        end do
        close(funit)

        open(newunit = funit, file = fpath // 'Rmwork_Mf.asc', status = "old")
        do mX = 1, NMARKET
            do pX1 = 1, NTYPE1
                read(funit, *, iostat = istat) mom%Rmwork_Mf(pX1, mX)
            end do
        end do
        close(funit)

        open(newunit = funit, file = fpath // 'Rfwork_mF.asc', status = "old")
        do mX = 1, NMARKET
            do pX2 = 1, NTYPE2
                read(funit, *, iostat = istat) mom%Rfwork_mF(pX2, mX)
            end do
        end do
        close(funit)

        ! joint labour supply allocation

        open(newunit = funit, file = fpath // 'mhours_M0.asc', status = "old")
        do kX = 0, 1
            do pX1 = 1, NTYPE1
                do hX1 = 0, 2
                    read(funit, *, iostat = istat) mom%mhours_M0(hX1, pX1, kX)
                end do
            end do
        end do
        close(funit)


        open(newunit = funit, file = fpath // 'fhours_0F.asc', status = "old")
        do kX = 0, 1
            do pX2 = 1, NTYPE2
                do hX2 = 0, 2
                    read(funit, *, iostat = istat) mom%fhours_0F(hX2, pX2, kX)
                end do
            end do
        end do
        close(funit)


        open(newunit = funit, file = fpath // 'mhours_MF.asc', status = "old")
        do kX = 0, 1
            do pX2 = 1, NTYPE2
                do pX1 = 1, NTYPE1
                    do hX1 = 0, 2
                        read(funit, *, iostat = istat) mom%mhours_MF(hX1, pX1, pX2, kX)
                    end do
                end do
            end do
        end do
        close(funit)

        open(newunit = funit, file = fpath // 'fhours_MF.asc', status = "old")
        do kX = 0, 1
            do pX2 = 1, NTYPE2
                do pX1 = 1, NTYPE1
                    do hX2 = 0, 2
                        read(funit, *, iostat = istat) mom%fhours_MF(hX2, pX1, pX2, kX)
                    end do
                end do
            end do
        end do
        close(funit)



        open(newunit = funit, file = fpath // 'mhoursmean_M0.asc', status = "old")
        do kX = 0, 1
            do pX1 = 1, NTYPE1
                read(funit, *, iostat = istat) mom%mhoursh_M0(pX1, kX)
            end do
        end do
        close(funit)

        open(newunit = funit, file = fpath // 'mhourssd_M0.asc', status = "old")
        do kX = 0, 1
            do pX1 = 1, NTYPE1
                read(funit, *, iostat = istat) mom%mhourshsd_M0(pX1, kX)
            end do
        end do
        close(funit)

        open(newunit = funit, file = fpath // 'fhoursmean_0F.asc', status = "old")
        do kX = 0, 1
            do pX2 = 1, NTYPE2
                read(funit, *, iostat = istat) mom%fhoursh_0F(pX2, kX)
            end do
        end do
        close(funit)

        open(newunit = funit, file = fpath // 'fhourssd_0F.asc', status = "old")
        do kX = 0, 1
            do pX2 = 1, NTYPE2
                read(funit, *, iostat = istat) mom%fhourshsd_0F(pX2, kX)
            end do
        end do
        close(funit)

        open(newunit = funit, file = fpath // 'mhoursmean_Mf.asc', status = "old")
        do kX = 0, 1
            do pX1 = 1, NTYPE1
                read(funit, *, iostat = istat) mom%mhoursh_Mf(pX1, kX)
            end do
        end do
        close(funit)

        open(newunit = funit, file = fpath // 'mhourssd_Mf.asc', status = "old")
        do kX = 0, 1
            do pX1 = 1, NTYPE1
                read(funit, *, iostat = istat) mom%mhourshsd_Mf(pX1, kX)
            end do
        end do
        close(funit)

        open(newunit = funit, file = fpath // 'fhoursmean_mF.asc', status = "old")
        do kX = 0, 1
            do pX2 = 1, NTYPE2
                read(funit, *, iostat = istat) mom%fhoursh_mF(pX2, kX)
            end do
        end do
        close(funit)

        open(newunit = funit, file = fpath // 'fhourssd_mF.asc', status = "old")
        do kX = 0, 1
            do pX2 = 1, NTYPE2
                read(funit, *, iostat = istat) mom%fhourshsd_mF(pX2, kX)
            end do
        end do
        close(funit)

        open(newunit = funit, file = fpath // 'mhoursh_M_F.asc', status = "old")
        do pX2 = 1, NTYPE2
            do pX1 = 1, NTYPE1
                read(funit, *, iostat = istat) mom%mhoursh_M_F(pX1, pX2)
            end do
        end do
        close(funit)

        open(newunit = funit, file = fpath // 'fhoursh_M_F.asc', status = "old")
        do pX2 = 1, NTYPE2
            do pX1 = 1, NTYPE1
                read(funit, *, iostat = istat) mom%fhoursh_M_F(pX1, pX2)
            end do
        end do
        close(funit)

        open(newunit = funit, file = fpath // 'mwork_M_F.asc', status = "old")
        do pX2 = 1, NTYPE2
            do pX1 = 1, NTYPE1
                read(funit, *, iostat = istat) mom%mwork_M_F(pX1, pX2)
            end do
        end do
        close(funit)

        open(newunit = funit, file = fpath // 'fwork_M_F.asc', status = "old")
        do pX2 = 1, NTYPE2
            do pX1 = 1, NTYPE1
                read(funit, *, iostat = istat) mom%fwork_M_F(pX1, pX2)
            end do
        end do
        close(funit)

        ! wage distribution all workers (single men, integrates over kids)

        open(newunit = funit, file = fpath // 'mwageMwAll_M0.asc', status = "old")
        do pX1 = 1, NTYPE1
            read(funit, *, iostat = istat) mom%mwageMwAll_M0(pX1)
        end do
        close(funit)

        open(newunit = funit, file = fpath // 'mwagesdMwAll_M0.asc', status = "old")
        do pX1 = 1, NTYPE1
            read(funit, *, iostat = istat) mom%mwagesdMwAll_M0(pX1)
        end do
        close(funit)


        ! wage distribution all workers (single women, integrates over kids)

        open(newunit = funit, file = fpath // 'fwageFwAll_0F.asc', status = "old")
        do pX1 = 1, NTYPE1
            read(funit, *, iostat = istat) mom%fwageFwAll_0F(pX1)
        end do
        close(funit)

        open(newunit = funit, file = fpath // 'fwagesdFwAll_0F.asc', status = "old")
        do pX2 = 1, NTYPE2
            read(funit, *, iostat = istat) mom%fwagesdFwAll_0F(pX2)
        end do
        close(funit)


        ! wage distribution all workers (married men, integrates over spouse type/state and kids)

        open(newunit = funit, file = fpath // 'mwageMwAll_Mf.asc', status = "old")
        do pX1 = 1, NTYPE1
            read(funit, *, iostat = istat) mom%mwageMwAll_Mf(pX1)
        end do
        close(funit)

        open(newunit = funit, file = fpath // 'mwagesdMwAll_Mf.asc', status = "old")
        do pX1 = 1, NTYPE1
            read(funit, *, iostat = istat) mom%mwagesdMwAll_Mf(pX1)
        end do
        close(funit)


        ! wage distribution all workers (married women, integrates over spouse type/state and kids)

        open(newunit = funit, file = fpath // 'fwageFwAll_mF.asc', status = "old")
        do pX2 = 1, NTYPE2
            read(funit, *, iostat = istat) mom%fwageFwAll_mF(pX2)
        end do
        close(funit)

        open(newunit = funit, file = fpath // 'fwagesdFwAll_mF.asc', status = "old")
        do pX2 = 1, NTYPE2
            read(funit, *, iostat = istat) mom%fwagesdFwAll_mF(pX2)
        end do
        close(funit)

        ! earnings distribution all workers (single men, integrates over kids)

        open(newunit = funit, file = fpath // 'mearnMwAll_M0.asc', status = "old")
        do pX1 = 1, NTYPE1
            read(funit, *, iostat = istat) mom%mearnMwAll_M0(pX1)
        end do
        close(funit)

        open(newunit = funit, file = fpath // 'mearnsdMwAll_M0.asc', status = "old")
        do pX1 = 1, NTYPE1
            read(funit, *, iostat = istat) mom%mearnsdMwAll_M0(pX1)
        end do
        close(funit)


        ! earnings distribution all workers (single women, integrates over kids)

        open(newunit = funit, file = fpath // 'fearnFwAll_0F.asc', status = "old")
        do pX1 = 1, NTYPE1
            read(funit, *, iostat = istat) mom%fearnFwAll_0F(pX1)
        end do
        close(funit)

        open(newunit = funit, file = fpath // 'fearnsdFwAll_0F.asc', status = "old")
        do pX2 = 1, NTYPE2
            read(funit, *, iostat = istat) mom%fearnsdFwAll_0F(pX2)
        end do
        close(funit)


        ! earnings distribution all workers (married men, integrates over spouse type/state and kids)

        open(newunit = funit, file = fpath // 'mearnMwAll_Mf.asc', status = "old")
        do pX1 = 1, NTYPE1
            read(funit, *, iostat = istat) mom%mearnMwAll_Mf(pX1)
        end do
        close(funit)

        open(newunit = funit, file = fpath // 'mearnsdMwAll_Mf.asc', status = "old")
        do pX1 = 1, NTYPE1
            read(funit, *, iostat = istat) mom%mearnsdMwAll_Mf(pX1)
        end do
        close(funit)


        ! earnings distribution all workers (married women, integrates over spouse type/state and kids)

        open(newunit = funit, file = fpath // 'fearnFwAll_mF.asc', status = "old")
        do pX2 = 1, NTYPE2
            read(funit, *, iostat = istat) mom%fearnFwAll_mF(pX2)
        end do
        close(funit)

        open(newunit = funit, file = fpath // 'fearnsdFwAll_mF.asc', status = "old")
        do pX2 = 1, NTYPE2
            read(funit, *, iostat = istat) mom%fearnsdFwAll_mF(pX2)
        end do
        close(funit)




        ! **** ATUS ****



        ! marginal distribution of home production time (single men/women)

        open(newunit = funit, file = fpath // 'mhome_M0.asc', status = "old")
        do kX = 0, 1
            do pX1 = 1, NTYPE1
                do hX1 = 1, NHH
                    read(funit, *, iostat = istat) mom%mhome_M0(hX1, pX1, kX)
                end do
            end do
        end do
        close(funit)

        open(newunit = funit, file = fpath // 'fhome_0F.asc', status = "old")
        do kX = 0, 1
            do pX2 = 1, NTYPE2
                do hX2 = 1, NHH
                    read(funit, *, iostat = istat) mom%fhome_0F(hX2, pX2, kX)
                end do
            end do
        end do
        close(funit)



        ! marginal distribution of home production time (married men/women)

        open(newunit = funit, file = fpath // 'mhome_Mf.asc', status = "old")
        do kX = 0, 1
            do pX1 = 1, NTYPE1
                do hX1 = 1, NHH
                    read(funit, *, iostat = istat) mom%mhome_Mf(hX1, pX1, kX)
                end do
            end do
        end do
        close(funit)

        open(newunit = funit, file = fpath // 'fhome_mF.asc', status = "old")
        do kX = 0, 1
            do pX2 = 1, NTYPE2
                do hX2 = 1, NHH
                    read(funit, *, iostat = istat) mom%fhome_mF(hX2, pX2, kX)
                end do
            end do
        end do
        close(funit)



        ! home production hours (single men/women)

        open(newunit = funit, file = fpath // 'mhomeh_M0.asc', status = "old")
        do kX = 0, 1
            do pX1 = 1, NTYPE1
                read(funit, *, iostat = istat) mom%mhomeh_M0(pX1, kX)
            end do
        end do
        close(funit)

        open(newunit = funit, file = fpath // 'mhomehsd_M0.asc', status = "old")
        do kX = 0, 1
            do pX1 = 1, NTYPE1
                read(funit, *, iostat = istat) mom%mhomehsd_M0(pX1, kX)
            end do
        end do
        close(funit)

        open(newunit = funit, file = fpath // 'fhomeh_0F.asc', status = "old")
        do kX = 0, 1
            do pX2 = 1, NTYPE2
                read(funit, *, iostat = istat) mom%fhomeh_0F(pX2, kX)
            end do
        end do
        close(funit)

        open(newunit = funit, file = fpath // 'fhomehsd_0F.asc', status = "old")
        do kX = 0, 1
            do pX2 = 1, NTYPE2
                read(funit, *, iostat = istat) mom%fhomehsd_0F(pX2, kX)
            end do
        end do
        close(funit)


        ! home production hours (married men/women)

        open(newunit = funit, file = fpath // 'mhomeh_Mf.asc', status = "old")
        do kX = 0, 1
            do pX1 = 1, NTYPE1
                read(funit, *, iostat = istat) mom%mhomeh_Mf(pX1, kX)
            end do
        end do
        close(funit)

        open(newunit = funit, file = fpath // 'mhomehsd_Mf.asc', status = "old")
        do kX = 0, 1
            do pX1 = 1, NTYPE1
                read(funit, *, iostat = istat) mom%mhomehsd_Mf(pX1, kX)
            end do
        end do
        close(funit)

        open(newunit = funit, file = fpath // 'fhomeh_mF.asc', status = "old")
        do kX = 0, 1
            do pX2 = 1, NTYPE2
                read(funit, *, iostat = istat) mom%fhomeh_mF(pX2, kX)
            end do
        end do
        close(funit)

        open(newunit = funit, file = fpath // 'fhomehsd_mF.asc', status = "old")
        do kX = 0, 1
            do pX2 = 1, NTYPE2
                read(funit, *, iostat = istat) mom%fhomehsd_mF(pX2, kX)
            end do
        end do
        close(funit)

        ! home hours by match distribution
        open(newunit = funit, file = fpath // 'mhomeh_M_F.asc', status = "old")
        do pX2 = 1, NTYPE2
            do pX1 = 1, NTYPE1
                read(funit, *, iostat = istat) mom%mhomeh_M_F(pX1, pX2)
            end do
        end do
        close(funit)

        open(newunit = funit, file = fpath // 'fhomeh_M_F.asc', status = "old")
        do pX2 = 1, NTYPE2
            do pX1 = 1, NTYPE1
                read(funit, *, iostat = istat) mom%fhomeh_M_F(pX1, pX2)
            end do
        end do
        close(funit)

    end subroutine readMomentFiles2

    ! calculate the empiricial covariance matrix for the moments

    subroutine readMomentVarianceMatrix(opt, vcov)

        use mathfn

        implicit none

        type(opt_t), intent(in) :: opt
        real(dp), intent(out) :: vcov(NMOM, NMOM)

        type(mom_t), allocatable :: mom(:)
        type(mom_t) :: momMode
        integer :: ixR, ixM0, ixM1
        real(dp) :: momModeVec(NMOM)
        real(dp), allocatable :: momVec(:, :)
        real(dp), allocatable :: momVecTr(:, :)
        real(dp) :: sd0, sd1, mean0, mean1, corr
        integer :: nr

        integer :: funit, istat

        open(newunit = funit, file = 'input/moments/' // opt%locname // '/nsamples.asc', status = "old")
        read(funit, *, iostat = istat) nr
        close(funit)

        allocate(momVec(NMOM,nr))
        allocate(momVecTr(nr,NMOM))
        allocate(mom(nr))

        ! read the mode (data source identifiers)
        call readMomentFiles2( 'input/moments/' // opt%locname, 'mode', momMode )
        momModeVec = transfer( momMode, (/1.0_dp/) )

        ! read moment files for each replication
        do ixR = 1, nr
            call readMomentFiles2( 'input/moments/' // opt%locname, 'sample/' // intToStr(ixR), mom(ixR) )
            momVec(:, ixR) = transfer( mom(ixR), (/1.0_dp/) )
        end do

        momVecTr = transpose(momVec)

        do ixM1 = 1, NMOM
            do ixM0 = 1, NMOM
                ! if different data sources set covariance to identically zero
                if ( momModeVec(ixM0) == momModeVec(ixM1) ) then
                    call vcorr( momVecTr(:, ixM0), momVecTr(:, ixM1), sd0, mean0, sd1, mean1, corr, vcov(ixM0, ixM1) )
                else
                    vcov(ixM0, ixM1) = 0.0_dp
                end if
            end do
        end do

    end subroutine readMomentVarianceMatrix

    subroutine aggregateMoments2(m, eallM, eallF, eall, mom, momExtra)
        use momentIntegrate
        implicit none

        type(m_t),     intent(in) :: m
        type(eall_t),  intent(in) :: eallM(NTYPE1, 0:1)
        type(eall_t),  intent(in) :: eallF(NTYPE2, 0:1)
        type(eall2_t), intent(in) :: eall(NTYPE1, NTYPE2, 0:1)

        type(mom_t),      intent(out) :: mom
        type(momExtra_t), intent(out), optional :: momExtra

        ! these parameters are used where there are no conditioning event
        real(dp), parameter :: onesMF(NTYPE1, NTYPE2, 0:1) = 1.0_dp
        real(dp), parameter :: onesM(NTYPE1, 0:1) = 1.0_dp
        real(dp), parameter :: onesF(NTYPE2, 0:1) = 1.0_dp

        real(dp) :: typeSum

        ! indexing parameters
        integer :: hX1, hX2

        ! total number of households
        typeSum = sum(m%numSingle1) + sum(m%numSingle2) + sum(m%typeDemand)

        ! IPUMS MOMENTS
        ! -------------

        ! distribution of household types
        ! note that we always use typeDemand for couples
        ! in the equilibrium this equals typeSupply

        mom%Rjtype_M0 = 0.0_dp
        mom%Rjtype_0F = 0.0_dp
        mom%Rjtype_MF = 0.0_dp
        mom%Rmhours_M0 = 0.0_dp
        mom%Rfhours_0F = 0.0_dp
        mom%Rmhours_Mf = 0.0_dp
        mom%Rfhours_mF = 0.0_dp
        mom%Rmwork_M0 = 0.0_dp
        mom%Rfwork_0F = 0.0_dp
        mom%Rmwork_Mf = 0.0_dp
        mom%Rfwork_mF = 0.0_dp

        mom%Rjtype_M0(:, 1) = m%numSingle1
        mom%Rjtype_0F(:, 1) = m%numSingle2
        mom%Rjtype_MF(:,:,1) = m%typeDemand

        ! hours
        call integrateSingleM_Kids( m%numSingleKid1, eallM%work(1), eallM%hourshW, mom%Rmhours_M0(:, 1) )
        call integrateSingleF_Kids( m%numSingleKid2, eallF%work(1), eallF%hourshW, mom%Rfhours_0F(:, 1) )
        call integrateCouple_KidsF( m%typeDemandKid, eall%mwork(1), eall%mhourshMw, mom%Rmhours_Mf(:, 1) )
        call integrateCouple_KidsM( m%typeDemandKid, eall%fwork(1), eall%fhourshFw, mom%Rfhours_mF(:, 1) )

        ! employment
        call integrateSingleM_Kids( m%numSingleKid1, onesM,  eallM%work(1), mom%Rmwork_M0(:, 1) )
        call integrateSingleF_Kids( m%numSingleKid2, onesF,  eallF%work(1), mom%Rfwork_0F(:, 1) )
        call integrateCouple_KidsF( m%typeDemandKid, onesMF, eall%mwork(1), mom%Rmwork_Mf(:, 1) )
        call integrateCouple_KidsM( m%typeDemandKid, onesMF, eall%fwork(1), mom%Rfwork_mF(:, 1) )

        ! distribution of UN/PT/FT (hX1 index) for single men, conditional on (pX1, kX)
        do hX1 = 0, 2
            call integrateSingleM_Null( eallM%hours(hX1), mom%mhours_M0(hX1, :, :) )
        end do

        ! distribution of UN/PT/FT (hX2 index) for single women, conditional on (pX2, kX)
        do hX2 = 0, 2
            call integrateSingleF_Null( eallF%hours(hX2), mom%fhours_0F(hX2, :, :) )
        end do

        ! distribution of UN/PT/FT (hX1 index) for married men, conditional on (pX1, pX2, kX)
        do hX1 = 0, 2
            call integrateCouple_Null( eall%mhours(hX1), mom%mhours_MF(hX1, :, :,:) )
        end do

        ! distribution of UN/PT/FT (hX2 index) for married women, conditional on (pX1, pX2, kX)
        do hX2 = 0, 2
            call integrateCouple_Null( eall%fhours(hX2), mom%fhours_MF(hX2, :, :,:) )
        end do

        ! mean and s.d. of conditional work hours for single men, conditional on (pX1, kX)
        ! when extending to market, will need to use work(1)
        call integrateSingleMeanSdM_Null( eallM%hourshW, eallM%hourshWsd, mom%mhoursh_M0, mom%mhourshsd_M0 )

        ! mean and s.d. of conditional work hours for single women, conditional on (pX2, kX)
        ! when extending to market, will need to use work(1)
        call integrateSingleMeanSdF_Null( eallF%hourshW, eallF%hourshWsd, mom%fhoursh_0F, mom%fhourshsd_0F )



        ! mean and s.d. of conditional work hours for married men, integrating over type and state of
        ! spouse, and conditional on (pX1, kX)
        call integrateCoupleMeanSd_F( m%typeDemandKid, eall%mwork(1), eall%mhourshMw, eall%mhourshMwsd, &
                                      mom%mhoursh_Mf, mom%mhourshsd_Mf )

        ! mean and s.d. of conditional work hours for married women, integrating over type and state of
        ! spouse, and conditional on (pX2, kX)
        call integrateCoupleMeanSd_M( m%typeDemandKid, eall%fwork(1), eall%fhourshFw, eall%fhourshFwsd, &
                                      mom%fhoursh_mF, mom%fhourshsd_mF )

        call integrateCouple_Kids( m%typeDemandKid, eall%mwork(1), eall%mhourshMw, mom%mhoursh_M_F )
        call integrateCouple_Kids( m%typeDemandKid, eall%fwork(1), eall%fhourshFw, mom%fhoursh_M_F )

        call integrateCouple_Kids( m%typeDemandKid, onesMF, eall%mwork(1), mom%mwork_M_F )
        call integrateCouple_Kids( m%typeDemandKid, onesMF, eall%fwork(1), mom%fwork_M_F )

        ! mean and s.d. of accepted wages for single men, integerating over kids, and conditional on pX1
        ! when extending to market, will need to use work(1)
        call integrateSingleMeanSdM_Kids( m%numSingleKid1, eallM%work(1), eallM%wageWork, eallM%wagesdWork, mom%mwageMwAll_M0, &
                                          mom%mwagesdMwAll_M0 )

        ! mean and s.d. of accepted wages for single women, integrating over kids, and conditional on pX2
        call integrateSingleMeanSdF_Kids( m%numSingleKid2, eallF%work(1), eallF%wageWork, eallF%wagesdWork, mom%fwageFwAll_0F, &
                                          mom%fwagesdFwAll_0F )

        ! mean and s.d. of accepted wages for married men, integrating over kids and type and state of spouse, and
        ! conditional on pX1
        call integrateCoupleMeanSd_KidsF( m%typeDemandKid, eall%mwork(1), eall%mwageMw, eall%mwagesdMw, mom%mwageMwAll_Mf, &
                                          mom%mwagesdMwAll_Mf )

        ! mean and s.d. of accepted wages for married women, integrating over type and state of spouse, and
        ! conditional on (pX1, kX)
        call integrateCoupleMeanSd_KidsM( m%typeDemandKid, eall%fwork(1), eall%fwageFw, eall%fwagesdFw, mom%fwageFwAll_mF, &
                                          mom%fwagesdFwAll_mF )

        ! mean and s.d. of earnings for single men, conditional on (pX1)
        call integrateSingleMeanSdM_Kids( m%numSingleKid1, eallM%work(1), eallM%earnW, eallM%earnWsd, mom%mearnMwAll_M0, &
                                          mom%mearnsdMwAll_M0 )

        ! mean and s.d. of earnings for single women, conditional on (pX2)
        call integrateSingleMeanSdF_Kids( m%numSingleKid2, eallF%work(1), eallF%earnW, eallF%earnWsd, mom%fearnFwAll_0F, &
                                          mom%fearnsdFwAll_0F )

        ! mean and s.d. of earnings for married men, integrating over type and state of spouse, and
        ! conditional on (pX1)
        call integrateCoupleMeanSd_KidsF( m%typeDemandKid, eall%mwork(1), eall%mearnMw, eall%mearnMwsd, mom%mearnMwAll_Mf, &
                                          mom%mearnsdMwAll_Mf )

        ! mean and s.d. of earnings for married women, integrating over type and state of spouse, and
        ! conditional on (pX2)
        call integrateCoupleMeanSd_KidsM( m%typeDemandKid, eall%fwork(1), eall%fearnFw, eall%fearnFwsd, mom%fearnFwAll_mF, &
                                          mom%fearnsdFwAll_mF )

        ! ATUS MOMENTS
        ! ------------


        ! distribution of home time (1,...,NHH) (hX1 index) for single men, conditional on (pX1, kX)
        do hX1 = 1, NHH
            call integrateSingleM_Null( eallM%home(hX1), mom%mhome_M0(hX1, :, :) )
        end do

        ! distribution of home time (1,...,NHH) (hX2 index) for single women, conditional on (pX1, kX)
        do hX2 = 1, NHH
            call integrateSingleF_Null( eallF%home(hX2), mom%fhome_0F(hX2, :, :) )
        end do

        ! distribution of home time (1,...,NHH) (hX1 index) for married men, integrating over type and
        ! state of spouse, conditional on (pX1, kX)
        do hX1 = 1, NHH
            call integrateCouple_F( m%typeDemandKid, onesMF, eall%mhome(hX1), mom%mhome_Mf(hX1, :, :) )
        end do

        ! distribution of home time (1,...,NHH) (hX2 index) for married women, integrating over type and
        ! state of spouse, conditional on (pX2, kX)
        do hX2 = 1, NHH
            call integrateCouple_M( m%typeDemandKid, onesMF, eall%fhome(hX2), mom%fhome_mF(hX2, :, :) )
        end do

        ! mean and s.d. of unconditional home time for single men, conditional on (pX1, kX)
        call integrateSingleMeanSdM_Null( eallM%homeh, eallM%homehsd, mom%mhomeh_M0, mom%mhomehsd_M0 )

        ! mean and s.d. of unconditional home time for single women, conditional on (pX2, kX)
        call integrateSingleMeanSdF_Null( eallF%homeh, eallF%homehsd, mom%fhomeh_0F, mom%fhomehsd_0F )

        ! mean and s.d. of unconditional home time for married men, integrating over type and state
        ! of spouse, conditional on (pX1, kX)
        call integrateCoupleMeanSd_F( m%typeDemandKid, onesMF, eall%mhomeh, eall%mhomehsd, mom%mhomeh_Mf, mom%mhomehsd_Mf )

        ! mean and s.d. of unconditional home time for married women, integrating over type and state
        ! of spouse, conditional on (pX2, kX)
        call integrateCoupleMeanSd_M( m%typeDemandKid, onesMF, eall%fhomeh, eall%fhomehsd, mom%fhomeh_mF, mom%fhomehsd_mF )

        ! mean and s.d. of unconditional home time for married men, integrating over kids, conditional on (pX1, pX2)
        call integrateCouple_Kids( m%typeDemandKid, onesMF, eall%mhomeh, mom%mhomeh_M_F ) !, mom%mhomehsd_M_F )

        ! mean and s.d. of unconditional home time for married women, integrating over kids, conditional on (pX1, pX2)
        call integrateCouple_Kids( m%typeDemandKid, onesMF, eall%fhomeh, mom%fhomeh_M_F )!, mom%fhomehsd_M_F )

        ! these moments are not targetted moments
        if ( present(momExtra) ) then

            ! mean and s.d. of unconditional work hours for single men, conditional on (pX1, kX)
            call integrateSingleMeanSdM_Null( eallM%hoursh, eallM%hourshsd, momExtra%mhoursuh_M0, momExtra%mhoursuhsd_M0 )

            ! mean and s.d. of unconditional work hours for single women, conditional on (pX2, kX)
            call integrateSingleMeanSdF_Null( eallF%hoursh, eallF%hourshsd, momExtra%fhoursuh_0F, momExtra%fhoursuhsd_0F )

            ! mean and s.d. of unconditional work hours for married men, integrating over type and state
            ! of spouse, conditional on (pX1, kX)
            call integrateCoupleMeanSd_F( m%typeDemandKid, onesMF, eall%mhoursh, eall%mhourshsd, momExtra%mhoursuh_Mf, &
                                          momExtra%mhoursuhsd_Mf )

            ! mean and s.d. of unconditional work hours for married men, conditional on (pX1, pX2, kX)
            call integrateCoupleMeanSd_Null( eall%mhoursh, eall%mhourshsd, momExtra%mhoursuh_M_F, momExtra%mhoursuhsd_M_F )

            ! mean and s.d. of unconditional work hours for married women, integrating over type and state
            ! of spouse, conditional on (pX2, kX)
            call integrateCoupleMeanSd_M( m%typeDemandKid, onesMF, eall%fhoursh, eall%fhourshsd, momExtra%fhoursuh_mF, &
                                          momExtra%fhoursuhsd_mF )

            ! mean and s.d. of unconditional work hours for married women, conditional on (pX1, pX2, kX)
            call integrateCoupleMeanSd_Null( eall%fhoursh, eall%fhourshsd, momExtra%fhoursuh_M_F, momExtra%fhoursuhsd_M_F )

            ! joint distribution of UN/PT/FT for married men (hX1 index) and married women (hX2) index, conditional on (pX1, pX2, kX)
            do hX2 = 0, 2
                do hX1 = 0, 2
                    call integrateCouple_Null( eall%jhours(hX1, hX2), momExtra%jhours_MF(hX1, hX2, :, :,:) )
                end do
            end do

            ! distribution of UN/PT/FT (hX1 index) for married men, integrating over type and state of spouse,
            ! conditional on (pX1, kX)
            do hX1 = 0, 2
                call integrateCouple_F( m%typeDemandKid, onesMF, eall%mhours(hX1), momExtra%mhours_Mf(hX1, :, :) )
            end do

            ! distribution of UN/PT/FT (hX2 index) for married women, integrating over type and state of spouse,
            ! conditional on (pX1, kX)
            do hX2 = 0, 2
                call integrateCouple_M( m%typeDemandKid, onesMF, eall%fhours(hX2), momExtra%fhours_mF(hX2, :, :) )
            end do

            ! mean and s.d. of accepted wages for single men by PT/FT (hX1 index), conditional on (pX1, kX)
            ! when extending to market, need to use hours
            do hX1 = 1, 2
                call integrateSingleMeanSdM_Null( eallM%wage(hX1), eallM%wagesd(hX1), &
                                                  momExtra%mwage_M0(hX1, :, :), momExtra%mwagesd_M0(hX1, :, :) )
            end do

            ! mean and s.d. of accepted wages for single women by PT/FT (hX2 index), conditional on (pX1, kX)
            ! when extending to market, need to use hours
            do hX2 = 1, 2
                call integrateSingleMeanSdF_Null( eallF%wage(hX2), eallF%wagesd(hX2), &
                                                  momExtra%fwage_0F(hX2, :, :), momExtra%fwagesd_0F(hX2, :, :) )
            end do

            ! mean and s.d. of accepted wages for married men by own PT/FT (hX1 index), integrating over type and state of spouse,
            ! conditional on (pX1, kX)
            do hX1 = 1, 2
                call integrateCoupleMeanSd_F( m%typeDemandKid, eall%mhours(hX1), eall%mwage(hX1), eall%mwagesd(hX1), &
                                              momExtra%mwage_Mf(hX1, :, :), momExtra%mwagesd_Mf(hX1, :, :) )
            end do

            ! mean and s.d. of accepted wages for married women by own PT/FT (hX2 index), integrating over type and state of spouse,
            ! conditional on (pX2, kX)
            do hX2 = 1, 2
                call integrateCoupleMeanSd_M( m%typeDemandKid, eall%fhours(hX2), eall%fwage(hX2), eall%fwagesd(hX2), &
                                              momExtra%fwage_mF(hX2, :, :), momExtra%fwagesd_mF(hX2, :, :) )
            end do

            ! mean and s.d. of accepted wages for married men by SPOUSE work state (hX2 index), integrating over type of spouse,
            ! conditional on (pX1, kX)
            do hX2 = 0, 1
                call integrateCoupleMeanSd_F( m%typeDemandKid, eall%fwork(hX2), eall%mwageFw(hX2), eall%mwageFwsd(hX2), &
                                              momExtra%mwageFw_Mf(hX2, :, :), momExtra%mwageFwsd_Mf(hX2, :, :) )
            end do

            ! mean and s.d. of accepted wages for married women by SPOUSE work state (hX1 index), integrating over type of spouse,
            ! conditional on (pX2, kX)
            do hX1 = 0, 1
                call integrateCoupleMeanSd_M( m%typeDemandKid, eall%mwork(hX1), eall%fwageMw(hX1), eall%fwageMwsd(hX1), &
                                              momExtra%fwageMw_mF(hX1, :, :), momExtra%fwageMwsd_mF(hX1, :, :) )
            end do

            ! covariance of accepted wage product within a match, conditional on (pX1, pX2, kX)
            ! when extending to market, will need to jhours(1:2,1:2)
            call integrateCouple_Null( eall%jwagecov, momExtra%jwagecov_MF )

            ! mean and s.d. of accepted wages for single men, conditional on (pX1, kX)
            ! when extending to market, will need to use work(1)
            call integrateSingleMeanSdM_Null( eallM%wageWork, eallM%wagesdWork, momExtra%mwageMw_M0, momExtra%mwagesdMw_M0 )

            ! mean and s.d. of accepted wages for single women, conditional on (pX2, kX)
            ! when extending to market, will need to use work(1)
            call integrateSingleMeanSdF_Null( eallF%wageWork, eallF%wagesdWork, momExtra%fwageFw_0F, momExtra%fwagesdFw_0F )

            ! mean and s.d. of accepted wages for married men, integrating over type and state of spouse, and
            ! conditional on (pX1, kX)
            call integrateCoupleMeanSd_F( m%typeDemandKid, eall%mwork(1), eall%mwageMw, eall%mwagesdMw, momExtra%mwageMw_Mf, &
                                          momExtra%mwagesdMw_Mf )

            ! mean and s.d. of accepted wages for married women, integrating over type and state of spouse, and
            ! conditional on (pX1, kX)
            call integrateCoupleMeanSd_M( m%typeDemandKid, eall%fwork(1), eall%fwageFw, eall%fwagesdFw, momExtra%fwageFw_mF, &
                                          momExtra%fwagesdFw_mF )

            ! mean and s.d. of accepted wage product within a match, conditional on (pX1, pX2, kX)
            ! when extending to market, will need to jwork
            call integrateCoupleMeanSd_Null( eall%jwage, eall%jwagesd, momExtra%jwage_MF, momExtra%jwagesd_MF )



            ! mean and s.d. of unconditional home time for single men by own work state (hX1 index), conditional on (pX1, kX)
            do hX1 = 0, 1
                call integrateSingleMeanSdM_M( m%numSingleKid1, eallM%work(hX1), eallM%homehW(hX1), eallM%homehWsd(hX1), &
                                               momExtra%mhomehMw_m0(hX1, :), momExtra%mhomehMwsd_m0(hX1, :) )
            end do

            ! mean and s.d. of unconditional home time for single women by own work state (hX2 index), conditional on (pX2, kX)
            do hX2 = 0, 2
                call integrateSingleMeanSdF_F( m%numSingleKid2, eallF%hours(hX2), eallF%homehH(hX2), eallF%homehHsd(hX2), &
                                               momExtra%fhomehFh_0f(hX2, :), momExtra%fhomehFhsd_0f(hX2, :) )
            end do

            ! mean and s.d. of unconditional home time for married men by own work state (hX1 index), integrating over type and
            ! state of spouse, conditional on (pX1, kX)
            do hX1 = 0, 1
                call integrateCoupleMeanSd_MF( m%typeDemandKid, eall%mwork(hX1), eall%mhomehMw(hX1), eall%mhomehMwsd(hX1), &
                                               momExtra%mhomehMw_mf(hX1, :), momExtra%mhomehMwsd_mf(hX1, :) )
            end do

            ! mean and s.d. of unconditional home time for married women by own work state (hX2 index), integrating over type and
            ! state of spouse, conditional on (pX2, kX)
            do hX2 = 0, 2
                call integrateCoupleMeanSd_MF( m%typeDemandKid, eall%fhours(hX2), eall%fhomehFh(hX2), eall%fhomehFhsd(hX2), &
                                               momExtra%fhomehFh_mf(hX2, :), momExtra%fhomehFhsd_mf(hX2, :) )
            end do

            ! mean and s.d. of unconditional home time for married men by SPOUSE work state (hX2 index), integrating over type
            ! of spouse, conditional on (pX1, kX)
            do hX2 = 0, 1
                call integrateCoupleMeanSd_MF( m%typeDemandKid, eall%fwork(hX2), eall%mhomehFw(hX2), eall%mhomehFwsd(hX2), &
                                               momExtra%mhomehFw_mf(hX2, :), momExtra%mhomehFwsd_mf(hX2, :) )
            end do

            ! mean and s.d. of unconditional home time for married men by SPOUSE work state (hX1 index), integrating over type
            ! of spouse, conditional on (pX2, kX)
            do hX1 = 0, 1
                call integrateCoupleMeanSd_MF( m%typeDemandKid, eall%mwork(hX1), eall%fhomehMw(hX1), eall%fhomehMwsd(hX1), &
                                               momExtra%fhomehMw_mf(hX1, :), momExtra%fhomehMwsd_mf(hX1, :) )
            end do

            ! aggregated work hours cell

            do hX1 = 0, 2
                call integrateSingleM_M( m%numSingleKid1, onesM, eallM%hours(hX1), momExtra%mhours_sin(hX1, :) )
            end do

            do hX2 = 0, 2
                call integrateSingleF_F( m%numSingleKid2, onesF, eallF%hours(hX2), momExtra%fhours_sin(hX2, :) )
            end do

            do hX1 = 0, 2
                call integrateCouple_MF( m%typeDemandKid, onesMF, eall%mhours(hX1), momExtra%mhours_cou(hX1, :) )
            end do

            do hX2 = 0, 2
                call integrateCouple_MF( m%typeDemandKid, onesMF, eall%fhours(hX2), momExtra%fhours_cou(hX2, :) )
            end do

            ! aggregated work home cell

            do hX1 = 1, NHH
                call integrateSingleM_M( m%numSingleKid1, onesM, eallM%home(hX1), momExtra%mhome_sin(hX1, :) )
            end do

            do hX2 = 1, NHH
                call integrateSingleF_F( m%numSingleKid2, onesF, eallF%home(hX2), momExtra%fhome_sin(hX2, :) )
            end do

            do hX1 = 1, NHH
                call integrateCouple_MF( m%typeDemandKid, onesMF, eall%mhome(hX1), momExtra%mhome_cou(hX1, :) )
            end do

            do hX2 = 1, NHH
                call integrateCouple_MF( m%typeDemandKid, onesMF, eall%fhome(hX2), momExtra%fhome_cou(hX2, :) )
            end do

            momExtra%jtype_M0 = m%numSingle1
            momExtra%jtype_0F = m%numSingle2
            momExtra%jtype_MF = m%typeDemand

            call integrateCouple_KidsMF( m%typeDemandKid, onesMF, eall%mwork(1), momExtra%mwork_cou_knk )
            call integrateCouple_KidsMF( m%typeDemandKid, onesMF, eall%mhoursh, momExtra%mhoursh_cou_knk )
            call integrateCouple_KidsMF( m%typeDemandKid, eall%mwork(1), eall%mhourshMw, momExtra%mhourshMw_cou_knk )
            call integrateCouple_KidsMF( m%typeDemandKid, onesMF, eall%mhomeh, momExtra%mhomeh_cou_knk )
            call integrateCouple_KidsMF( m%typeDemandKid, eall%mwork(1), eall%Eearn1, momExtra%mearnMw_cou_knk )

            call integrateCouple_KidsMF( m%typeDemandKid, onesMF, eall%fwork(1), momExtra%fwork_cou_knk )
            call integrateCouple_KidsMF( m%typeDemandKid, onesMF, eall%fhoursh, momExtra%fhoursh_cou_knk )
            call integrateCouple_KidsMF( m%typeDemandKid, eall%fwork(1), eall%fhourshFw, momExtra%fhourshFw_cou_knk )
            call integrateCouple_KidsMF( m%typeDemandKid, onesMF, eall%fhomeh, momExtra%fhomeh_cou_knk )
            call integrateCouple_KidsMF( m%typeDemandKid, eall%fwork(1), eall%Eearn2, momExtra%fearnFw_cou_knk )

            call integrateSingleM_KidsM( m%numSingleKid1, onesM, eallM%work(1), momExtra%mwork_sin_knk )
            call integrateSingleM_KidsM( m%numSingleKid1, onesM, eallM%hoursh, momExtra%mhoursh_sin_knk )
            call integrateSingleM_KidsM( m%numSingleKid1, eallM%work(1), eallM%hourshW, momExtra%mhourshMw_sin_knk )
            call integrateSingleM_KidsM( m%numSingleKid1, onesM, eallM%homeh, momExtra%mhomeh_sin_knk )
            call integrateSingleM_KidsM( m%numSingleKid1, eallM%work(1), eallM%Eearn, momExtra%mearnMw_sin_knk )

            call integrateSingleF_KidsF( m%numSingleKid2, onesF, eallF%work(1), momExtra%fwork_sin_knk )
            call integrateSingleF_KidsF( m%numSingleKid2, onesF, eallF%hoursh, momExtra%fhoursh_sin_knk )
            call integrateSingleF_KidsF( m%numSingleKid2, eallF%work(1), eallF%hourshW, momExtra%fhourshFw_sin_knk )
            call integrateSingleF_KidsF( m%numSingleKid2, onesF, eallF%homeh, momExtra%fhomeh_sin_knk )
            call integrateSingleF_KidsF( m%numSingleKid2, eallF%work(1), eallF%Eearn, momExtra%fearnFw_sin_knk )

            ! moved from mom
            call integrateSingleM_Kids( m%numSingleKid1, eallM%work(1), eallM%hourshW, momExtra%mhoursh_M_0 )
            call integrateSingleF_Kids( m%numSingleKid2, eallF%work(1), eallF%hourshW, momExtra%fhoursh_0_F )

            call integrateSingleM_Kids( m%numSingleKid1, onesM, eallM%work(1), momExtra%mwork_M_0 )
            call integrateSingleF_Kids( m%numSingleKid2, onesF, eallF%work(1), momExtra%fwork_0_F )

            ! mean and s.d. of unconditional home time for single men, integrating over kids, conditional on (pX1)
            call integrateSingleM_Kids( m%numSingleKid1, onesM, eallM%homeh, momExtra%mhomeh_M_0 ) !, momExtra%mhomehsd_M_F )

            ! mean and s.d. of unconditional home time for single women, integrating over kids, conditional on (pX2)
            call integrateSingleF_Kids( m%numSingleKid2, onesF, eallF%homeh, momExtra%fhomeh_0_F ) !, momExtra%mhomehsd_M_F )

        end if ! present(momExtra)

    end subroutine aggregateMoments2


    subroutine aggregateMomentsMarket2(m, eallM, eallF, eall, mom, momExtra)
        use momentIntegrate
        implicit none
        type(m_t),     intent(in) :: m(NMARKET)
        type(eall_t),  intent(in) :: eallM(NTYPE1, 0:1, NMARKET)
        type(eall_t),  intent(in) :: eallF(NTYPE2, 0:1, NMARKET)
        type(eall2_t), intent(in) :: eall(NTYPE1, NTYPE2, 0:1, NMARKET)

        type(mom_t),      intent(out) :: mom
        type(momExtra_t), intent(out), optional :: momExtra

        ! these parameters are used where there are no conditioning event
        real(dp), parameter :: onesMF(NTYPE1, NTYPE2, 0:1, NMARKET) = 1.0_dp
        real(dp), parameter :: onesM(NTYPE1, 0:1, NMARKET) = 1.0_dp
        real(dp), parameter :: onesF(NTYPE2, 0:1, NMARKET) = 1.0_dp

        real(dp) :: typeSum(NMARKET)
        real(dp) :: typeSumAll

        real(dp) :: measureSingleM(NTYPE1, 0:1, NMARKET)
        real(dp) :: measureSingleF(NTYPE2, 0:1, NMARKET)
        real(dp) :: measureCouple(NTYPE1, NTYPE2, 0:1, NMARKET)

        ! indexing parameters
        integer :: mX, hX1, hX2, pX1, pX2

        real(dp) :: Ewmwf, Ewm, Ewf
        real(dp) :: Sdwm, Sdwf

        integer :: ixF, ixM, ixKid

        ! total number of households in each market mX
        typeSumAll = 0.0_dp
        do mX = 1, NMARKET
            typeSum(mX) = sum(m(mX)%numSingle1) + sum(m(mX)%numSingle2) + sum(m(mX)%typeDemand)
            typeSumAll = typeSumAll + m(mX)%marketSize * typeSum(mX)
        end do

        ! total measures across markets mX
        do mX = 1, NMARKET
            measureCouple(:, :, :, mX) = m(mX)%marketSize * m(mX)%typeDemandKid
        end do

        do mX = 1, NMARKET
            measureSingleM(:, :, mX) = m(mX)%marketSize * m(mX)%numSingleKid1
        end do

        do mX = 1, NMARKET
            measureSingleF(:, :, mX) = m(mX)%marketSize * m(mX)%numSingleKid2
        end do

        call integrateCouple_KidsMarketMF( measureCouple, eall%jwork, eall%jwage, Ewmwf )
        call integrateCoupleMeanSd_KidsMarketMF( measureCouple, eall%jwork, eall%mwageJw, eall%mwagesdJw, Ewm, Sdwm )
        call integrateCoupleMeanSd_KidsMarketMF( measureCouple, eall%jwork, eall%fwageJw, eall%fwagesdJw, Ewf, Sdwf )

        ! IPUMS MOMENTS
        ! -------------

        ! distribution of household types by market
        ! note that we always use typeDemand for couples
        ! in the equilibrium this equals typeSupply

        do mX = 1, NMARKET
            mom%Rjtype_M0(:, mX)   = m(mX)%numSingle1
            mom%Rjtype_0F(:, mX)   = m(mX)%numSingle2
            mom%Rjtype_MF(:, :, mX) = m(mX)%typeDemand
        end do

        ! hours
        do mX = 1, NMARKET
            call integrateSingleM_Kids( m(mX)%numSingleKid1, eallM(:, :, mX)%work(1), eallM(:, :, mX)%hourshW, mom%Rmhours_M0(:, mX) )
            call integrateSingleF_Kids( m(mX)%numSingleKid2, eallF(:, :, mX)%work(1), eallF(:, :, mX)%hourshW, mom%Rfhours_0F(:, mX) )
            call integrateCouple_KidsF( m(mX)%typeDemandKid, eall(:, :, :, mX)%mwork(1), eall(:, :, :, mX)%mhourshMw, &
                                        mom%Rmhours_Mf(:, mX) )
            call integrateCouple_KidsM( m(mX)%typeDemandKid, eall(:, :, :, mX)%fwork(1), eall(:, :, :, mX)%fhourshFw, &
                                        mom%Rfhours_mF(:, mX) )
        end do

        ! employment
        do mX = 1, NMARKET
            call integrateSingleM_Kids( m(mX)%numSingleKid1, onesM(:, :, mX),  eallM(:, :, mX)%work(1), mom%Rmwork_M0(:, mX) )
            call integrateSingleF_Kids( m(mX)%numSingleKid2, onesF(:, :, mX),  eallF(:, :, mX)%work(1), mom%Rfwork_0F(:, mX) )
            call integrateCouple_KidsF( m(mX)%typeDemandKid, onesMF(:, :, :, mX), eall(:, :, :, mX)%mwork(1), mom%Rmwork_Mf(:, mX) )
            call integrateCouple_KidsM( m(mX)%typeDemandKid, onesMF(:, :, :, mX), eall(:, :, :, mX)%fwork(1), mom%Rfwork_mF(:, mX) )
        end do

        ! distribution of UN/PT/FT (hX1 index) for single men, conditional on (pX1, kX)
        do hX1 = 0, 2
            call integrateSingleM_Market( measureSingleM, onesM, eallM%hours(hX1), mom%mhours_M0(hX1, :, :) )
        end do

        ! distribution of UN/PT/FT (hX2 index) for single women, conditional on (pX2, kX)
        do hX2 = 0, 2
            call integrateSingleF_Market( measureSingleF, onesF, eallF%hours(hX2), mom%fhours_0F(hX2, :, :) )
        end do

        ! distribution of UN/PT/FT (hX1 index) for married men, conditional on (pX1, pX2, kX)
        do hX1 = 0, 2
            call integrateCouple_Market( measureCouple, onesMF, eall%mhours(hX1), mom%mhours_MF(hX1, :, :, :) )
        end do

        ! distribution of UN/PT/FT (hX2 index) for married women, conditional on (pX1, pX2, kX)
        do hX2 = 0, 2
            call integrateCouple_Market( measureCouple, onesMF, eall%fhours(hX2), mom%fhours_MF(hX2, :, :, :) )
        end do

        ! mean and s.d. of conditional work hours for single men, conditional on (pX1, kX)
        call integrateSingleMeanSdM_Market( measureSingleM, eallM%work(1), eallM%hourshW, eallM%hourshWsd, mom%mhoursh_M0, &
                                            mom%mhourshsd_M0 )

        ! mean and s.d. of conditional work hours for single women, conditional on (pX2, kX)
        call integrateSingleMeanSdF_Market( measureSingleF, eallF%work(1), eallF%hourshW, eallF%hourshWsd, mom%fhoursh_0F, &
                                            mom%fhourshsd_0F )

        ! mean and s.d. of conditional work hours for married men, integrating over type and state of
        ! spouse, and conditional on (pX1, kX)
        call integrateCoupleMeanSd_MarketF( measureCouple, eall%mwork(1), eall%mhourshMw, eall%mhourshMwsd, &
                                            mom%mhoursh_Mf, mom%mhourshsd_Mf )

        ! mean and s.d. of conditional work hours for married women, integrating over type and state of
        ! spouse, and conditional on (pX2, kX)
        call integrateCoupleMeanSd_MarketM( measureCouple, eall%fwork(1), eall%fhourshFw, eall%fhourshFwsd, &
                                            mom%fhoursh_mF, mom%fhourshsd_mF )

        call integrateCouple_KidsMarket( measureCouple, eall%mwork(1), eall%mhourshMW, mom%mhoursh_M_F )
        call integrateCouple_KidsMarket( measureCouple, eall%fwork(1), eall%fhourshFW, mom%fhoursh_M_F )

        call integrateCouple_KidsMarket( measureCouple, onesMF, eall%mwork(1), mom%mwork_M_F )
        call integrateCouple_KidsMarket( measureCouple, onesMF, eall%fwork(1), mom%fwork_M_F )

        ! mean and s.d. of accepted wages for single men, conditional on (pX1, kX)
        call integrateSingleMeanSdM_KidsMarket( measureSingleM, eallM%work(1), eallM%wageWork, eallM%wagesdWork, &
                                                mom%mwageMwAll_M0, mom%mwagesdMwAll_M0 )

        ! mean and s.d. of accepted wages for single women, conditional on (pX2, kX)
        call integrateSingleMeanSdF_KidsMarket( measureSingleF, eallF%work(1), eallF%wageWork, eallF%wagesdWork, &
                                                mom%fwageFwAll_0F, mom%fwagesdFwAll_0F )

        ! mean and s.d. of accepted wages for married men, integrating over type and state of spouse, and
        ! conditional on (pX1, kX)
        call integrateCoupleMeanSd_KidsMarketF( measureCouple, eall%mwork(1), eall%mwageMw, eall%mwagesdMw, &
                                                mom%mwageMwAll_Mf, mom%mwagesdMwAll_Mf )

        ! mean and s.d. of accepted wages for married women, integrating over type and state of spouse, and
        ! conditional on (pX1, kX)
        call integrateCoupleMeanSd_KidsMarketM( measureCouple, eall%fwork(1), eall%fwageFw, eall%fwagesdFw, mom%fwageFwAll_mF, &
                                                mom%fwagesdFwAll_mF )

        ! mean and s.d. of earnings for single men, conditional on (pX1)
        call integrateSingleMeanSdM_KidsMarket( measureSingleM, eallM%work(1), eallM%earnW, eallM%earnWsd, mom%mearnMwAll_M0, &
                                                mom%mearnsdMwAll_M0 )

        ! mean and s.d. of earnings for single women, conditional on (pX2)
        call integrateSingleMeanSdF_KidsMarket( measureSingleF, eallF%work(1), eallF%earnW, eallF%earnWsd, mom%fearnFwAll_0F, &
                                                mom%fearnsdFwAll_0F )

        ! mean and s.d. of earnings for married men, integrating over type and state of spouse, and
        ! conditional on (pX1)
        call integrateCoupleMeanSd_KidsMarketF( measureCouple, eall%mwork(1), eall%mearnMw, eall%mearnMwsd, mom%mearnMwAll_Mf, &
                                                mom%mearnsdMwAll_Mf )

        ! mean and s.d. of earnings for married women, integrating over type and state of spouse, and
        ! conditional on (pX2)
        call integrateCoupleMeanSd_KidsMarketM( measureCouple, eall%fwork(1), eall%fearnFw, eall%fearnFwsd, mom%fearnFwAll_mF, &
                                                mom%fearnsdFwAll_mF )


        ! ATUS MOMENTS
        ! ------------


        ! distribution of home time (1,...,NHH) (hX1 index) for single men, conditional on (pX1, kX)
        do hX1 = 1, NHH
            call integrateSingleM_Market( measureSingleM, onesM, eallM%home(hX1), mom%mhome_M0(hX1, :, :) )
        end do

        ! distribution of home time (1,...,NHH) (hX2 index) for single women, conditional on (pX1, kX)
        do hX2 = 1, NHH
            call integrateSingleF_Market( measureSingleF, onesF, eallF%home(hX2), mom%fhome_0F(hX2, :, :) )
        end do

        ! distribution of home time (1,...,NHH) (hX1 index) for married men, integrating over type and
        ! state of spouse, conditional on (pX1, kX)
        do hX1 = 1, NHH
            call integrateCouple_MarketF( measureCouple, onesMF, eall%mhome(hX1), mom%mhome_Mf(hX1, :, :) )
        end do

        ! distribution of home time (1,...,NHH) (hX2 index) for married women, integrating over type and
        ! state of spouse, conditional on (pX2, kX)
        do hX2 = 1, NHH
            call integrateCouple_MarketM( measureCouple, onesMF, eall%fhome(hX2), mom%fhome_mF(hX2, :, :) )
        end do

        ! mean and s.d. of unconditional home time for single men, conditional on (pX1, kX)
        call integrateSingleMeanSdM_Market( measureSingleM, onesM, eallM%homeh, eallM%homehsd, mom%mhomeh_M0, mom%mhomehsd_M0 )

        ! mean and s.d. of unconditional home time for single women, conditional on (pX2, kX)
        call integrateSingleMeanSdF_Market( measureSingleF, onesF, eallF%homeh, eallF%homehsd, mom%fhomeh_0F, mom%fhomehsd_0F )

        ! mean and s.d. of unconditional home time for married men, integrating over type and state
        ! of spouse, conditional on (pX1, kX)
        call integrateCoupleMeanSd_MarketF( measureCouple, onesMF, eall%mhomeh, eall%mhomehsd, mom%mhomeh_Mf, mom%mhomehsd_Mf )


        ! mean and s.d. of unconditional home time for married women, integrating over type and state
        ! of spouse, conditional on (pX2, kX)
        call integrateCoupleMeanSd_MarketM( measureCouple, onesMF, eall%fhomeh, eall%fhomehsd, mom%fhomeh_mF, mom%fhomehsd_mF )


        ! mean and s.d. of unconditional home time for married men, integrating over kids, conditional on (pX1, pX2)
        call integrateCouple_KidsMarket( measureCouple, onesMF, eall%mhomeh, mom%mhomeh_M_F )

        ! mean and s.d. of unconditional home time for married women, integrating over kids, conditional on (pX1, pX2)
        call integrateCouple_KidsMarket( measureCouple, onesMF, eall%fhomeh, mom%fhomeh_M_F )

        ! these moments are not targetted moments
        if ( present(momExtra) ) then

            ! mean and s.d. of unconditional work hours for single men, conditional on (pX1, kX)
            call integrateSingleMeanSdM_Market( measureSingleM, onesM, eallM%hoursh, eallM%hourshsd, momExtra%mhoursuh_M0, &
                                                momExtra%mhoursuhsd_M0 )

            ! mean and s.d. of unconditional work hours for single women, conditional on (pX2, kX)
            call integrateSingleMeanSdF_Market( measureSingleF, onesF, eallF%hoursh, eallF%hourshsd, momExtra%fhoursuh_0F, &
                                                momExtra%fhoursuhsd_0F )

            ! mean and s.d. of unconditional work hours for married men, integrating over type and state
            ! of spouse, conditional on (pX1, kX)
            call integrateCoupleMeanSd_MarketF( measureCouple, onesMF, eall%mhoursh, eall%mhourshsd, momExtra%mhoursuh_Mf, &
                                                momExtra%mhoursuhsd_Mf )

            ! mean and s.d. of unconditional work hours for married men, conditional on (pX1, pX2, kX)
            call integrateCoupleMeanSd_Market( measureCouple, onesMF, eall%mhoursh, eall%mhourshsd, momExtra%mhoursuh_M_F, &
                                               momExtra%mhoursuhsd_M_F )

            ! mean and s.d. of unconditional work hours for married women, integrating over type and state
            ! of spouse, conditional on (pX2, kX)
            call integrateCoupleMeanSd_MarketM( measureCouple, onesMF, eall%fhoursh, eall%fhourshsd, momExtra%fhoursuh_mF, &
                                                momExtra%fhoursuhsd_mF )

            ! mean and s.d. of unconditional work hours for married women, conditional on (pX1, pX2, kX)
            call integrateCoupleMeanSd_Market( measureCouple, onesMF, eall%fhoursh, eall%fhourshsd, momExtra%fhoursuh_M_F, &
                                               momExtra%fhoursuhsd_M_F )

            ! joint distribution of UN/PT/FT for married men (hX1 index) and married women (hX2) index, conditional on (pX1, pX2, kX)
            do hX2 = 0, 2
                do hX1 = 0, 2
                    call integrateCouple_Market( measureCouple, onesMF, eall%jhours(hX1, hX2), momExtra%jhours_MF(hX1, hX2, :, :, :) )
                end do
            end do

            ! distribution of UN/PT/FT (hX1 index) for married men, integrating over type and state of spouse,
            ! conditional on (pX1, kX)
            do hX1 = 0, 2
                call integrateCouple_MarketF( measureCouple, onesMF, eall%mhours(hX1), momExtra%mhours_Mf(hX1, :, :) )
            end do

            ! distribution of UN/PT/FT (hX2 index) for married women, integrating over type and state of spouse,
            ! conditional on (pX1, kX)
            do hX2 = 0, 2
                call integrateCouple_MarketM( measureCouple, onesMF, eall%fhours(hX2), momExtra%fhours_mF(hX2, :, :) )
            end do

            ! mean and s.d. of accepted wages for single men by PT/FT (hX1 index), conditional on (pX1, kX)
            ! when extending to market, need to use hours
            do hX1 = 1, 2
                call integrateSingleMeanSdM_Market( measureSingleM, eallM%hours(hX1), eallM%wage(hX1), eallM%wagesd(hX1), &
                                                    momExtra%mwage_M0(hX1, :, :), momExtra%mwagesd_M0(hX1, :, :) )
            end do

            ! mean and s.d. of accepted wages for single women by PT/FT (hX2 index), conditional on (pX1, kX)
            ! when extending to market, need to use hours
            do hX2 = 1, 2
                call integrateSingleMeanSdF_Market( measureSingleF, eallF%hours(hX2), eallF%wage(hX2), eallF%wagesd(hX2), &
                                                    momExtra%fwage_0F(hX2, :, :), momExtra%fwagesd_0F(hX2, :, :) )
            end do

            ! mean and s.d. of accepted wages for married men by own PT/FT (hX1 index), integrating over type and state of spouse,
            ! conditional on (pX1, kX)
            do hX1 = 1, 2
                call integrateCoupleMeanSd_MarketF( measureCouple, eall%mhours(hX1), eall%mwage(hX1), eall%mwagesd(hX1), &
                                                    momExtra%mwage_Mf(hX1, :, :), momExtra%mwagesd_Mf(hX1, :, :) )
            end do

            ! mean and s.d. of accepted wages for married women by own PT/FT (hX2 index), integrating over type and state of spouse,
            ! conditional on (pX2, kX)
            do hX2 = 1, 2
                call integrateCoupleMeanSd_MarketM( measureCouple, eall%fhours(hX2), eall%fwage(hX2), eall%fwagesd(hX2), &
                                                    momExtra%fwage_mF(hX2, :, :), momExtra%fwagesd_mF(hX2, :, :) )
            end do

            ! mean and s.d. of accepted wages for married men by SPOUSE work state (hX2 index), integrating over type of spouse,
            ! conditional on (pX1, kX)
            do hX2 = 0, 1
                call integrateCoupleMeanSd_MarketF( measureCouple, eall%fwork(hX2), eall%mwageFw(hX2), eall%mwageFwsd(hX2), &
                                                    momExtra%mwageFw_Mf(hX2, :, :), momExtra%mwageFwsd_Mf(hX2, :, :) )
            end do

            ! mean and s.d. of accepted wages for married women by SPOUSE work state (hX1 index), integrating over type of spouse,
            ! conditional on (pX2, kX)
            do hX1 = 0, 1
                call integrateCoupleMeanSd_MarketM( measureCouple, eall%mwork(hX1), eall%fwageMw(hX1), eall%fwageMwsd(hX1), &
                                                    momExtra%fwageMw_mF(hX1, :, :), momExtra%fwageMwsd_mF(hX1, :, :) )
            end do

            ! covariance of accepted wage product within a match, conditional on (pX1, pX2, kX)
            ! when extending to market, will need to jhours(1:2,1:2)
            call integrateCouple_Market( measureCouple, eall%jwork, eall%jwagecov, momExtra%jwagecov_MF )


            ! mean and s.d. of accepted wages for single men, conditional on (pX1, kX)
            call integrateSingleMeanSdM_Market( measureSingleM, eallM%work(1), eallM%wageWork, eallM%wagesdWork, &
                                                momExtra%mwageMw_M0, momExtra%mwagesdMw_M0 )

            ! mean and s.d. of accepted wages for single women, conditional on (pX2, kX)
            call integrateSingleMeanSdF_Market( measureSingleF, eallF%work(1), eallF%wageWork, eallF%wagesdWork, &
                                                momExtra%fwageFw_0F, momExtra%fwagesdFw_0F )

            ! mean and s.d. of accepted wages for married men, integrating over type and state of spouse, and
            ! conditional on (pX1, kX)
            call integrateCoupleMeanSd_MarketF( measureCouple, eall%mwork(1), eall%mwageMw, eall%mwagesdMw, momExtra%mwageMw_Mf, &
                                                momExtra%mwagesdMw_Mf )

            ! mean and s.d. of accepted wages for married women, integrating over type and state of spouse, and
            ! conditional on (pX1, kX)
            call integrateCoupleMeanSd_MarketM( measureCouple, eall%fwork(1), eall%fwageFw, eall%fwagesdFw, momExtra%fwageFw_mF, &
                                                momExtra%fwagesdFw_mF )

            ! mean and s.d. of accepted wage product within a match, conditional on (pX1, pX2, kX)
            ! when extending to market, will need to jhours(1:2,1:2)
            call integrateCoupleMeanSd_Market( measureCouple, eall%jwork, eall%jwage, eall%jwagesd, momExtra%jwage_MF, &
                                               momExtra%jwagesd_MF )


            ! mean and s.d. of unconditional home time for single men by own work state (hX1 index), conditional on (pX1, kX)
            do hX1 = 0, 1
                call integrateSingleMeanSdM_MarketM( measureSingleM, eallM%work(hX1), eallM%homehW(hX1), eallM%homehWsd(hX1), &
                                                     momExtra%mhomehMw_m0(hX1, :), momExtra%mhomehMwsd_m0(hX1, :) )
            end do

            ! mean and s.d. of unconditional home time for single women by own work state (hX2 index), conditional on (pX2, kX)
            do hX2 = 0, 2
                call integrateSingleMeanSdF_MarketF( measureSingleF, eallF%hours(hX2), eallF%homehH(hX2), eallF%homehHsd(hX2), &
                                                     momExtra%fhomehFh_0f(hX2, :), momExtra%fhomehFhsd_0f(hX2, :) )
            end do

            ! mean and s.d. of unconditional home time for married men by own work state (hX1 index), integrating over type and
            ! state of spouse, conditional on (pX1, kX)
            do hX1 = 0, 1
                call integrateCoupleMeanSd_MarketMF( measureCouple, eall%mwork(hX1), eall%mhomehMw(hX1), eall%mhomehMwsd(hX1), &
                                                     momExtra%mhomehMw_mf(hX1, :), momExtra%mhomehMwsd_mf(hX1, :) )
            end do

            ! mean and s.d. of unconditional home time for married women by own work state (hX2 index), integrating over type and
            ! state of spouse, conditional on (pX2, kX)
            do hX2 = 0, 2
                call integrateCoupleMeanSd_MarketMF( measureCouple, eall%fhours(hX2), eall%fhomehFh(hX2), eall%fhomehFhsd(hX2), &
                                                     momExtra%fhomehFh_mf(hX2, :), momExtra%fhomehFhsd_mf(hX2, :) )
            end do

            ! mean and s.d. of unconditional home time for married men by SPOUSE work state (hX2 index), integrating over type
            ! of spouse, conditional on (pX1, kX)
            do hX2 = 0, 1
                call integrateCoupleMeanSd_MarketMF( measureCouple, eall%fwork(hX2), eall%mhomehFw(hX2), eall%mhomehFwsd(hX2), &
                                                     momExtra%mhomehFw_mf(hX2, :), momExtra%mhomehFwsd_mf(hX2, :) )
            end do

            ! mean and s.d. of unconditional home time for married men by SPOUSE work state (hX1 index), integrating over type
            ! of spouse, conditional on (pX2, kX)
            do hX1 = 0, 1
                call integrateCoupleMeanSd_MarketMF( measureCouple, eall%mwork(hX1), eall%fhomehMw(hX1), eall%fhomehMwsd(hX1), &
                                                     momExtra%fhomehMw_mf(hX1, :), momExtra%fhomehMwsd_mf(hX1, :) )
            end do


            call integrateCoupleMeanSd_Market( measureCouple, onesMF, eall%mhomeh, eall%mhourshsd, &
                                               momExtra%mhomeh_M_F, momExtra%mhomehsd_M_F )

            call integrateCoupleMeanSd_Market( measureCouple, onesMF, eall%fhomeh, eall%fhourshsd, &
                                               momExtra%fhomeh_M_F, momExtra%fhomehsd_M_F )

            ! aggregated measures for elasticities

            call integrateCouple_MarketMF( measureCouple, onesMF, eall%mhoursh, momExtra%mhoursuh_cou )
            call integrateCouple_MarketMF( measureCouple, onesMF, eall%fhoursh, momExtra%fhoursuh_cou )
            call integrateCouple_MarketMF( measureCouple, onesMF, eall%mhourshMw, momExtra%mhoursh_cou )
            call integrateCouple_MarketMF( measureCouple, onesMF, eall%fhourshFw, momExtra%fhoursh_cou )
            call integrateCouple_MarketMF( measureCouple, onesMF, eall%mwork(1), momExtra%mwork_cou )
            call integrateCouple_MarketMF( measureCouple, onesMF, eall%fwork(1), momExtra%fwork_cou )
            call integrateCouple_MarketMF( measureCouple, onesMF, eall%mhomeh, momExtra%mhomeuh_cou )
            call integrateCouple_MarketMF( measureCouple, onesMF, eall%fhomeh, momExtra%fhomeuh_cou )


            call integrateSingleM_MarketM( measureSingleM, onesM, eallM%hoursh, momExtra%mhoursuh_sin )
            call integrateSingleF_MarketF( measureSingleF, onesF, eallF%hoursh, momExtra%fhoursuh_sin )
            call integrateSingleM_MarketM( measureSingleM, onesM, eallM%hourshW, momExtra%mhoursh_sin )
            call integrateSingleF_MarketF( measureSingleF, onesF, eallF%hourshW, momExtra%fhoursh_sin )
            call integrateSingleM_MarketM( measureSingleM, onesM, eallM%work(1), momExtra%mwork_sin )
            call integrateSingleF_MarketF( measureSingleF, onesF, eallF%work(1), momExtra%fwork_sin )
            call integrateSingleM_MarketM( measureSingleM, onesM, eallM%homeh, momExtra%mhomeuh_sin )
            call integrateSingleF_MarketF( measureSingleF, onesF, eallF%homeh, momExtra%fhomeuh_sin )

            ! aggregated work hours cell

            do hX1 = 0, 2
                call integrateSingleM_MarketM( measureSingleM, onesM, eallM%hours(hX1), momExtra%mhours_sin(hX1, :) )
            end do

            do hX2 = 0, 2
                call integrateSingleF_MarketF( measureSingleF, onesF, eallF%hours(hX2), momExtra%fhours_sin(hX2, :) )
            end do

            do hX1 = 0, 2
                call integrateCouple_MarketMF( measureCouple, onesMF, eall%mhours(hX1), momExtra%mhours_cou(hX1, :) )
            end do

            do hX2 = 0, 2
                call integrateCouple_MarketMF( measureCouple, onesMF, eall%fhours(hX2), momExtra%fhours_cou(hX2, :) )
            end do

            ! aggregated work home cell

            do hX1 = 1, NHH
                call integrateSingleM_MarketM( measureSingleM, onesM, eallM%home(hX1), momExtra%mhome_sin(hX1, :) )
            end do

            do hX2 = 1, NHH
                call integrateSingleF_MarketF( measureSingleF, onesF, eallF%home(hX2), momExtra%fhome_sin(hX2, :) )
            end do

            do hX1 = 1, NHH
                call integrateCouple_MarketMF( measureCouple, onesMF, eall%mhome(hX1), momExtra%mhome_cou(hX1, :) )
            end do

            do hX2 = 1, NHH
                call integrateCouple_MarketMF( measureCouple, onesMF, eall%fhome(hX2), momExtra%fhome_cou(hX2, :) )
            end do



            do pX1 = 1, NTYPE1
                momExtra%jtype_M0(pX1) = sum( measureSingleM(pX1, :, :) )
            end do

            do pX2 = 1, NTYPE2
                momExtra%jtype_0F(pX2) = sum( measureSingleF(pX2, :, :) )
            end do

            do pX2 = 1, NTYPE2
                do pX1 = 1, NTYPE1
                    momExtra%jtype_MF(pX1, pX2) = sum( measureCouple(pX1, pX2, :, :) )
                end do
            end do

            call integrateCouple_KidsMarketMF( measureCouple, onesMF, eall%mwork(1), momExtra%mwork_cou_knk )
            call integrateCouple_KidsMarketMF( measureCouple, onesMF, eall%mhoursh, momExtra%mhoursh_cou_knk )
            call integrateCouple_KidsMarketMF( measureCouple, eall%mwork(1), eall%mhourshMw, momExtra%mhourshMw_cou_knk )
            call integrateCouple_KidsMarketMF( measureCouple, onesMF, eall%mhomeh, momExtra%mhomeh_cou_knk )
            call integrateCouple_KidsMarketMF( measureCouple, eall%mwork(1), eall%mearnMw, momExtra%mearnMw_cou_knk )

            call integrateCouple_KidsMarketMF( measureCouple, onesMF, eall%fwork(1), momExtra%fwork_cou_knk )
            call integrateCouple_KidsMarketMF( measureCouple, onesMF, eall%fhoursh, momExtra%fhoursh_cou_knk )
            call integrateCouple_KidsMarketMF( measureCouple, eall%fwork(1), eall%fhourshFw, momExtra%fhourshFw_cou_knk )
            call integrateCouple_KidsMarketMF( measureCouple, onesMF, eall%fhomeh, momExtra%fhomeh_cou_knk )
            call integrateCouple_KidsMarketMF( measureCouple, eall%fwork(1), eall%fearnFw, momExtra%fearnFw_cou_knk )

            call integrateSingleM_KidsM( measureSingleM, onesM, eallM%work(1), momExtra%mwork_sin_knk )
            call integrateSingleM_KidsM( measureSingleM, onesM, eallM%hoursh, momExtra%mhoursh_sin_knk )
            call integrateSingleM_KidsM( measureSingleM, eallM%work(1), eallM%hourshW, momExtra%mhourshMw_sin_knk )
            call integrateSingleM_KidsM( measureSingleM, onesM, eallM%homeh, momExtra%mhomeh_sin_knk )
            call integrateSingleM_KidsM( measureSingleM, eallM%work(1), eallM%earnW, momExtra%mearnMw_sin_knk )

            call integrateSingleF_KidsF( measureSingleF, onesF, eallF%work(1), momExtra%fwork_sin_knk )
            call integrateSingleF_KidsF( measureSingleF, onesF, eallF%hoursh, momExtra%fhoursh_sin_knk )
            call integrateSingleF_KidsF( measureSingleF, eallF%work(1), eallF%hourshW, momExtra%fhourshFw_sin_knk )
            call integrateSingleF_KidsF( measureSingleF, onesF, eallF%homeh, momExtra%fhomeh_sin_knk )
            call integrateSingleF_KidsF( measureSingleF, eallF%work(1), eallF%earnW, momExtra%fearnFw_sin_knk )

            ! moved from mom
            call integrateSingleM_KidsMarket( measureSingleM, eallM%work(1), eallM%hourshW, momExtra%mhoursh_M_0 )
            call integrateSingleF_KidsMarket( measureSingleF, eallF%work(1), eallF%hourshW, momExtra%fhoursh_0_F )

            call integrateSingleM_KidsMarket( measureSingleM, onesM, eallM%work(1), momExtra%mwork_M_0 )
            call integrateSingleF_KidsMarket( measureSingleF, onesF, eallF%work(1), momExtra%fwork_0_F )

            ! mean and s.d. of unconditional home time for single men, integrating over kids, conditional on (pX1)
            call integrateSingleM_KidsMarket( measureSingleM, onesM, eallM%homeh, momExtra%mhomeh_M_0 )

            ! mean and s.d. of unconditional home time for single women, integrating over kids, conditional on (pX2)
            call integrateSingleF_KidsMarket( measureSingleF, onesF, eallF%homeh, momExtra%fhomeh_0_F )

        end if ! present(momExtra)

    end subroutine aggregateMomentsMarket2

end module moments
